/*
 * Velocity - A GNOME File Manager
 *
 *
 * Copyright (C) (2002) (Kyle Davis) <kyled@blueriver.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>

#include <config.h>
#include <glib.h>
#include <gdk/gdk.h>
#include <velocity-module-skel.h>
#include <velocity-sys.h>

//TODO:We need to allow configuring of all of these programs, adding of more, etc. (in which case we could make this a generic preview module)
gint preview_pid=(gint)NULL;
static gint spawnvp(char**argv){
	gint pid;
	if(g_spawn_async(NULL,argv,NULL,G_SPAWN_SEARCH_PATH|G_SPAWN_STDOUT_TO_DEV_NULL|G_SPAWN_STDERR_TO_DEV_NULL,NULL,NULL,&pid,NULL))return(pid);
	return((gint)NULL);
}
static void preview_ac3(char*filename){
	char*arg_list[]={"ac3dec",filename,NULL};
	preview_pid=spawnvp(arg_list);
}
static void preview_mp3(char*filename){
	char*arg_list[]={"mpg123",filename,NULL};
	preview_pid=spawnvp(arg_list);
}
static void preview_ogg(char*filename){
	char*arg_list[]={"ogg123","-d","esd",filename,NULL};	//TODO:  This should be in the options really...
	preview_pid=spawnvp(arg_list);
}
static void preview_wav(char*filename){   
	char*arg_list[]={"sox",filename,"-t","ossdsp","/dev/dsp",NULL};
	preview_pid=spawnvp(arg_list);
}
static void preview_midi(char*filename){
	char*arg_list[]={"timidity","-Oe",filename,NULL};	/*default to ESD*/
	preview_pid=(gint)NULL;
	preview_pid=spawnvp(arg_list);
	if(!preview_pid){	/*resort to direct DSP if we must*/
		arg_list[1]="-Od";
		preview_pid=spawnvp(arg_list);
	}
}
void stop_preview(velocity_vfs_obj*vfsobj){
	int retval=0;
	if(preview_pid){
		kill(preview_pid,SIGTERM);
		waitpid(preview_pid,&retval,WUNTRACED);
		preview_pid=(gint)NULL;
		if(vfsobj)velocity_icon_emblem_remove(vfsobj->icon,VELOCITY_EMBLEM_SOUND);
	}
}
void start_preview(velocity_vfs_obj*vfsobj){
	char*uri=velocity_vfs_get_uri_unescaped(vfsobj);
	stop_preview(NULL);	/*Stop any old audio processes*/
	//TODO: Shouldn't we use a look up array or something....
	if(strcasecmp("audio/ac3",vfsobj->info->mime_type)==0){	/*AC3 preview*/
		preview_ac3(uri);
	}else if(strcasecmp("audio/x-mp3",vfsobj->info->mime_type)==0){	/*MP3 preview*/
		preview_mp3(uri);
	}else if(strcasecmp("audio/x-riff",vfsobj->info->mime_type)==0){	/*WAV preview*/
		preview_wav(uri);
	}else if(strcasecmp("audio/x-wav",vfsobj->info->mime_type)==0){	/*WAV preview*/
		preview_wav(uri);
	}else if(strcasecmp("audio/x-midi",vfsobj->info->mime_type)==0){
		preview_midi(uri);
	}else if(strcasecmp("application/x-ogg",vfsobj->info->mime_type)==0){	/*OGG preview*/
		preview_ogg(uri);
	}
	if(uri){
		velocity_icon_emblem_add(vfsobj->icon,VELOCITY_EMBLEM_SOUND);
		g_free(uri);
	}
}

void*on_view_status_event(velocity_view*vview,velocity_view_status_event vevent,const char*uri,GnomeVFSResult result,gpointer user_data){
	switch(vevent){
		case VELOCITY_VIEW_STATUS_EVENT_STOP:
			stop_preview(NULL);
			break;
		default:
			break;
	}
	return(NULL);
}
void*on_view_behavior_event(velocity_view*vview,velocity_view_behavior_event vevent,GdkEvent*event,velocity_vfs_obj*vfsobj,gpointer user_data){
	switch(vevent){
		case VELOCITY_VIEW_BEHAVIOR_EVENT_OBJECT_LOADED:
			VELOCITY_MSG("ADD SONG INFO TO VFSOBJ_EXT HERE");
			break;
		case VELOCITY_VIEW_BEHAVIOR_EVENT_MOUSE_OVER:
			if(velocity_module_preferences_get_bool(module_info.id,"general","preview",TRUE)){
				if(velocity_vfs_is_local(vfsobj))start_preview(vfsobj);
			}
			break;
		case VELOCITY_VIEW_BEHAVIOR_EVENT_MOUSE_OUT:
			if(velocity_vfs_is_local(vfsobj))stop_preview(vfsobj);
			break;
		default:
			break;
	}
	return(NULL);	/*Because we'd rather act transparently*/
}
GtkWidget*preferences_new(velocity_preferences_dialog*vprefsdlg,const gchar*vuri){
	GtkWidget*b=gtk_vbox_new(FALSE,0);
	GtkWidget*t=gtk_check_button_new_with_label(_("Automatic audio preview"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(t),velocity_module_preferences_get_bool(module_info.id,"general","preview",TRUE));
	g_object_set_data(G_OBJECT(b),"preview",(gpointer)t);
	gtk_box_pack_start(GTK_BOX(b),t,FALSE,FALSE,0);	
	gtk_widget_show_all(b);
	return(b);
}
void preferences_free(velocity_preferences_dialog*vprefsdlg,GtkWidget*dlgcomponent,gint gtkresponse){
	gboolean preview=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(g_object_get_data(G_OBJECT(dlgcomponent),"preview")));
	switch(gtkresponse){
		case GTK_RESPONSE_OK:
			velocity_module_preferences_set_bool(module_info.id,"general","preview",preview);
			break;
		case GTK_RESPONSE_CANCEL:
		default:
			break;
	}
}
velocity_module_return init(){
	VELOCITY_MODULE_INFO(audio,_("Audio handler"),
		_("Handles previewing and getting info on audio files."),"http://velocity.sourceforge.net/");
	VELOCITY_MODULE_VERSION(1,0,0);
	VELOCITY_MODULE_AUTHORS("Kyle Davis");
	VELOCITY_MODULE_MIME_TYPES("audio/ac3","audio/x-mp3","audio/x-riff","audio/x-wav","audio/x-midi","application/x-ogg");
	return(VELOCITY_MODULE_OK);
}
void about(){
	VELOCITY_MSG("about");
}
void unload(){
	stop_preview(NULL);
}
