/*
 * GFileRunner - A GNOME File Manager
 *
 *
 * Copyright (C) (2002) (Kyle Davis) <kyled@blueriver.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <config.h>
#include <libgnome/libgnome.h>
#include <libgnomeui/libgnomeui.h>
#include <velocity-module-skel.h>
#include <velocity-module-gui-skel.h>
#include <velocity-sys.h>
#include <velocity-factories.h>

#include <sys/ioctl.h>
#include <linux/cdrom.h>
#include <fcntl.h>
#include <fstab.h>
#include <mntent.h>


#define _PATH_MTAB	"/etc/mtab"

void*on_view_status_event(velocity_view*vview,velocity_view_status_event vevent,const char*uri,GnomeVFSResult result,gpointer user_data){
	switch(vevent){
		case VELOCITY_VIEW_STATUS_EVENT_STOP:
			break;
		default:
			break;
	}
	return(NULL);
}
void*on_view_behavior_event(velocity_view*vview,velocity_view_behavior_event vevent,GdkEvent*event,velocity_vfs_obj*vfsobj,gpointer user_data){
	switch(vevent){
		case VELOCITY_VIEW_BEHAVIOR_EVENT_OBJECT_LOADED:
			break;
		case VELOCITY_VIEW_BEHAVIOR_EVENT_MOUSE_OVER:
			break;
		case VELOCITY_VIEW_BEHAVIOR_EVENT_MOUSE_OUT:
			break;
		default:
			break;
	}
	return(NULL);	/*Because we'd rather act transparently*/
}

typedef enum{DISC_INVALID=0,	/*not used*/
	DISC_DNE,	/*disc does not exist*/
	DISC_AUDIO,
	DISC_DATA,
	DISC_NUM}disc_state;
disc_state get_drive_status(const char*fsdev){
	struct cdrom_tochdr toch;
	struct cdrom_tocentry entry;
	int fd=open(fsdev,O_RDONLY|O_NONBLOCK);
	int status;
	if((fd<0)||(ioctl(fd,CDROMREADTOCHDR,&toch)==0)){
		close(fd);
		return(DISC_DATA);
	}
	for(entry.cdte_track=1;entry.cdte_track<=toch.cdth_trk1;entry.cdte_track++){
		entry.cdte_format=CDROM_LBA;
		if(ioctl(fd,CDROMREADTOCENTRY,&entry)==0){
			if(entry.cdte_ctrl&CDROM_DATA_TRACK){
				close(fd);
				return(DISC_AUDIO);
			}
		}
	}
	return(DISC_DNE);
}
gboolean is_mounted(const char*fsdir){
	FILE*f=fopen(_PATH_MTAB,"r");
	struct mntent*m=getmntent(f);
	while(m){
		if(strcmp(m->mnt_dir,fsdir)==0){
			fclose(f);
			VELOCITY_MSG("is_mounted(%s)",fsdir);
			return(TRUE);
		}
		m=getmntent(f);	
	}
	fclose(f);
	VELOCITY_MSG("not_mounted(%s)",fsdir);
	return(FALSE);
}
gboolean check_all(gpointer data){
	FILE*f=fopen(_PATH_FSTAB,"r");
	struct mntent*m=getmntent(f);
	while(m){
		if(m->mnt_dir){	/*If it has a configured mount point*/
			char*fsdev=g_strdup(m->mnt_fsname);
			char*fsdir=g_strdup(m->mnt_dir);
			if((strcmp(m->mnt_type,"iso9660")==0)&&(!is_mounted(fsdir))){
				char*cmd=NULL;
				VELOCITY_MSG("cd(%s),type(%i)",fsdev,get_drive_status(fsdev));
				switch(get_drive_status(fsdev)){
					case DISC_DATA:
						cmd=g_strdup_printf("mount %s",fsdev);
						break;
					default:
						break;
				}
				if(cmd){
					char*tcmd=g_strconcat(cmd," &",NULL);
					VELOCITY_MSG("Running(%s)",tcmd);
					system(tcmd);
					g_free(tcmd);
					g_free(cmd);
				}
			}
			g_free(fsdir);
			g_free(fsdev);
		}
		m=getmntent(f);
	}
	fclose(f);
	return(TRUE);
}
velocity_module_return init(){
	VELOCITY_MODULE_INFO("automount",_("Auto-CD mounter"),
		_("This module will automatically mount CDs when you putthem into your CD drive"),
		"http://velocity.sourceforge.net/");
	VELOCITY_MODULE_VERSION(1,0,0);
	VELOCITY_MODULE_AUTHORS("Kyle Davis");
	VELOCITY_MODULE_MIME_TYPES("internal/all");
	//gui_ext.context=automount_module_gui_ext_context;
	//if(velocity_module_preferences_get_bool(module_info.id,"general","firstload",TRUE)){
	//}
	g_timeout_add(3000,(GSourceFunc)check_all,(gpointer)NULL);
	return(VELOCITY_MODULE_OK);
}
void about(){
	VELOCITY_MSG("about automount");	//TODO;
}
void unload(){
}
