/*
 * GFileRunner - A GNOME File Manager
 *
 *
 * Copyright (C) (2002) (Kyle Davis) <kyled@blueriver.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>

#include <config.h>
#include <velocity-module-skel.h>
#include <velocity-module-gui-skel.h>
#include <velocity-sys.h>
#include <velocity-factories.h>
#include "callbacks.h"

#define VELOCITY_VFS_MIME_TYPE_ISO_IMAGE	"application/x-iso-image"

/*A modified sendto for this module*/
static void burn_sendto_cb(GtkMenuItem*menuitem,gpointer user_data){
	velocity_view*vview=(velocity_view*)g_object_get_data(G_OBJECT(menuitem),"velocity_view");   /*Always set by the factories for convenience*/
	GList*n=vview->selection->items;
	GList*sl=NULL;
	GList*dl=NULL;
	char*dest_prefix=velocity_vfs_ensure_dir_uri(gtk_widget_get_name(GTK_WIDGET(menuitem)));
	/*Build source and destination uri lists, and work around a dumbass GNOME VFS oddity*/
	while(n){
		if(n->data){
			velocity_vfs_obj*vfsobj=(velocity_vfs_obj*)n->data;
			char*uri=velocity_vfs_get_uri(vfsobj);
			gboolean hacked=FALSE;
			if(velocity_vfs_is_symlink(vfsobj)){
				if(uri[strlen(uri)-1]=='/'){
					uri[strlen(uri)-1]=0;
					hacked=TRUE;
				}
			}
			sl=g_list_prepend(sl,gnome_vfs_uri_new(uri));
			if(hacked)uri[strlen(uri)+1]='-';
			g_free(uri);
			hacked=FALSE;
			uri=g_strconcat(dest_prefix,vfsobj->info->name,NULL);
			if(velocity_vfs_is_symlink(vfsobj)){
				if(uri[strlen(uri)-1]=='/'){
					uri[strlen(uri)-1]=0;
					hacked=TRUE;
				}
			}
			dl=g_list_prepend(dl,gnome_vfs_uri_new(uri));
			if(hacked)uri[strlen(uri)+1]='-';
			g_free(uri);
		}
		n=n->next;
	}
	VELOCITY_MSG("BURN::SendTo(%s)",dest_prefix);
	velocity_vfs_xfer(GTK_WINDOW(vview->parent),sl,dl,VELOCITY_VFS_XFER_COPY,TRUE,NULL,NULL);
	g_free(dest_prefix);
}

static void burn_module_gui_ext_sendto_cb(GtkMenuItem*menuitem,gpointer user_data){
	/*This will generate the directory-in-menu for the BURN URI*/
	velocity_view*vview=(velocity_view*)g_object_get_data(G_OBJECT(menuitem),"velocity_view");	/*Always set by the factories for convenience*/
	velocity_vfs_filter_flags filter=VELOCITY_VFS_FILTER_HIDDEN|VELOCITY_VFS_FILTER_FILES;
	velocity_menu_factory_directory_scan(menuitem,VELOCITY_VFS_BURN_URI,filter,2,TRUE,TRUE,G_CALLBACK(velocity_menu_sendto_cb),(gpointer)vview);
}
static void burn_module_gui_ext_context_cb(GtkMenuItem*menuitem,gpointer user_data){
	velocity_view*vview=(velocity_view*)g_object_get_data(G_OBJECT(menuitem),"velocity_view");	/*Always set by the factories for convenience*/
	char*uri=velocity_vfs_get_uri_unescaped((velocity_vfs_obj*)vview->selection->items->data);
	char*argv[]={"nautilus-cd-burner",uri,NULL};
	gnome_execute_async(NULL,2,argv);
	g_free(uri);
}

gpointer burn_module_gui_ext_tools[]={VELOCITY_MENU_FACTORY_SUBMENU,NULL,N_("gnome-dev-cdrom"),N_("Burning Queue"),
		VELOCITY_FACTORY_FALSE,NULL,NULL,
	NULL};
gpointer burn_module_gui_ext_sendto[]={VELOCITY_MENU_FACTORY_SUBMENU,NULL,N_("gnome-dev-cdrom"),N_("Burning Queue"),
		VELOCITY_FACTORY_TRUE,G_CALLBACK(burn_module_gui_ext_sendto_cb),NULL,
	NULL};
gpointer burn_module_gui_ext_context[]={VELOCITY_MENU_FACTORY_PIXBUF,N_("gnome-dev-cdrom"),N_("Burn as CD"),
		VELOCITY_FACTORY_TRUE,G_CALLBACK(burn_module_gui_ext_context_cb),NULL,
	NULL};

void*on_view_behavior_event(velocity_view*vview,velocity_view_behavior_event vevent,GdkEvent*event,velocity_vfs_obj*vfsobj,gpointer user_data){
	switch(vevent){
		case VELOCITY_VIEW_BEHAVIOR_EVENT_CONTEXT:
			if((!velocity_clipboard_has_multiple(vview->selection)&&(strcasecmp(vfsobj->info->mime_type,VELOCITY_VFS_MIME_TYPE_ISO_IMAGE)==0))){
				gui_ext.context=burn_module_gui_ext_context;
			}else{
				gui_ext.context=NULL;
			}
			break;
		default:
			break;
	}
	return(NULL);	/*Because we'd rather act transparently*/
}

velocity_module_return init(){
	VELOCITY_MODULE_INFO(burn,_("CD-Writer handler"),
		_("Handles burning files to CDs via detected CD-Writers."),"http://velocity.sourceforge.net/");
	VELOCITY_MODULE_VERSION(1,0,0);
	VELOCITY_MODULE_AUTHORS("Kyle Davis");
	VELOCITY_MODULE_MIME_TYPES("internal/all");
	gui_ext.tools=burn_module_gui_ext_tools;
	gui_ext.sendto=burn_module_gui_ext_sendto;
	if(velocity_module_preferences_get_bool(module_info.id,"general","firstload",TRUE)){
		GnomeVFSFileInfo*info=gnome_vfs_file_info_new();
		velocity_module_preferences_set_bool(module_info.id,"general","firstload",FALSE);
		VELOCITY_MSG("CD-Writer module first time loaded");
		if(gnome_vfs_get_file_info(VELOCITY_VFS_BURN_URI,info,GNOME_VFS_FILE_INFO_DEFAULT)==GNOME_VFS_OK){
			VELOCITY_MSG("BURN EXISTS!!");
			gnome_vfs_file_info_unref(info);
		}else{
			/*Burn:/// not available*/
			VELOCITY_ERR("CD-Writer module has been blacklisted because " VELOCITY_VFS_BURN_URI " is invalid.");
			return(VELOCITY_MODULE_UNLOAD_PERSISTANT);
		}
	}else{
	}
	return(VELOCITY_MODULE_OK);
}
void about(){
	VELOCITY_MSG("about burn");	//TODO;
}
void unload(){
}
