/*
 * Velocity - A GNOME File Manager
 *
 *
 * Copyright (C) (2002) (Kyle Davis) <kyled@blueriver.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <velocity-module-skel.h>
#include <gnome.h>
#include <velocity-vfs.h>
#include <libgtkhtml/gtkhtml.h>
#include <libgtkhtml/layout/htmlstyle.h>
#include <velocity-factories.h>
#include <velocity-sys.h>
#include <velocity-windowman.h>

#define GTKHTML_IO_BUFFER_SIZE	4096	/*4KB should be okay for now*/
typedef struct{
	GtkWidget*widget;
	GtkWidget*gtkhtml;
	HtmlDocument*document;
	GnomeVFSAsyncHandle*handle;
	char*url;
	char*base;
	velocity_window*vwin;
}gtkhtml_module_viewer_info;

void gtkhtml_module_viewer_goto(gtkhtml_module_viewer_info*vi,const gchar*url);

static void gtkhtml_module_view_cb(GtkMenuItem*menuitem,gpointer user_data){
	velocity_view*vview=(velocity_view*)g_object_get_data(G_OBJECT(menuitem),"velocity_view");	/*Always set by the factories for convenience*/
	if(vview){
		velocity_window*vwin=(velocity_window*)vview->user_data;
		velocity_vfs_obj*vfsobj=(velocity_vfs_obj*)vview->selection->items->data;
		char*furi=velocity_vfs_get_uri(vfsobj);
		char*uri=g_strdup_printf(VELOCITY_MODULE_URI_PREFIX "%s:%s",module_info.id,furi);
		if(!vwin){
			velocity_windowman_new(uri);
		}else{
			velocity_window_set_location(vwin,uri);
		}
		g_free(uri);
		g_free(furi);
	}
}

velocity_module_gui_ext gui_ext;
void*gtkhtml_module_gui_ext_context[]={"View",VELOCITY_FACTORY_TRUE,gtkhtml_module_view_cb,NULL,
	NULL};

velocity_module_return init(void){
	VELOCITY_MODULE_INFO(gtkhtml,_("GTK HTML Viewer"),
		_("For quick viewing of HTML files."),"http://velocity.sourceforge.net/");
	VELOCITY_MODULE_VERSION(1,0,0);
	VELOCITY_MODULE_AUTHORS("Kyle Davis");
	VELOCITY_MODULE_MIME_TYPES("text/html");
	gui_ext.context=gtkhtml_module_gui_ext_context;
	return(VELOCITY_MODULE_OK);
}
void about(void){
	VELOCITY_MSG("an about box goes here...");
}
void unload(void){
}

static void gtkhtml_module_viewer_info_close(gtkhtml_module_viewer_info*vi){
	if(vi){
		if(vi->handle){
			gnome_vfs_async_cancel(vi->handle);
			vi->handle=NULL;
		}
/*		if(vi->document){
			g_object_unref(vi->document);
			vi->document=NULL;
		}*/
	}
}
static void gtkhtml_module_viewer_info_free(gtkhtml_module_viewer_info*vi){
	if(vi){
		gtkhtml_module_viewer_info_close(vi);
		g_free(vi);
	}
}
static void gtkhtml_module_async_close(GnomeVFSAsyncHandle*handle,GnomeVFSResult result,HtmlStream*s){
	gtkhtml_module_viewer_info*vi=(gtkhtml_module_viewer_info*)g_object_get_data(G_OBJECT(s),"viewer_info");
	if(result!=GNOME_VFS_OK){
		gtkhtml_module_viewer_info_free(vi);
	}else{
		gtkhtml_module_viewer_info_close(vi);
	}
	html_stream_close(s);
}
static void gtkhtml_module_async_read(GnomeVFSAsyncHandle*handle,GnomeVFSResult result,gpointer buffer,
	GnomeVFSFileSize bytes_requested,GnomeVFSFileSize bytes_read,HtmlStream*s){
	gtkhtml_module_viewer_info*vi=(gtkhtml_module_viewer_info*)g_object_get_data(G_OBJECT(s),"viewer_info");
	if(vi){	/*Good ol' sanity*/
		if(result==GNOME_VFS_OK){
			VELOCITY_MSG("read %i",bytes_read);
			html_stream_write(s,buffer,bytes_read);
			//html_document_write_stream(si->document,buffer,bytes_read);
			gnome_vfs_async_read(handle,buffer,bytes_requested,(GnomeVFSAsyncReadCallback)gtkhtml_module_async_read,(gpointer)s);
		}else{
			gnome_vfs_async_close(handle,(GnomeVFSAsyncCloseCallback)gtkhtml_module_async_close,(gpointer)s);
			g_free(buffer);
			buffer=NULL;
		}
	}
}
static void gtkhtml_module_viewer_vfs_open(GnomeVFSAsyncHandle*handle,GnomeVFSResult result,gpointer user_data){
	if(result==GNOME_VFS_OK){
		gchar*buffer=g_malloc(GTKHTML_IO_BUFFER_SIZE);
		gnome_vfs_async_read(handle,buffer,GTKHTML_IO_BUFFER_SIZE,(GnomeVFSAsyncReadCallback)gtkhtml_module_async_read,user_data);
	}else{
		VELOCITY_MSG("unable to open");
	}
}
static gboolean gtkhtml_module_doc_request_url(HtmlDocument*document,const gchar*url,HtmlStream*stream,gpointer user_data){
	gtkhtml_module_viewer_info*vi=(gtkhtml_module_viewer_info*)user_data;
	char*furl=gnome_vfs_uri_make_full_from_relative(vi->base,url);
	GnomeVFSAsyncHandle*h;
	VELOCITY_MSG("requested(%s)",furl);
	g_object_set_data(G_OBJECT(stream),"viewer_info",user_data);
	gnome_vfs_async_open(&h,furl,GNOME_VFS_OPEN_READ,GNOME_VFS_PRIORITY_DEFAULT,
		(GnomeVFSAsyncOpenCallback)gtkhtml_module_viewer_vfs_open,(gpointer)stream);
//	html_stream_set_cancel_func(stream,(HtmlStreamCancelFunc)gtkhtml_module_viewer_stream_cancel_cb,c);
	g_free(furl);
	return(TRUE);
}
static void gtkhtml_module_doc_link_clicked(HtmlDocument*document,const gchar*url,gpointer user_data){
	gtkhtml_module_viewer_info*vi=(gtkhtml_module_viewer_info*)user_data;
	char*furl=gnome_vfs_uri_make_full_from_relative(vi->base,url);
	VELOCITY_MSG("clicked(%s)",furl);
	gtkhtml_module_viewer_goto(vi,furl);
	g_free(furl);
}
static void gtkhtml_module_doc_title_changed(HtmlDocument*document,const gchar*title,gpointer user_data){
	VELOCITY_MSG("set_title(%s)",title);
}
static void gtkhtml_module_doc_submit(HtmlDocument*document,const gchar*method,const gchar*url,const gchar*encoding,gpointer user_data){
	VELOCITY_MSG("submit (%s,%s,%s)",method,url,encoding);
}
static void gtkhtml_module_async_open(GnomeVFSAsyncHandle*handle,GnomeVFSResult result,gpointer user_data){
	gtkhtml_module_viewer_info*vi=(gtkhtml_module_viewer_info*)user_data;
	if(!vi->document){
		vi->document=html_document_new();
		g_signal_connect(vi->document,"request_url",G_CALLBACK(gtkhtml_module_doc_request_url),vi);
		g_signal_connect(vi->document,"link_clicked",G_CALLBACK(gtkhtml_module_doc_link_clicked),vi);
		g_signal_connect(vi->document,"title_changed",G_CALLBACK(gtkhtml_module_doc_title_changed),vi);
		g_signal_connect(vi->document,"submit",G_CALLBACK(gtkhtml_module_doc_submit),vi);
	}else{
		html_document_clear(vi->document);
	}
	if(vi->base)g_free(vi->base);
	vi->base=g_path_get_dirname(vi->url);
	if(!vi->base)vi->base=g_strdup(vi->url);
	VELOCITY_MSG("set_base(%s)",vi->base);
	html_document_open_stream(vi->document,"text/html");
	if(result==GNOME_VFS_OK){
		char*buffer=g_malloc(GTKHTML_IO_BUFFER_SIZE);
		html_view_set_document(HTML_VIEW(vi->gtkhtml),vi->document);
		g_object_set_data(G_OBJECT(vi->document->current_stream),"viewer_info",(gpointer)vi);
		gnome_vfs_async_read(handle,buffer,GTKHTML_IO_BUFFER_SIZE,(GnomeVFSAsyncReadCallback)gtkhtml_module_async_read,(gpointer)vi->document->current_stream);
	}else{
		VELOCITY_MSG("VFS-ERROR: %s",velocity_vfs_get_error_string(result));
		html_document_write_stream(vi->document,"Unable to display page",23);
		html_document_close_stream(vi->document);
	}
}
static gboolean gtkhtml_module_viewer_request_object(HtmlView*html_view,HtmlEmbedded*widget,gpointer user_data){
	VELOCITY_MSG("request obj(%s)","unfinishished");
	return(FALSE);
}
void gtkhtml_module_viewer_goto(gtkhtml_module_viewer_info*vi,const gchar*url){
	gtkhtml_module_viewer_info_close(vi);	/*Stop anything going on*/
	if(vi->url)g_free(vi->url);
	vi->url=g_strdup(url);
	gnome_vfs_async_open(&vi->handle,url,GNOME_VFS_OPEN_READ,GNOME_VFS_PRIORITY_DEFAULT,(GnomeVFSAsyncOpenCallback)gtkhtml_module_async_open,(gpointer)vi);
}
GtkWidget*on_viewer_new(velocity_window*vwin,const char*uri){
	gtkhtml_module_viewer_info*vi=(gtkhtml_module_viewer_info*)g_new0(gtkhtml_module_viewer_info,1);
	if(vi){
		vi->widget=gtk_scrolled_window_new(NULL,NULL);
		vi->gtkhtml=html_view_new();
		vi->vwin=vwin;
		
		gtk_container_add(GTK_CONTAINER(vi->widget),vi->gtkhtml);
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(vi->widget),GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
		g_object_set_data(G_OBJECT(vi->widget),"viewer_info",(gpointer)vi);
		g_signal_connect(G_OBJECT(vi->gtkhtml),"request_object",G_CALLBACK(gtkhtml_module_viewer_request_object),(gpointer)vi);
		gtk_widget_show_all(vi->widget);
		VELOCITY_MSG("opening");
		gtkhtml_module_viewer_goto(vi,uri);
		return(vi->widget);
	}
	return(NULL);
}
void on_viewer_free(GtkWidget*viewer){
	gtkhtml_module_viewer_info*vi=(gtkhtml_module_viewer_info*)g_object_get_data(G_OBJECT(viewer),"viewer_info");
	if(vi)gtkhtml_module_viewer_info_free(vi);
}
