/*
 * Velocity - A GNOME File Manager
 *
 *
 * Copyright (C) (2002) (Kyle Davis) <kyled@blueriver.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <config.h>
#include <gdk/gdkkeysyms.h>
#include <ctype.h>
#include <libgnome/gnome-desktop-item.h>
#include <libbonobo.h>
#include <libbonoboui.h>
#include <orbit/orbit.h>
#include "callbacks.h"
#include "velocity-sys.h"
#include "velocity-cursor.h"
#include "velocity-clipboard.h"
#include "velocity-icon.h"
#include "velocity-vfs-object.h"
#include "velocity-script.h"
#include "velocity-template.h"
#include "velocity-file-properties-window.h"
#include "velocity-icons.h"
#include "velocity-factories.h"
#include "velocity-module-private.h"
#include "velocity-thumbnail.h"
#include "velocity-windowman.h"

//TODO:  Still need to test and integrate into everything else
/*static GList*velocity_vfs_mime_get_short_list_applications_ids(const char*mime_type){
	GList*r=NULL;
	GList*sl=gnome_vfs_mime_get_short_list_applications(mime_type);
	GList*n=sl;
	while(n){
		if(n->data){
			GnomeVFSMimeApplication*ma=(GnomeVFSMimeApplication*)n->data;
			r=g_list_prepend(r,(gpointer)g_strdup(ma->id));
		}
		n=n->next;
	}
	gnome_vfs_mime_application_list_free(sl);
	return(r);
}
static GList*velocity_get_mime_type_combined_applications_list(GList*unique_mime_types){
	guint mime_types=g_list_length(unique_mime_types);
	GList**applists=g_malloc0(GList*,mime_types);
	GList*n=unique_mime_types;
	GList*r=NULL;
	guint m=0;
	guint i;
	while(n){
		GList*sl=velocity_vfs_mime_get_short_list_applications_ids((char*)n->data);
		if(sl){
			applists[m]=sl;
			m++;
		}
		n=n->next;
	}
	for(i=0;i<m;i++){
		n=applists[i];
		while(n){
			guint matches=0;
			guint j;
			for(j=0;j<m;j++){
				if(j!=i){
					if(g_list_find_custom(applists[j],n->data,strcasecmp))matches++;
				}
			}
			n=n->next;
		}
	}
	g_free(applists);
}*/
static GtkWidget*velocity_menu_openwith(velocity_view*vview){
	GtkWidget*submenu=gtk_menu_new();
	GtkWidget*subitem2=NULL;
	if(velocity_clipboard_has_items(vview->selection)){
		GList*n=vview->selection->items;
		velocity_vfs_obj*vfsobj=(velocity_vfs_obj*)n->data;
		gboolean allthesame=TRUE;
		while(n){
			velocity_vfs_obj*vfsobj2=(velocity_vfs_obj*)n->data;
			if(strcasecmp(vfsobj->info->mime_type,vfsobj2->info->mime_type)!=0){
				allthesame=FALSE;
				break;
			}
			n=n->next;
		}
		if(allthesame){
			GnomeVFSMimeAction*act=gnome_vfs_mime_get_default_action(vfsobj->info->mime_type);
			//TODO:  Eventually I'll need to add this component support
//			if(act->action_type==GNOME_VFS_MIME_ACTION_TYPE_APPLICATION){
				n=gnome_vfs_mime_get_short_list_applications(vfsobj->info->mime_type);
//			}else{
//				n=gnome_vfs_mime_get_short_list_components(vfsobj->info->mime_type);
//			}
			if(act){
				while(n){
					GnomeVFSMimeApplication*ma=(GnomeVFSMimeApplication*)n->data;
					if(ma){
						if(strcmp(ma->id,act->action.application->id)==0){
							char*tmp=g_strdup_printf("<span weight=\"heavy\">%s</span>",ma->name);
							GtkWidget*w=gtk_label_new(tmp);
							subitem2=gtk_menu_item_new();
							gtk_label_set_use_markup(GTK_LABEL(w),TRUE);
							gtk_misc_set_alignment(GTK_MISC(w),0.0,0.0);
							gtk_container_add(GTK_CONTAINER(subitem2),w);
							g_free(tmp);
						}else{
							subitem2=gtk_menu_item_new_with_label(ma->name);
						}
						g_object_set_data(G_OBJECT(subitem2),"velocity_view",(gpointer)vview);
						g_signal_connect(subitem2,"event",G_CALLBACK(velocity_menu_openwith_cb),vview);
						g_object_set_data_full(G_OBJECT(subitem2),"mimeapp_id",g_strdup(ma->id),(GDestroyNotify)g_free);
						gtk_menu_shell_append(GTK_MENU_SHELL(submenu),subitem2);
						gnome_vfs_mime_application_free(ma);
					}
					n=n->next;
				}
				gnome_vfs_mime_action_free(act);
			}
			gtk_menu_shell_append(GTK_MENU_SHELL(submenu),gtk_menu_item_new());
			subitem2=gtk_menu_item_new_with_mnemonic(_("_Other..."));
			g_object_set_data(G_OBJECT(subitem2),"velocity_view",(gpointer)vview);
			g_signal_connect(subitem2,"activate",G_CALLBACK(velocity_menu_openwith_other_cb),vview);
			gtk_menu_shell_append(GTK_MENU_SHELL(submenu),subitem2);
		}else{
			//TODO:  Handle multi-typed file lists HERE DAMN YOU!!
			VELOCITY_MSG("Handle multi-typed file lists HERE");
		}
	}
	return(submenu);
}
static gboolean velocity_supports_mime_list(gchar**mime_types,GList*l){
	guint mismatched=g_list_length(l);
	while(l){
		if(l->data){
			velocity_vfs_obj*vfsobj=(velocity_vfs_obj*)l->data;
			if(velocity_mime_type_vector_check(mime_types,vfsobj->info->mime_type))mismatched--;
		}
		l=l->next;
	}
	return(mismatched==0);
}
static GtkWidget*velocity_menu_addons(velocity_view*vview){
	velocity_script*vscript=NULL;
	GtkWidget*submenu=NULL;
	GList*slist=NULL;
	GList*m=velocity_module_tracker_get_list();	/*module addons*/
	char*script=NULL;
	int mime_type_offset=0;
	char*puri[]={PACKAGE_SCRIPTS_DIR GNOME_VFS_URI_PATH_STR,vprefs->locations->scripts,NULL};
	int d=0;
	gboolean useselection=velocity_clipboard_has_items(vview->selection);
	if(useselection){
		slist=vview->selection->items;
	}else{
		GnomeVFSFileInfo*vfsinfo=gnome_vfs_file_info_new();
		if(gnome_vfs_get_file_info(vview->clocation,vfsinfo,GNOME_VFS_FILE_INFO_GET_MIME_TYPE|GNOME_VFS_FILE_INFO_FORCE_FAST_MIME_TYPE|GNOME_VFS_FILE_INFO_FOLLOW_LINKS)==GNOME_VFS_OK){
			char*puri=velocity_vfs_get_uri_parent(vview->clocation);
			velocity_vfs_obj*vfsobj=velocity_vfs_obj_new(puri,vfsinfo,NULL,NULL);
			slist=g_list_prepend(NULL,vfsobj);
			g_free(puri);
			gnome_vfs_file_info_unref(vfsinfo);
		}else{
			return(NULL);
		}
	}
	while(m){
		if(m->data){
			velocity_module*vmod=(velocity_module*)m->data;
			if((vmod->gui_ext)&&(vmod->gui_ext->addons)&&(velocity_supports_mime_list(vmod->info->mime_types,slist))){
				if(!submenu)submenu=gtk_menu_new();
				velocity_menu_factory_append(vview,submenu,vmod->gui_ext->addons);
			}
		}
		m=m->next;
	}
	if(submenu)gtk_menu_shell_append(GTK_MENU_SHELL(submenu),gtk_menu_item_new());
	//TODO:  Cache this info or something...probably cache it and use your vfs_dir_monitor to refresh the info
	while(puri[d]){
		GList*files=NULL;
		if(gnome_vfs_directory_list_load(&files,puri[d],GNOME_VFS_FILE_INFO_FOLLOW_LINKS)==GNOME_VFS_OK){
			GList*n=files;
			while(n){
				if(n->data){
					GnomeVFSFileInfo*vfsinfo=(GnomeVFSFileInfo*)n->data;
					if(!velocity_vfs_filter(vfsinfo,NULL,VELOCITY_VFS_FILTER_DIRS)){
						script=g_strconcat(puri[d],vfsinfo->name,NULL);
						vscript=velocity_script_new_from_file(script);
						if(!submenu)submenu=gtk_menu_new();
						if(vscript){
							if((vscript->name)&&(vscript->mime_types)&&(velocity_supports_mime_list(vscript->mime_types,slist))){
								void*t[]={vscript->name,VELOCITY_FACTORY_TRUE,G_CALLBACK(velocity_menu_scripts_cb),(gpointer)velocity_script_dup(vscript),NULL};
								velocity_menu_factory_append(vview,submenu,t);
							}
							velocity_script_free(vscript);
						}
						g_free(script);
					}
					gnome_vfs_file_info_unref(vfsinfo);
				}
				n=n->next;
			}
		}else{
			VELOCITY_ERR("An error occured trying to read %s",puri[d]);
		}
		d++;
	}
	if(!useselection){
		velocity_vfs_obj_destroy((velocity_vfs_obj*)slist->data);
		g_list_free(slist);
	}
	return(submenu);
}
static GtkWidget*velocity_menu_templates(velocity_view*vview){
	velocity_template*vtemplate=NULL;
	GtkWidget*submenu=NULL;
	GtkWidget*subitem=NULL;
	char*puri[]={PACKAGE_TEMPLATES_DIR GNOME_VFS_URI_PATH_STR,vprefs->locations->templates,NULL};
	int d=0;
	GList*m=velocity_module_tracker_get_list();	/*module templates*/
	GList*n=velocity_static_templates;	/*our static templates*/
	while(n){
		if(n->data){
			velocity_template*vtemplate=(velocity_template*)n->data;
			char*iconuri=velocity_vfs_get_mime_icon_uri(vtemplate->mime_type,NULL,VELOCITY_MENU_FACTORY_PIXBUF_LARGE_SIZE);
			void*t[]={VELOCITY_MENU_FACTORY_PIXBUF_LARGE,iconuri,vtemplate->name,VELOCITY_FACTORY_TRUE,G_CALLBACK(velocity_menu_templates_cb),(gpointer)velocity_template_dup(vtemplate),
				NULL};
			if(!submenu)submenu=gtk_menu_new();
			velocity_menu_factory_append(vview,submenu,t);
			g_free(iconuri);
		}
		n=n->next;
	}
	while(m){
		if(m->data){
			velocity_module*vmod=(velocity_module*)m->data;
			if(vmod->gui_ext){
				GList*n=vmod->gui_ext->templates;
				while(n){
					if(n->data){
						velocity_template*vtemplate=(velocity_template*)n->data;
						char*iconuri=velocity_vfs_get_mime_icon_uri(vtemplate->mime_type,NULL,VELOCITY_MENU_FACTORY_PIXBUF_LARGE_SIZE);
						void*t[]={VELOCITY_MENU_FACTORY_PIXBUF_LARGE,iconuri,vtemplate->name,VELOCITY_FACTORY_TRUE,G_CALLBACK(velocity_menu_templates_cb),(gpointer)velocity_template_dup(vtemplate),
							NULL};
						if(!submenu)submenu=gtk_menu_new();
						velocity_menu_factory_append(vview,submenu,t);
						g_free(iconuri);
					}
					n=n->next;
				}
			}
		}
		m=m->next;
	}
	if(submenu)gtk_menu_shell_append(GTK_MENU_SHELL(submenu),gtk_menu_item_new());
	//TODO:  Cache this info or something...probably cache it and use your vfs_dir_monitor to refresh the info
	while(puri[d]){
		GList*files=NULL;
		if(gnome_vfs_directory_list_load(&files,puri[d],GNOME_VFS_FILE_INFO_FOLLOW_LINKS)==GNOME_VFS_OK){
			GList*n=files;
			while(n){
				if(n->data){
					GnomeVFSFileInfo*vfsinfo=(GnomeVFSFileInfo*)n->data;
					if(!velocity_vfs_filter(vfsinfo,NULL,VELOCITY_VFS_FILTER_DIRS)){
						char*script=g_strconcat(puri[d],vfsinfo->name,NULL);
						vtemplate=velocity_template_new_from_file(script);
						if(vtemplate){
							if(vtemplate->name){  /* [semi] make sure that we've got a real template made for Velocity */
								if(!submenu)submenu=gtk_menu_new();
								if(vtemplate->have_dependancies){	/*If all dependancies passed*/
									char*iconuri=velocity_vfs_get_mime_icon_uri(vtemplate->mime_type,NULL,VELOCITY_MENU_FACTORY_PIXBUF_LARGE_SIZE);
									void*t[]={VELOCITY_MENU_FACTORY_PIXBUF_LARGE,iconuri,vtemplate->name,VELOCITY_FACTORY_TRUE,G_CALLBACK(velocity_menu_templates_cb),(gpointer)velocity_template_dup(vtemplate),
										NULL};
									velocity_menu_factory_append(vview,submenu,t);
									g_free(iconuri);
								}else{
									/* we have an error, so let the user know */
									char*errtxt=g_strconcat("ERROR: ",vtemplate->name,NULL);
									subitem=gtk_menu_item_new_with_label(errtxt);
									g_object_set_data(G_OBJECT(subitem),"velocity_view",(gpointer)vview);
									g_signal_connect(subitem,"activate",G_CALLBACK(velocity_menu_templates_error_cb),velocity_template_dup(vtemplate));
									gtk_menu_shell_append(GTK_MENU_SHELL(submenu),subitem);
									g_free(errtxt);
								}
							}
							velocity_template_free(vtemplate);
						}
						g_free(script);
					}
					gnome_vfs_file_info_unref(vfsinfo);
				}
				n=n->next;
			}
		}else{
			VELOCITY_ERR("An error occured trying to read %s",puri[d]);
		}
		d++;
	}
	return(submenu);
}
static GtkWidget*velocity_menu_sendto(velocity_view*vview){
	GtkWidget*submenu=NULL;
	GtkWidget*subitem=NULL;
	char*puri[]={PACKAGE_SENDTO_DIR GNOME_VFS_URI_PATH_STR,vprefs->locations->sendto,NULL};
	int d=0;
	GList*m=velocity_module_tracker_get_list();	/*module sendto'*/
	while(m){
		if(m->data){
			velocity_module*vmod=(velocity_module*)m->data;
			if((vmod->gui_ext)&&(vmod->gui_ext->sendto)){
				if(!submenu)submenu=gtk_menu_new();
				velocity_menu_factory_append(vview,submenu,vmod->gui_ext->sendto);
			}
		}
		m=m->next;
	}
	if(submenu)gtk_menu_shell_append(GTK_MENU_SHELL(submenu),gtk_menu_item_new());
	while(puri[d]){
		void*fs[]={VELOCITY_MENU_FACTORY_DIRECTORY,(gpointer)puri[d],(gpointer)VELOCITY_VFS_FILTER_FILES,(gpointer)1,(gpointer)FALSE,
				NULL,(gpointer)TRUE,G_CALLBACK(velocity_menu_sendto_cb),(gpointer)vview,
			NULL};
		velocity_menu_factory_append(vview,submenu,fs);
		d++;
	}
	return(submenu);
}

void velocity_menu_refresh_cb(GtkWidget*widget,gpointer user_data){
	velocity_view*vview=(velocity_view*)user_data;
	velocity_view_set_location(vview,vview->clocation);
}
void velocity_menu_sorting_reorganize_cb(GtkWidget*widget,gpointer user_data){
	velocity_view*vview=(velocity_view*)user_data;
	velocity_view_soft_refresh(vview,FALSE);
}
void velocity_menu_sorting_dirs_first_cb(GtkWidget*widget,gpointer user_data){
	velocity_view*vview=(velocity_view*)user_data;
	vview->prefs->dirs_first^=1;
	vprefs->view->dirs_first=vview->prefs->dirs_first;
	if(vview->user_data){
		velocity_window*vwin=(velocity_window*)vview->user_data;
		gtk_widget_destroy(GTK_WIDGET(gnome_app_get_dock_item_by_name(GNOME_APP(vwin->widget),GNOME_APP_MENUBAR_NAME)));
		GNOME_APP(vwin->widget)->menubar=NULL;	/*gtk_widget_destroy doesn't do this and gnome_app_set_menus wants it*/
		velocity_window_menubar_create(vwin);
		velocity_toolbar_refresh_cb(NULL,(gpointer)vwin);
	}else{
		velocity_view_set_location(vview,vview->clocation);
	}
	velocity_preferences_sync();
}
void velocity_menu_sorting_cb(GtkWidget*widget,gpointer user_data){
	velocity_view*vview=(velocity_view*)g_object_get_data(G_OBJECT(widget),"velocity_view");
	vview->prefs->sorting=(velocity_view_sort_method)GPOINTER_TO_INT(user_data);
	VELOCITY_MSG("This currently sets the global sorting method, this should probably change!");
	vprefs->view->sorting=vview->prefs->sorting;
	if(vview->user_data){
		velocity_window*vwin=(velocity_window*)vview->user_data;
		gtk_widget_destroy(GTK_WIDGET(gnome_app_get_dock_item_by_name(GNOME_APP(vwin->widget),GNOME_APP_MENUBAR_NAME)));
		GNOME_APP(vwin->widget)->menubar=NULL;	/*gtk_widget_destroy doesn't do this and gnome_app_set_menus wants it*/
		velocity_window_menubar_create(vwin);	
	}
	velocity_view_soft_refresh(vview,FALSE);
	velocity_preferences_sync();
}
void velocity_menu_clipboard(velocity_view*vview,GtkWidget*menu){
	gboolean clipboard=velocity_clipboard_has_items(vclipboard);
	gboolean selection=velocity_clipboard_has_items(vview->selection);
	gboolean alldirs=TRUE;
	void*menus_sel1[]={VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_CUT,_("C_ut"),VELOCITY_FACTORY_TRUE,velocity_menu_cut_cb,vview,
			VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_COPY,_("_Copy"),VELOCITY_FACTORY_TRUE,velocity_menu_copy_cb,vview,
			VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_COPY,_("C_opy Location"),VELOCITY_FACTORY_FALSE,velocity_menu_copy_cb,vview,
			NULL};
	void*menus_dir[]={VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_PASTE,_("_Paste"),(gpointer)clipboard,velocity_menu_paste_cb,vview,
			VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_PASTE,_("Paste _Link"),(gpointer)((clipboard)&&(vclipboard->mode==VELOCITY_CLIPBOARD_COPY)),velocity_menu_paste_link_cb,vview,
			NULL};
	void*menus_trash[]={VELOCITY_MENU_FACTORY_PIXBUF,VELOCITY_TRASH,_("Move to _Trash"),VELOCITY_FACTORY_TRUE,velocity_menu_trash_cb,vview,
			NULL};
	void*menus_delete[]={VELOCITY_MENU_FACTORY_PIXBUF,VELOCITY_TRASH,_("_Delete"),VELOCITY_FACTORY_TRUE,velocity_menu_delete_cb,vview,
			NULL};
	void*menus_sel2[]={VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_CONVERT,_("Re_name"),VELOCITY_FACTORY_TRUE,velocity_menu_rename_cb,vview,
			VELOCITY_MENU_FACTORY_EMBEDDED_FACTORY,(velocity_vfs_uris_match(vview->clocation,vprefs->locations->trash)?menus_delete:menus_trash),
			VELOCITY_MENU_FACTORY_SEPERATOR,
			VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_PROPERTIES,_("P_roperties"),VELOCITY_FACTORY_TRUE,velocity_menu_properties_cb,vview,
			NULL};
	if(selection){
		GList*n=vview->selection->items;
		while(n){
			velocity_vfs_obj*vfsobj2=n->data;
			if(!velocity_vfs_is_directory(vfsobj2)){
				alldirs=FALSE;
				break;
			}
			n=n->next;
		}
		velocity_menu_factory_append(vview,menu,menus_sel1);
	}
	if(alldirs)velocity_menu_factory_append(vview,menu,menus_dir);
	if(selection)velocity_menu_factory_append(vview,menu,menus_sel2);
}

static void velocity_menu_openwith_lc(GtkMenuItem*menuitem,gpointer user_data){
	velocity_view*vview=(velocity_view*)user_data;
	GnomeVFSMimeApplication*ma=gnome_vfs_mime_application_new_from_id(g_object_get_data(G_OBJECT(menuitem),"mimeapp_id"));
	if(ma->can_open_multiple_files){
		GList*uris=velocity_clipboard_to_uri_list(vview->selection);
		velocity_vfs_mime_application_open_list(ma,uris);
		gnome_vfs_list_deep_free(uris);
	}else{
		GList*n=vview->selection->items;
		while(n){
			if(n->data){
				velocity_vfs_obj*vfsobj=(velocity_vfs_obj*)n->data;
				char*uri=velocity_vfs_get_uri_unescaped(vfsobj);
				velocity_vfs_mime_application_open(ma,uri);
				g_free(uri);
			}
			n=n->next;
		}
	}
	gnome_vfs_mime_application_free(ma);
}
static void velocity_menu_openwith_rc_setdefault(GtkMenuItem*menuitem,gpointer user_data){
	velocity_view*vview=(velocity_view*)g_object_get_data(G_OBJECT(menuitem),"velocity_view");
	GtkWidget*mi=(GtkWidget*)user_data;
	GnomeVFSMimeApplication*ma=gnome_vfs_mime_application_new_from_id(g_object_get_data(G_OBJECT(mi),"mimeapp_id"));
	if((vview)&&(vview->selection)&&(vview->selection->items)){
		velocity_vfs_obj*vfsobj=vview->selection->items->data;
		if(vfsobj){
			GnomeVFSResult r=gnome_vfs_mime_set_default_action_type(vfsobj->info->mime_type,GNOME_VFS_MIME_ACTION_TYPE_APPLICATION);
			VELOCITY_MSG("set action to application (%s)",gnome_vfs_result_to_string(r));
			r=gnome_vfs_mime_set_default_application(vfsobj->info->mime_type,ma->id);
			VELOCITY_MSG("set to default application (%s)",gnome_vfs_result_to_string(r));
		}
	}
	gnome_vfs_mime_application_free(ma);
}
static void velocity_menu_openwith_rc_hideitem(GtkMenuItem*menuitem,gpointer user_data){
	velocity_view*vview=(velocity_view*)g_object_get_data(G_OBJECT(menuitem),"velocity_view");
	GtkWidget*mi=(GtkWidget*)user_data;
	GnomeVFSMimeApplication*ma=gnome_vfs_mime_application_new_from_id(g_object_get_data(G_OBJECT(mi),"mimeapp_id"));
	if(vview){
		velocity_vfs_obj*vfsobj=vview->selection->items->data;
		if(vfsobj){
			GnomeVFSResult r=gnome_vfs_mime_remove_application_from_short_list(vfsobj->info->mime_type,ma->id);
			if(r==GNOME_VFS_OK)gtk_widget_hide(mi);
			VELOCITY_MSG("removing from list:  %s",gnome_vfs_result_to_string(r));
		}
	}
	gnome_vfs_mime_application_free(ma);
}
static void velocity_menu_openwith_rc_run(GtkMenuItem*menuitem,gpointer user_data){
	velocity_menu_openwith_lc(GTK_MENU_ITEM(user_data),g_object_get_data(G_OBJECT(menuitem),"velocity_view"));
}
static void velocity_menu_openwith_rc(GtkMenuItem*menuitem,GdkEvent*event,gpointer user_data){
	velocity_view*vview=(velocity_view*)user_data;
	velocity_vfs_obj*vfsobj=(velocity_vfs_obj*)vview->selection->items->data;
	//TODO:  Finish these....
	void*menufs[]={VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_EXECUTE,_("Run"),VELOCITY_FACTORY_TRUE,velocity_menu_openwith_rc_run,(gpointer)menuitem,
		VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_EXECUTE,_("Run in _terminal"),VELOCITY_FACTORY_FALSE,NULL,NULL,
		VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_EXECUTE,_("Run in as _root"),VELOCITY_FACTORY_FALSE,NULL,NULL,
		VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_EXECUTE,_("_Modify the command line"),VELOCITY_FACTORY_FALSE,NULL,NULL,
		VELOCITY_MENU_FACTORY_SEPERATOR,
		VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_YES,_("Set as _default"),VELOCITY_FACTORY_TRUE,velocity_menu_openwith_rc_setdefault,menuitem,
		VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_DELETE,_("_Remove this item"),VELOCITY_FACTORY_TRUE,velocity_menu_openwith_rc_hideitem,menuitem,
		NULL};
	GtkWidget*menu=velocity_menu_factory(vview,GTK_WIDGET(menuitem),menufs);
	gtk_menu_item_set_submenu(menuitem,menu);
	gtk_widget_show_all(menu);
	gtk_menu_item_select(menuitem);
//	gtk_menu_popup(GTK_MENU(menu),NULL,NULL,NULL,NULL,event->button.button,event->button.time);
}
static void velocity_menu_openwith_other_changed_cb(GtkOptionMenu*o,GtkDialog*dlg){
	const gchar*ida[]={"known","custom"};	/*Widget ID Array*/
	const gchar*id=ida[gtk_option_menu_get_history(o)];
	GtkWidget*active=g_object_get_data(G_OBJECT(dlg),"active");
	GtkWidget*selected=g_object_get_data(G_OBJECT(dlg),id);
	gtk_widget_hide_all(active);
	gtk_widget_show_all(selected);
	if(id==ida[0]){
		gtk_widget_hide(GTK_WIDGET(g_object_get_data(G_OBJECT(dlg),"term")));
		gtk_widget_hide(GTK_WIDGET(g_object_get_data(G_OBJECT(dlg),"multiple")));
	}else{
		gtk_widget_show(GTK_WIDGET(g_object_get_data(G_OBJECT(dlg),"term")));
		gtk_widget_show(GTK_WIDGET(g_object_get_data(G_OBJECT(dlg),"multiple")));
	}
	g_object_set_data(G_OBJECT(dlg),"active",(gpointer)selected);
}
void velocity_menu_openwith_other_cb(GtkMenuItem*widget,gpointer*user_data){
	velocity_view*vview=(velocity_view*)user_data;
	velocity_vfs_obj*vfsobj=(velocity_vfs_obj*)vview->selection->items->data;
	GnomeVFSMimeApplication*ma=NULL;
	gchar*findby[]={_("List of known programs"),_("Manual"),NULL};
	GtkWidget*w=gtk_dialog_new_with_buttons(_("Open with program"),NULL,0,
		GTK_STOCK_CANCEL,GTK_RESPONSE_CANCEL,GTK_STOCK_OK,GTK_RESPONSE_OK,NULL);
	GtkWidget*h=gtk_hbox_new(TRUE,6);
	GtkWidget*t=gtk_label_new(_("Find program mode:"));
	GtkWidget*t2=gtk_image_new_from_stock(GTK_STOCK_ADD,GTK_ICON_SIZE_BUTTON);
	GtkWidget*o=velocity_option_menu_factory(findby);
	GtkWidget*known=gtk_scrolled_window_new(NULL,NULL);
	GtkWidget*custom=gnome_file_entry_new(NULL,_("Locate a program to open this type of file"));
	GtkWidget*add=gtk_check_button_new();
	GtkWidget*makedefault=gtk_check_button_new();
	GtkWidget*term=gtk_check_button_new();
	GtkWidget*multiple=gtk_check_button_new();
	GtkListStore*store=gtk_list_store_new(3,GDK_TYPE_PIXBUF,G_TYPE_STRING,G_TYPE_POINTER);
	GtkTreeView*view=GTK_TREE_VIEW(gtk_tree_view_new_with_model(GTK_TREE_MODEL(store)));
	GtkCellRenderer*renderer=gtk_cell_renderer_pixbuf_new();
	GtkTreeViewColumn*column=gtk_tree_view_column_new_with_attributes(_("Icon"),renderer,"pixbuf",0,NULL);
	g_signal_connect(o,"changed",G_CALLBACK(velocity_menu_openwith_other_changed_cb),w);
	gtk_tree_view_append_column(view,column);
	renderer=gtk_cell_renderer_text_new();
	column=gtk_tree_view_column_new_with_attributes(_("Program"),renderer,"text",1,NULL);
	gtk_tree_view_append_column(view,column);
	gtk_tree_view_set_rules_hint(view,TRUE);
	gtk_tree_view_set_headers_visible(view,FALSE);
	gtk_box_set_homogeneous(GTK_BOX(GTK_DIALOG(w)->vbox),FALSE);
	gtk_container_set_border_width(GTK_CONTAINER(GTK_DIALOG(w)->vbox),4);
	gtk_container_add(GTK_CONTAINER(h),t);
	gtk_container_add(GTK_CONTAINER(h),o);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(w)->vbox),h,FALSE,TRUE,4);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(known),GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
	gtk_container_add(GTK_CONTAINER(known),GTK_WIDGET(view));
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(w)->vbox),known);
	g_object_set_data(G_OBJECT(w),"known",(gpointer)known);
	g_object_set_data(G_OBJECT(w),"active",(gpointer)known);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(w)->vbox),custom);
	g_object_set_data(G_OBJECT(w),"custom",(gpointer)custom);

	h=gtk_hbox_new(FALSE,6);
	t=gtk_image_new_from_stock(GTK_STOCK_EXECUTE,GTK_ICON_SIZE_BUTTON);
	gtk_container_add(GTK_CONTAINER(h),t);
	t=gtk_label_new(_("Needs terminal?"));
	gtk_container_add(GTK_CONTAINER(h),t);
	gtk_container_add(GTK_CONTAINER(term),h);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(w)->vbox),term,FALSE,TRUE,4);
	g_object_set_data(G_OBJECT(w),"term",(gpointer)term);

	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(multiple),TRUE);
	h=gtk_hbox_new(FALSE,6);
	t=gtk_image_new_from_stock(GTK_STOCK_DND_MULTIPLE,GTK_ICON_SIZE_BUTTON);
	gtk_container_add(GTK_CONTAINER(h),t);
	t=gtk_label_new(_("Can open multiple files?"));
	gtk_container_add(GTK_CONTAINER(h),t);
	gtk_container_add(GTK_CONTAINER(multiple),h);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(w)->vbox),multiple,FALSE,TRUE,4);
	g_object_set_data(G_OBJECT(w),"multiple",(gpointer)multiple);

	h=gtk_hbox_new(FALSE,6);
	t=gtk_image_new_from_stock(GTK_STOCK_ADD,GTK_ICON_SIZE_BUTTON);
	gtk_container_add(GTK_CONTAINER(h),t);
	t=gtk_label_new(_("Add to Open menu?"));
	gtk_container_add(GTK_CONTAINER(h),t);
	gtk_container_add(GTK_CONTAINER(add),h);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(w)->vbox),add,FALSE,TRUE,4);

	h=gtk_hbox_new(FALSE,6);
	t=gtk_image_new_from_stock(GTK_STOCK_DIALOG_QUESTION,GTK_ICON_SIZE_BUTTON);
	gtk_container_add(GTK_CONTAINER(h),t);
	t=gtk_label_new(_("Make default?"));
	gtk_container_add(GTK_CONTAINER(h),t);
	gtk_container_add(GTK_CONTAINER(makedefault),h);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(w)->vbox),makedefault,FALSE,TRUE,4);
	{
		GList*l=velocity_vfs_directory_list_load_recursive(VELOCITY_VFS_APPLICATIONS_ALL_USERS_URI,GNOME_VFS_FILE_INFO_GET_MIME_TYPE|GNOME_VFS_FILE_INFO_FORCE_FAST_MIME_TYPE|GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
		if(l){
			GList*n=l;
			while(n){
				velocity_vfs_obj*vfsobj=n->data;
				if(strcmp(vfsobj->info->name,".directory")!=0){
					GtkTreeIter iter;
					char*uri=velocity_vfs_get_uri(vfsobj);
					GnomeDesktopItem*di=gnome_desktop_item_new_from_uri(uri,GNOME_DESKTOP_ITEM_LOAD_ONLY_IF_EXISTS,NULL);
					if((di)&&(gnome_desktop_item_attr_exists(di,GNOME_DESKTOP_ITEM_NAME))){
						gtk_list_store_append(store,&iter);
						gtk_list_store_set(store,&iter,
							1,g_strdup(gnome_desktop_item_get_string(di,GNOME_DESKTOP_ITEM_NAME)),
							2,di,
							-1);
						if(gnome_desktop_item_attr_exists(di,GNOME_DESKTOP_ITEM_ICON)){
							GdkPixbuf*p=velocity_image_cache_get(gnome_desktop_item_get_string(di,GNOME_DESKTOP_ITEM_ICON),16,16,NULL,VELOCITY_IMAGE_CACHE_PRIORITY_MEDIUM);
							if(!p){
								char*iuri=velocity_image_get_uri(gnome_desktop_item_get_string(di,GNOME_DESKTOP_ITEM_ICON),16);
								if(iuri){
									p=velocity_image_cache_get(iuri,16,16,NULL,VELOCITY_IMAGE_CACHE_PRIORITY_MEDIUM);
									g_free(iuri);
								}
							}
							gtk_list_store_set(store,&iter,0,p,-1);
						}
					}
					g_free(uri);
				}
				velocity_vfs_obj_free(vfsobj);
				n=n->next;
			}
			g_list_free(l);
		}
		//TODO:  We should add items from the MIME app list here too
	}
	gtk_widget_show_all(w);
	gtk_widget_hide(custom);
	gtk_widget_hide(term);
	gtk_widget_hide(multiple);
	switch(gtk_dialog_run(GTK_DIALOG(w))){
		case GTK_RESPONSE_OK:
			if(g_object_get_data(G_OBJECT(w),"active")==known){
				GtkTreeSelection*selection=gtk_tree_view_get_selection(view);
				GtkTreeModel*model;
				GtkTreeIter iter;
				if(gtk_tree_selection_get_selected(selection,&model,&iter)){
					GnomeDesktopItem*di=NULL;
					gtk_tree_model_get(model,&iter,2,&di,-1);
					if(di){
						GList*uris=velocity_clipboard_to_uri_list(vview->selection);
						gnome_desktop_item_launch(di,uris,GNOME_DESKTOP_ITEM_LAUNCH_APPEND_URIS,NULL);
						gnome_vfs_list_deep_free(uris);
						if(gnome_desktop_item_attr_exists(di,GNOME_DESKTOP_ITEM_EXEC)){
							gboolean diterm=(gnome_desktop_item_attr_exists(di,GNOME_DESKTOP_ITEM_TERMINAL))?
								gnome_desktop_item_get_boolean(di,GNOME_DESKTOP_ITEM_TERMINAL):FALSE;
							const char*tcmd=gnome_desktop_item_get_string(di,GNOME_DESKTOP_ITEM_EXEC);
							char*pu=strstr(tcmd,"%");
							char*cmd=NULL;
							if(pu){
								cmd=g_strndup(tcmd,pu-tcmd-1);
							}else{
								cmd=g_strdup(cmd);
							}
							if(cmd){
								if(gnome_vfs_is_executable_command_string(cmd)){
									ma=velocity_vfs_mime_application_new_from_cmd(cmd);
									if(!ma){
										VELOCITY_MSG("MIME APP not found in application registry.  Adding...");
										ma=velocity_vfs_mime_application_new(cmd,vfsobj->info->mime_type,diterm,TRUE);
									}
								}
								g_free(cmd);
							}
						}
					}
				}
			}else{	/*custom application*/
				const char*cmd=gtk_entry_get_text(GTK_ENTRY(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(custom))));
				ma=velocity_vfs_mime_application_new_from_cmd(cmd);
				if((!ma)&&(gnome_vfs_is_executable_command_string(cmd))){	/*If this app isn't in the registry we'll create add it*/
					VELOCITY_MSG("MIME APP not found in application registry.  Adding...");
					ma=velocity_vfs_mime_application_new(cmd,vfsobj->info->mime_type,
						gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(term)),
						gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(multiple)));
				}
				if(ma){	/*Double check that we have a MIMEApp in case some weirdness happened*/
					GList*uris=velocity_clipboard_to_uri_list(vview->selection);
					velocity_vfs_mime_application_open_list(ma,uris);
					gnome_vfs_list_deep_free(uris);
				}
				//TODO: Now make sure:
				//		open_with list follows MIME types selected (and make a extract_unique_mime_types_from_selection function)
				//		make sure open_with_other doesn't show for more than one mime type (to keep sanity for now)
			}
			if(ma){
				VELOCITY_MSG("MIME APP    ID:  %s",ma->id);
				VELOCITY_MSG("MIME APP  NAME:  %s",ma->name);
				VELOCITY_MSG("MIME APP  TERM:  %i",ma->requires_terminal);
				VELOCITY_MSG("MIME APP MULTI:  %i",ma->can_open_multiple_files);
				if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(add))){
					if(gnome_vfs_mime_add_application_to_short_list(vfsobj->info->mime_type,ma->id)==GNOME_VFS_OK){
						VELOCITY_MSG("Application(%s) now associated with MIME type(%s)",ma->name,vfsobj->info->mime_type);
						if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(makedefault))){
							if(gnome_vfs_mime_set_default_application(vfsobj->info->mime_type,ma->id)==GNOME_VFS_OK){
								VELOCITY_MSG("Application now default for this MIME type",ma->name,vfsobj->info->mime_type);
							}else{
								VELOCITY_ERR("Error setting default application(%s) for MIME type(%s)",ma->name,vfsobj->info->mime_type);
							}
						}
					}else{
						VELOCITY_ERR("Error associating application(%s) with MIME type(%s)",ma->name,vfsobj->info->mime_type);
					}
				}
				gnome_vfs_mime_application_free(ma);
			}
			break;
		case GTK_RESPONSE_CANCEL:
		default:
			break;
	}
	gtk_widget_destroy(w);
}
gboolean velocity_menu_openwith_cb(GtkMenuItem*widget,GdkEvent*event,gpointer*user_data){
	if(event->type==GDK_BUTTON_PRESS){
		switch(event->button.button){
			case 1:  /*Left Click*/
				velocity_menu_openwith_lc(widget,user_data);
				break;  
			case 3: 
				velocity_menu_openwith_rc(widget,event,user_data);
				break;  
		}       
	}
	return(FALSE);
}
static GSList*velocity_get_lang_list(void){
	GSList*r=NULL;
	const GList*n=gnome_i18n_get_language_list(NULL);
	while(n){
		if(n->data){
			const char*l=(const char*)n->data;
			if((strlen(l)>0)&&(strcmp(l,"C")!=0)){	/*Skip C local and empty strings*/
				r=g_slist_prepend(r,n->data);
			}
		}
		n=n->next;
	}
	return(r);
}
void velocity_menu_open_cb(GtkMenuItem*menuitem,gpointer user_data){
	velocity_view*vview=(velocity_view*)user_data;
/*	//TODO:  This needs moved to the view's open/activate section, but for now this is just testing
	if(!velocity_clipboard_has_multiple(vview->selection)){
		velocity_vfs_obj*vfsobj=(velocity_vfs_obj*)vview->selection->items->data;
		if(vfsobj){
			GnomeVFSMimeAction*act=gnome_vfs_mime_get_default_action(vfsobj->info->mime_type);
			if(act){
				if(act->action_type==GNOME_VFS_MIME_ACTION_TYPE_COMPONENT){
					Bonobo_ActivationID aid;
					Bonobo_Unknown*obj=bonobo_activation_activate_from_id(act->action.component->iid,0,&aid,NULL);
					if(obj!=CORBA_OBJECT_NIL){
						CORBA_Environment ev;
						BonoboControl*control;
						char*name=bonobo_server_info_prop_lookup(act->action.component,"name",velocity_get_lang_list());
						VELOCITY_MSG("iid(%s),aid(%s)",act->action.component->iid,aid);
						//create control
						CORBA_exception_init(&ev);
						control=Bonobo_Unknown_queryInterface(obj,"IDL:Bonobo/Control:1.0",&ev);
						if(BONOBO_EX(&ev))control=CORBA_OBJECT_NIL;
						if(control!=CORBA_OBJECT_NIL){
							GtkWidget*w=gtk_window_new(GTK_WINDOW_POPUP);
							GtkWidget*c=bonobo_control_get_widget(control);
							VELOCITY_MSG("is control");
							gtk_container_add(GTK_CONTAINER(w),c);
							gtk_widget_show(w);
							gtk_widget_show(c);
						}else{
							bonobo_object_release_unref(control,NULL);
						}
						CORBA_exception_free(&ev);
						
					}
				}
				gnome_vfs_mime_action_free(act);
			}
		}
	}else{
*/
		velocity_view_activate(vview);
//	}
}
void velocity_menu_open_in_window_cb(GtkMenuItem*menuitem,gpointer user_data){
	if(user_data){
		velocity_view*vview=(velocity_view*)user_data;
		GList*n=vview->selection->items;
		if(n){
			char*uri=NULL;
			while(n){
				velocity_vfs_obj*vfsobj=n->data;
				if(vfsobj){
					if(velocity_vfs_is_directory(vfsobj)){
						uri=velocity_vfs_get_uri(vfsobj);
						velocity_windowman_new(uri);
						g_free(uri);
					}
				}
				n=n->next;
			}
		}else{
			VELOCITY_MSG("Nothing selected.  Duplicating window.");
			velocity_windowman_new(vview->clocation);
		}
	}else{
		VELOCITY_MSG("Nothing selected or invalid item.  Opening $HOME");
		velocity_windowman_new(NULL);
	}
}
void velocity_menu_scripts_cb(GtkMenuItem*menuitem,gpointer user_data){
	velocity_view*vview=(velocity_view*)g_object_get_data(G_OBJECT(menuitem),"velocity_view");
	velocity_script*vscript=(velocity_script*)user_data;
	velocity_vfs_obj*vfsobj=NULL;
	char*puri_txt=NULL;
	char*argstr=NULL;
	char*argdeplist=NULL;
	guint i=0;
	while(vscript->checked_dependancies[i]){
		if(!argdeplist){
			argdeplist=g_strdup_printf((vscript->needs_terminal)?"\\\"%s\\\"":"\"%s\"",vscript->checked_dependancies[i]);
		}else{
			char*tmp=argdeplist;
			argdeplist=g_strdup_printf((vscript->needs_terminal)?"%s \\\"%s\\\"":"%s \"%s\"",argdeplist,vscript->checked_dependancies[i]);
			g_free(tmp);
		}
		i++;
	}
	velocity_view_stop(vview);
	if(vview->selection->items){
		GList*n=vview->selection->items;
		while(n){
			vfsobj=(velocity_vfs_obj*)n->data;
			if(vfsobj){
				if(vscript->local_only){	/* make sure that our uri string is local only */
					puri_txt=velocity_vfs_uri_reformat(vfsobj->prefix);
				}else{
					puri_txt=g_strdup(vfsobj->prefix);
				}
				if(vscript->needs_terminal){
					if(!argstr){
						argstr=g_strdup_printf("%s -e \"\\\"%s\\\" \\\"%s\\\" %i %i %i %i \\\"%s\\\" %s \\\"%s\\\" \\\"%s\\\"",
							vprefs->locations->terminal,vscript->file,VELOCITY_SCRIPT_FLAG_EXECUTE,50,50,51,51,puri_txt,argdeplist,vfsobj->info->name,vfsobj->info->mime_type);
					}else{
						char*tmp=argstr;
						argstr=g_strdup_printf("%s \\\"%s\\\" \\\"%s\\\"",tmp,vfsobj->info->name,vfsobj->info->mime_type);
						g_free(tmp);
					}
				}else{
					if(!argstr){
						argstr=g_strdup_printf("\"%s\" \"%s\" %i %i %i %i \"%s\" %s \"%s\" \"%s\"",
							vscript->file,VELOCITY_SCRIPT_FLAG_EXECUTE,50,50,51,51,puri_txt,argdeplist,vfsobj->info->name,vfsobj->info->mime_type);
					}else{
						char*tmp=argstr;
						argstr=g_strdup_printf("%s \"%s\" \"%s\"",tmp,vfsobj->info->name,vfsobj->info->mime_type);
						g_free(tmp);
					}
				}
			}
			n=n->next;
			if((!n)||(!vscript->multiple_files)){	/*If end of list or if no multiple file support*/
				if(vscript->needs_terminal){	/*Make sure we end our command line properly*/
					char*tmp=argstr;
					argstr=g_strdup_printf("%s\"",tmp);
					g_free(tmp);
				}else{
					char*tmp=argstr;
					argstr=g_strdup_printf("%s",tmp);
					g_free(tmp);
				}
				VELOCITY_MSG("Spawning [%s]",argstr);
				g_spawn_command_line_async(argstr,NULL);
				g_free(argstr);
				g_free(puri_txt);
				argstr=NULL;
			}
		}
	}else{
		/*If there isn't anything selected we can assume that they right clicked on the background (the directory that they are in) for now...*/
		char*mime_type=VELOCITY_VFS_MIME_TYPE_DIRECTORY;
		char*name=velocity_vfs_uri_get_tail(vview->clocation);
		puri_txt=velocity_vfs_get_uri_parent(vview->clocation);
		if(vscript->local_only){	/* make sure that our uri string is local only */
			char*tmp=puri_txt;
			puri_txt=velocity_vfs_uri_reformat(tmp);
			g_free(tmp);
		}
		if(vscript->needs_terminal){
			argstr=g_strdup_printf("%s -e \"\\\"%s\\\" \\\"%s\\\" %i %i %i %i \\\"%s\\\" %s \\\"%s\\\" \\\"%s\\\"\"",
				vprefs->locations->terminal,vscript->file,VELOCITY_SCRIPT_FLAG_EXECUTE,50,50,51,51,puri_txt,argdeplist,name,mime_type);
		}else{
			argstr=g_strdup_printf("\"%s\" \"%s\" %i %i %i %i \"%s\" %s \"%s\" \"%s\"",
				vscript->file,VELOCITY_SCRIPT_FLAG_EXECUTE,50,50,51,51,puri_txt,argdeplist,name,mime_type);
		}
		VELOCITY_MSG("Spawning [%s]",argstr);
		g_spawn_command_line_async(argstr,NULL);
		g_free(argstr);
		g_free(name);
		g_free(puri_txt);
	}
	velocity_script_free(vscript);
	g_free(argdeplist);
}
static void velocity_menu_templates_new_entry_cb(GtkWidget*w,gpointer user_data){
	velocity_menu_templates_new_cb(GTK_DIALOG(g_object_get_data(G_OBJECT(w),"dialog")),GTK_RESPONSE_OK,user_data);
}
void velocity_menu_templates_new_cb(GtkDialog*dlg,gint response,gpointer user_data){
	velocity_template*vtemplate=(velocity_template*)user_data;
	velocity_view*vview=(velocity_view*)vtemplate->user_data;
	const char*str=gtk_entry_get_text(GTK_ENTRY(g_object_get_data(G_OBJECT(dlg),"entry")));
	char*tmp;
	if(response==GTK_RESPONSE_OK){
		if(!str)return;
		if((!vtemplate->file)&&(vtemplate->activate)){	/*Static template*/
			char*uri=g_strconcat(vview->clocation,str,NULL);
			gboolean ret=vtemplate->activate(uri,str);
			g_free(uri);
			if(!ret){
				GtkWidget*w=gtk_message_dialog_new(GTK_WINDOW(vview->parent),GTK_DIALOG_MODAL,GTK_MESSAGE_QUESTION,GTK_BUTTONS_OK_CANCEL,_("New name:  "));
				GtkWidget*t=gtk_entry_new();
				gtk_entry_set_text(GTK_ENTRY(t),str);
				gtk_container_add(GTK_CONTAINER(GTK_DIALOG(w)->vbox),t);
				g_object_set_data(G_OBJECT(t),"dialog",(gpointer)w);
				g_object_set_data(G_OBJECT(w),"entry",(gpointer)t);
				g_signal_connect(t,"activate",G_CALLBACK(velocity_menu_templates_new_entry_cb),vtemplate);
				g_signal_connect(w,"response",G_CALLBACK(velocity_menu_templates_new_cb),vtemplate);
				gtk_widget_show_all(w);
				return;
			}
		}else{	/*Normal template/script file*/
			/*count how many deps there are, and create an ordered list of deps to be passed to template exec*/
			char*argdeplist=NULL;
			char*argstr=NULL;
			guint i=0;
			while(vtemplate->checked_dependancies[i]){
				if(!argdeplist){
					argdeplist=g_strdup_printf("\"%s\"",vtemplate->checked_dependancies[i]);
				}else{
					tmp=argdeplist;
					argdeplist=g_strdup_printf("%s \"%s\"",argdeplist,vtemplate->checked_dependancies[i]);
					g_free(tmp);
				}
				i++;
			}
			/*If we want to going to modify the requested file name*/
			argstr=g_strdup_printf("\"%s\" \"%s\" \"%s\" \"%s\" %s",vtemplate->file,VELOCITY_TEMPLATE_FLAG_OVERIDE_DESIRED,str,vview->clocation,argdeplist);
			g_spawn_command_line_async(argstr,NULL);
			g_free(argdeplist);
			g_free(argstr);
		}
		/*Then we can tell everyone that we added a file*/
		tmp=g_strconcat(vview->clocation,str,NULL);
		velocity_vfs_inform_event(tmp,GNOME_VFS_MONITOR_EVENT_CREATED);
		g_free(tmp);
		velocity_template_free(vtemplate);
	}
	gtk_widget_destroy(GTK_WIDGET(dlg));
}
void velocity_menu_templates_cb(GtkMenuItem*menuitem,gpointer user_data){
	velocity_view*vview=(velocity_view*)g_object_get_data(G_OBJECT(menuitem),"velocity_view");
	velocity_template*vtemplate=(velocity_template*)user_data;
	GtkWidget*w;
	GtkWidget*t;
	char*prefix=gnome_vfs_unescape_string(vview->clocation,GNOME_VFS_URI_PATH_STR);
	char*filename=NULL;
	char*tmp=NULL;
	GnomeVFSURI*guri=NULL;
	int filen=0;
	vtemplate->user_data=vview;
	do{
		if(filen>1){
			g_free(filename);
			filename=g_strdup_printf(_("Copy %i of %s"),filen,vtemplate->desired);
		}
		if(tmp)g_free(tmp);
		if(filename){
			tmp=g_strconcat(prefix,filename,NULL);
		}else{
			tmp=g_strconcat(prefix,vtemplate->desired,NULL);
		}
		if(guri)gnome_vfs_uri_unref(guri);
		guri=gnome_vfs_uri_new(tmp);
		filen++;
	}while(gnome_vfs_uri_exists(guri));
	if(tmp)g_free(tmp);
	if(!filename)filename=g_strdup(vtemplate->desired);
	w=gtk_message_dialog_new(GTK_WINDOW(vview->parent),GTK_DIALOG_MODAL,GTK_MESSAGE_QUESTION,GTK_BUTTONS_OK_CANCEL,_("New name:  "));
	t=gtk_entry_new();
	gtk_entry_set_text(GTK_ENTRY(t),filename);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(w)->vbox),t);
	g_object_set_data(G_OBJECT(t),"dialog",(gpointer)w);
	g_object_set_data(G_OBJECT(w),"entry",(gpointer)t);
	g_signal_connect(t,"activate",G_CALLBACK(velocity_menu_templates_new_entry_cb),vtemplate);
	g_signal_connect(w,"response",G_CALLBACK(velocity_menu_templates_new_cb),vtemplate);
	gtk_widget_show_all(w);
	g_free(filename);
	g_free(prefix);
}
void velocity_menu_templates_error_cb(GtkMenuItem*menuitem,gpointer user_data){
	velocity_view*vview=(velocity_view*)g_object_get_data(G_OBJECT(menuitem),"velocity_view");
	velocity_template*vtemplate=(velocity_template*)user_data;
	GtkWidget*w;
	char*depsok=g_strdup("");
	char*depsfail=g_strdup("");
	guint i=0;
	while(vtemplate->checked_dependancies[i]){
		if(strcmp(vtemplate->checked_dependancies[i],VELOCITY_DEPENDANCY_VECTOR_ERROR)==0){
			char*tmp=depsfail;
			depsfail=g_strconcat(tmp,vtemplate->dependancies[i],",",NULL);
			g_free(tmp);
		}else{
			char*tmp=depsok;
			depsok=g_strconcat(tmp,vtemplate->dependancies[i],",",NULL);
			g_free(tmp);
		}
		i++;
	}
	w=gtk_message_dialog_new(GTK_WINDOW(vview->parent),GTK_DIALOG_MODAL,GTK_MESSAGE_INFO,GTK_BUTTONS_OK,
		_("Failed Template:  %s\n"
		"Passed Dependancies:  %s\n"
		"Failed Dependancies:  %s\n\n"
		"This template cannot be used because it requires a program, or programs,\n"
		"that you do not have installed.\n"
		"If these programs are installed, please check that they are installed properly\n"
		"and that they can be found in one of the directories specified by your PATH\n"
		"environment variable."),vtemplate->file,depsok,depsfail);
	gtk_dialog_run(GTK_DIALOG(w));
	gtk_widget_destroy(w);
	g_free(depsfail);
	g_free(depsok);
}
void velocity_menu_sendto_cb(GtkMenuItem*menuitem,gpointer user_data){
	velocity_view*vview=(velocity_view*)user_data;
	GList*n=vview->selection->items;
	GList*sl=NULL;
	GList*dl=NULL;
	char*dest_prefix=velocity_vfs_ensure_dir_uri(gtk_widget_get_name(GTK_WIDGET(menuitem)));
	/*Build source and destination uri lists, and work around a dumbass GNOME VFS oddity*/
	while(n){
		velocity_vfs_obj*vfsobj=(velocity_vfs_obj*)n->data;
		if(vfsobj){
			char*uri=velocity_vfs_get_uri(vfsobj);
			gboolean hacked=FALSE;
			if(velocity_vfs_is_symlink(vfsobj)){
				if(uri[strlen(uri)-1]=='/'){
					uri[strlen(uri)-1]=0;
					hacked=TRUE;
				}
			}
			sl=g_list_prepend(sl,gnome_vfs_uri_new(uri));
			if(hacked)uri[strlen(uri)+1]='-';
			g_free(uri);
			hacked=FALSE;
			uri=g_strconcat(dest_prefix,vfsobj->info->name,NULL);
			if(velocity_vfs_is_symlink(vfsobj)){
				if(uri[strlen(uri)-1]=='/'){
					uri[strlen(uri)-1]=0;
					hacked=TRUE;
				}
			}
			dl=g_list_prepend(dl,gnome_vfs_uri_new(uri));
			if(hacked)uri[strlen(uri)+1]='-';
			g_free(uri);
		}
		n=n->next;
	}
	VELOCITY_MSG("SendTo(%s)",dest_prefix);
	velocity_vfs_xfer(GTK_WINDOW(vview->parent),sl,dl,VELOCITY_VFS_XFER_MOVE,TRUE,NULL,NULL);
	g_free(dest_prefix);
}
gboolean velocity_menu_newfolder_cb(const gchar*uri,const gchar*name){
	if(uri){
		VELOCITY_MSG("New dir(%s)",name);
		//TODO:  velocity_vfs_new_dir here instead (as to let our one func handle error checking....)
		return(gnome_vfs_make_directory(uri,GNOME_VFS_PERM_USER_ALL)==GNOME_VFS_OK);
	}
	return(FALSE);
}
void velocity_menu_set_animation_cb(GtkMenuItem*menuitem,gpointer user_data){
	velocity_view*vview=(velocity_view*)g_object_get_data(G_OBJECT(menuitem),"velocity_view");
	vview->prefs->animation_mode=GPOINTER_TO_INT(user_data);
	vprefs->view->animation_mode=vview->prefs->animation_mode;
	if(vview->user_data){
		velocity_window*vwin=(velocity_window*)vview->user_data;
		gtk_widget_destroy(GTK_WIDGET(gnome_app_get_dock_item_by_name(GNOME_APP(vwin->widget),GNOME_APP_MENUBAR_NAME)));
		GNOME_APP(vwin->widget)->menubar=NULL;	/*gtk_widget_destroy doesn't do this and gnome_app_set_menus wants it*/
		velocity_window_menubar_create(vwin);
	}
	velocity_preferences_sync();
}
void velocity_menu_file_types_cb(GtkMenuItem*menuitem,gpointer user_data){
	g_spawn_command_line_async(GNOME_FILE_TYPES_PROPERTIES,NULL);
}
void velocity_menu_preferences_cb(GtkMenuItem*menuitem,gpointer user_data){
	velocity_view*vview=(velocity_view*)user_data;
	velocity_preferences_window_new(GTK_WIDGET(vview->parent));
}
void velocity_window_tree_cb(velocity_tree_item*vti,gpointer user_data){
	velocity_window*vwin=(velocity_window*)user_data;
	velocity_window_set_location(vwin,vti->uri);
}
void velocity_window_on_quit_cb(GtkWidget*w,GdkEvent*event,gpointer user_data){
	velocity_window*vwin=(velocity_window*)user_data;
	velocity_view_stop(vwin->view);
	velocity_windowman_free(vwin);
}
void velocity_menu_file_quit_cb(GtkMenuItem*menuitem,gpointer user_data){
	velocity_windowman_free_all();
}
void velocity_menu_edit_selectall_cb(GtkMenuItem*menuitem,gpointer user_data){
	velocity_window*vwin=(velocity_window*)user_data;
	GList*n=vwin->view->dirs;
	guint li=0;
	velocity_window_clear_selection(vwin);
	velocity_clipboard_set_details(vwin->view->selection,vwin->view->clocation,VELOCITY_CLIPBOARD_COPY);
velocity_menu_edit_selectall_jmp:
	while(n){
		velocity_vfs_obj*vfsobj=(velocity_vfs_obj*)n->data;
		if(vfsobj){
			velocity_clipboard_prepend_file(vwin->view->selection,vfsobj);
			velocity_icon_highlight(vfsobj->icon);
		}
		n=n->next;
	}
	if((li<1)&&(vwin->view->dirs!=vwin->view->files)){
		n=vwin->view->files;
		li++;
		goto velocity_menu_edit_selectall_jmp;
	}
}
void velocity_menu_cut_cb(GtkWidget*menuitem,gpointer user_data){
	velocity_view*vview=(velocity_view*)user_data;
	velocity_vfs_obj*vfsobj=NULL;
	GList*n=NULL;
	if(velocity_clipboard_has_items(vview->selection)){
		velocity_clipboard_flush(vclipboard);
		velocity_clipboard_set_details(vclipboard,vview->clocation,VELOCITY_CLIPBOARD_CUT);
		n=vview->selection->items;
		while(n){
			vfsobj=(velocity_vfs_obj*)n->data;
			if(vfsobj)velocity_clipboard_prepend_file(vclipboard,vfsobj);
			n=n->next;
		}
	}
	velocity_windowman_set_all(VELOCITY_WINDOWMAN_PASTE_ON);
}
void velocity_menu_copy_cb(GtkWidget*menuitem,gpointer user_data){
	velocity_view*vview=(velocity_view*)user_data;
	velocity_vfs_obj*vfsobj=NULL;
	GList*n=NULL;
	velocity_clipboard_flush(vclipboard);
	velocity_clipboard_set_details(vclipboard,vview->clocation,VELOCITY_CLIPBOARD_COPY);
	n=vview->selection->items;
	while(n){
		vfsobj=(velocity_vfs_obj*)n->data;
		if(vfsobj)velocity_clipboard_prepend_file(vclipboard,vfsobj);
		n=n->next;
	}
	velocity_windowman_set_all(VELOCITY_WINDOWMAN_PASTE_ON);
}
void velocity_menu_paste_cb(GtkWidget*menuitem,gpointer user_data){
	velocity_view*vview=(velocity_view*)user_data;
	GList*n=vclipboard->items;
	velocity_vfs_obj*vfsobj=NULL;
	char*dest_prefix=vview->clocation; /*If we are unsure put it in the current directory; a decent failsafe*/
	GList*sl=NULL;
	GList*dl=NULL;
	//TODO:  Check list of special folders....give _module_defined_error_ if somebody is trying to move it....
	if(velocity_clipboard_has_items(vclipboard)){
		if((velocity_clipboard_has_items(vview->selection))&&(!velocity_clipboard_has_multiple(vview->selection))){
			vfsobj=(velocity_vfs_obj*)vview->selection->items->data;
			if(velocity_vfs_is_directory(vfsobj)){
				char*tmp=velocity_vfs_get_uri(vfsobj);
				dest_prefix=velocity_vfs_uri_reformat(tmp);
				g_free(tmp);
			}
		}
		/*Build source and destination uri lists, and work around a dumbass GNOME VFS oddity*/
		n=vclipboard->items;
		while(n){
			vfsobj=(velocity_vfs_obj*)n->data;
			if(vfsobj){
				char*uri=velocity_vfs_get_uri(vfsobj);
				gboolean hacked=FALSE;
				if(velocity_vfs_is_symlink(vfsobj)){
					if(uri[strlen(uri)-1]=='/'){
						uri[strlen(uri)-1]=0;
						hacked=TRUE;
					}
				}
				sl=g_list_prepend(sl,gnome_vfs_uri_new(uri));
				if(hacked)uri[strlen(uri)+1]='-';
				g_free(uri);
				hacked=FALSE;
				uri=g_strconcat(dest_prefix,vfsobj->info->name,NULL);
				if(velocity_vfs_is_symlink(vfsobj)){
					if(uri[strlen(uri)-1]=='/'){
						uri[strlen(uri)-1]=0;
						hacked=TRUE;
					}
				}
				dl=g_list_prepend(dl,gnome_vfs_uri_new(uri));
				if(hacked)uri[strlen(uri)+1]='-';
				g_free(uri);
			}
			n=n->next;
		}
		switch(vclipboard->mode){
			case VELOCITY_CLIPBOARD_CUT:	/*Move the files*/
				velocity_vfs_xfer(GTK_WINDOW(vview->parent),sl,dl,VELOCITY_VFS_XFER_MOVE,TRUE,NULL,NULL);
				break;
			case VELOCITY_CLIPBOARD_LINK:	/*Symlink the new files to the sources*/
				velocity_vfs_xfer(GTK_WINDOW(vview->parent),sl,dl,VELOCITY_VFS_XFER_LINK,TRUE,NULL,NULL);
				break;
			case VELOCITY_CLIPBOARD_COPY:	/*Copy the files*/
			default:
				velocity_vfs_xfer(GTK_WINDOW(vview->parent),sl,dl,VELOCITY_VFS_XFER_COPY,TRUE,NULL,NULL);
				break;
		}
		velocity_clipboard_flush(vclipboard);
		velocity_windowman_set_all(VELOCITY_WINDOWMAN_PASTE_OFF);
	}
}
void velocity_menu_paste_link_cb(GtkWidget*menuitem,gpointer user_data){
	vclipboard->mode=VELOCITY_CLIPBOARD_LINK;
	velocity_menu_paste_cb(menuitem,user_data);
}
void velocity_menu_delete_cb(GtkWidget*menuitem,gpointer user_data){
	velocity_view*vview=(velocity_view*)user_data;
	if(velocity_clipboard_has_items(vview->selection)){
		GList*uris;
		if(vprefs->trash->ask){
			GtkWidget*w=gtk_message_dialog_new(NULL,GTK_DIALOG_MODAL,GTK_MESSAGE_QUESTION,GTK_BUTTONS_YES_NO,
				_("Are you sure you would like to delete these files?"),NULL);
			GtkWidget*ask=gtk_check_button_new();
			GtkWidget*h=gtk_hbox_new(FALSE,6);
			GtkWidget*t=gtk_image_new_from_stock(GTK_STOCK_DIALOG_QUESTION,GTK_ICON_SIZE_BUTTON);
			gtk_container_add(GTK_CONTAINER(h),t);
			t=gtk_label_new(_("Never ask this again"));
			gtk_container_add(GTK_CONTAINER(h),t);
			gtk_container_add(GTK_CONTAINER(ask),h);
			gtk_container_add(GTK_CONTAINER(GTK_DIALOG(w)->vbox),ask);
			gtk_widget_show_all(GTK_DIALOG(w)->vbox);
			if(gtk_dialog_run(GTK_DIALOG(w))==GTK_RESPONSE_NO){
				gtk_widget_destroy(w);
				return;
			}
			if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ask)))vprefs->trash->ask=FALSE;
			gtk_widget_destroy(w);
		}
		uris=velocity_clipboard_to_uri_list(vview->selection);
		velocity_vfs_delete(uris,GTK_WINDOW(vview->parent));
		gnome_vfs_list_deep_free(uris);
	}
}
void velocity_menu_trash_cb(GtkWidget*menuitem,gpointer user_data){
	velocity_view*vview=(velocity_view*)user_data;
	if(velocity_clipboard_has_items(vview->selection)){
		GList*n=vview->selection->items;
		GList*sl=NULL;
		GList*dl=NULL;
		if(vprefs->trash->ask){
			GtkWidget*w=gtk_message_dialog_new(NULL,GTK_DIALOG_MODAL,GTK_MESSAGE_QUESTION,GTK_BUTTONS_YES_NO,
				_("Are you sure you would like to send these files to the trash?"),NULL);
			GtkWidget*ask=gtk_check_button_new();
			GtkWidget*h=gtk_hbox_new(FALSE,6);
			GtkWidget*t=gtk_image_new_from_stock(GTK_STOCK_DIALOG_QUESTION,GTK_ICON_SIZE_BUTTON);
			gtk_container_add(GTK_CONTAINER(h),t);
			t=gtk_label_new(_("Never ask this again"));
			gtk_container_add(GTK_CONTAINER(h),t);
			gtk_container_add(GTK_CONTAINER(ask),h);
			gtk_container_add(GTK_CONTAINER(GTK_DIALOG(w)->vbox),ask);
			gtk_widget_show_all(GTK_DIALOG(w)->vbox);
			if(gtk_dialog_run(GTK_DIALOG(w))==GTK_RESPONSE_NO){
				gtk_widget_destroy(w);
				return;
			}
			if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ask)))vprefs->trash->ask=FALSE;
			gtk_widget_destroy(w);
		}
		while(n){
			velocity_vfs_obj*vfsobj=(velocity_vfs_obj*)n->data;
			if(vfsobj){
				char*uri=velocity_vfs_get_uri(vfsobj);
				gboolean hacked=FALSE;
				if(velocity_vfs_is_symlink(vfsobj)){
					if(uri[strlen(uri)-1]=='/'){
						uri[strlen(uri)-1]=0;
						hacked=TRUE;
					}
				}
				sl=g_list_prepend(sl,gnome_vfs_uri_new(uri));
				if(hacked)uri[strlen(uri)+1]='-';
				g_free(uri);
				uri=g_strconcat(vprefs->locations->trash,vfsobj->info->name,NULL);
				dl=g_list_prepend(dl,gnome_vfs_uri_new(uri));
				g_free(uri);
			}
			n=n->next;
		}
		velocity_vfs_xfer(GTK_WINDOW(vview->parent),sl,dl,VELOCITY_VFS_XFER_MOVE_TO_TRASH,TRUE,NULL,NULL);
	}
}
static void velocity_menu_rename_response_cb(GtkDialog*dlg,gint response_id,gpointer user_data){
	if(response_id==GTK_RESPONSE_OK){
		const gchar*str=gtk_entry_get_text(GTK_ENTRY(g_object_get_data(G_OBJECT(dlg),"entry")));
		if(str){
			velocity_vfs_obj*vfsobj=(velocity_vfs_obj*)user_data;
			velocity_view*vview=vfsobj->data;	/*This is feels like such a dirty hack...*/
			char*ouri=velocity_vfs_get_uri(vfsobj);
			GnomeVFSResult r=velocity_vfs_rename(vfsobj,str);
			if(r==GNOME_VFS_OK){
				velocity_view_set_icon_caption(vview,vfsobj,FALSE);
				if(velocity_vfs_is_directory(vfsobj)){
					char*uri=velocity_vfs_get_uri(vfsobj);
					velocity_windowman_trees_unregister(ouri);
					velocity_windowman_trees_register(uri);
					g_free(uri);
				}
				gtk_widget_destroy(GTK_WIDGET(dlg));
			}else{
				//TODO:  An error dialog would be more helpful...
				VELOCITY_ERR("VFS-ERROR:  %s",velocity_vfs_get_error_string(r));
			}
			g_free(ouri);
		}
	}else{
		gtk_widget_destroy(GTK_WIDGET(dlg));
	}
}
void velocity_menu_rename_cb(GtkMenuItem*menuitem,gpointer user_data){
	velocity_view*vview=(velocity_view*)user_data;
	GList*n=vview->selection->items;
	while(n){
		velocity_vfs_obj*vfsobj=(velocity_vfs_obj*)n->data;
		if(vfsobj){
/*			GtkWidget*w=gnome_request_dialog(FALSE,_("New name:  "),vfsobj->info->name,255,
				(GnomeStringCallback)velocity_menu_renamedlg_cb,(gpointer)vfsobj,(gpointer)vview->parent);
			gtk_widget_show(w);*/
			GtkWidget*e=gtk_entry_new();
			GtkWidget*dlg=gtk_message_dialog_new(vview->parent,GTK_DIALOG_MODAL,GTK_MESSAGE_QUESTION,GTK_BUTTONS_OK_CANCEL,_("New name:  "));
			gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dlg)->vbox),e);
			gtk_entry_set_text(GTK_ENTRY(e),vfsobj->info->name);
			g_object_set_data(G_OBJECT(dlg),"entry",(gpointer)e);
			g_signal_connect(dlg,"response",G_CALLBACK(velocity_menu_rename_response_cb),vfsobj);
			gtk_widget_show_all(dlg);
		}
		n=n->next;
	}
}
void velocity_menu_properties_cb(GtkMenuItem*menuitem,gpointer user_data){
	velocity_view*vview=(velocity_view*)user_data;
	velocity_vfs_obj*vfsobj=NULL;
	GList*n=vview->selection->items;
	while(n){
		//TODO:  This needs to be done properly at some point in time....
		vfsobj=(velocity_vfs_obj*)n->data;
		if(vfsobj)dlgFileProperties_new(vfsobj);
		n=n->next;
	}
}
void velocity_menu_help_about_cb(GtkMenuItem*menuitem,gpointer user_data){
	const char*authors[]={"Kyle Davis",NULL};
	GtkWidget*w=gnome_about_new("Velocity",VERSION,"Copyright(c) 2002-2003 Kyle Davis",
		_("Manage your data with power, precision, and Velocity.\n\n"
		"Velocity is a GNOME file manager designed to be a powerful alternative to Nautilus.\n\n"
		"Thank you for trying Velocity!"),
		authors,NULL,NULL,
		gdk_pixbuf_new_from_file(PACKAGE_PIXMAPS_DIR G_DIR_SEPARATOR_S VELOCITY_ICON,NULL));
	gtk_widget_show(w);
}
void velocity_menu_show_hidden_cb(GtkMenuItem*menuitem,gpointer user_data){
	velocity_view*vview=(velocity_view*)user_data;
	vview->prefs->filter^=VELOCITY_VFS_FILTER_HIDDEN;
	if(velocity_vfs_is_desktop_dir_uri(vview->clocation)){
		vprefs->desktop->filter=vview->prefs->filter;
	}else{
		vprefs->view->filter=vview->prefs->filter;
	}
	if(vview->user_data){
		velocity_window*vwin=(velocity_window*)vview->user_data;
		velocity_toolbar_refresh_cb(NULL,(gpointer)vwin);
	}else{
		char*tmp=g_strdup(vview->clocation);
		velocity_view_set_location(vview,tmp);
		g_free(tmp);
	}
	velocity_preferences_sync();
}
void velocity_toolbar_back_cb(GtkButton*button,gpointer user_data){
	velocity_window_history_back((velocity_window*)user_data);
}
void velocity_toolbar_forward_cb(GtkButton*button,gpointer user_data){
	velocity_window_history_forward((velocity_window*)user_data);
}
void velocity_toolbar_up_cb(GtkButton*button,gpointer user_data){
	velocity_window*vwin=(velocity_window*)user_data;
	char*uri=vwin->view->clocation;
	VELOCITY_MSG("current URI=%s",vwin->view->clocation);
	//TODO:  All of this should go into a module URI aware function (velocity_window_get_uri_parent) 
	if((uri)&&(strncmp(uri,VELOCITY_MODULE_URI_PREFIX,strlen(VELOCITY_MODULE_URI_PREFIX))==0)){
		char*substr=uri+strlen(VELOCITY_MODULE_URI_PREFIX);
		GList*n=velocity_module_tracker_get_list();
		while(n){
			if(n->data){
				velocity_module*vmod=(velocity_module*)n->data;
				if(vmod->info->id){
					char*tmp=g_strconcat(vmod->info->id,":",NULL);
					int l=strlen(tmp);
					if(strncmp(substr,tmp,l)==0){
						g_free(tmp);
						uri=substr+l;
						VELOCITY_MSG("this is a URI for mod(%s)",vmod->info->id);
						break;
					}
					g_free(tmp);
				}
			}
			n=n->next;
		}
	}
	if(uri){
		char*tmp=gnome_vfs_unescape_string(uri,GNOME_VFS_URI_PATH_STR);
		if(velocity_vfs_uri_has_parent(tmp)){
			char*puri=velocity_vfs_get_uri_parent(tmp);
			VELOCITY_MSG("parent URI=%s",puri);
			if(puri)velocity_window_set_location(vwin,puri);
			g_free(puri);
		}
		g_free(tmp);
	}
}
void velocity_toolbar_stop_cb(GtkButton*button,gpointer user_data){
	velocity_window*vwin=(velocity_window*)user_data;
	velocity_view_stop(vwin->view);
}
void velocity_toolbar_refresh_cb(GtkButton*button,gpointer user_data){
	velocity_window*vwin=(velocity_window*)user_data;
	char*tmp=gnome_vfs_unescape_string(vwin->view->clocation,GNOME_VFS_URI_PATH_STR);
	VELOCITY_MSG("going_to(%s)",tmp);
	velocity_window_set_location(vwin,tmp);
	g_free(tmp);
}
gboolean velocity_toolbar_refresh_click_cb(GtkButton*button,GdkEventButton*event,gpointer user_data){
	velocity_window*vwin=(velocity_window*)user_data;
	char*tmp;
	switch(event->button){
		case 2:
			tmp=gnome_vfs_unescape_string(vwin->view->clocation,GNOME_VFS_URI_PATH_STR);
			velocity_windowman_new(tmp);
			g_free(tmp);
			return(TRUE);
			break;
		default:
			break;
	}
	return(FALSE);
}
void velocity_toolbar_cut_cb(GtkButton*button,gpointer user_data){
	velocity_window*vwin=(velocity_window*)user_data;
	velocity_menu_cut_cb(GTK_WIDGET(button),(gpointer)vwin->view);
}
void velocity_toolbar_copy_cb(GtkButton*button,gpointer user_data){
	velocity_window*vwin=(velocity_window*)user_data;
	velocity_menu_copy_cb(GTK_WIDGET(button),(gpointer)vwin->view);
}
void velocity_toolbar_paste_cb(GtkButton*button,gpointer user_data){
	velocity_window*vwin=(velocity_window*)user_data;
	velocity_menu_paste_cb(GTK_WIDGET(button),(gpointer)vwin->view);
}
void velocity_toolbar_trash_cb(GtkButton*button,gpointer user_data){
	velocity_window*vwin=(velocity_window*)user_data;
	velocity_menu_trash_cb(GTK_WIDGET(button),(gpointer)vwin->view);
}
static void velocity_toolbar_trash_goto_cb(GtkMenuItem*menuitem,gpointer user_data){
	velocity_view*vview=(velocity_view*)user_data;
	velocity_view_set_location(vview,vprefs->locations->trash);
}
static void velocity_toolbar_trash_new_win_cb(GtkMenuItem*menuitem,gpointer user_data){
	velocity_view*vview=(velocity_view*)user_data;
	velocity_windowman_new(vprefs->locations->trash);
}
gboolean velocity_toolbar_trash_click_cb(GtkButton*button,GdkEventButton*event,gpointer user_data){
	velocity_window*vwin=(velocity_window*)user_data;
	switch(event->button){
		case 3:
			{
			void*mstream[]={VELOCITY_MENU_FACTORY_PIXBUF,VELOCITY_TRASH_FULL,_("_Empty Trash"),VELOCITY_FACTORY_TRUE,velocity_menu_empty_trash_cb,vwin->view,
				VELOCITY_MENU_FACTORY_SEPERATOR,
				VELOCITY_MENU_FACTORY_PIXBUF,VELOCITY_FOLDER,_("Open Trash"),VELOCITY_FACTORY_TRUE,velocity_toolbar_trash_goto_cb,vwin->view,
				VELOCITY_MENU_FACTORY_PIXBUF,VELOCITY_FOLDER,_("Open in New Window"),VELOCITY_FACTORY_TRUE,velocity_toolbar_trash_new_win_cb,vwin->view,
				NULL};
				GtkWidget*menu=velocity_menu_factory(vwin->view,GTK_WIDGET(vwin->view->parent),mstream);
				gtk_menu_popup(GTK_MENU(menu),NULL,NULL,NULL,NULL,event->button,event->time);
			}
			return(TRUE);
			break;
		case 2:
			velocity_toolbar_trash_new_win_cb(NULL,vwin->view);
			return(TRUE);
			break;
		default:
			break;
	}
	return(FALSE);
}
void velocity_toolbar_undo_cb(GtkButton*button,gpointer user_data){
	VELOCITY_MSG("FIXME:  This doesn't do anything yet!\n");
}
void velocity_toolbar_search_cb(GtkButton*button,gpointer user_data){
	velocity_window*vwin=(velocity_window*)user_data;
	char*path=gnome_vfs_unescape_string(vwin->view->clocation,GNOME_VFS_URI_PATH_STR);
	char*tmp=g_strdup_printf("--path=%s",path);
	char*argv[]={"gnome-search-tool",tmp,NULL};
	VELOCITY_MSG("spawning GNOME's search tool (for now)");
	g_spawn_async(NULL,argv,NULL,G_SPAWN_SEARCH_PATH|G_SPAWN_STDOUT_TO_DEV_NULL|G_SPAWN_STDERR_TO_DEV_NULL,NULL,NULL,NULL,NULL);
	g_free(tmp);
	g_free(path);
}
void velocity_toolbar_home_cb(GtkButton*button,gpointer user_data){
	velocity_window*vwin=(velocity_window*)user_data;
	velocity_window_set_location(vwin,NULL);
}
static void velocity_toolbar_home_new_win_cb(GtkButton*button,gpointer user_data){
	velocity_windowman_new(NULL);
}
gboolean velocity_toolbar_home_click_cb(GtkButton*button,GdkEventButton*event,gpointer user_data){
	velocity_window*vwin=(velocity_window*)user_data;
	switch(event->button){
		case 3:
			{
			void*mstream[]={VELOCITY_MENU_FACTORY_PIXBUF,VELOCITY_FOLDER,_("Open Home"),VELOCITY_FACTORY_TRUE,velocity_toolbar_home_cb,vwin,
				VELOCITY_MENU_FACTORY_PIXBUF,VELOCITY_FOLDER,_("Open in New Window"),VELOCITY_FACTORY_TRUE,velocity_toolbar_home_new_win_cb,vwin->view,
				NULL};
				GtkWidget*menu=velocity_menu_factory(vwin->view,GTK_WIDGET(vwin->view->parent),mstream);
				gtk_menu_popup(GTK_MENU(menu),NULL,NULL,NULL,NULL,event->button,event->time);
			}
			break;
		case 2:
			velocity_toolbar_home_new_win_cb(NULL,NULL);
			return(TRUE);
			break;
		case 1:
		default:
			break;
	}
	return(FALSE);
}
gint velocity_window_progress_cb(gpointer user_data){
	velocity_window*vwin=(velocity_window*)user_data;
	GtkProgressBar*progress=gnome_appbar_get_progress(GNOME_APPBAR(GNOME_APP(vwin->widget)->statusbar));
	char*strbuf=g_strdup_printf(_("Reading... (%i of %i)"),vwin->view->read.objects-(vwin->view->total.images-vwin->view->read.images),vwin->view->total.objects);
	if((vwin->view->read.objects>0)&&(vwin->view->total.objects>0)){
		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(progress),(double)(vwin->view->read.objects+vwin->view->read.images)/(double)(vwin->view->total.objects+vwin->view->total.images));
	}
	gnome_appbar_set_status(GNOME_APPBAR(GNOME_APP(vwin->widget)->statusbar),strbuf);
	g_free(strbuf);
	return(TRUE);
}
void velocity_menu_empty_trash_cb(GtkMenuItem*mi,gpointer user_data){
	velocity_view*vview=(velocity_view*)user_data;
	velocity_vfs_empty_trash(GTK_WINDOW(vview->parent),TRUE,NULL,NULL);
}
void velocity_menu_tool_network_cb(GtkMenuItem*mi,gpointer user_data){
	velocity_view*vview=(velocity_view*)user_data;
	velocity_view_set_location(vview,VELOCITY_VFS_NETWORK_URI);
}
void velocity_menu_tool_applications_cb(GtkMenuItem*mi,gpointer user_data){
	velocity_view*vview=(velocity_view*)user_data;
	velocity_view_set_location(vview,VELOCITY_VFS_APPLICATIONS_URI);
}
void velocity_menu_tool_preferences_cb(GtkMenuItem*mi,gpointer user_data){
	velocity_view*vview=(velocity_view*)user_data;
	velocity_view_set_location(vview,VELOCITY_VFS_PREFERENCES_URI);
}
void*velocity_window_on_status_event_cb(velocity_view*vview,velocity_view_status_event vevent,const char*uri,GnomeVFSResult result,gpointer user_data){
	velocity_window*vwin=(velocity_window*)user_data;
	GList*n=velocity_module_tracker_get_list();
	while(n){
		if(n->data){
			velocity_module*vmod=(velocity_module*)n->data;
			if(vmod->on_view_status_event){
				gpointer ret=vmod->on_view_status_event(vview,vevent,uri,result,user_data);
				if(ret){
					VELOCITY_MSG("module(%s)::status_event(%i) caught",vmod->info->id,(int)vevent);
					return(ret);
				}
			}
		}
		n=n->next;
	}
	switch(vevent){
		case VELOCITY_VIEW_STATUS_EVENT_VALIDATE:
			VELOCITY_MSG("VALIDATE");
			if(vwin){
				char*sname=velocity_vfs_get_human_readable(uri);
				char*status=g_strdup_printf(_("Opening %s ..."),sname);
				set_cursor_busy(vwin->widget);
				gtk_widget_set_sensitive(velocity_toolbar_factory_get_button(vwin->toolbar,GTK_STOCK_STOP),TRUE);
				gnome_appbar_clear_stack(GNOME_APPBAR(GNOME_APP(vwin->widget)->statusbar));
				gnome_appbar_set_status(GNOME_APPBAR(GNOME_APP(vwin->widget)->statusbar),status);
				g_free(status);
				g_free(sname);
			}
			break;
		case VELOCITY_VIEW_STATUS_EVENT_ERROR:
			if((vwin)&&(result)){
				char*status=g_strdup_printf(_("Error:  %s"),velocity_vfs_get_error_string(result));
				gnome_appbar_clear_stack(GNOME_APPBAR(GNOME_APP(vwin->widget)->statusbar));
				gnome_appbar_set_status(GNOME_APPBAR(GNOME_APP(vwin->widget)->statusbar),status);
				velocity_tree_unregister_item(vwin->tree,uri);
				g_free(status);
			}
			break;
		case VELOCITY_VIEW_STATUS_EVENT_GO:
			VELOCITY_MSG("GO");
			if(vwin){
				char*sname=velocity_vfs_get_human_readable(uri);
				char*title=g_strdup_printf(_("Velocity - %s"),sname);
				char*status=g_strdup_printf(_("Scanning %s ..."),sname);
				velocity_tree_item*vti=velocity_tree_register_item(vwin->tree,uri);
//TODO:  We have issues with this one...major ones...
				if(vti){
					if(vprefs->sync_tree){
						GtkTreeSelection*ts=gtk_tree_view_get_selection(GTK_TREE_VIEW(vti->tree->tree));
						if(ts){
							if(!gtk_tree_selection_iter_is_selected(ts,&vti->iter)){
								velocity_tree_item_select_func tmp=vti->tree->select_func;
								vti->tree->select_func=NULL;
								gtk_tree_selection_select_iter(ts,&vti->iter);
								vti->tree->select_func=tmp;
							}
						}
					}
				}
				gnome_appbar_clear_stack(GNOME_APPBAR(GNOME_APP(vwin->widget)->statusbar));
				gnome_appbar_set_status(GNOME_APPBAR(GNOME_APP(vwin->widget)->statusbar),status);
				velocity_window_update_location_entry(vwin,uri);
				gtk_window_set_title(GTK_WINDOW(vwin->widget),title);
				velocity_window_history_append_uri(vwin,uri);
				g_free(status);
				g_free(title);
				g_free(sname);
			}
			break;
		case VELOCITY_VIEW_STATUS_EVENT_LOADING:
			if(vwin){
				if(vwin->progress_handle)gtk_timeout_remove(vwin->progress_handle);
				vwin->progress_handle=gtk_timeout_add(250,(GtkFunction)velocity_window_progress_cb,(gpointer)vwin);
			}
			break;
		case VELOCITY_VIEW_STATUS_EVENT_DONE:
			if(vwin){
				GnomeVFSFileSize sz=velocity_vfs_get_free_space(vview->clocation);
				char*szstr=gnome_vfs_format_file_size_for_display(sz);
				char*strbuf;
				char*s1=NULL;
				char*s2=NULL;
				char*s3=NULL;
				if(vview->total.dirs>0)s1=g_strdup_printf("%i folders",vview->total.dirs);
				if(vview->total.files>0)s2=g_strdup_printf("%i files",vview->total.files);
				if(vview->total.filtered>0)s3=g_strdup_printf("%i filtered",vview->total.filtered);
				strbuf=g_strdup_printf(_("Disk free: %s  -  %s%s%s%s%s"),szstr,
					(s1)?s1:"",((s1)&&((s2)||(s3)))?", ":"",
					(s2)?s2:"",((s2)&&(s3))?", ":"",
					(s3)?s3:"");
				if(vwin->progress_handle)gtk_timeout_remove(vwin->progress_handle);
				vwin->progress_handle=(guint)NULL;
				velocity_window_progress_cb((gpointer)vwin);	/*Update once more to ensure everything got updated properly*/
				set_cursor_normal(vwin->widget);
				gnome_appbar_clear_stack(GNOME_APPBAR(GNOME_APP(vwin->widget)->statusbar));
				gnome_appbar_set_status(GNOME_APPBAR(GNOME_APP(vwin->widget)->statusbar),strbuf);
				gtk_widget_set_sensitive(velocity_toolbar_factory_get_button(vwin->toolbar,GTK_STOCK_STOP),FALSE);
				if(s1)g_free(s1);
				if(s2)g_free(s2);
				if(s3)g_free(s3);
				g_free(strbuf);
				g_free(szstr);
			}
			break;
		case VELOCITY_VIEW_STATUS_EVENT_STOP:
			if(vwin){
				if(vwin->progress_handle){
					gtk_timeout_remove(vwin->progress_handle);
					vwin->progress_handle=(guint)NULL;
				}
				set_cursor_normal(vwin->widget);
				gtk_widget_set_sensitive(velocity_toolbar_factory_get_button(vwin->toolbar,GTK_STOCK_STOP),FALSE);
			}
			break;
		case VELOCITY_VIEW_STATUS_EVENT_NEW_WINDOW:
			velocity_windowman_new(uri);
			break;
		default:
			VELOCITY_MSG("recieved unknown event");
			break;
	}
	return(NULL);
}
void*velocity_window_on_behavior_event_cb(velocity_view*vview,velocity_view_behavior_event vevent,GdkEvent*event,velocity_vfs_obj*vfsobj,gpointer user_data){
	velocity_window*vwin=(velocity_window*)user_data;
	/*We always need to pass this signal to our modules for sanity's sake usually*/
	if((vfsobj)||(vevent==VELOCITY_VIEW_BEHAVIOR_EVENT_BACKGROUND_CONTEXT)){
		GList*n=velocity_module_tracker_get_list();
		while(n){
			if(n->data){
				velocity_module*vmod=(velocity_module*)n->data;
				if(vmod->on_view_behavior_event){
					if((!vfsobj)||(velocity_module_supports_mime_type(vmod,vfsobj->info->mime_type))){
						gpointer ret=vmod->on_view_behavior_event(vview,vevent,event,vfsobj,user_data);
						if(ret){
							VELOCITY_MSG("module(%s)::behavior_event(%i) caught",vmod->info->id,(int)vevent);
							return(ret);
						}
					}
				}
			}
			n=n->next;
		}
	}
	switch(vevent){
		case VELOCITY_VIEW_BEHAVIOR_EVENT_OBJECT_LOADED:
			if(vfsobj){
				if(velocity_vfs_is_directory(vfsobj)){
					GnomeVFSURI*curi=gnome_vfs_uri_new(vview->clocation);
					char*urilocation;
					curi=gnome_vfs_uri_append_string(curi,vfsobj->info->name);
					urilocation=gnome_vfs_uri_to_string(curi,GNOME_VFS_URI_HIDE_USER_NAME|GNOME_VFS_URI_HIDE_PASSWORD);
					velocity_windowman_trees_register(urilocation);
					g_free(urilocation);
					gnome_vfs_uri_unref(curi);
				}
			}
			break;
		case VELOCITY_VIEW_BEHAVIOR_EVENT_OBJECT_GET_ICON:
			if(velocity_vfs_is_desktop_dir_uri(vview->clocation)&&(velocity_vfs_is_symlink(vfsobj))){
				char*uri=velocity_vfs_get_mime_icon_uri(vfsobj->info->mime_type,vfsobj->info->symlink_name,
					MAX(vview->prefs->profile->icon_size.x,vview->prefs->profile->icon_size.y));
				GdkPixbuf*r=NULL;
				if(uri){
					r=velocity_image_cache_get(uri,vview->prefs->profile->icon_size.x,vview->prefs->profile->icon_size.y,NULL,VELOCITY_IMAGE_CACHE_PRIORITY_HIGH);
					g_free(uri);
				}
				return(r);
			}
			break;
		case VELOCITY_VIEW_BEHAVIOR_EVENT_CLICK:
			if(vwin){
				if(velocity_vfs_is_directory(vfsobj)){
					VELOCITY_MSG("make get_size work for directories");
				}else{
					char*szstr=gnome_vfs_format_file_size_for_display(vfsobj->info->size);
					char*strbuf=g_strdup_printf(_("Size:  %s    MIME Type:  %s"),szstr,vfsobj->info->mime_type);
					gnome_appbar_set_status(GNOME_APPBAR(GNOME_APP(vwin->widget)->statusbar),strbuf);
					g_free(szstr);
					g_free(strbuf);
				}
			}
			break;
		case VELOCITY_VIEW_BEHAVIOR_EVENT_ACTIVATE:
			//TODO:  This is kind of a hack.  Ideally we would only show the selection when the window/view has focus.
			//	the issue with this is how to go about watching the focus...
			if(velocity_vfs_is_desktop_dir_uri(vview->clocation)&&(velocity_vfs_is_symlink(vfsobj))){
				//TODO:  This is the hackish way of doing it, I should probably do this properly at some point in time...
				char*uri=vfsobj->prefix;
				char*name=vfsobj->info->name;
				vfsobj->prefix=velocity_vfs_get_uri_parent(vfsobj->info->symlink_name);
				vfsobj->info->name=velocity_vfs_uri_get_tail(vfsobj->info->symlink_name);
				velocity_view_open(vview,vfsobj);
				g_free(vfsobj->prefix);
				g_free(vfsobj->info->name);
				vfsobj->prefix=uri;
				vfsobj->info->name=name;
				velocity_view_clear_selection(velocity_windowman_desktop());
				return((gpointer)TRUE);
			}
			break;
		case VELOCITY_VIEW_BEHAVIOR_EVENT_CONTEXT:
			if(vview){
				//TODO:  This whole thing needs redone to work with a URI if possible (for special toolbar buttons and tree items)
				char*uri=NULL;	/*used for hack for symlinks on the desktop*/
				char*name=NULL;	/*used for hack for symlinks on the desktop*/
				GtkWidget*menu=NULL;
				gboolean selection=velocity_clipboard_has_items(vview->selection);
				gboolean alldirs=FALSE;
				if(velocity_vfs_is_desktop_dir_uri(vview->clocation)&&(velocity_vfs_is_symlink(vfsobj))){
					uri=vfsobj->prefix;
					name=vfsobj->info->name;
					vfsobj->prefix=velocity_vfs_get_uri_parent(vfsobj->info->symlink_name);
					vfsobj->info->name=velocity_vfs_uri_get_tail(vfsobj->info->symlink_name);
				}
				{
					gboolean istrash=velocity_vfs_is_trash_dir(vfsobj);
					/*Possible menu factory streams*/
					void*mitrash[]={VELOCITY_MENU_FACTORY_PIXBUF,VELOCITY_TRASH_FULL,_("_Empty Trash"),VELOCITY_FACTORY_TRUE,velocity_menu_empty_trash_cb,vview,
						NULL};
					GList*n=velocity_module_tracker_get_list();
					guint mods=g_list_length(n);
					void**modmi=g_new0(void*,mods*2+1);	/*Module menu items*/
					guint modmii=0;	/*Module menu item iterator*/
					while(n){
						if(n->data){
							velocity_module*vmod=(velocity_module*)n->data;
							if((vmod->gui_ext)&&(vmod->gui_ext->context)){
								if(velocity_module_supports_mime_type(vmod,vfsobj->info->mime_type)){
									modmi[modmii++]=VELOCITY_MENU_FACTORY_EMBEDDED_FACTORY;
									modmi[modmii++]=vmod->gui_ext->context;
								}
							}
						}
						n=n->next;
					}
					{
						void*miopen[]={VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_OPEN,_("_Open"),VELOCITY_FACTORY_TRUE,velocity_menu_open_cb,vview,
							NULL};
						void*miaddons[]={VELOCITY_MENU_FACTORY_SUBMENU,velocity_menu_addons(vview),GTK_STOCK_EXECUTE,_("_Add-ons"),VELOCITY_FACTORY_TRUE,NULL,NULL,
							NULL};
						void*misendto[]={VELOCITY_MENU_FACTORY_SUBMENU,velocity_menu_sendto(vview),GTK_STOCK_JUMP_TO,_("Send _to"),VELOCITY_FACTORY_TRUE,NULL,NULL,
							NULL};
						void*menus[]={VELOCITY_MENU_FACTORY_EMBEDDED_FACTORY,miopen,
							VELOCITY_MENU_FACTORY_EMBEDDED_FACTORY,(istrash)?mitrash:NULL,
							VELOCITY_MENU_FACTORY_EMBEDDED_FACTORY,(istrash)?NULL:modmi,
							VELOCITY_MENU_FACTORY_SUBMENU,velocity_menu_openwith(vview),GTK_STOCK_OPEN,_("Open _With"),VELOCITY_FACTORY_TRUE,NULL,NULL,
							VELOCITY_MENU_FACTORY_SEPERATOR,
							VELOCITY_MENU_FACTORY_EMBEDDED_FACTORY,miaddons,
							VELOCITY_MENU_FACTORY_EMBEDDED_FACTORY,(istrash)?NULL:misendto,
							VELOCITY_MENU_FACTORY_SEPERATOR,
							NULL};
						void*menus_dir_newwin[]={VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_OPEN,_("Open in New _Window"),VELOCITY_FACTORY_TRUE,velocity_menu_open_in_window_cb,vview,
							NULL};
						void*menus_dir[]={VELOCITY_MENU_FACTORY_EMBEDDED_FACTORY,miopen,
							VELOCITY_MENU_FACTORY_EMBEDDED_FACTORY,(vview==velocity_windowman_desktop())?NULL:menus_dir_newwin,
							VELOCITY_MENU_FACTORY_EMBEDDED_FACTORY,(istrash)?mitrash:NULL,
							VELOCITY_MENU_FACTORY_EMBEDDED_FACTORY,(istrash)?NULL:modmi,
							VELOCITY_MENU_FACTORY_SEPERATOR,
							VELOCITY_MENU_FACTORY_EMBEDDED_FACTORY,miaddons,
							VELOCITY_MENU_FACTORY_EMBEDDED_FACTORY,(istrash)?NULL:misendto,
							VELOCITY_MENU_FACTORY_SEPERATOR,
							NULL};
						if(selection){
							GList*n=vview->selection->items;
							alldirs=TRUE;
							while(n){
								velocity_vfs_obj*vfsobj2=n->data;
								if(!velocity_vfs_is_directory(vfsobj2)){
									alldirs=FALSE;
									break;
								}
								n=n->next;
							}
						}
						if(alldirs){
							menu=velocity_menu_factory(vview,GTK_WIDGET(vview->parent),menus_dir);
						}else{
							menu=velocity_menu_factory(vview,GTK_WIDGET(vview->parent),menus);
						}
						if(!istrash)velocity_menu_clipboard(vview,menu);
						if(menu)gtk_menu_popup(GTK_MENU(menu),NULL,NULL,NULL,NULL,event->button.button,event->button.time);
					}
					if(modmi)g_free(modmi);
					//TODO:  Another idea may be to "Edit symlink" on symlinks and "Fix symlink" on broken symlinks
					/*				if(vfsobj->icon->emblem==GFR_ICON_DEC_SYMLINK){
										subitem=gtk_menu_item_new_with_label("FIXME: Follow Symlink");
										gtk_menu_shell_append(GTK_MENU_SHELL(menu),subitem);
										gtk_widget_show(subitem);
								}
					*/
				}
				if((uri)||(name)){
					g_free(vfsobj->prefix);
					g_free(vfsobj->info->name);
					vfsobj->prefix=uri;
					vfsobj->info->name=name;
				}
			}
			break;
		case VELOCITY_VIEW_BEHAVIOR_EVENT_BACKGROUND_CONTEXT:
			if(vview){
				//TODO:  This sorting struct is duplicated:  its also in velocity-window.c for the main menu...FIX IT...WE _HATE_ CODE BLOAT!!
				//		and when you figure out how to do that work on doing the same kind of thing for the clipboard...
				void*desktopadd[]={VELOCITY_MENU_FACTORY_SEPERATOR,
					VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_PROPERTIES,_("Preferences"),VELOCITY_FACTORY_TRUE,velocity_menu_preferences_cb,vview,
					NULL};
				void*viewopts[]={VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_REFRESH,_("Re_organize"),VELOCITY_FACTORY_TRUE,velocity_menu_sorting_reorganize_cb,vview,
					VELOCITY_MENU_FACTORY_SEPERATOR,
					VELOCITY_MENU_FACTORY_CHECKBOX,(gpointer)vview->prefs->dirs_first,_("_Directories first"),VELOCITY_FACTORY_TRUE,velocity_menu_sorting_dirs_first_cb,vview,
					VELOCITY_MENU_FACTORY_CHECKBOX,(gpointer)(!vview->prefs->filter&VELOCITY_VFS_FILTER_HIDDEN),_("Show _hidden"),VELOCITY_FACTORY_TRUE,velocity_menu_show_hidden_cb,vview,
					VELOCITY_MENU_FACTORY_SEPERATOR,
					VELOCITY_MENU_FACTORY_CHECKBOX,(gpointer)(vview->prefs->sorting==VELOCITY_VIEW_SORT_NAME),_("by _Name"),VELOCITY_FACTORY_TRUE,velocity_menu_sorting_cb,GINT_TO_POINTER(VELOCITY_VIEW_SORT_NAME),
					VELOCITY_MENU_FACTORY_CHECKBOX,(gpointer)(vview->prefs->sorting==VELOCITY_VIEW_SORT_TYPE),_("by _Type"),VELOCITY_FACTORY_TRUE,velocity_menu_sorting_cb,GINT_TO_POINTER(VELOCITY_VIEW_SORT_TYPE),
					VELOCITY_MENU_FACTORY_CHECKBOX,(gpointer)(vview->prefs->sorting==VELOCITY_VIEW_SORT_SIZE),_("by _Size"),VELOCITY_FACTORY_TRUE,velocity_menu_sorting_cb,GINT_TO_POINTER(VELOCITY_VIEW_SORT_SIZE),
					VELOCITY_MENU_FACTORY_CHECKBOX,(gpointer)(vview->prefs->sorting==VELOCITY_VIEW_SORT_MODIFIED),_("by Last _Modified"),VELOCITY_FACTORY_TRUE,velocity_menu_sorting_cb,GINT_TO_POINTER(VELOCITY_VIEW_SORT_MODIFIED),
					VELOCITY_MENU_FACTORY_CHECKBOX,(gpointer)(vview->prefs->sorting==VELOCITY_VIEW_SORT_ACCESSED),("by Last A_ccessed"),VELOCITY_FACTORY_TRUE,velocity_menu_sorting_cb,GINT_TO_POINTER(VELOCITY_VIEW_SORT_ACCESSED),
					VELOCITY_MENU_FACTORY_CHECKBOX,(gpointer)(vview->prefs->sorting==VELOCITY_VIEW_SORT_CREATED),_("by Date Created"),VELOCITY_FACTORY_TRUE,velocity_menu_sorting_cb,GINT_TO_POINTER(VELOCITY_VIEW_SORT_CREATED),
					NULL};		
				void*menus[]={VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_OPEN,_("Open New Window _Here"),VELOCITY_FACTORY_TRUE,velocity_menu_open_in_window_cb,vview,
					VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_OPEN,_("Open New _Window"),VELOCITY_FACTORY_TRUE,velocity_menu_open_in_window_cb,NULL,
					VELOCITY_MENU_FACTORY_SEPERATOR,
					VELOCITY_MENU_FACTORY_SUBMENU,velocity_menu_templates(vview),GTK_STOCK_NEW,_("_New"),VELOCITY_FACTORY_TRUE,NULL,NULL,
					VELOCITY_MENU_FACTORY_SEPERATOR,
					VELOCITY_MENU_FACTORY_SUBMENU,velocity_menu_addons(vview),GTK_STOCK_EXECUTE,_("_Add-ons"),VELOCITY_FACTORY_TRUE,NULL,NULL,
					VELOCITY_MENU_FACTORY_SEPERATOR,
					VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_REFRESH,_("_Refresh"),VELOCITY_FACTORY_TRUE,velocity_menu_refresh_cb,(gpointer)vview,
					VELOCITY_MENU_FACTORY_SUBFACTORY,viewopts,GTK_STOCK_PREFERENCES,_("_View Options"),VELOCITY_FACTORY_TRUE,NULL,NULL,
					VELOCITY_MENU_FACTORY_SEPERATOR,
					NULL};
				GtkWidget*menu=velocity_menu_factory(vview,GTK_WIDGET(vview->parent),menus);
				velocity_menu_clipboard(vview,menu);
				if(velocity_vfs_is_desktop_dir_uri(vview->clocation))velocity_menu_factory_append(vview,menu,desktopadd);
				gtk_menu_popup(GTK_MENU(menu),NULL,NULL,NULL,NULL,event->button.button,event->button.time);
			}
			break;
		case VELOCITY_VIEW_BEHAVIOR_EVENT_KEYBOARD:
			if(velocity_clipboard_has_items(vview->selection)){
				switch(event->key.keyval){
					case GDK_Return:
						velocity_view_activate(vview);
						break;
					case GDK_Delete:
						velocity_menu_trash_cb(NULL,(gpointer)vview);
						break;
					case GDK_Escape:
						if(vview->user_data){
							velocity_window*vwin=(velocity_window*)vview->user_data;
							velocity_window_clear_selection(vwin);
						}else{
							velocity_view_clear_selection(vview);
						}
						break;
					case GDK_x:
					case GDK_X:
						if(event->key.state&GDK_CONTROL_MASK)velocity_menu_cut_cb(NULL,(gpointer)vview);
						break;
					case GDK_c:
					case GDK_C:
						if(event->key.state&GDK_CONTROL_MASK)velocity_menu_copy_cb(NULL,(gpointer)vview);
						break;
					case GDK_v:
					case GDK_V:
						if(event->key.state&GDK_CONTROL_MASK)velocity_menu_paste_cb(NULL,(gpointer)vview);
						break;
					default:
						break;
				}
			}
			break;
		default:
			break;
	}
	return(NULL);	//TODO:  May want to use a true || false depending if the event was handled (if nothing is required to be returned by event)
}
