/*
 * Velocity - A GNOME File Manager
 *
 *
 * Copyright (C) (2002) (Kyle Davis) <kyled@blueriver.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <glib.h>
#include "velocity-clipboard.h"
#include "velocity-vfs-object.h"

velocity_clipboard*velocity_clipboard_new(){
	velocity_clipboard*vclip=g_new0(velocity_clipboard,1);
	g_assert(vclip!=NULL);
	vclip->mode=VELOCITY_CLIPBOARD_COPY;
	return(vclip);
}
void velocity_clipboard_set_details(velocity_clipboard*vclip,const char*uri_prefix,velocity_clipboard_mode mode){
	if(vclip)g_free(vclip->uri_prefix);
	vclip->uri_prefix=g_strdup(uri_prefix);
	vclip->mode=mode;
}
void velocity_clipboard_prepend_file(velocity_clipboard*vclip,velocity_vfs_obj*obj){
	if(vclip){
		vclip->items=g_list_prepend(vclip->items,(gpointer)velocity_vfs_obj_dup(obj));
	}
}
void velocity_clipboard_remove_file(velocity_clipboard*vclip,velocity_vfs_obj*obj){
	if(vclip){
		GList*n=vclip->items;
		GList*nl=NULL;
		while(n){
			velocity_vfs_obj*tobj=(velocity_vfs_obj*)n->data;
			if(tobj){
				if(tobj!=obj)nl=g_list_prepend(nl,n->data);
			}
			n=n->next;
		}
		if(vclip->items)g_list_free(vclip->items);
		velocity_vfs_obj_free(obj);
		vclip->items=nl;
	}
}
GList*velocity_clipboard_to_uri_list(velocity_clipboard*vclip){
	GList*r=NULL;
	GList*n=vclip->items;
	while(n){
		if(n->data){
			velocity_vfs_obj*vfsobj=(velocity_vfs_obj*)n->data;
			r=g_list_prepend(r,(gpointer)velocity_vfs_get_uri_unescaped(vfsobj));
		}
		n=n->next;
	}
	return(r);
}
GList*velocity_clipboard_get_unique_mime_types(velocity_clipboard*vclip){
	GList*r=NULL;
	GList*n=vclip->items;
	while(n){
		if(n->data){
			velocity_vfs_obj*vfsobj=(velocity_vfs_obj*)n->data;
			GList*rn=r;
			while(rn){
				if(strcasecmp((char*)rn->data,vfsobj->info->mime_type)==0)break;
				rn=rn->next;
			}
			if(!rn){
				r=g_list_prepend(r,g_strdup(vfsobj->info->mime_type));
			}
		}
		n=n->next;
	}
	return(r);
}
gboolean velocity_clipboard_has_items(velocity_clipboard*vclip){
	return((vclip)?(vclip->items!=NULL):FALSE);
}
gboolean velocity_clipboard_has_multiple(velocity_clipboard*vclip){
	return(((vclip)&&(vclip->items)&&(vclip->items->next)));
}
void velocity_clipboard_flush(velocity_clipboard*vclip){
	GList*n=NULL;
	if(vclip){
		if(vclip->uri_prefix){
			g_free(vclip->uri_prefix);
			vclip->uri_prefix=NULL;
		}
		if(vclip->items){
			n=vclip->items;
			while(n){
				velocity_vfs_obj*vfsobj=(velocity_vfs_obj*)n->data;
				if(vfsobj) velocity_vfs_obj_free(vfsobj);
				n=n->next;
			}
			vclip->items=NULL;
		}
	}
}
void velocity_clipboard_free(velocity_clipboard*vclip){
	if(vclip){
		velocity_clipboard_flush(vclip);
		g_free(vclip);
		vclip=NULL;
	}
}
