/*
 * Velocity - A GNOME File Manager
 *
 *
 * Copyright (C) (2002) (Kyle Davis) <kyled@blueriver.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <config.h>
#include <glib.h>
#include <libgnome/libgnome.h>
#include <libgnomeui/libgnomeui.h>

#include "velocity-ditem.h"
#include "velocity-module.h"
#include "velocity-factories.h"
#include "velocity-template.h"
#include "velocity-view.h"
#include "velocity-windowman.h"

static void velocity_ditem_response_cb(GtkDialog*dialog,gint response,gpointer user_data){
	velocity_ditem_edit*vdie=(velocity_ditem_edit*)user_data;
	switch(response){
		case GTK_RESPONSE_OK:{
			GnomeDesktopItem*di=gnome_ditem_edit_get_ditem(vdie->die);
			gnome_desktop_item_save(di,vdie->location,TRUE,NULL);
			VELOCITY_MSG("TODO:  This should update the icon & caption");
			}break;
		case GTK_RESPONSE_CANCEL:
		default:
			break;
	}
	gtk_widget_destroy(vdie->window);
	velocity_ditem_edit_free(vdie);
}
static velocity_ditem_edit*velocity_ditem_edit_widget_new(void){
	velocity_ditem_edit*vdie;
	vdie=g_new0(velocity_ditem_edit,1);
	vdie->die=(GnomeDItemEdit*)gnome_ditem_edit_new();
	return(vdie);
}
static velocity_ditem_edit*velocity_ditem_edit_window_new(void){
	velocity_ditem_edit*vdie=velocity_ditem_edit_widget_new();
	vdie->window=gtk_dialog_new_with_buttons(_("Edit Launcher"),NULL,GTK_DIALOG_MODAL,
		GTK_STOCK_HELP,GTK_RESPONSE_HELP,GTK_STOCK_CANCEL,GTK_RESPONSE_CANCEL,GTK_STOCK_OK,GTK_RESPONSE_OK,NULL);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(vdie->window)->vbox),GTK_WIDGET(vdie->die));
	g_signal_connect(vdie->window,"response",G_CALLBACK(velocity_ditem_response_cb),vdie);
	gtk_widget_show_all(vdie->window);
	return(vdie);
}
void velocity_ditem_edit_new(const char*location){
	velocity_ditem_edit*vdie=velocity_ditem_edit_window_new();
	VELOCITY_MSG("loading %s",location);
	vdie->location=g_strdup(location);
	gnome_ditem_edit_load_uri(GNOME_DITEM_EDIT(vdie->die),location,NULL);
}
void velocity_ditem_edit_free(velocity_ditem_edit*vdie){
	//if(vdie->die)gtk_widget_destroy(GTK_WIDGET(vdie->die));
	if(vdie->location)g_free(vdie->location);
	g_free(vdie);
}

/*Static module code starts here*/
VELOCITY_STATIC_MODULE_INFO_DEFINE(ditem);
VELOCITY_STATIC_MODULE_GUI_DEFINE(ditem);

static void velocity_ditem_edit_cb(GtkMenuItem*menuitem,gpointer user_data){
	velocity_view*vview=(velocity_view*)g_object_get_data(G_OBJECT(menuitem),"velocity_view");	/*Always set by the factories for convenience*/
	if(vview){
		GList*n=vview->selection->items;
		while(n){
			if(n->data){
				velocity_vfs_obj*vfsobj=(velocity_vfs_obj*)n->data;
				char*uri=velocity_vfs_get_uri(vfsobj);
				velocity_ditem_edit_new(uri);
				g_free(uri);
			}
			n=n->next;
		}
	}
}
static GtkWidget*velocity_ditem_module_page1_new(velocity_vfs_obj*vfsobj){
	velocity_ditem_edit*vdie=velocity_ditem_edit_widget_new();
	GnomeDesktopItem*ditem=(GnomeDesktopItem*)velocity_vfs_obj_ext_get_data(vfsobj,VELOCITY_STATIC_MODULE_INFO_VAR(ditem).id);
	vdie->location=velocity_vfs_get_uri(vfsobj);
	if(ditem){	/*attempt to load from cache first*/
		gnome_ditem_edit_set_ditem(GNOME_DITEM_EDIT(vdie->die),gnome_desktop_item_copy(ditem));
	}else{	/*resort to loading from file if needed*/
		gnome_ditem_edit_load_uri(GNOME_DITEM_EDIT(vdie->die),vdie->location,NULL);
	}
	return(GTK_WIDGET(vdie->die));
}
static void velocity_ditem_module_page1_ok(GtkWidget*page1){
	VELOCITY_MSG("OK not done");
}
static void velocity_ditem_module_page1_cancel(GtkWidget*page1){
	VELOCITY_MSG("Cancel not done");
}
//TODO: Localization needed...somehow...
const gpointer velocity_ditem_module_gui_ext_context[]={N_("Edit"),VELOCITY_FACTORY_TRUE,velocity_ditem_edit_cb,NULL,
	NULL};
velocity_module_gui_ext_page ditem_module_page1;

static gboolean velocity_ditem_template_cb(const gchar*uri,const gchar*name){
	//TODO: Check for valid name?
	if((uri)&&(name)){
		GnomeVFSHandle*h;
		//TODO:  This should use the async functions before we allow it on a networked FS
		if(gnome_vfs_create(&h,uri,GNOME_VFS_OPEN_WRITE,FALSE,GNOME_VFS_PERM_USER_ALL)==GNOME_VFS_OK){
			GnomeVFSFileSize w;
			if(gnome_vfs_write(h,"[Desktop Entry]\n",16,&w)==GNOME_VFS_OK){
				gnome_vfs_close(h);
				velocity_ditem_edit_new(uri);
				return(TRUE);
			}
			gnome_vfs_close(h);
		}
	}
	return(FALSE);
}
velocity_module_return velocity_ditem_init(void){
	VELOCITY_STATIC_MODULE_INFO(ditem,_("Desktop Entry"),
		_("Handles executing creating, and editing of desktop entry files.  (Both GNOME and KDE)"),
		"http://velocity.sourceforge.net/");
	VELOCITY_STATIC_MODULE_VERSION(ditem,1,0,0);
	VELOCITY_STATIC_MODULE_AUTHORS(ditem,"Kyle Davis");
	VELOCITY_STATIC_MODULE_MIME_TYPES(ditem,GNOME_APP_INFO,KDE_APP_INFO,VELOCITY_VFS_MIME_TYPE_VFOLDER);//,VELOCITY_VFS_MIME_TYPE_VFOLDER);
	VELOCITY_STATIC_MODULE_GUI_VAR(ditem).context=(gpointer*)velocity_ditem_module_gui_ext_context;
	ditem_module_page1.page_title=_("Edit");
	ditem_module_page1.page_new=(velocity_module_gui_ext_page_new)velocity_ditem_module_page1_new;
	ditem_module_page1.page_ok=(velocity_module_ext_callback)velocity_ditem_module_page1_ok;
	ditem_module_page1.page_cancel=(velocity_module_ext_callback)velocity_ditem_module_page1_cancel;
	VELOCITY_STATIC_MODULE_GUI_VAR(ditem).pages=g_list_prepend(NULL,&ditem_module_page1);
	VELOCITY_STATIC_MODULE_GUI_VAR(ditem).templates=g_list_prepend(VELOCITY_STATIC_MODULE_GUI_VAR(ditem).templates,
		(gpointer)velocity_template_new(_("GNOME Shortcut"),"1.0",GNOME_APP_INFO,_("New GNOME Shortcut.desktop"),
			_("Handles executing, editing, and creating GNOME Shortcut files."),TRUE,
			(velocity_template_activate_cb)velocity_ditem_template_cb));
	return(VELOCITY_MODULE_OK);
}
void velocity_ditem_about(void){
	VELOCITY_MSG("an about box goes here...");
}
void velocity_ditem_unload(void){
}
static void velocity_ditem_vfs_obj_ext_destroy(void*ext_data){
	GnomeDesktopItem*ditem=(GnomeDesktopItem*)ext_data;
	gnome_desktop_item_unref(ditem);
}
static char*velocity_ditem_comment_cb(velocity_vfs_obj*vfsobj){
	GnomeDesktopItem*di=(GnomeDesktopItem*)velocity_vfs_obj_ext_get_data(vfsobj,VELOCITY_STATIC_MODULE_INFO_VAR(ditem).id);
	if(di){
		if(gnome_desktop_item_attr_exists(di,GNOME_DESKTOP_ITEM_COMMENT)){
			return(g_strdup(gnome_desktop_item_get_localestring(di,GNOME_DESKTOP_ITEM_COMMENT)));
		}
	}
	return(NULL);
}
void*velocity_ditem_on_view_behavior_event(velocity_view*vview,velocity_view_behavior_event vevent,GdkEvent*event,velocity_vfs_obj*vfsobj,gpointer user_data){
	switch(vevent){
		case VELOCITY_VIEW_BEHAVIOR_EVENT_OBJECT_LOADED:
			if(vfsobj){
				char*uri=velocity_vfs_get_uri(vfsobj);
				if(uri){
					GnomeDesktopItem*ditem;
					char*furi;
					if(strcmp(vfsobj->info->mime_type,VELOCITY_VFS_MIME_TYPE_VFOLDER)==0){
						furi=g_strconcat(uri,".directory",NULL);
					}else{
						furi=g_strdup(uri);
					}
					ditem=gnome_desktop_item_new_from_uri(furi,GNOME_DESKTOP_ITEM_LOAD_ONLY_IF_EXISTS,NULL);
					if(ditem){
						velocity_vfs_obj_ext_add(vfsobj,VELOCITY_STATIC_MODULE_INFO_VAR(ditem).id,(gpointer)ditem,(velocity_vfs_obj_ext_destroy)velocity_ditem_vfs_obj_ext_destroy);
						velocity_tooltip_remove(vfsobj,"desktop_item_info");	/*Remove any old info*/
						velocity_tooltip_add(vfsobj,"desktop_item_info",_("Comment"),VELOCITY_TOOLTIP_ITEM_TYPE_TEXT,(velocity_tooltip_item_text_cb)velocity_ditem_comment_cb);
					}
					g_free(furi);
					g_free(uri);
				}
			}
			break;
		case VELOCITY_VIEW_BEHAVIOR_EVENT_OBJECT_GET_CAPTION:
			if(vfsobj){
				GnomeDesktopItem*di=(GnomeDesktopItem*)velocity_vfs_obj_ext_get_data(vfsobj,VELOCITY_STATIC_MODULE_INFO_VAR(ditem).id);
				if((di)&&(gnome_desktop_item_attr_exists(di,GNOME_DESKTOP_ITEM_NAME))){
					return(g_strdup(gnome_desktop_item_get_string(di,GNOME_DESKTOP_ITEM_NAME)));
				}
			}
			break;
		case VELOCITY_VIEW_BEHAVIOR_EVENT_OBJECT_GET_ICON:
			if(vfsobj){
				char*uri=velocity_vfs_get_uri(vfsobj);
				if(strcmp(vfsobj->info->mime_type,VELOCITY_VFS_MIME_TYPE_VFOLDER)==0)velocity_icon_emblem_add(vfsobj->icon,VELOCITY_FOLDER);
				if(uri){
					GnomeDesktopItem*di=(GnomeDesktopItem*)velocity_vfs_obj_ext_get_data(vfsobj,VELOCITY_STATIC_MODULE_INFO_VAR(ditem).id);
					if((di)&&(gnome_desktop_item_attr_exists(di,GNOME_DESKTOP_ITEM_ICON))){
						GdkPixbuf*p=velocity_image_cache_get(gnome_desktop_item_get_string(di,GNOME_DESKTOP_ITEM_ICON),vview->prefs->profile->icon_size.x,vview->prefs->profile->icon_size.y,NULL,VELOCITY_IMAGE_CACHE_PRIORITY_MEDIUM);
						if(!p){
							char*iuri=velocity_image_get_uri(gnome_desktop_item_get_string(di,GNOME_DESKTOP_ITEM_ICON),72);
							if(iuri){
								p=velocity_image_cache_get(iuri,vview->prefs->profile->icon_size.x,vview->prefs->profile->icon_size.y,NULL,VELOCITY_IMAGE_CACHE_PRIORITY_MEDIUM);
								g_free(iuri);
							}
						}
						if(p){
							g_free(uri);
							return(p);
						}else{
							VELOCITY_MSG("Unable to find(%s)",gnome_desktop_item_get_string(di,GNOME_DESKTOP_ITEM_ICON));
						}
					}
					g_free(uri);
				}
			}
			break;
		case VELOCITY_VIEW_BEHAVIOR_EVENT_ACTIVATE:
			if(vfsobj){
				GnomeDesktopItem*di=(GnomeDesktopItem*)velocity_vfs_obj_ext_get_data(vfsobj,VELOCITY_STATIC_MODULE_INFO_VAR(ditem).id);
				if(di){
					if((gnome_desktop_item_get_entry_type(di)==GNOME_DESKTOP_ITEM_TYPE_LINK)&&(gnome_desktop_item_attr_exists(di,GNOME_DESKTOP_ITEM_URL))){	/*force internal handling of URL's until I devise a better method*/
						const char*exec=gnome_desktop_item_get_string(di,GNOME_DESKTOP_ITEM_URL);
						if(!exec){
							VELOCITY_MSG("URL not found, trying to get EXEC.");
							gnome_desktop_item_get_string(di,GNOME_DESKTOP_ITEM_EXEC);
						}
						if(exec){
							GnomeVFSURI*guri=gnome_vfs_uri_new(exec);
							//TODO:  I don't like SMB and FTP being hard coded here....
							if(gnome_vfs_uri_is_local(guri)||(strcmp(guri->method_string,"smb")==0)||(strcmp(guri->method_string,"ftp")==0)){
								if(vview==velocity_windowman_desktop()){	/*Local links are handled internally*/
									if(vview->on_status_event){
										VELOCITY_MSG("opening %s",exec);
										vview->on_status_event(vview,VELOCITY_VIEW_STATUS_EVENT_NEW_WINDOW,exec,GNOME_VFS_OK,vview->user_data);
									}else{
										VELOCITY_ERR("URL's not handled because an on_new_window callback was not registered!");
									}
								}else{
									velocity_view_set_location(vview,exec);
								}
							}else{
								if(!gnome_url_show(exec,NULL)){
									VELOCITY_ERR("Unable to handle URL \"%s\"!",exec);
								}
							}
							gnome_vfs_uri_unref(guri);
						}else{
							VELOCITY_ERR("Unable to read GNOME desktop file \"%s\".  Corrupt or malformed file.",vfsobj->info->name);
						}
					}else{
						gnome_desktop_item_launch(di,NULL,GNOME_DESKTOP_ITEM_LAUNCH_APPEND_URIS,NULL);
					}
				}
			}
			break;
		default:
			break;
	}
	return(NULL);
}
