/*
 * Velocity - A GNOME File Manager
 *
 *
 * Copyright (C) (2002) (Kyle Davis) <kyled@blueriver.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <gtk/gtk.h>
#include "velocity-factories.h"

void velocity_menu_factory_directory_scan(GtkMenuItem*mi,const char*uri,velocity_vfs_filter_flags filter,
	guint depth,gboolean show_self,gboolean enabled,gpointer callback,gpointer callback_data){
	if(!g_object_get_data(G_OBJECT(mi),"velocity_menu_factory_dir_list")){
		velocity_view*vview=(velocity_view*)g_object_get_data(G_OBJECT(mi),"velocity_view");   /*Always set by the factories for convenience*/
		GtkWidget*sm=gtk_menu_item_get_submenu(mi);
		gboolean empty;
		if(!sm){
			sm=gtk_menu_new();
			gtk_menu_item_set_submenu(mi,sm);
		}
		void*fs[]={VELOCITY_MENU_FACTORY_DIRECTORY,(gpointer)uri,(gpointer)filter,
				(gpointer)show_self,(gpointer)depth,NULL,(gpointer)enabled,
				callback,callback_data,
			NULL};
		velocity_menu_factory_append(vview,sm,fs);
		empty=(gboolean)g_object_get_data(G_OBJECT(sm),"velocity_menu_factory_dir_empty");
		if(empty){
			gtk_menu_item_deselect(mi);
			gtk_menu_item_remove_submenu(mi);
			gtk_menu_item_select(mi);
			if(callback)g_signal_connect_after(mi,"activate",callback,callback_data);
			sm=(GtkWidget*)1;	/*To avoid reading this item again*/
		}
		g_object_set_data(G_OBJECT(mi),"velocity_menu_factory_dir_list",(gpointer)sm);
	}
	
}

static GtkWidget*velocity_menu_factory_directory_ent_new(const char*prefix,GnomeVFSFileInfo*info){
	char*tfuri=g_strconcat(prefix,info->name,NULL);
	char*furi=velocity_vfs_ensure_dir_uri(tfuri);
	char*iuri=velocity_vfs_get_mime_icon_uri(info->mime_type,furi,VELOCITY_MENU_FACTORY_PIXBUF_SIZE);
	GdkPixbuf*p=velocity_image_cache_get(iuri,VELOCITY_MENU_FACTORY_PIXBUF_SIZE,VELOCITY_MENU_FACTORY_PIXBUF_SIZE,NULL,VELOCITY_IMAGE_CACHE_PRIORITY_HIGH);
	GtkWidget*img=gtk_image_new_from_pixbuf(p);
	GtkWidget*subitem=gtk_image_menu_item_new_with_label(info->name);
	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(subitem),img);
	gtk_widget_set_name(subitem,furi);
	g_object_unref(G_OBJECT(p));
	g_free(iuri);
	g_free(furi);
	g_free(tfuri);
	return(subitem);
}
static void velocity_menu_factory_directory_submenu_cb(GtkMenuItem*mi){
	velocity_menu_factory_directory_scan(mi,gtk_widget_get_name(GTK_WIDGET(mi)),
		(velocity_vfs_filter_flags)g_object_get_data(G_OBJECT(mi),"velocity_menu_factory_dir_filter"),
		(guint)g_object_get_data(G_OBJECT(mi),"velocity_menu_factory_dir_depth"),
		(gboolean)g_object_get_data(G_OBJECT(mi),"velocity_menu_factory_dir_show_self"),
		(gboolean)g_object_get_data(G_OBJECT(mi),"velocity_menu_factory_dir_enabled"),
		g_object_get_data(G_OBJECT(mi),"velocity_menu_factory_dir_callback"),
		g_object_get_data(G_OBJECT(mi),"velocity_menu_factory_dir_callback_data"));
}

static GtkWidget*_velocity_menu_factory(velocity_view*vview,GtkWidget*parent,gboolean newmenu,void**factorystream){
	GtkWidget*menu;
	GtkWidget*mi;
	guint i=0;
	if(!factorystream)return(NULL);
	menu=(newmenu)?gtk_menu_new():parent;
	while(factorystream[i]){
		if((guint)factorystream[i]==(guint)VELOCITY_MENU_FACTORY_SEPERATOR){
			mi=gtk_menu_item_new();
			gtk_widget_set_sensitive(mi,FALSE);			
		}else if((guint)factorystream[i]==(guint)VELOCITY_MENU_FACTORY_EMBEDDED_FACTORY){
			_velocity_menu_factory(vview,menu,FALSE,factorystream[++i]);
			mi=NULL;	/*So we avoid trying to use it down below*/
		}else{
			guint type=(guint)factorystream[i];
			mi=NULL;
			if(type<=VELOCITY_MENU_FACTORY_NUM)i++;
			switch(type){
				case (guint)VELOCITY_MENU_FACTORY_CHECKBOX:
					{
						gboolean checked;
						checked=(gboolean)factorystream[i++];
						mi=gtk_check_menu_item_new();
						gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mi),checked);
					}
					break;
				case (guint)VELOCITY_MENU_FACTORY_SUBMENU:
				case (guint)VELOCITY_MENU_FACTORY_SUBFACTORY:
					{
						GtkWidget*submenu;
						if(type==(guint)VELOCITY_MENU_FACTORY_SUBMENU){
							submenu=(factorystream[i])?factorystream[i]:gtk_menu_new();
						}else{
							submenu=(factorystream[i])?_velocity_menu_factory(vview,mi,TRUE,factorystream[i]):NULL;
						}
						mi=gtk_image_menu_item_new();
						if(submenu)gtk_menu_item_set_submenu(GTK_MENU_ITEM(mi),submenu);
						i++;
					}
				case (guint)VELOCITY_MENU_FACTORY_PIXBUF:
				case (guint)VELOCITY_MENU_FACTORY_PIXBUF_LARGE:
					{
						GtkIconSize gsize=(type==(guint)VELOCITY_MENU_FACTORY_PIXBUF_LARGE)?GTK_ICON_SIZE_LARGE_TOOLBAR:GTK_ICON_SIZE_MENU;
						guint size=(type==(guint)VELOCITY_MENU_FACTORY_PIXBUF_LARGE)?VELOCITY_MENU_FACTORY_PIXBUF_LARGE_SIZE:VELOCITY_MENU_FACTORY_PIXBUF_SIZE;
						char*iuri=(char*)factorystream[i++];
						if(!mi)mi=gtk_image_menu_item_new();
						if(iuri){
							GtkWidget*img=NULL;
							if(iuri[0]==GNOME_VFS_URI_PATH_CHR){
								GdkPixbuf*p=velocity_image_cache_get(iuri,size,size,NULL,VELOCITY_IMAGE_CACHE_PRIORITY_HIGH);
								if(p){
									img=gtk_image_new_from_pixbuf(p);
									g_object_unref(G_OBJECT(p));
								}
							}else{
								GtkStockItem si;
								if(gtk_stock_lookup(iuri,&si)){
									img=gtk_image_new_from_stock(iuri,gsize);
								}else{
									char*fiuri=velocity_image_get_uri(iuri,size);
									GdkPixbuf*p=velocity_image_cache_get(fiuri,size,size,NULL,VELOCITY_IMAGE_CACHE_PRIORITY_HIGH);
									if(p){
										img=gtk_image_new_from_pixbuf(p);
										g_object_unref(G_OBJECT(p));
									}
									g_free(fiuri);
								}
							}
							if(img)gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mi),img);
						}
					}
					break;
				case (guint)VELOCITY_MENU_FACTORY_DIRECTORY:	//URI,filter,show_self,depth,text,enabled,cb,cb_data
					{
						char*uri=(char*)factorystream[i++];
						velocity_vfs_filter_flags filter=(velocity_vfs_filter_flags)factorystream[i++];
						guint depth=(guint)factorystream[i++];
						gboolean show_self=(gboolean)factorystream[i++];
						gboolean enabled=(gboolean)factorystream[i+1];
						GtkWidget*submenu;
						GList*files;
						if(show_self){
							GnomeVFSFileInfo*info=gnome_vfs_file_info_new();
							if(gnome_vfs_get_file_info(uri,info,GNOME_VFS_FILE_INFO_GET_MIME_TYPE|GNOME_VFS_FILE_INFO_FORCE_FAST_MIME_TYPE|GNOME_VFS_FILE_INFO_FOLLOW_LINKS)==GNOME_VFS_OK){
								char*iuri=velocity_vfs_get_mime_icon_uri(info->mime_type,uri,VELOCITY_MENU_FACTORY_PIXBUF_SIZE);
								GdkPixbuf*p=velocity_image_cache_get(iuri,VELOCITY_MENU_FACTORY_PIXBUF_SIZE,VELOCITY_MENU_FACTORY_PIXBUF_SIZE,NULL,VELOCITY_IMAGE_CACHE_PRIORITY_HIGH);
								GtkWidget*img=gtk_image_new_from_pixbuf(p);
								GtkWidget*t=gtk_label_new(_("<span weight=\"heavy\">[This Directory]</span>"));
								GtkWidget*subitem=gtk_image_menu_item_new();
								gtk_label_set_use_markup(GTK_LABEL(t),TRUE);
								gtk_container_add(GTK_CONTAINER(subitem),t);
								gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(subitem),img);
								gtk_widget_set_name(subitem,uri);
								g_object_unref(G_OBJECT(p));
								g_free(iuri);
								gtk_widget_set_sensitive(subitem,enabled);
								g_object_set_data(G_OBJECT(subitem),"velocity_view",(gpointer)vview);
								if(factorystream[i+2])g_signal_connect(subitem,"activate",factorystream[i+2],factorystream[i+3]);
								gtk_menu_shell_append(GTK_MENU_SHELL(menu),subitem);
								gtk_menu_shell_append(GTK_MENU_SHELL(menu),gtk_menu_item_new());
								gnome_vfs_file_info_unref(info);
							}
						}
						if(gnome_vfs_directory_list_load(&files,uri,GNOME_VFS_FILE_INFO_GET_MIME_TYPE|GNOME_VFS_FILE_INFO_FORCE_FAST_MIME_TYPE|GNOME_VFS_FILE_INFO_FOLLOW_LINKS)==GNOME_VFS_OK){
							GList*n=files;
							gboolean empty=TRUE;
							while(n){
								if(n->data){
									GnomeVFSFileInfo*info=(GnomeVFSFileInfo*)n->data;
									//TODO:  Finish GNOME_APP_INFO support!!!
									if((!velocity_vfs_filter(info,NULL,filter))||(strcmp(info->mime_type,GNOME_APP_INFO)==0)){
										GtkWidget*subitem=velocity_menu_factory_directory_ent_new(uri,info);
										if((((info->valid_fields&GNOME_VFS_FILE_INFO_FIELDS_TYPE)&&(info->type&GNOME_VFS_FILE_TYPE_DIRECTORY)))&&(depth>0)){
											GtkWidget*sm=gtk_menu_new();
											gtk_menu_item_set_submenu(GTK_MENU_ITEM(subitem),sm);
											g_object_set_data(G_OBJECT(sm),"velocity_menu_factory_dir_empty",(gpointer)TRUE);
											g_object_set_data(G_OBJECT(subitem),"velocity_menu_factory_dir_filter",(gpointer)filter);
											g_object_set_data(G_OBJECT(subitem),"velocity_menu_factory_dir_show_self",(gpointer)show_self);
											g_object_set_data(G_OBJECT(subitem),"velocity_menu_factory_dir_depth",(gpointer)(depth-1));
											g_object_set_data(G_OBJECT(subitem),"velocity_menu_factory_dir_enabled",(gpointer)enabled);
											g_object_set_data(G_OBJECT(subitem),"velocity_menu_factory_dir_callback",factorystream[i+2]);
											g_object_set_data(G_OBJECT(subitem),"velocity_menu_factory_dir_callback_data",factorystream[i+3]);
											g_signal_connect(subitem,"activate",G_CALLBACK(velocity_menu_factory_directory_submenu_cb),NULL);
										}else{
											if(factorystream[i+2])g_signal_connect(subitem,"activate",factorystream[i+2],factorystream[i+3]);
										}
										gtk_widget_set_sensitive(subitem,enabled);
										g_object_set_data(G_OBJECT(subitem),"velocity_view",(gpointer)vview);
										gtk_menu_shell_append(GTK_MENU_SHELL(menu),subitem);
										empty=FALSE;
									}
									gnome_vfs_file_info_unref(info);
								}
								n=n->next;
							}
							g_object_set_data(G_OBJECT(menu),"velocity_menu_factory_dir_empty",(gpointer)empty);
						}else{
							VELOCITY_ERR("An error occured trying to read %s",uri);
						}
					}
					break;
				default:
					mi=gtk_menu_item_new();
					break;
			}
			if(mi){
				if(factorystream[i]){
					GtkWidget*label=gtk_label_new_with_mnemonic(factorystream[i++]);
					gtk_misc_set_alignment(GTK_MISC(label),0.0,0.5);
					gtk_misc_set_padding(GTK_MISC(label),(type==(int)VELOCITY_MENU_FACTORY_PIXBUF_LARGE)?8:4,0);
					gtk_container_add(GTK_CONTAINER(mi),label);
				}else{
					i++;
				}
				gtk_widget_set_sensitive(mi,(gboolean)factorystream[i++]);
				if(factorystream[i])g_signal_connect(mi,"activate",G_CALLBACK(factorystream[i]),factorystream[i+1]);
			}else{
				i+=2;
			}
			i++;
		}
		if(mi){
			g_object_set_data(G_OBJECT(mi),"velocity_view",(gpointer)vview);	/*This is quite handy...*/
			gtk_menu_shell_append(GTK_MENU_SHELL(menu),mi);
		}
		i++;
	}
	if(menu)gtk_widget_show_all(menu);
	return(menu);
}
GtkWidget*velocity_menu_factory(velocity_view*vview,GtkWidget*parent,void**factorystream){
	return(_velocity_menu_factory(vview,parent,TRUE,factorystream));
}
void velocity_menu_factory_append(velocity_view*vview,GtkWidget*parent,void**factorystream){
	_velocity_menu_factory(vview,parent,FALSE,factorystream);
}
//TODO: may need to finish the toolbar factory some day...
GtkWidget*velocity_toolbar_factory(GtkWidget*parent,void**factorystream,gpointer user_data){
	GtkWidget*tb=gtk_toolbar_new();
	GtkWidget*tbi=NULL;
	int i=0;
	while(factorystream[i]){
		switch((int)factorystream[i]){
			case (int)VELOCITY_TOOLBAR_FACTORY_SEPERATOR:
				gtk_toolbar_append_space(GTK_TOOLBAR(tb));
				break;
			case (int)VELOCITY_TOOLBAR_FACTORY_PIXBUF:
				i++;
				if(factorystream[i]){
					GtkWidget*img=NULL;
					char*iuri=(char*)factorystream[i];
					if(iuri[0]==GNOME_VFS_URI_PATH_CHR){
						GdkPixbuf*p=velocity_image_cache_get(iuri,VELOCITY_TOOLBAR_FACTORY_SIZE,VELOCITY_TOOLBAR_FACTORY_SIZE,NULL,VELOCITY_IMAGE_CACHE_PRIORITY_HIGH);
						if(p){
							img=gtk_image_new_from_pixbuf(p);
							g_object_unref(G_OBJECT(p));
						}
					}else{
						GtkStockItem si;
						if(gtk_stock_lookup(iuri,&si)){
							img=gtk_image_new_from_stock(iuri,GTK_ICON_SIZE_LARGE_TOOLBAR);
						}else{
							char*fiuri=velocity_image_get_uri(iuri,VELOCITY_TOOLBAR_FACTORY_SIZE);
							GdkPixbuf*p=velocity_image_cache_get(fiuri,VELOCITY_TOOLBAR_FACTORY_SIZE,VELOCITY_TOOLBAR_FACTORY_SIZE,NULL,VELOCITY_IMAGE_CACHE_PRIORITY_HIGH);
							if(p){
								img=gtk_image_new_from_pixbuf(p);
								g_object_unref(G_OBJECT(p));
							}
							g_free(fiuri);
						}
					}
					i++;
					tbi=gtk_toolbar_append_item(GTK_TOOLBAR(tb),factorystream[i],factorystream[i+1],NULL,img,factorystream[i+2],(gpointer)user_data);
					i+=3;
					gtk_widget_set_name(tbi,factorystream[i]);
				}
				break;
			default:
				break;
		}
		i++;
	}
	return(tb);
}
GtkWidget*velocity_toolbar_factory_get_button(GtkWidget*toolbar,const char*buttonname){
	GList*n=gtk_container_get_children(GTK_CONTAINER(toolbar));
	while(n){
		GtkWidget*btn=GTK_WIDGET(n->data);
		if(btn){
			if(strcmp(gtk_widget_get_name(GTK_WIDGET(btn)),buttonname)==0){
				return(btn);
			}
		}
		n=n->next;
	}
	return(NULL);
}

GtkWidget*velocity_option_menu_factory(gchar**vmnemonic){
	GtkWidget*o=gtk_option_menu_new();
	GtkWidget*m=gtk_menu_new();
	guint i=0;
	gtk_option_menu_set_menu(GTK_OPTION_MENU(o),m);
	while(vmnemonic[i]){
		GtkWidget*mi=gtk_menu_item_new_with_mnemonic(vmnemonic[i]);
		gtk_menu_shell_append(GTK_MENU_SHELL(m),mi);
		i++;
	}
	gtk_option_menu_set_history(GTK_OPTION_MENU(o),0);
	gtk_widget_show_all(o);
	return(o);
}
