/*
 * Velocity - A GNOME File Manager
 *
 *
 * Copyright (C) (2002) (Kyle Davis) <kyled@blueriver.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <config.h>
#include <pwd.h>
#include <grp.h>

#include "callbacks.h"
#include "velocity-file-properties-window.h"
#include "velocity-vfs.h"
#include "velocity-view.h"
#include "velocity-icons.h"
#include "velocity-module-private.h"

permission_btns*new_permission_btns(void){
	 int struct_size=PERM_EXECUTE-PERM_READ+1;
	 permission_btns*permissions=g_new(permission_btns,1);
	 permissions->Nobody=g_new(GtkWidget*,struct_size);
	 permissions->Owner=g_new(GtkWidget*,struct_size);
	 permissions->Group=g_new(GtkWidget*,struct_size);
	 permissions->Other=g_new(GtkWidget*,struct_size);
	 permissions->Everyone=g_new(GtkWidget*,struct_size);
	 return(permissions);
}

void free_permission_btns(permission_btns*permissions){
	/*We don't free the GtkToggleButton's because GTK+ will (and wants to) do it for us!  : ) */
	g_free(permissions->Nobody);
	g_free(permissions->Owner);
	g_free(permissions->Group);
	g_free(permissions->Other);
	g_free(permissions->Everyone);
	g_free(permissions);
}

static void dlgFileProperties_response(GtkDialog*dlg,gint response,gpointer user_data){
	velocity_file_properties*vprops=(velocity_file_properties*)user_data;
	switch(response){
		case GTK_RESPONSE_OK:{
			char*origiconpath=gnome_icon_entry_get_filename(GNOME_ICON_ENTRY(vprops->iconentry));
			char*uri=velocity_vfs_get_uri(vprops->file_list);
			GnomeVFSFileInfo*info=gnome_vfs_file_info_new();
			if(origiconpath){
				GdkPixbuf*mimeicon=velocity_image_cache_get(origiconpath,72,72,NULL,VELOCITY_IMAGE_CACHE_PRIORITY_HIGH);
				if(mimeicon){
					char*iconpath=origiconpath;
					gnome_vfs_mime_set_icon(vprops->file_list->info->mime_type,iconpath);
					velocity_icon_set_icon(vprops->file_list->icon,mimeicon);
					g_object_unref(mimeicon);
					g_free(origiconpath);
				}
			}
			if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(vprops->permissions->Owner[PERM_READ])))info->permissions|=GNOME_VFS_PERM_USER_READ;
			if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(vprops->permissions->Owner[PERM_WRITE])))info->permissions|=GNOME_VFS_PERM_USER_WRITE;
			if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(vprops->permissions->Owner[PERM_EXECUTE])))info->permissions|=GNOME_VFS_PERM_USER_EXEC;
			if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(vprops->permissions->Group[PERM_READ])))info->permissions|=GNOME_VFS_PERM_GROUP_READ;
			if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(vprops->permissions->Group[PERM_WRITE])))info->permissions|=GNOME_VFS_PERM_GROUP_WRITE;
			if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(vprops->permissions->Group[PERM_EXECUTE])))info->permissions|=GNOME_VFS_PERM_GROUP_EXEC;
			if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(vprops->permissions->Other[PERM_READ])))info->permissions|=GNOME_VFS_PERM_OTHER_READ;
			if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(vprops->permissions->Other[PERM_WRITE])))info->permissions|=GNOME_VFS_PERM_OTHER_WRITE;
			if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(vprops->permissions->Other[PERM_EXECUTE])))info->permissions|=GNOME_VFS_PERM_OTHER_EXEC;
			if(gnome_vfs_set_file_info(uri,info,GNOME_VFS_SET_FILE_INFO_PERMISSIONS)!=GNOME_VFS_OK){
				//TODO:  Better error handling...
				VELOCITY_MSG("Unable to set permissions on file %s\n",uri);
			}
			vprops->file_list->info->permissions=info->permissions;
			g_free(uri);
		 	}break;
		case GTK_RESPONSE_CANCEL:
		default:
			break;
	}
	dlgFileProperties_free(vprops);
}
static void dlgFileProperties_on_btnModify_clicked(GtkButton*button,gpointer user_data){
	char*argstr=g_strdup_printf("%s \"%s\"",GNOME_FILE_TYPES_PROPERTIES,(char*)user_data);
	g_spawn_command_line_async(argstr,NULL);
	g_free(argstr);
}

//TODO:  Shoule take a GList* filled with GnomeVFSFileInfo*'s
//TODO:  Set User ID, Set Group ID, Sticky _ALL_ need to go into a struct!
//TODO:  Don't forget to only show groups that a user is allowed to change ownership to...(they must be a member of it or be root...)
GtkWidget*dlgFileProperties_new(velocity_vfs_obj*file_list){
	GtkWidget*notebookProperties;
	GtkWidget*vbox2;
	GtkWidget*frameObjectInfo;
	GtkWidget*tableObjectInformation;
	GtkWidget*lblTotalSize;
	GtkWidget*scrolledwin_txtFileName;
	GtkWidget*viewport1;
	GtkWidget*lstFileNames;
	GtkWidget*frameMIMEInfo;
	GtkWidget*tableMIMEInfo;
	GtkWidget*lblMIMEType;
	GtkWidget*lblDefaultApp;
	GtkWidget*txtMIMEType;
	GtkWidget*hbox1;
	GtkWidget*txtDefaultApp=NULL;
	GtkWidget*btnModify;
	GtkWidget*tablblGeneral;
	GtkWidget*tablblPermissions;
	GtkWidget*tablePermissions;
	GtkWidget*lblPermR;
	GtkWidget*lblPermW;
	GtkWidget*lblPermX;
	GtkWidget*lblNobody;
	GtkWidget*lblOwner;
	GtkWidget*lblGroup;
	GtkWidget*lblOther;
	GtkWidget*lblEveryone;
	GtkWidget*lblSpecial;
	GtkWidget*chkSUID;
	GtkWidget*chkSGID;
	GtkWidget*chkSticky;
	GtkWidget*lblFileOwner;
	GtkWidget*lblFileGroup;
	GtkWidget*dialog_action_area1;
	GtkTooltips*tooltips;
	GtkWidget*listitem;
	GtkWidget*w;	/*Generic widget holder used in many places*/
	GList*namelist=NULL;
	GnomeVFSMimeApplication*vfsmimeapp;
	velocity_file_properties*vFileProperties=g_new(velocity_file_properties,1);
	vFileProperties->permissions=new_permission_btns();
	/*Ensure access to the information*/
	vFileProperties->file_list=velocity_vfs_obj_dup(file_list);	//TODO:  make this a list....
	tooltips=gtk_tooltips_new();
   
	vFileProperties->window=gtk_dialog_new_with_buttons(_("Properties"),NULL,GTK_DIALOG_MODAL,
		GTK_STOCK_HELP,GTK_RESPONSE_HELP,GTK_STOCK_CANCEL,GTK_RESPONSE_CANCEL,GTK_STOCK_OK,GTK_RESPONSE_OK,NULL);
	gtk_dialog_set_default_response(GTK_DIALOG(vFileProperties->window),GTK_RESPONSE_CANCEL);
	g_signal_connect(vFileProperties->window,"response",G_CALLBACK(dlgFileProperties_response),vFileProperties);
	gtk_window_set_position(GTK_WINDOW(vFileProperties->window),GTK_WIN_POS_CENTER);
	gtk_window_set_default_size(GTK_WINDOW(vFileProperties->window),340,400);

	notebookProperties=gtk_notebook_new();
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(vFileProperties->window)->vbox),notebookProperties,TRUE,TRUE,0);
	gtk_container_set_border_width(GTK_CONTAINER(notebookProperties),2);

	vbox2=gtk_vbox_new(FALSE,12);
	gtk_container_add(GTK_CONTAINER(notebookProperties),vbox2);

	frameObjectInfo=gtk_frame_new(_("Object Information"));
	gtk_box_pack_start(GTK_BOX(vbox2),frameObjectInfo,TRUE,TRUE,0);
	gtk_container_set_border_width(GTK_CONTAINER(frameObjectInfo),4);

	tableObjectInformation=gtk_table_new(3,2,FALSE);
	gtk_container_add(GTK_CONTAINER(frameObjectInfo),tableObjectInformation);

	//TODO:  This should be "insensitive" (disabled) if there are more than one mime type in the list
	vFileProperties->iconentry=gnome_icon_entry_new(NULL,_("Choose a new icon for this file type"));
	//TODO:  memleak on next line...
	gnome_icon_entry_set_filename(GNOME_ICON_ENTRY(vFileProperties->iconentry),velocity_image_get_mime_type_uri(file_list->info->mime_type,72));
	gtk_table_attach(GTK_TABLE(tableObjectInformation),vFileProperties->iconentry,0,1,0,1,(GtkAttachOptions)(GTK_FILL),(GtkAttachOptions)(GTK_EXPAND|GTK_FILL),0,0);
	gtk_container_set_border_width(GTK_CONTAINER(vFileProperties->iconentry),1);
	
	w=gtk_button_new_with_label(_("Decorations..."));
	gtk_table_attach(GTK_TABLE(tableObjectInformation),w,0,1,1,2,(GtkAttachOptions)(GTK_FILL),(GtkAttachOptions)(GTK_FILL),10,2);
	gtk_tooltips_set_tip(tooltips,w,_("Modify modify the emblems attached to this object"),NULL);
	g_signal_connect(w,"clicked",G_CALLBACK(dlgFileProperties_emblems_cb),vFileProperties);

/*
	if(velocity_vfs_is_directory(file_list)&&velocity_vfs_is_symlink(file_list)){
		lblFileNames = gtk_label_new(_("Symbolic Directory Name(s):"));
	}else if(velocity_vfs_is_symlink(file_list)){
		lblFileNames = gtk_label_new(_("Symbolic File Name(s):"));
	}else if(velocity_vfs_is_directory(file_list)){
		lblFileNames = gtk_label_new(_("Directory Name(s):"));
	}else{
		lblFileNames = gtk_label_new(_("File Name(s):"));
	}
	gtk_table_attach(GTK_TABLE(tableObjectInformation),lblFileNames,0,1,1,2,(GtkAttachOptions)(GTK_FILL),(GtkAttachOptions)(0),0,0);
	gtk_misc_set_alignment(GTK_MISC(lblFileNames),0,0.5);
	gtk_misc_set_padding(GTK_MISC(lblFileNames),10,2);*/

{
	char*tmp=NULL;
	if(velocity_vfs_is_directory(file_list)){
		char*sz="--NOT IMPLEMENTED YET--";
		VELOCITY_MSG("get dir size not done yet");	//TODO: do this sometime...
		tmp=g_strdup_printf(_("Directory Size:  %s"),sz);
	}else{
		char*sz=gnome_vfs_format_file_size_for_display(file_list->info->size);
		tmp=g_strdup_printf(_("Total Size:  %s"),sz);
		g_free(sz);
	}
	lblTotalSize=gtk_label_new(tmp);
	gtk_table_attach(GTK_TABLE(tableObjectInformation),lblTotalSize,1,2,2,3,(GtkAttachOptions)(GTK_FILL),(GtkAttachOptions)(0),4,0);
	gtk_misc_set_alignment(GTK_MISC(lblTotalSize),0,0.5);
	g_free(tmp);
}

	//TODO: This needs to be a GtkListView
	//TODO: This needs to really be (capable of) a list of files...
	lstFileNames=gtk_label_new(file_list->info->name);
	gtk_table_attach(GTK_TABLE(tableObjectInformation),lstFileNames,1,2,0,2,(GtkAttachOptions)(GTK_FILL),(GtkAttachOptions)(GTK_FILL),4,0);
	
	frameMIMEInfo=gtk_frame_new(_("MIME Information"));
	gtk_box_pack_start(GTK_BOX(vbox2),frameMIMEInfo,TRUE,TRUE,0);
	gtk_container_set_border_width(GTK_CONTAINER(frameMIMEInfo),4);
	
	tableMIMEInfo=gtk_table_new(2,2,FALSE);
	gtk_container_add(GTK_CONTAINER(frameMIMEInfo),tableMIMEInfo);
	
	lblMIMEType=gtk_label_new(_("MIME Type:"));
	gtk_table_attach(GTK_TABLE(tableMIMEInfo),lblMIMEType,0,1,0,1,(GtkAttachOptions)(GTK_FILL),(GtkAttachOptions)(0),0,0);
	gtk_misc_set_alignment(GTK_MISC(lblMIMEType),0,0.5);
	gtk_misc_set_padding(GTK_MISC(lblMIMEType),10,2);
	
	lblDefaultApp=gtk_label_new(_("Default Application:"));
	gtk_table_attach(GTK_TABLE(tableMIMEInfo),lblDefaultApp,0,1,1,2,(GtkAttachOptions)(GTK_FILL),(GtkAttachOptions)(0),0,0);
	gtk_misc_set_alignment(GTK_MISC(lblDefaultApp),0,0.5);
	gtk_misc_set_padding(GTK_MISC(lblDefaultApp),10,2);
	
	txtMIMEType=gtk_label_new(_(file_list->info->mime_type));
	gtk_table_attach(GTK_TABLE(tableMIMEInfo),txtMIMEType,1,2,0,1,(GtkAttachOptions)(GTK_FILL),(GtkAttachOptions)(0),0,0);
	gtk_misc_set_alignment(GTK_MISC(txtMIMEType),0,0.5);
	gtk_misc_set_padding(GTK_MISC(txtMIMEType),4,0);
	
	hbox1=gtk_hbox_new(FALSE,12);
	gtk_table_attach(GTK_TABLE(tableMIMEInfo),hbox1,1,2,1,2,(GtkAttachOptions)(GTK_FILL),(GtkAttachOptions)(GTK_FILL),0,0);

	vfsmimeapp=gnome_vfs_mime_get_default_application(file_list->info->mime_type);
	if(vfsmimeapp){
		if(vfsmimeapp->name){
			txtDefaultApp=gtk_label_new(_(vfsmimeapp->name));
		}
	}
	if(!txtDefaultApp)txtDefaultApp=gtk_label_new(_("None"));
	gtk_box_pack_start(GTK_BOX(hbox1),txtDefaultApp,FALSE,FALSE,0);
	gtk_misc_set_alignment(GTK_MISC(txtDefaultApp),0,0.5);
	gtk_misc_set_padding(GTK_MISC(txtDefaultApp),4,0);
	
	btnModify=gtk_button_new_with_label(_("Modify"));
	gtk_box_pack_start(GTK_BOX(hbox1),btnModify,FALSE,FALSE,0);
	gtk_container_set_border_width(GTK_CONTAINER(btnModify),4);
	gtk_tooltips_set_tip(tooltips,btnModify,_("Modify GNOME's Default Application for this File Type"),NULL);
	
	tablblGeneral=gtk_label_new(_("General"));
	gtk_notebook_set_tab_label(GTK_NOTEBOOK(notebookProperties),gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebookProperties),0),tablblGeneral);
	
	tablePermissions=gtk_table_new(8,4,TRUE);
	tablblPermissions = gtk_label_new (_("Permissions"));
	gtk_container_set_border_width(GTK_CONTAINER(tablePermissions), 8);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebookProperties),tablePermissions,tablblPermissions);
	
	lblPermW=gtk_label_new(_("Write"));
	gtk_table_attach(GTK_TABLE(tablePermissions),lblPermW,2,3,0,1,(GtkAttachOptions)(GTK_FILL),(GtkAttachOptions)(0),0,0);
	gtk_misc_set_alignment(GTK_MISC(lblPermW),0,0.5);
	
	lblPermX=gtk_label_new(_("Executable"));
	gtk_table_attach(GTK_TABLE(tablePermissions),lblPermX,3,4,0,1,(GtkAttachOptions)(GTK_FILL),(GtkAttachOptions)(0),0,0);
	gtk_misc_set_alignment(GTK_MISC(lblPermX),0,0.5);
	
	lblNobody=gtk_label_new(_("Nobody"));
	gtk_table_attach(GTK_TABLE(tablePermissions),lblNobody,0,1,1,2,(GtkAttachOptions)(GTK_FILL),(GtkAttachOptions)(0),0,0);
	gtk_misc_set_alignment(GTK_MISC(lblNobody),0,0.5);
	gtk_misc_set_padding(GTK_MISC(lblNobody),0,8);
	
	lblOwner=gtk_label_new(_("Owner"));
	gtk_table_attach(GTK_TABLE(tablePermissions),lblOwner,0,1,2,3,(GtkAttachOptions)(GTK_FILL),(GtkAttachOptions)(0),0,0);
	gtk_misc_set_alignment(GTK_MISC(lblOwner),0,0.5);
	gtk_misc_set_padding(GTK_MISC(lblOwner),0,8);
	
	lblGroup=gtk_label_new(_("Group"));
	gtk_table_attach(GTK_TABLE(tablePermissions),lblGroup,0,1,3,4,(GtkAttachOptions)(GTK_FILL),(GtkAttachOptions)(0),0,0);
	gtk_misc_set_alignment(GTK_MISC(lblGroup),0,0.5);
	gtk_misc_set_padding(GTK_MISC(lblGroup),0,8);
	
	lblOther=gtk_label_new(_("Other"));
	gtk_table_attach(GTK_TABLE(tablePermissions),lblOther,0,1,4,5,(GtkAttachOptions)(GTK_FILL),(GtkAttachOptions)(0),0,0);
	gtk_misc_set_alignment(GTK_MISC(lblOther),0,0.5);
	gtk_misc_set_padding(GTK_MISC(lblOther),0,8);
	
	lblPermR = gtk_label_new (_("Read"));
	gtk_table_attach (GTK_TABLE (tablePermissions), lblPermR, 1, 2, 0, 1,(GtkAttachOptions) (GTK_FILL),(GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (lblPermR), 0, 0.5);
	gtk_misc_set_padding (GTK_MISC (lblPermR), 4, 2);
	
	lblEveryone = gtk_label_new (_("Everyone"));
	gtk_table_attach (GTK_TABLE (tablePermissions), lblEveryone, 0, 1, 5, 6,(GtkAttachOptions) (GTK_FILL),(GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (lblEveryone), 0, 0.5);
	gtk_misc_set_padding (GTK_MISC (lblEveryone), 0, 8);

/*This is a nice way to fill the permissions table*/
{
	int x=1;
	int y=1;
	int i=0; /*item iterator*/
	int s=0; /*sets iterator*/
	int p=PERM_READ;
	GtkWidget**sets[]={vFileProperties->permissions->Nobody,
		vFileProperties->permissions->Owner,
		vFileProperties->permissions->Group,
		vFileProperties->permissions->Other,
		vFileProperties->permissions->Everyone};
	char*tips[]={_("Deny everyone Read access"),_("Deny everyone Write access"),_("Deny everyone Execution access"),
		_("Allow owner Read access"),_("Allow owner Write access"),_("Allow owner Execution access"),
		_("Allow group members Read access"),_("Allow group members Write access"),_("Allow group members Execution access"),
		_("Allow others Read access"),_("Allow others Write access"),_("Allow others Execution access"),
		_("Allow everyone Read access"),_("Allow everyone Write access"),_("Allow everyone Execution access")};
	for(s=0;s<5;s++){
		GtkWidget**plist=sets[s];
		x=1;
		for(p=PERM_READ;p<PERM_NUM;p++){
			plist[p]=gtk_check_button_new();
			gtk_table_attach(GTK_TABLE(tablePermissions),plist[p],x,x+1,y,y+1,(GtkAttachOptions)(GTK_FILL),(GtkAttachOptions)(0),0,0);
			gtk_container_set_border_width(GTK_CONTAINER(plist[p]),2);
			gtk_tooltips_set_tip(tooltips,plist[p],tips[i],NULL);
			i++;
			x++;
		}
		y++;
	}
}
	
	lblSpecial = gtk_label_new(_("Special"));
	gtk_table_attach(GTK_TABLE(tablePermissions), lblSpecial, 0, 1, 6, 7,(GtkAttachOptions) (GTK_FILL),(GtkAttachOptions) (0), 0, 0);
	gtk_label_set_justify(GTK_LABEL(lblSpecial), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment(GTK_MISC(lblSpecial), 0, 0.5);
	gtk_misc_set_padding(GTK_MISC(lblSpecial), 0, 5);
	
	chkSUID = gtk_check_button_new_with_label (_("SUID"));
	gtk_table_attach (GTK_TABLE (tablePermissions), chkSUID, 1, 2, 6, 7,(GtkAttachOptions) (GTK_FILL),(GtkAttachOptions) (0), 0, 0);
	gtk_container_set_border_width (GTK_CONTAINER (chkSUID), 2);
	gtk_tooltips_set_tip (tooltips, chkSUID, _("Set SUID bit"), NULL);
	gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (chkSUID), FALSE);
	
	chkSGID = gtk_check_button_new_with_label (_("SGID"));
	gtk_table_attach (GTK_TABLE (tablePermissions), chkSGID, 2, 3, 6, 7,(GtkAttachOptions) (GTK_FILL),(GtkAttachOptions) (0), 0, 0);
	gtk_container_set_border_width (GTK_CONTAINER (chkSGID), 2);
	gtk_tooltips_set_tip (tooltips, chkSGID, _("Set SGID bit"), NULL);
	gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (chkSGID), FALSE);
	
	chkSticky = gtk_check_button_new_with_label (_("Sticky"));
	gtk_table_attach (GTK_TABLE (tablePermissions), chkSticky, 3, 4, 6, 7,(GtkAttachOptions) (GTK_FILL),(GtkAttachOptions) (0), 0, 0);
	gtk_container_set_border_width (GTK_CONTAINER (chkSticky), 2);
	gtk_tooltips_set_tip (tooltips, chkSticky, _("Set Sticky bit"), NULL);
	gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (chkSticky), FALSE);
	
	lblFileOwner = gtk_label_new (_("File Owner:"));
	gtk_table_attach (GTK_TABLE (tablePermissions), lblFileOwner, 0, 1, 7, 8,(GtkAttachOptions) (GTK_FILL),(GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_padding (GTK_MISC (lblFileOwner), 0, 8);
	
	lblFileGroup = gtk_label_new (_("File Group:"));
	gtk_table_attach (GTK_TABLE (tablePermissions), lblFileGroup, 2, 3, 7, 8,(GtkAttachOptions) (GTK_FILL),(GtkAttachOptions) (0), 0, 0);

{//TODO:  only show users that user is allowed to set permissions for (root: show all, users: show themselves)
	GtkWidget*om=gtk_option_menu_new();
	GtkWidget*m=gtk_menu_new();
	struct passwd*pwent=getpwent();
	struct group*grent=getgrent();
	guint i=0;
	gtk_option_menu_set_menu(GTK_OPTION_MENU(om),m);
	gtk_table_attach(GTK_TABLE(tablePermissions),om,1,2,7,8,(GtkAttachOptions)(GTK_EXPAND|GTK_FILL),(GtkAttachOptions)(0),0,0);
	while(pwent){
		GtkWidget*mi=gtk_menu_item_new_with_label(pwent->pw_name);
		gtk_menu_shell_append(GTK_MENU_SHELL(m),mi);
		if(pwent->pw_uid==file_list->info->uid)gtk_option_menu_set_history(GTK_OPTION_MENU(om),i);
		i++;
		pwent=getpwent();
	}
	endpwent();

//TODO:  only show groups user is in (or allowed to change to actually)
	om=gtk_option_menu_new();
	m=gtk_menu_new();
	i=0;
	gtk_option_menu_set_menu(GTK_OPTION_MENU(om),m);
	gtk_table_attach(GTK_TABLE(tablePermissions),om,3,4,7,8,(GtkAttachOptions)(GTK_EXPAND|GTK_FILL),(GtkAttachOptions)(0),0,0);
	while(grent){
		GtkWidget*mi=gtk_menu_item_new_with_label(grent->gr_name);
		gtk_menu_shell_append(GTK_MENU_SHELL(m),mi);
		if(grent->gr_gid==file_list->info->gid)gtk_option_menu_set_history(GTK_OPTION_MENU(om),i);
		i++;
		grent=getgrent();
	}
	endgrent();
}
{
	GList*l=velocity_module_tracker_get_list();
	while(l){
		velocity_module*vmod=(velocity_module*)l->data;
		if((velocity_module_supports_mime_type(vmod,file_list->info->mime_type))&&(vmod->gui_ext)){
			GList*n=vmod->gui_ext->pages;
			while(n){
				if(n->data){
					velocity_module_gui_ext_page*page=(velocity_module_gui_ext_page*)n->data;
					if(page->page_new){
						GtkWidget*c=page->page_new(file_list);
						if(c){
							GtkWidget*l=gtk_label_new(page->page_title);
							gtk_notebook_append_page(GTK_NOTEBOOK(notebookProperties),c,l);
						}
					}
				}
				n=n->next;
			}
		}
		l=l->next;
	}
}

	/*Set buttons to show permissions properly*/
	 if(file_list->info->permissions&GNOME_VFS_PERM_USER_READ)
		  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(vFileProperties->permissions->Owner[PERM_READ]),TRUE);
	 if(file_list->info->permissions&GNOME_VFS_PERM_GROUP_READ)
		  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(vFileProperties->permissions->Group[PERM_READ]),TRUE);
	 if(file_list->info->permissions&GNOME_VFS_PERM_OTHER_READ)
		  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(vFileProperties->permissions->Other[PERM_READ]),TRUE);

	 if(file_list->info->permissions&GNOME_VFS_PERM_USER_WRITE)
		  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(vFileProperties->permissions->Owner[PERM_WRITE]),TRUE);
	 if(file_list->info->permissions&GNOME_VFS_PERM_GROUP_WRITE)
		  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(vFileProperties->permissions->Group[PERM_WRITE]),TRUE);
	 if(file_list->info->permissions&GNOME_VFS_PERM_OTHER_WRITE)
		  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(vFileProperties->permissions->Other[PERM_WRITE]),TRUE);

	 if(file_list->info->permissions&GNOME_VFS_PERM_USER_EXEC)
		  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(vFileProperties->permissions->Owner[PERM_EXECUTE]),TRUE);
	 if(file_list->info->permissions&GNOME_VFS_PERM_GROUP_EXEC)
		  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(vFileProperties->permissions->Group[PERM_EXECUTE]),TRUE);
	 if(file_list->info->permissions&GNOME_VFS_PERM_OTHER_EXEC)
		  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(vFileProperties->permissions->Other[PERM_EXECUTE]),TRUE);

	 if(file_list->info->permissions&GNOME_VFS_PERM_SUID)
		 gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chkSUID),TRUE);
	 if(file_list->info->permissions&GNOME_VFS_PERM_SGID)
		 gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chkSGID),TRUE);
	 if(file_list->info->permissions&GNOME_VFS_PERM_STICKY)
		 gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chkSticky),TRUE);
	 
	g_signal_connect(vFileProperties->permissions->Nobody[PERM_READ], "toggled",G_CALLBACK(dlgFileProperties_on_chkNobodyR_cb),vFileProperties->permissions);
	g_signal_connect(vFileProperties->permissions->Nobody[PERM_WRITE], "toggled",G_CALLBACK(dlgFileProperties_on_chkNobodyW_cb),vFileProperties->permissions);
	g_signal_connect(vFileProperties->permissions->Nobody[PERM_EXECUTE], "toggled",G_CALLBACK(dlgFileProperties_on_chkNobodyX_cb),vFileProperties->permissions);
	g_signal_connect(vFileProperties->permissions->Owner[PERM_READ], "toggled",G_CALLBACK(dlgFileProperties_on_chkOwnerR_cb),vFileProperties->permissions);
	g_signal_connect(vFileProperties->permissions->Owner[PERM_WRITE], "toggled",G_CALLBACK(dlgFileProperties_on_chkOwnerW_cb),vFileProperties->permissions);
	g_signal_connect(vFileProperties->permissions->Owner[PERM_EXECUTE], "toggled",G_CALLBACK(dlgFileProperties_on_chkOwnerX_cb),vFileProperties->permissions);
	g_signal_connect(vFileProperties->permissions->Group[PERM_READ], "toggled",G_CALLBACK(dlgFileProperties_on_chkGroupR_cb),vFileProperties->permissions);
	g_signal_connect(vFileProperties->permissions->Group[PERM_WRITE], "toggled",G_CALLBACK(dlgFileProperties_on_chkGroupW_cb),vFileProperties->permissions);
	g_signal_connect(vFileProperties->permissions->Group[PERM_EXECUTE], "toggled",G_CALLBACK(dlgFileProperties_on_chkGroupX_cb),vFileProperties->permissions);
	g_signal_connect(vFileProperties->permissions->Other[PERM_READ], "toggled",G_CALLBACK(dlgFileProperties_on_chkOtherR_cb),vFileProperties->permissions);
	g_signal_connect(vFileProperties->permissions->Other[PERM_WRITE], "toggled",G_CALLBACK(dlgFileProperties_on_chkOtherW_cb),vFileProperties->permissions);
	g_signal_connect(vFileProperties->permissions->Other[PERM_EXECUTE], "toggled",G_CALLBACK(dlgFileProperties_on_chkOtherX_cb),vFileProperties->permissions);
	g_signal_connect(vFileProperties->permissions->Everyone[PERM_READ], "toggled",G_CALLBACK(dlgFileProperties_on_chkEveryoneR_cb),vFileProperties->permissions);
	g_signal_connect(vFileProperties->permissions->Everyone[PERM_WRITE], "toggled",G_CALLBACK(dlgFileProperties_on_chkEveryoneW_cb),vFileProperties->permissions);
	g_signal_connect(vFileProperties->permissions->Everyone[PERM_EXECUTE], "toggled",G_CALLBACK(dlgFileProperties_on_chkEveryoneX_cb),vFileProperties->permissions);
	g_signal_connect(btnModify,"clicked",G_CALLBACK(dlgFileProperties_on_btnModify_clicked),file_list->info->mime_type);
/*  TODO:  Activate all signals....
	g_signal_connect(notebookProperties, "switch_page",G_CALLBACK(dlgFileProperties_on_notebookProperties_switch_page),NULL);
	g_signal_connect(lstFileNames, "selection_changed",G_CALLBACK(dlgFileProperties_on_lstFileNames_selection_changed),NULL);
	g_signal_connect(btnHelp, "clicked",G_CALLBACK(dlgFileProperties_on_btnHelp_clicked),NULL);
*/
	
	gtk_widget_show_all(vFileProperties->window);
	return(vFileProperties->window);
}
void dlgFileProperties_free(velocity_file_properties*vprops){
	free_permission_btns(vprops->permissions);
	velocity_vfs_obj_free(vprops->file_list);
	gtk_widget_destroy(GTK_WIDGET(vprops->window));
	g_free(vprops);
}
static void*dlgFileProperties_emblems_on_behavior_event_cb(velocity_view*vview,velocity_view_behavior_event vevent,GdkEvent*event,velocity_vfs_obj*vfsobj,gpointer user_data){
	/*This is used as a filtering device*/
	//TODO:  objects should be able to either have a list of events to accept, or a list of events to deny....
	switch(vevent){
		case VELOCITY_VIEW_BEHAVIOR_EVENT_OBJECT_GET_ICON:
			velocity_window_on_behavior_event_cb(vview,vevent,event,vfsobj,user_data);
			break;
		default:
			break;
	}
	return(NULL);
}
void dlgFileProperties_emblems_cb(GtkWidget*widget,gpointer user_data){
	velocity_file_properties*vprops=(velocity_file_properties*)user_data;
	if(vprops){
		GtkWidget*w=gtk_window_new(GTK_WINDOW_TOPLEVEL);
		gtk_window_set_title(GTK_WINDOW(w),_("Set Decorations - UNFINISHED"));
		gtk_window_set_default_size(GTK_WINDOW(w),400,300);
		gtk_window_set_transient_for(GTK_WINDOW(w),GTK_WINDOW(vprops->window));
		gtk_window_set_modal(GTK_WINDOW(w),TRUE);
		vprops->view=velocity_view_new(GTK_WINDOW(w),TRUE);
		gtk_container_add(GTK_CONTAINER(w),vprops->view->widget);
		gtk_widget_show_all(w);
		g_free(vprops->view->prefs);
		vprops->view->prefs=g_memdup((gconstpointer)vprefs->view,sizeof(velocity_view_preferences));/*Copy prefs over from our main prefs handler*/
		vprops->view->prefs->can_open=FALSE;
		vprops->view->prefs->can_drag=FALSE;
		vprops->view->prefs->can_lasso=FALSE;
		vprops->view->prefs->can_cancel=FALSE;
		vprops->view->prefs->sorting=VELOCITY_VIEW_SORT_NAME;
		vprops->view->prefs->profile->draw_start=GTK_ANCHOR_NW;
		vprops->view->prefs->profile->draw_direction=GTK_ANCHOR_EAST;
		vprops->view->prefs->profile->caption_placement=GTK_ANCHOR_SOUTH;
		vprops->view->prefs->profile->icon_size.x=64;
		vprops->view->prefs->profile->icon_size.y=64;
		vprops->view->prefs->profile->icon_spacing.x=72;
		vprops->view->prefs->profile->icon_spacing.y=96;
		vprops->view->prefs->preview_images=TRUE;
		vprops->view->user_data=NULL;
		vprops->view->on_behavior_event=(velocity_view_behavior_event_ucb)dlgFileProperties_emblems_on_behavior_event_cb;
		//vprops->view->on_behavior_event=(velocity_view_behavior_event_ucb)velocity_window_on_behavior_event_cb;
		//velocity_view_set_location(vprops->view,PACKAGE_THEMES_DIR G_DIR_SEPARATOR_S "/emblem-*");
	}
}
/*Nobody Permissions*/
void dlgFileProperties_on_chkNobodyR_cb(GtkToggleButton *togglebutton,gpointer user_data){
	permission_btns*permissions=(permission_btns*)user_data;
	if(gtk_toggle_button_get_active(togglebutton)){
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(permissions->Owner[PERM_READ]),FALSE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(permissions->Group[PERM_READ]),FALSE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(permissions->Other[PERM_READ]),FALSE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(permissions->Everyone[PERM_READ]),FALSE);
	}
}
void dlgFileProperties_on_chkNobodyW_cb(GtkToggleButton *togglebutton,gpointer user_data){
	permission_btns*permissions=(permission_btns*)user_data;
	if(gtk_toggle_button_get_active(togglebutton)){
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(permissions->Owner[PERM_WRITE]),FALSE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(permissions->Group[PERM_WRITE]),FALSE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(permissions->Other[PERM_WRITE]),FALSE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(permissions->Everyone[PERM_WRITE]),FALSE);
	}
}
void dlgFileProperties_on_chkNobodyX_cb(GtkToggleButton *togglebutton,gpointer user_data){
	permission_btns*permissions=(permission_btns*)user_data;
	if(gtk_toggle_button_get_active(togglebutton)){
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(permissions->Owner[PERM_EXECUTE]),FALSE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(permissions->Group[PERM_EXECUTE]),FALSE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(permissions->Other[PERM_EXECUTE]),FALSE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(permissions->Everyone[PERM_EXECUTE]),FALSE);
	}
}

/*Owner Permissions*/
void dlgFileProperties_on_chkOwnerR_cb(GtkToggleButton *togglebutton,gpointer user_data){
	 permission_btns*permissions=(permission_btns*)user_data;
	 if(gtk_toggle_button_get_active(togglebutton)){
		  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(permissions->Nobody[PERM_READ]),FALSE);
	 }else{
		  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(permissions->Everyone[PERM_READ]),FALSE);
	 }
}
void dlgFileProperties_on_chkOwnerW_cb(GtkToggleButton *togglebutton,gpointer user_data){
	 permission_btns*permissions=(permission_btns*)user_data;
	 if(gtk_toggle_button_get_active(togglebutton)){
		  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(permissions->Nobody[PERM_WRITE]),FALSE);
	 }else{
		  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(permissions->Everyone[PERM_WRITE]),FALSE);
	 }
}
void dlgFileProperties_on_chkOwnerX_cb(GtkToggleButton *togglebutton,gpointer user_data){
	 permission_btns*permissions=(permission_btns*)user_data;
	 if(gtk_toggle_button_get_active(togglebutton)){
		  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(permissions->Nobody[PERM_EXECUTE]),FALSE);
	 }else{
		  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(permissions->Everyone[PERM_EXECUTE]),FALSE);
	 }
}

/*Group Permissions*/
void dlgFileProperties_on_chkGroupR_cb(GtkToggleButton *togglebutton,gpointer user_data){
	 permission_btns*permissions=(permission_btns*)user_data;
	 if(gtk_toggle_button_get_active(togglebutton)){
		  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(permissions->Nobody[PERM_READ]),FALSE);
	 }else{
		  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(permissions->Everyone[PERM_READ]),FALSE);
	 }
}
void dlgFileProperties_on_chkGroupW_cb(GtkToggleButton *togglebutton,gpointer user_data){
	 permission_btns*permissions=(permission_btns*)user_data;
	 if(gtk_toggle_button_get_active(togglebutton)){
		  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(permissions->Nobody[PERM_WRITE]),FALSE);
	 }else{
		  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(permissions->Everyone[PERM_WRITE]),FALSE);
	 }
}
void dlgFileProperties_on_chkGroupX_cb(GtkToggleButton *togglebutton,gpointer user_data){
	 permission_btns*permissions=(permission_btns*)user_data;
	 if(gtk_toggle_button_get_active(togglebutton)){
		  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(permissions->Nobody[PERM_EXECUTE]),FALSE);
	 }else{
		  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(permissions->Everyone[PERM_EXECUTE]),FALSE);
	 }
}

/*Other Permissions*/
void dlgFileProperties_on_chkOtherR_cb(GtkToggleButton *togglebutton,gpointer user_data){
	 permission_btns*permissions=(permission_btns*)user_data;
	 if(gtk_toggle_button_get_active(togglebutton)){
		  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(permissions->Nobody[PERM_READ]),FALSE);
	 }else{
		  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(permissions->Everyone[PERM_READ]),FALSE);
	 }
}
void dlgFileProperties_on_chkOtherW_cb(GtkToggleButton *togglebutton,gpointer user_data){
	 permission_btns*permissions=(permission_btns*)user_data;
	 if(gtk_toggle_button_get_active(togglebutton)){
		  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(permissions->Nobody[PERM_WRITE]),FALSE);
	 }else{
		  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(permissions->Everyone[PERM_WRITE]),FALSE);
	 }
}
void dlgFileProperties_on_chkOtherX_cb(GtkToggleButton *togglebutton,gpointer user_data){
	 permission_btns*permissions=(permission_btns*)user_data;
	 if(gtk_toggle_button_get_active(togglebutton)){
		  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(permissions->Nobody[PERM_EXECUTE]),FALSE);
	 }else{
		  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(permissions->Everyone[PERM_EXECUTE]),FALSE);
	 }
}

/*Everyone Permissions*/
void dlgFileProperties_on_chkEveryoneR_cb(GtkToggleButton *togglebutton,gpointer user_data){
	 permission_btns*permissions=(permission_btns*)user_data;
	 if(gtk_toggle_button_get_active(togglebutton)){
		  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(permissions->Nobody[PERM_READ]),FALSE);
		  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(permissions->Owner[PERM_READ]),TRUE);
		  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(permissions->Group[PERM_READ]),TRUE);
		  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(permissions->Other[PERM_READ]),TRUE);
	 }
}
void dlgFileProperties_on_chkEveryoneW_cb(GtkToggleButton *togglebutton,gpointer user_data){
	 permission_btns*permissions=(permission_btns*)user_data;
	 if(gtk_toggle_button_get_active(togglebutton)){
		  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(permissions->Nobody[PERM_WRITE]),FALSE);
		  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(permissions->Owner[PERM_WRITE]),TRUE);
		  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(permissions->Group[PERM_WRITE]),TRUE);
		  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(permissions->Other[PERM_WRITE]),TRUE);
	 }
}
void dlgFileProperties_on_chkEveryoneX_cb(GtkToggleButton *togglebutton,gpointer user_data){
	 permission_btns*permissions=(permission_btns*)user_data;
	 if(gtk_toggle_button_get_active(togglebutton)){
		  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(permissions->Nobody[PERM_EXECUTE]),FALSE);
		  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(permissions->Owner[PERM_EXECUTE]),TRUE);
		  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(permissions->Group[PERM_EXECUTE]),TRUE);
		  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(permissions->Other[PERM_EXECUTE]),TRUE);
	 }
}
