/*
 * Velocity - A GNOME File Manager
 *
 *
 * Copyright (C) (2002) (Kyle Davis) <kyled@blueriver.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <config.h>
#include <glib.h>
#include <libgnomecanvas/gnome-canvas.h>
#include <libgnomevfs/gnome-vfs-mime-handlers.h>
#include <libgnomevfs/gnome-vfs-utils.h>
#include <pango/pango.h>

#include "velocity-icon.h"
#include "velocity-sys.h"
#include "velocity-image.h"
#include "velocity-icons.h"

static void velocity_icon_ensure_selector(velocity_icon*icon){
	/*Stipple for every other of pixel....this way it looks semi-transparent in GDK mode*/
	/*0101 0101 = 170*/
	char mystipple[]={170,0};
	GdkBitmap*stipple=gdk_bitmap_create_from_data(NULL,mystipple,4,2);
	if(icon->selector)gtk_object_destroy(GTK_OBJECT(icon->selector));
	switch(icon->selector_type){
		case VELOCITY_ICON_SELECTION_IMAGE:
		case VELOCITY_ICON_SELECTION_PRELIGHT:
			if(icon->selector2)gtk_object_destroy(GTK_OBJECT(icon->selector2));
			icon->selector=gnome_canvas_item_new(GNOME_CANVAS_GROUP(icon->icon_group),gnome_canvas_pixbuf_get_type(),
				NULL);
			icon->selector2=gnome_canvas_item_new(GNOME_CANVAS_GROUP(icon->icon_group),gnome_canvas_pixbuf_get_type(),
				NULL);
			gnome_canvas_item_hide(GNOME_CANVAS_ITEM(icon->selector2));		
			break;
		case VELOCITY_ICON_SELECTION_CIRCLE:
		case VELOCITY_ICON_SELECTION_SQUARE:
		default:
			icon->selector=gnome_canvas_item_new(GNOME_CANVAS_GROUP(icon->icon_group),
				(icon->selector_type==VELOCITY_ICON_SELECTION_CIRCLE)?gnome_canvas_ellipse_get_type():gnome_canvas_rect_get_type(),
				"x1",(gfloat)(-icon->size.x>>1),"y1",(gfloat)(-icon->size.y>>1),
				"x2",(gfloat)(icon->size.x>>1),"y2",(gfloat)(icon->size.y>>1),
				"fill_color_rgba",0xAAAAFF22,
				"fill_stipple",stipple,
				"outline_color_rgba",0x00006622,
				"width_pixels",0,
				NULL);	
			break;
	}
	gnome_canvas_item_hide(GNOME_CANVAS_ITEM(icon->selector));
	gdk_drawable_unref(stipple);
}

velocity_icon*velocity_icon_new(const GnomeCanvasGroup*owner,gdouble x,gdouble y,GdkPoint size,GdkPoint spacing,GtkAnchorType caption_placement){
	velocity_icon*icon=(velocity_icon*)g_new0(velocity_icon,1);
	icon->parent=GNOME_CANVAS_GROUP(owner);
	icon->size.x=size.x;
	icon->size.y=size.y;
	icon->spacing.x=spacing.x;
	icon->spacing.y=spacing.y;
	icon->caption_placement=caption_placement;
	icon->selector_type=VELOCITY_ICON_SELECTION_PRELIGHT;//VELOCITY_ICON_SELECTION_SQUARE;
	if(!icon->parent)return(NULL);

/* this causes grouping of the following objects */
	icon->icon_group=(GnomeCanvasGroup*)gnome_canvas_item_new(GNOME_CANVAS_GROUP(icon->parent),gnome_canvas_group_get_type(),
		"x",(gdouble)(x+(icon->size.x>>1)),"y",(gdouble)(y+(icon->size.y>>1)),
		NULL);
	if(!icon->icon_group)return(NULL);
		
	velocity_icon_ensure_selector(icon);
		
	/*Hide our selector by default*/
	icon->x1=x;
	icon->y1=y;
	icon->x2=x+icon->size.x;
	icon->y2=y+icon->size.y;
	icon->effect_type=VELOCITY_ICON_ANIMATION_ROTATE; /*These must be different here so that the second is the real one that will get the initialized vars*/
	velocity_icon_set_animation_type(icon,VELOCITY_ICON_ANIMATION_BOUNCE);
	return(icon);
}
/*This will ensure that our icon objects are stacked properly*/
static void velocity_icon_restack(velocity_icon*icon){
	GList*n=icon->emblems;
	gboolean sot=((icon->selector_type!=VELOCITY_ICON_SELECTION_PRELIGHT)&&(icon->selector_type!=VELOCITY_ICON_SELECTION_IMAGE));	/*Selector on Top?*/
	if(icon->pixbuf)gnome_canvas_item_raise_to_top(GNOME_CANVAS_ITEM(icon->pixbuf));
	if(!sot){
		gnome_canvas_item_raise_to_top(GNOME_CANVAS_ITEM(icon->selector));
		gnome_canvas_item_raise_to_top(GNOME_CANVAS_ITEM(icon->selector2));
	}
	/*Decorations stacking*/
	while(n){
		velocity_icon_emblem*bdec=(velocity_icon_emblem*)n->data;
		if((bdec)&&(bdec->pixbuf))gnome_canvas_item_raise_to_top(GNOME_CANVAS_ITEM(bdec->pixbuf));
		n=n->next;
	}
	if(sot)gnome_canvas_item_raise_to_top(GNOME_CANVAS_ITEM(icon->selector));
}
void velocity_icon_emblem_add(velocity_icon*icon,const char*emblem){
	if(emblem){
		char*puri=(emblem[0]==GNOME_VFS_URI_PATH_CHR)?g_strdup(emblem):velocity_image_get_uri(emblem,icon->size.x);
		if(puri){
			/*I don't care what anybody says, an emblem should never take up more than 25% of an entire icon*/
			GdkPixbuf*p=velocity_image_cache_get(puri,icon->size.x>>1,icon->size.y>>1,NULL,VELOCITY_IMAGE_CACHE_PRIORITY_HIGH);
			if(p){
				/*All emblems are now to be 64x64 and mostly transparent*/
				velocity_icon_emblem*bdec=(velocity_icon_emblem*)g_new(velocity_icon_emblem,1);
				bdec->emblem=g_strdup(emblem);
				bdec->pixbuf=gnome_canvas_item_new(GNOME_CANVAS_GROUP(icon->icon_group),gnome_canvas_pixbuf_get_type(),
					"pixbuf",p,
					"x",(gdouble)((icon->size.x>>1)-(gdk_pixbuf_get_width(p)>>1)-8),"y",(gdouble)((icon->size.y>>1)-(gdk_pixbuf_get_height(p)>>1)-8),
					NULL);
				icon->emblems=g_list_prepend(icon->emblems,(gpointer)bdec);
				g_object_unref(p);
				velocity_icon_restack(icon);
			}
			g_free(puri);
		}
	}
}
void velocity_icon_emblem_remove(velocity_icon*icon,const char*emblem){
	if(emblem){
		GList*nl=NULL;	/*NOTE:  nl means new list*/
		GList*n=icon->emblems;
		if(icon->emblems){
			while(n){
				velocity_icon_emblem*bdec=(velocity_icon_emblem*)n->data;
				if(bdec){
					if(strcmp(bdec->emblem,emblem)==0){
						g_free(bdec->emblem);
						if(bdec->pixbuf)gtk_object_destroy(GTK_OBJECT(bdec->pixbuf));
						g_free(bdec);
					}else{
						nl=g_list_prepend(nl,(gpointer)bdec);
					}
				}
				n=n->next;
			}
			g_list_free(icon->emblems);
			icon->emblems=nl;
		}
	}
}

//TODO:  This needs to go into a velocity-image-effects.[ch] pair or something (someday)
/*This code borrowed from libeel and modified (and improved?) for Velocity*/
static guchar pixel_effect_brighten(guchar c){
	return((guchar)((c<240)?c+16:255));
}
GdkPixbuf*velocity_image_effect_brighten(GdkPixbuf*c){
	GdkPixbuf*n=NULL;	/*New pixbuf*/
	int x,y,w,h,crs,nrs;
	gboolean alpha=gdk_pixbuf_get_has_alpha(c);
	guchar*np,*cp;
	w=gdk_pixbuf_get_width(c);
	h=gdk_pixbuf_get_height(c);
	cp=gdk_pixbuf_get_pixels(c);
	crs=gdk_pixbuf_get_rowstride(c);
	n=gdk_pixbuf_new(gdk_pixbuf_get_colorspace(c),alpha,gdk_pixbuf_get_bits_per_sample(c),w,h);
	np=gdk_pixbuf_get_pixels(n);
	nrs=gdk_pixbuf_get_rowstride(n);
	for(y=0;y<h;y++){
		guchar*pn=np+y*nrs;
		guchar*pc=cp+y*crs;
		for(x=0;x<w;x++){
			*pn++=pixel_effect_brighten(*pc++);
			*pn++=pixel_effect_brighten(*pc++);
			*pn++=pixel_effect_brighten(*pc++);
			if(alpha)*pn++=*pc++;
		}
	}
	return(n);
}

void velocity_icon_set_icon(velocity_icon*icon,GdkPixbuf*pixbuf){
	if(pixbuf){
		GdkPoint*scaled_size=g_new0(GdkPoint,1);
		GdkPixbuf*scaled_pixbuf=NULL;
		guint w=gdk_pixbuf_get_width(pixbuf);
		guint h=gdk_pixbuf_get_height(pixbuf);
		if((w>icon->size.x)||(h>icon->size.y)){
			scaled_size=get_scaled_aspect(w,h,icon->size.x,icon->size.y);
			if(pixbuf)scaled_pixbuf=gdk_pixbuf_scale_simple(pixbuf,scaled_size->x,scaled_size->y,GDK_INTERP_NEAREST);
		}else{
			g_object_ref(G_OBJECT(pixbuf));
			scaled_pixbuf=pixbuf;
			scaled_size->x=w;
			scaled_size->y=h;
		}
		if(scaled_pixbuf){
			/*Center the image if it is smaller than (width X height)*/
			gdouble x=((scaled_size->x<icon->size.x)?(icon->size.x-scaled_size->x)>>1:0)-(icon->size.x>>1);
			gdouble y=((scaled_size->y<icon->size.y)?(icon->size.y-scaled_size->y)>>1:0)-(icon->size.y>>1);
			if(icon->pixbuf)gtk_object_destroy(GTK_OBJECT(icon->pixbuf));
			icon->pixbuf=gnome_canvas_item_new(GNOME_CANVAS_GROUP(icon->icon_group),gnome_canvas_pixbuf_get_type(),
				"pixbuf",scaled_pixbuf,
				"x",x,"y",y,
				"width",(gdouble)icon->size.x,"height",(gdouble)icon->size.y,
				NULL);
			if(icon->selector_type==VELOCITY_ICON_SELECTION_PRELIGHT){
				GdkPixbuf*p=velocity_image_effect_brighten(scaled_pixbuf);
				GdkPixbuf*h=velocity_image_effect_brighten(p);
				gnome_canvas_item_set(GNOME_CANVAS_ITEM(icon->selector),
					"pixbuf",p,
					"x",x,"y",y,
					"width",(gdouble)icon->size.x,"height",(gdouble)icon->size.y,
					NULL);
				gnome_canvas_item_set(GNOME_CANVAS_ITEM(icon->selector2),
					"pixbuf",h,
					"x",x,"y",y,
					"width",(gdouble)icon->size.x,"height",(gdouble)icon->size.y,
					NULL);
				g_object_unref(G_OBJECT(h));
				g_object_unref(G_OBJECT(p));
			}
			g_object_unref(scaled_pixbuf);
		}
		g_free(scaled_size);
		velocity_icon_restack(icon);
	}
}
void velocity_icon_set_font_colors(velocity_icon*icon,gulong fg,gulong bg,gulong pfg,gulong pbg,gulong hfg,gulong hbg){
	icon->fg=fg;
	icon->bg=bg;
	icon->pfg=pfg;
	icon->pbg=pbg;
	icon->hfg=hfg;
	icon->hbg=hbg;
}
void velocity_icon_set_caption(velocity_icon*icon,const char*caption,PangoFontDescription*font,gboolean confine){
	if(caption){
		char*utfcap=g_locale_to_utf8(caption,-1,NULL,NULL,NULL);
		char*tmp=caption;
		guint l=0;
		int t;
		int x;
		int y;
		while(tmp){	/*Count the number of lines in the caption*/
			char*tmp2=strchr(tmp,'\n');
			tmp=(tmp2!=tmp)?tmp2:NULL;
			l++;
		}
		if(icon->caption_fg){
			gnome_canvas_item_hide(GNOME_CANVAS_ITEM(icon->caption_fg));
			gtk_object_destroy(GTK_OBJECT(icon->caption_fg));
			icon->caption_fg=NULL;
		}
		if(icon->caption_bg){
			gnome_canvas_item_hide(GNOME_CANVAS_ITEM(icon->caption_bg));
			gtk_object_destroy(GTK_OBJECT(icon->caption_bg));
			icon->caption_bg=NULL;
		}
		/*Set up the caption for the icons if needed*/
		switch(icon->caption_placement){
			case GTK_ANCHOR_EAST:
				/*The 1.5 may just be my font...but it works perfectly for my font...I dunno...*/
				//TODO:  Handle caption_is_multilined!!!
				t=((PANGO_PIXELS(pango_font_description_get_size(font))*1.5)*l);
				icon->caption_bg=gnome_canvas_item_new(GNOME_CANVAS_GROUP(icon->icon_group),gnome_canvas_text_get_type(),
					"x",(gdouble)((icon->size.x>>1)+3),"y",(gdouble)-(t>>1),
					"fill_color_rgba",icon->bg,
					"anchor",GTK_ANCHOR_NW,
					"clip",confine,
					"clip_width",(gdouble)(icon->spacing.x-icon->size.x-4),
					"clip_height",(gdouble)t,
					NULL);
				icon->caption_fg=gnome_canvas_item_new(GNOME_CANVAS_GROUP(icon->icon_group),gnome_canvas_text_get_type(),
					"x",(gdouble)((icon->size.x>>1)+2),"y",(gdouble)-(t>>1),
					"fill_color_rgba",icon->fg,
					"anchor",GTK_ANCHOR_NW,
					"clip",confine,
					"clip_width",(gdouble)(icon->spacing.x-icon->size.x-4),
					"clip_height",(gdouble)t,
					NULL);
				icon->caption_bg->max_width=icon->caption_fg->max_width=-1;	/*Disable wrapping*/
				pango_layout_set_width(icon->caption_bg->layout,icon->caption_bg->max_width);
				pango_layout_set_width(icon->caption_fg->layout,icon->caption_bg->max_width);
				break;
			case GTK_ANCHOR_SOUTH:
			default:
				icon->caption_bg=gnome_canvas_item_new(GNOME_CANVAS_GROUP(icon->icon_group),gnome_canvas_text_get_type(),
					"x",(gdouble)+1,"y",(gdouble)(icon->size.y>>1)+1,
					"fill_color_rgba",icon->bg,
					"anchor",GTK_ANCHOR_N|GTK_ANCHOR_CENTER,
					"clip",confine,
					"clip_width",(gdouble)(icon->spacing.x),
					"clip_height",(gdouble)(icon->spacing.y-icon->size.y-4),
					NULL);
				icon->caption_fg=gnome_canvas_item_new(GNOME_CANVAS_GROUP(icon->icon_group),gnome_canvas_text_get_type(),
					"x",(gdouble)0,"y",(gdouble)(icon->size.y>>1),
					"fill_color_rgba",icon->fg,
					"anchor",GTK_ANCHOR_N|GTK_ANCHOR_CENTER,
					"clip",confine,
					"clip_width",(gdouble)(icon->spacing.x),
					"clip_height",(gdouble)(icon->spacing.y-icon->size.y-4),
					NULL);
				/*Somebody PLEASE document this!! (mainly the PANGO_SCALE part)*/
				icon->caption_bg->max_width=icon->caption_fg->max_width=(icon->size.x+((icon->spacing.x-icon->size.x)>>1))*PANGO_SCALE;
				pango_layout_set_width(icon->caption_bg->layout,icon->caption_bg->max_width);
				pango_layout_set_width(icon->caption_fg->layout,icon->caption_bg->max_width);
				break;
		}
		pango_layout_set_wrap(icon->caption_bg->layout,PANGO_WRAP_WORD_CHAR);
		pango_layout_set_wrap(icon->caption_fg->layout,PANGO_WRAP_WORD_CHAR);
		gnome_canvas_item_set(GNOME_CANVAS_ITEM(icon->caption_bg),
			"text",utfcap,
			"font_desc",font,
			NULL);
		gnome_canvas_item_set(GNOME_CANVAS_ITEM(icon->caption_fg),
			"text",utfcap,
			"font_desc",font,
			NULL);
		/*Basic tidiness around caption area*/
		pango_layout_get_pixel_size(icon->caption_bg->layout,&x,&y);
		switch(icon->caption_placement){
			case GTK_ANCHOR_EAST:
				if(x<(icon->spacing.x-icon->size.x)){
					gnome_canvas_item_set(GNOME_CANVAS_ITEM(icon->caption_bg),
						"clip_width",(gdouble)x,
						NULL);
					gnome_canvas_item_set(GNOME_CANVAS_ITEM(icon->caption_fg),
						"clip_width",(gdouble)x,
						NULL);
				}
				break;
			case GTK_ANCHOR_SOUTH:
			default:
				if(y<(icon->spacing.y-icon->size.y)){
					gnome_canvas_item_set(GNOME_CANVAS_ITEM(icon->caption_bg),
						"clip_height",(gdouble)y,
						NULL);
					gnome_canvas_item_set(GNOME_CANVAS_ITEM(icon->caption_fg),
						"clip_height",(gdouble)y,
						NULL);
				}
				break;
		}
		g_free(utfcap);
	}
}
void velocity_icon_move(velocity_icon*icon,gdouble xoff,gdouble yoff){
	icon->x1+=xoff;
	icon->y1+=yoff;
	icon->x2+=xoff;
	icon->y2+=yoff;
	gnome_canvas_item_move(GNOME_CANVAS_ITEM(icon->icon_group),xoff,yoff);
}
void velocity_icon_move_to(velocity_icon*icon,gdouble x,gdouble y){
	gdouble dx=x-icon->x1;
	gdouble dy=y-icon->y1;
	velocity_icon_move(icon,dx,dy);
}
void velocity_icon_hide(velocity_icon*icon){
	gnome_canvas_item_hide(GNOME_CANVAS_ITEM(icon->icon_group));
}
void velocity_icon_show(velocity_icon*icon){
	gnome_canvas_item_show(GNOME_CANVAS_ITEM(icon->icon_group));
}
void velocity_icon_prelight(velocity_icon*icon){
	if((!icon->prelighted)&&(!icon->highlighted)){
		gnome_canvas_item_show(GNOME_CANVAS_ITEM(icon->selector));
		if(icon->caption_fg)gnome_canvas_item_set(GNOME_CANVAS_ITEM(icon->caption_fg),"fill_color_rgba",icon->pfg,NULL);
		if(icon->caption_bg)gnome_canvas_item_set(GNOME_CANVAS_ITEM(icon->caption_bg),"fill_color_rgba",icon->pbg,NULL);
		icon->prelighted=TRUE;
	}
}
void velocity_icon_unprelight(velocity_icon*icon){
   if((!icon->highlighted)&&(icon->prelighted)){
		gnome_canvas_item_hide(GNOME_CANVAS_ITEM(icon->selector));
		if(icon->caption_fg)gnome_canvas_item_set(GNOME_CANVAS_ITEM(icon->caption_fg),"fill_color_rgba",icon->fg,NULL);
		if(icon->caption_bg)gnome_canvas_item_set(GNOME_CANVAS_ITEM(icon->caption_bg),"fill_color_rgba",icon->bg,NULL);
		icon->prelighted=FALSE;
   }
}

void velocity_icon_highlight(velocity_icon*icon){
	if(!icon->highlighted){
		if((icon->selector_type==VELOCITY_ICON_SELECTION_IMAGE)||(icon->selector_type==VELOCITY_ICON_SELECTION_PRELIGHT)){
			gnome_canvas_item_show(GNOME_CANVAS_ITEM(icon->selector2));
		}else{
			gnome_canvas_item_set(GNOME_CANVAS_ITEM(icon->selector),"fill_color_rgba",0x44BBFF22,NULL);
			gnome_canvas_item_show(GNOME_CANVAS_ITEM(icon->selector));
		}
		if(icon->caption_fg)gnome_canvas_item_set(GNOME_CANVAS_ITEM(icon->caption_fg),"fill_color_rgba",icon->hfg,NULL);
		if(icon->caption_bg)gnome_canvas_item_set(GNOME_CANVAS_ITEM(icon->caption_bg),"fill_color_rgba",icon->hbg,NULL);
		icon->highlighted=TRUE;
	}
}
void velocity_icon_unhighlight(velocity_icon*icon){
	if(icon->highlighted){
		if((icon->selector_type==VELOCITY_ICON_SELECTION_CIRCLE)||(icon->selector_type==VELOCITY_ICON_SELECTION_SQUARE)){
			gnome_canvas_item_set(GNOME_CANVAS_ITEM(icon->selector),"fill_color_rgba",0xAAAAFF22,NULL);
		}
		if(!icon->prelighted){
			if((icon->selector_type==VELOCITY_ICON_SELECTION_IMAGE)||(icon->selector_type==VELOCITY_ICON_SELECTION_PRELIGHT)){
				gnome_canvas_item_hide(GNOME_CANVAS_ITEM(icon->selector2));
			}else{
				gnome_canvas_item_hide(GNOME_CANVAS_ITEM(icon->selector));
			}
			if(icon->caption_fg)gnome_canvas_item_set(GNOME_CANVAS_ITEM(icon->caption_fg),"fill_color_rgba",icon->fg,NULL);
			if(icon->caption_bg)gnome_canvas_item_set(GNOME_CANVAS_ITEM(icon->caption_bg),"fill_color_rgba",icon->bg,NULL);
		}else{
			if(icon->caption_fg)gnome_canvas_item_set(GNOME_CANVAS_ITEM(icon->caption_fg),"fill_color_rgba",icon->pfg,NULL);
			if(icon->caption_bg)gnome_canvas_item_set(GNOME_CANVAS_ITEM(icon->caption_bg),"fill_color_rgba",icon->pbg,NULL);
		}
	}
	icon->highlighted=FALSE;
}
/*Animate a basic_icon*/
typedef void(*gnome_canvas_affine_func)(GnomeCanvasItem *item, const double affine[6]);
void velocity_icon_animate(velocity_icon*icon,double speed){
	GList*n=icon->emblems;
	gboolean absolute=TRUE;
	gdouble affine[6];
	gnome_canvas_affine_func affine_func;
	affine[0]=affine[1]=affine[2]=affine[3]=affine[4]=affine[5]=0.0; /*happily set all to zero*/
	switch(icon->effect_type){
		case VELOCITY_ICON_ANIMATION_BOUNCE:
			icon->effect_data[0]-=(speed*icon->effect_data[1]);
			if(icon->effect_data[0]>=0.0)icon->effect_data[1]=-icon->effect_data[1];
			if(icon->effect_data[0]<=(-icon->size.y>>1))icon->effect_data[1]=-icon->effect_data[1];
			art_affine_translate(affine,(double)0.0,(double)icon->effect_data[0]);
			break;
		case VELOCITY_ICON_ANIMATION_PULSATE:
			icon->effect_data[0]+=(speed*icon->effect_data[1]);
			if(icon->effect_data[0]>1.25)icon->effect_data[1]=-icon->effect_data[1];
			if(icon->effect_data[0]<0.75)icon->effect_data[1]=-icon->effect_data[1];
			art_affine_scale(affine,(gdouble)icon->effect_data[0],(gdouble)icon->effect_data[0]);
			affine[4]=(icon->size.x>>1)-((icon->size.x>>1)*icon->effect_data[0]);
			affine[5]=(icon->size.y>>1)-((icon->size.y>>1)*icon->effect_data[0]);
			break;
		case VELOCITY_ICON_ANIMATION_ROTATE:
		default:
			icon->effect_data[0]+=speed;
			if(icon->effect_data[0]>=360.0)icon->effect_data[0]-=360.0;
			art_affine_rotate(affine,(gdouble)speed);
			absolute=FALSE;
	}
	affine_func=(absolute)?gnome_canvas_item_affine_absolute:gnome_canvas_item_affine_relative;
	switch(icon->selector_type){
		case VELOCITY_ICON_SELECTION_IMAGE:
			affine_func(GNOME_CANVAS_ITEM(icon->pixbuf),affine);
		case VELOCITY_ICON_SELECTION_PRELIGHT:
			if(icon->highlighted){
				affine_func(GNOME_CANVAS_ITEM(icon->selector2),affine);
				gnome_canvas_item_hide(GNOME_CANVAS_ITEM(icon->pixbuf));	//TODO:  Hack, this shouldn't go here...but sleep was calling....
			}else if(icon->prelighted){
				affine_func(GNOME_CANVAS_ITEM(icon->selector),affine);
			}else{
				if(icon->selector_type==VELOCITY_ICON_SELECTION_PRELIGHT)affine_func(GNOME_CANVAS_ITEM(icon->pixbuf),affine);
			}
			break;
		case VELOCITY_ICON_SELECTION_SQUARE:
		case VELOCITY_ICON_SELECTION_CIRCLE:
		default:
			affine_func(GNOME_CANVAS_ITEM(icon->pixbuf),affine);
			break;
	}
	while(n){
		velocity_icon_emblem*dec=(velocity_icon_emblem*)n->data;
		if(dec)affine_func(GNOME_CANVAS_ITEM(dec->pixbuf),affine);
		n=n->next;
	}		
}
void velocity_icon_reset_animation(velocity_icon*icon){
	int tmp=icon->effect_type;
	icon->effect_type=-1;
	velocity_icon_set_animation_type(icon,tmp);	/*Hack to reset variables*/
	velocity_icon_animate(icon,0.0);
}
/*BREAKDOWN:  icon, desired animation mode     RETURNS:  whether effect mode was changed*/
gboolean velocity_icon_set_animation_type(velocity_icon*icon,velocity_icon_animation_mode animode){
	if(icon->effect_type!=animode){
		switch(animode){
			case VELOCITY_ICON_ANIMATION_BOUNCE:
				icon->effect_data[0]=0.0;  /*offset (negated for simpler math)*/
				icon->effect_data[1]=1.0;  /*delta*/
				break;
			case VELOCITY_ICON_ANIMATION_PULSATE:
				icon->effect_data[0]=1.0;  /*scale*/
				icon->effect_data[1]=1.0;  /*delta*/
				break;
			case VELOCITY_ICON_ANIMATION_ROTATE:
			default:
				icon->effect_data[0]=0.0;
				icon->effect_data[1]=0.0;
		}
		icon->effect_type=animode;
		return(TRUE);
	}
	return(FALSE);
}
void velocity_icon_set_selector_type(velocity_icon*icon,velocity_icon_selector_type selector_type){
	gboolean h=icon->highlighted;
	gboolean p=icon->prelighted;
	if(h)velocity_icon_unhighlight(icon);
	if(p)velocity_icon_unprelight(icon);
	icon->selector_type=selector_type;
	velocity_icon_ensure_selector(icon);
	if(p)velocity_icon_prelight(icon);
	if(h)velocity_icon_highlight(icon);
}
gboolean velocity_icon_is_child(velocity_icon*icon,GnomeCanvasItem*i){
	if(i){
		GList*n=icon->emblems;
		if(((icon->selector)&&(i==GNOME_CANVAS_ITEM(icon->selector)))||
			((icon->selector2)&&(i==GNOME_CANVAS_ITEM(icon->selector2)))||
			((icon->pixbuf)&&(i==GNOME_CANVAS_ITEM(icon->pixbuf)))||
			((icon->caption_fg)&&(i==GNOME_CANVAS_ITEM(icon->caption_fg)))||
			((icon->caption_bg)&&(i==GNOME_CANVAS_ITEM(icon->caption_bg))))return(TRUE);
		while(n){
			velocity_icon_emblem*bdec=(velocity_icon_emblem*)n->data;
			if((bdec->pixbuf)&&(i==GNOME_CANVAS_ITEM(bdec->pixbuf)))return(TRUE);
			n=n->next;
		}
	}
	return(FALSE);
}
/*Frees our objects and all related resources*/
void velocity_icon_free(velocity_icon*icon){
	if(icon){
		if(icon->pixbuf)gtk_object_destroy(GTK_OBJECT(icon->pixbuf));
		if(icon->emblems){
			GList*n=icon->emblems;
			while(n){
				velocity_icon_emblem*bdec=(velocity_icon_emblem*)n->data;
				g_free(bdec->emblem);
				if(bdec->pixbuf)gtk_object_destroy(GTK_OBJECT(bdec->pixbuf));
				g_free(bdec);
				n=n->next;
			}
			g_list_free(icon->emblems);
		}
		if(icon->selector)gtk_object_destroy(GTK_OBJECT(icon->selector));
		if(icon->selector2)gtk_object_destroy(GTK_OBJECT(icon->selector2));
		if(icon->caption_fg)gtk_object_destroy(GTK_OBJECT(icon->caption_fg));
		if(icon->caption_bg)gtk_object_destroy(GTK_OBJECT(icon->caption_bg));
		if(icon->icon_group)gtk_object_destroy(GTK_OBJECT(icon->icon_group));
		g_free(icon);
	}
}
