/*
 * Velocity - A GNOME File Manager
 *
 *
 * Copyright (C) (2002) (Kyle Davis) <kyled@blueriver.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef VELOCITY_ICON_H
#define VELOCITY_ICON_H

#include <glib.h>
#include <gnome.h>
#include <libgnomecanvas/gnome-canvas-pixbuf.h>
#include <libgnomecanvas/gnome-canvas.h>
#include <pango/pango.h>
#include "velocity-image.h"

/*TODO:  Add 3D spin, FadeIn&Out, etc. ??*/
typedef enum{VELOCITY_ICON_ANIMATION_INVALID=0, 
	VELOCITY_ICON_ANIMATION_BOUNCE,
	VELOCITY_ICON_ANIMATION_ROTATE,
	VELOCITY_ICON_ANIMATION_PULSATE,
	VELOCITY_ICON_ANIMATION_NUM}velocity_icon_animation_mode;

typedef enum{VELOCITY_ICON_SELECTION_INVALID=0,
	VELOCITY_ICON_SELECTION_CIRCLE,
	VELOCITY_ICON_SELECTION_SQUARE,
	VELOCITY_ICON_SELECTION_PRELIGHT,
	VELOCITY_ICON_SELECTION_IMAGE,
	VELOCITY_ICON_SELECTION_NUM}velocity_icon_selector_type;

typedef struct{
	char*emblem;
	GnomeCanvasPixbuf*pixbuf;
}velocity_icon_emblem;

typedef struct{
	GnomeCanvasGroup*parent;
	GnomeCanvasGroup*icon_group;
	GnomeCanvasPixbuf*pixbuf;
	GList*emblems;	/*List of emblems*/
	GnomeCanvasText*caption_fg;
	GnomeCanvasText*caption_bg;
	GnomeCanvasItem*selector;
	GnomeCanvasItem*selector2;	/*Needed for some selector types (PRELIGHT and IMAGE)*/
	gulong fg;	/*normal caption foreground*/
	gulong bg;	/*normal caption background*/
	gulong pfg;	/*prelit caption foreground*/
	gulong pbg;	/*prelit caption background*/
	gulong hfg;	/*highlighted caption foreground*/
	gulong hbg;	/*highlighted caption background*/
	gdouble x1;
	gdouble y1;
	gdouble x2;
	gdouble y2;
	GdkPoint size;	/*Dimensions for the icon*/
	GdkPoint spacing;	/*Space that the icon+caption can occupy*/
	GtkAnchorType caption_placement;	/*This cannot be a corner...Only N,E,S,W are allowed*/
	velocity_icon_animation_mode effect_type;
	gdouble effect_data[2];  /*Array for keeping special-effect specific data on each individual icon*/
	velocity_icon_selector_type selector_type;
	gboolean prelighted;
	gboolean highlighted;
}velocity_icon;

/*Basic stuff*/
velocity_icon*velocity_icon_new(const GnomeCanvasGroup*owner,gdouble x,gdouble y,GdkPoint size,GdkPoint spacing,GtkAnchorType caption_placement);
void velocity_icon_emblem_add(velocity_icon*icon,const char*emblem);
void velocity_icon_emblem_remove(velocity_icon*icon,const char*emblem);
void velocity_icon_set_icon(velocity_icon*icon,GdkPixbuf*pixbuf);
void velocity_icon_set_font_colors(velocity_icon*icon,gulong fg,gulong bg,gulong pfg,gulong pbg,gulong hfg,gulong hbg);
void velocity_icon_set_caption(velocity_icon*icon,const char*caption,PangoFontDescription*font,gboolean confine);
void velocity_icon_set_selector_type(velocity_icon*icon,velocity_icon_selector_type selector_type);
gboolean velocity_icon_is_child(velocity_icon*icon,GnomeCanvasItem*i);
void velocity_icon_free(velocity_icon*icon);

/*Simple visuals for basic_icons*/
void velocity_icon_move(velocity_icon*icon,gdouble xoff,gdouble yoff);
void velocity_icon_move_to(velocity_icon*icon,gdouble x,gdouble y);
void velocity_icon_hide(velocity_icon*icon);
void velocity_icon_show(velocity_icon*icon);
void velocity_icon_prelight(velocity_icon*icon);
void velocity_icon_unprelight(velocity_icon*icon);
void velocity_icon_highlight(velocity_icon*icon);
void velocity_icon_unhighlight(velocity_icon*icon);

/* Special effects for basic_icons */
void velocity_icon_animate(velocity_icon*icon,double);
void velocity_icon_reset_animation(velocity_icon*icon);
gboolean velocity_icon_set_animation_type(velocity_icon*icon,velocity_icon_animation_mode);

#endif
