/*
 * Velocity - A GNOME File Manager
 *
 *
 * Copyright (C) (2002) (Kyle Davis) <kyled@blueriver.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef VELOCITY_IMAGE_H
#define VELOCITY_IMAGE_H

#include <glib.h>
#include <gdk/gdk.h>
#include <gdk-pixbuf/gdk-pixbuf.h>

/*This controls how long an image stays in the cache*/
typedef enum{VELOCITY_IMAGE_CACHE_PRIORITY_INVALID=0,
	VELOCITY_IMAGE_CACHE_PRIORITY_NONE,	/*don't cache at all*/
	VELOCITY_IMAGE_CACHE_PRIORITY_LOW,	/*cache, but free soon*/
	VELOCITY_IMAGE_CACHE_PRIORITY_MEDIUM,	/*a nice medium*/
	VELOCITY_IMAGE_CACHE_PRIORITY_HIGH,	/*keep in cache as long as possible*/
	VELOCITY_IMAGE_CACHE_PRIORITY_NUM}velocity_image_cache_priority;

typedef struct{
	int n_channels;
	gboolean has_alpha;
	int bits_per_sample;
	int width;
	int height;
	int rowstride;
}velocity_image_info;

typedef struct{
	GdkPoint size;
	gpointer data;
	/*For freeing stale data*/
	velocity_image_cache_priority priority;
	guint staleness;
}velocity_image_cachelet;

typedef struct{
	/*GSList's of cachelets*/
	GSList*pixbufs;
	/*Information about original image*/
	velocity_image_info*info;
}velocity_image_cache_object;

gchar*velocity_image_get_uri(const gchar*image_name,guint base_size);
gchar*velocity_image_get_mime_type_uri(const gchar*mime_type,guint base_size);
velocity_image_info*velocity_image_info_new_from_pixbuf(GdkPixbuf*p);	/*Returned info should be g_free'd*/
GdkPoint*get_scaled_aspect(const guint w,const guint h,const guint scaledw,const guint scaledh);
GdkPixbuf*velocity_image_cache_get(const gchar*uri,const guint width,const guint height,velocity_image_info**info_ret,velocity_image_cache_priority priority);
void velocity_image_cache_shutdown(void);

#endif
