/*
 * Velocity - A GNOME File Manager
 *
 *
 * Copyright (C) (2002) (Kyle Davis) <kyled@blueriver.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <glib.h>
#include "velocity-preferences-util.h"
#include "velocity-preferences.h"
#include "velocity-factories.h"

static void velocity_view_profile_editor_size_cb(GtkSpinButton*s,GtkSpinButton*s2){
	if(gtk_spin_button_get_value_as_int(s)>gtk_spin_button_get_value_as_int(s2)){
		gtk_spin_button_set_value(s2,gtk_spin_button_get_value(s));
	}
}
static void velocity_view_profile_editor_spacing_cb(GtkSpinButton*s,GtkSpinButton*s2){
	if(gtk_spin_button_get_value_as_int(s)<gtk_spin_button_get_value_as_int(s2)){
		gtk_spin_button_set_value(s2,gtk_spin_button_get_value(s));
	}
}
/*This has two functions:
*  if edit is FALSE then profile is NULL or a real view profile to be used as a template in a new view profile
*  if edit is TRUE then profile is the view profile to be modified*/
static velocity_view_profile*velocity_view_profile_editor_new(GtkWindow*parent,velocity_view_profile*profile,gboolean edit){
	velocity_view_profile*vvp=(edit)?profile:g_new0(velocity_view_profile,1);
	/*Option menu arrays*/
	gchar*add[]={_("Top-left, draw right"),_("Top-left, draw down"),_("Top-right, draw left"),_("Top-right, draw down"),
		_("Bottom-left, draw right"),_("Bottom-left, draw up"),_("Bottom-right, draw left"),_("Bottom-right, draw up"),
		NULL};
	gchar*acp[]={_("Below"),_("Right"),NULL};
	gchar*act[]={_("Filename"),_("Filename, Size"),_("Filename, Type"),
		_("Filename, Type, Size"),_("Filename, Size, Type"),NULL};
	/*Translated-value arrays*/
	GtkAnchorType tdd_s[]={GTK_ANCHOR_NW,GTK_ANCHOR_NW,GTK_ANCHOR_NE,GTK_ANCHOR_NE,
		GTK_ANCHOR_SW,GTK_ANCHOR_SW,GTK_ANCHOR_SE,GTK_ANCHOR_SE};
	GtkAnchorType tdd_d[]={GTK_ANCHOR_EAST,GTK_ANCHOR_SOUTH,GTK_ANCHOR_WEST,GTK_ANCHOR_SOUTH,
		GTK_ANCHOR_EAST,GTK_ANCHOR_NORTH,GTK_ANCHOR_WEST,GTK_ANCHOR_NORTH};
	GtkAnchorType tcp[]={GTK_ANCHOR_SOUTH,GTK_ANCHOR_EAST};
	velocity_view_caption_type tct[]={VELOCITY_VIEW_CAPTION_NAME,VELOCITY_VIEW_CAPTION_NAME_SIZE,VELOCITY_VIEW_CAPTION_NAME_TYPE,
	  VELOCITY_VIEW_CAPTION_NAME_TYPE_SIZE,VELOCITY_VIEW_CAPTION_NAME_SIZE_TYPE};
	GtkWidget*w=gtk_dialog_new_with_buttons((edit)?_("Edit view profile"):_("New view profile"),parent,GTK_DIALOG_MODAL,
		GTK_STOCK_CANCEL,GTK_RESPONSE_CANCEL,GTK_STOCK_OK,GTK_RESPONSE_OK,NULL);
	GtkWidget*h=gtk_hbox_new(TRUE,12);
	GtkWidget*v=gtk_vbox_new(TRUE,6);
	GtkWidget*t=gtk_label_new(_("Name:"));
	GtkWidget*f=gtk_frame_new(_("Icon size"));
	GtkWidget*name=gtk_entry_new();
	GtkWidget*ct=velocity_option_menu_factory(act);
	GtkWidget*cp=velocity_option_menu_factory(acp);
	GtkWidget*cw=gtk_check_button_new();
	GtkWidget*dd=velocity_option_menu_factory(add);
	GtkWidget*sizex=gtk_spin_button_new(GTK_ADJUSTMENT(gtk_adjustment_new(64.0,16.0,128.0,1.0,8.0,0.0)),1.0,0);
	GtkWidget*sizey=gtk_spin_button_new(GTK_ADJUSTMENT(gtk_adjustment_new(64.0,16.0,128.0,1.0,8.0,0.0)),1.0,0);
	GtkWidget*spacingx=gtk_spin_button_new(GTK_ADJUSTMENT(gtk_adjustment_new(64.0,16.0,256.0,1.0,8.0,0.0)),1.0,0);
	GtkWidget*spacingy=gtk_spin_button_new(GTK_ADJUSTMENT(gtk_adjustment_new(64.0,16.0,256.0,1.0,8.0,0.0)),1.0,0);
	g_signal_connect(sizex,"changed",G_CALLBACK(velocity_view_profile_editor_size_cb),spacingx);
	g_signal_connect(sizey,"changed",G_CALLBACK(velocity_view_profile_editor_size_cb),spacingy);
	g_signal_connect(spacingx,"changed",G_CALLBACK(velocity_view_profile_editor_spacing_cb),sizex);
	g_signal_connect(spacingy,"changed",G_CALLBACK(velocity_view_profile_editor_spacing_cb),sizey);
	gtk_widget_set_size_request(GTK_WIDGET(w),320,480);
	gtk_box_pack_start(GTK_BOX(h),t,FALSE,FALSE,6);
	gtk_box_pack_start(GTK_BOX(h),name,FALSE,FALSE,6);
	gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(w)->vbox),6);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(w)->vbox),h);

	h=gtk_hbox_new(TRUE,12);
	t=gtk_label_new(_("Icon layout:"));
	gtk_box_pack_start(GTK_BOX(h),t,FALSE,FALSE,6);
	gtk_box_pack_start(GTK_BOX(h),dd,FALSE,FALSE,6);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(w)->vbox),h);

	h=gtk_hbox_new(TRUE,12);
	t=gtk_label_new(_("Caption placement:"));
	gtk_box_pack_start(GTK_BOX(h),t,FALSE,FALSE,6);
	gtk_box_pack_start(GTK_BOX(h),cp,FALSE,FALSE,6);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(w)->vbox),h);

	h=gtk_hbox_new(TRUE,12);
	t=gtk_label_new(_("Caption type:"));
	gtk_box_pack_start(GTK_BOX(h),t,FALSE,FALSE,6);
	gtk_box_pack_start(GTK_BOX(h),ct,FALSE,FALSE,6);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(w)->vbox),h);

	h=gtk_hbox_new(FALSE,6);
	t=gtk_image_new_from_stock(GTK_STOCK_SELECT_FONT,GTK_ICON_SIZE_BUTTON);
	gtk_container_add(GTK_CONTAINER(h),t);
	t=gtk_label_new(_("Caption can be wrapped"));
	gtk_container_add(GTK_CONTAINER(h),t);
	gtk_container_add(GTK_CONTAINER(cw),h);
	h=gtk_hbox_new(TRUE,12);
	gtk_box_pack_start(GTK_BOX(h),cw,FALSE,FALSE,6);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(w)->vbox),h);

	//TODO:  Need hooks so this can't be bigger than icon spacing
	h=gtk_hbox_new(TRUE,12);
	t=gtk_label_new(_("Width:"));
	gtk_box_pack_start(GTK_BOX(h),t,FALSE,FALSE,6);
	gtk_box_pack_start(GTK_BOX(h),sizex,FALSE,FALSE,6);
	gtk_box_pack_start(GTK_BOX(v),h,FALSE,FALSE,6);
	h=gtk_hbox_new(TRUE,12);
	t=gtk_label_new(_("Height:"));
	gtk_box_pack_start(GTK_BOX(h),t,FALSE,FALSE,6);
	gtk_box_pack_start(GTK_BOX(h),sizey,FALSE,FALSE,6);
	gtk_box_pack_start(GTK_BOX(v),h,FALSE,FALSE,6);
	gtk_container_add(GTK_CONTAINER(f),v);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(w)->vbox),f);

	//TODO:  This needs hooks so it can't be smaller the icon size (watch values and adjust accordingly
	f=gtk_frame_new(_("Icon spacing"));
	v=gtk_vbox_new(TRUE,6);
	h=gtk_hbox_new(TRUE,12);
	t=gtk_label_new(_("Width:"));
	gtk_box_pack_start(GTK_BOX(h),t,FALSE,FALSE,6);
	gtk_box_pack_start(GTK_BOX(h),spacingx,FALSE,FALSE,6);
	gtk_box_pack_start(GTK_BOX(v),h,FALSE,FALSE,6);
	h=gtk_hbox_new(TRUE,12);
	t=gtk_label_new(_("Height:"));
	gtk_box_pack_start(GTK_BOX(h),t,FALSE,FALSE,6);
	gtk_box_pack_start(GTK_BOX(h),spacingy,FALSE,FALSE,6);
	gtk_box_pack_start(GTK_BOX(v),h,FALSE,FALSE,6);
	gtk_container_add(GTK_CONTAINER(f),v);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(w)->vbox),f);

	f=gtk_frame_new(_("Preview"));
	h=gtk_scrolled_window_new(NULL,NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(h),GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
	t=gtk_label_new("TODO:  preview needed here");
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(h),t);
	gtk_container_add(GTK_CONTAINER(f),h);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(w)->vbox),f);

	if((profile)&&(profile!=vvp)){
		vvp->caption_type=profile->caption_type;
		vvp->caption_placement=profile->caption_placement;
		vvp->caption_wrap=profile->caption_wrap;
		vvp->draw_start=profile->draw_start;
		vvp->draw_direction=profile->draw_direction;
		vvp->icon_size.x=profile->icon_size.x;
		vvp->icon_size.y=profile->icon_size.y;
		vvp->icon_spacing.x=profile->icon_spacing.x;
		vvp->icon_spacing.y=profile->icon_spacing.y;
	}
	if((profile)||(edit)){
		guint i=0;
		if(vvp->name)gtk_entry_set_text(GTK_ENTRY(name),vvp->name);
		while(act[i]){
			if(tct[i]==vvp->caption_type){
				gtk_option_menu_set_history(GTK_OPTION_MENU(ct),i);
				break;
			}
			i++;
		}
		i=0;
		while(acp[i]){
			if(tcp[i]==vvp->caption_placement){
				gtk_option_menu_set_history(GTK_OPTION_MENU(cp),i);
				i++;
			}
			i++;
		}
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(cw),vvp->caption_wrap);
		i=0;
		while(add[i]){
			if((tdd_s[i]==vvp->draw_start)&&(tdd_d[i]==vvp->draw_direction)){
				gtk_option_menu_set_history(GTK_OPTION_MENU(dd),i);
				break;
			}
			i++;
		}
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(sizex),(gdouble)vvp->icon_size.x);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(sizey),(gdouble)vvp->icon_size.y);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(spacingx),(gdouble)vvp->icon_spacing.x);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(spacingy),(gdouble)vvp->icon_spacing.y);
	}

	gtk_widget_show_all(w);	
velocity_view_profile_editor_new_run:;
	switch(gtk_dialog_run(GTK_DIALOG(w))){
		case GTK_RESPONSE_OK:
			if(strlen(gtk_entry_get_text(GTK_ENTRY(name)))==0){
				GtkWidget*dlg=gtk_message_dialog_new(GTK_WINDOW(w),GTK_DIALOG_MODAL,GTK_MESSAGE_WARNING,GTK_BUTTONS_OK,_("View profile name is invalid."));
				gtk_dialog_run(GTK_DIALOG(dlg));
				gtk_widget_destroy(dlg);
				goto velocity_view_profile_editor_new_run;
			}else{
				if(edit){
					if(vvp->name)g_free(vvp->name);
				}
				vvp->name=g_strdup(gtk_entry_get_text(GTK_ENTRY(name)));
				vvp->caption_type=tct[gtk_option_menu_get_history(GTK_OPTION_MENU(ct))];
				vvp->caption_placement=tcp[gtk_option_menu_get_history(GTK_OPTION_MENU(cp))];
				vvp->caption_wrap=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cw));
				vvp->draw_start=tdd_s[gtk_option_menu_get_history(GTK_OPTION_MENU(dd))];
				vvp->draw_direction=tdd_d[gtk_option_menu_get_history(GTK_OPTION_MENU(dd))];
				vvp->icon_size.x=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(sizex));
				vvp->icon_size.y=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(sizey));
				vvp->icon_spacing.x=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spacingx));
				vvp->icon_spacing.y=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spacingy));
			}
			break;
		case GTK_RESPONSE_CANCEL:
		default:
			if(!edit)g_free(vvp);
			vvp=NULL;
			break;
	}
	gtk_widget_destroy(w);
	return(vvp);
}
static void velocity_view_profile_manager_new_cb(GtkWidget*widget,gpointer user_data){
	velocity_view_profile_manager_info*vvpmi=(velocity_view_profile_manager_info*)user_data;
	GtkTreeSelection*selection=gtk_tree_view_get_selection(vvpmi->tree);
	GtkTreeModel*model;
	GtkTreeIter iter;
	if((selection)&&(gtk_tree_selection_get_selected(selection,&model,&iter))){
		velocity_view_profile*vvp=NULL;
		GtkWidget*w;
		gint r;
		gtk_tree_model_get(model,&iter,1,&vvp,-1);
		w=gtk_message_dialog_new(NULL,GTK_DIALOG_MODAL,GTK_MESSAGE_QUESTION,GTK_BUTTONS_YES_NO,
			_("Would you like to use the selected view profile (%s) as a template for the new one?"),vvp->name);
		r=gtk_dialog_run(GTK_DIALOG(w));
		gtk_widget_destroy(w);
		switch(r){
			case GTK_RESPONSE_YES:
				vvp=velocity_view_profile_editor_new(NULL,vvp,FALSE);
				break;
			case GTK_RESPONSE_NO:
			default:
				vvp=velocity_view_profile_editor_new(NULL,NULL,FALSE);
				break;
		}
		if(vvp){
			GtkTreeIter iter;
			vvpmi->add=g_list_prepend(vvpmi->add,vvp);
			gtk_list_store_append(vvpmi->store,&iter);
			gtk_list_store_set(vvpmi->store,&iter,0,vvp->name,1,vvp,-1);
		}
	}
}
static void velocity_view_profile_manager_properties_cb(GtkWidget*widget,gpointer user_data){
	velocity_view_profile_manager_info*vvpmi=(velocity_view_profile_manager_info*)user_data;
	GtkTreeSelection*selection=gtk_tree_view_get_selection(vvpmi->tree);
	GtkTreeModel*model;
	GtkTreeIter iter;
	if((selection)&&(gtk_tree_selection_get_selected(selection,&model,&iter))){
		velocity_view_profile*vvp=NULL;
		gtk_tree_model_get(model,&iter,1,&vvp,-1);
		vvp=velocity_view_profile_editor_new(NULL,vvp,TRUE);
	}
}
static void velocity_view_profile_manager_delete_cb(GtkWidget*widget,gpointer user_data){
	velocity_view_profile_manager_info*vvpmi=(velocity_view_profile_manager_info*)user_data;
	GtkTreeSelection*selection=gtk_tree_view_get_selection(vvpmi->tree);
	GtkTreeModel*model;
	GtkTreeIter iter;
	if((selection)&&(gtk_tree_selection_get_selected(selection,&model,&iter))){
		velocity_view_profile*vvp=NULL;
		gtk_tree_model_get(model,&iter,1,&vvp,-1);
		gtk_list_store_remove(vvpmi->store,&iter);
		if(g_list_find(vvpmi->add,vvp)){	/*If newly added item*/
			vvpmi->add=g_list_remove(vvpmi->add,vvp);
			velocity_view_profile_free(vvp);
		}else{
			vvpmi->remove=g_list_prepend(vvpmi->remove,vvp);
		}
	}
}
GtkWidget*velocity_view_profile_manager_widget_new(){
	velocity_view_profile_manager_info*vvpmi=g_new0(velocity_view_profile_manager_info,1);
	GtkCellRenderer*lr=gtk_cell_renderer_text_new();
	GtkTreeViewColumn*lc=gtk_tree_view_column_new_with_attributes(_("Name"),lr,"text",0,NULL);
	GtkWidget*bb=gtk_hbutton_box_new();
	GtkWidget*t=gtk_scrolled_window_new(NULL,NULL);
	GList*n=velocity_view_profile_list();
	GtkTreeIter iter;
	vvpmi->widget=gtk_vbox_new(FALSE,12);
	vvpmi->store=gtk_list_store_new(2,G_TYPE_STRING,G_TYPE_POINTER);	/*View profile name, velocity_view_profile*/
	vvpmi->tree=GTK_TREE_VIEW(gtk_tree_view_new_with_model(GTK_TREE_MODEL(vvpmi->store)));
	gtk_tree_view_append_column(vvpmi->tree,lc);
	gtk_tree_view_set_headers_visible(vvpmi->tree,FALSE);	//TODO:  we may turn this back on later....
	gtk_tree_view_set_rules_hint(vvpmi->tree,TRUE);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(t),GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
	gtk_container_add(GTK_CONTAINER(t),GTK_WIDGET(vvpmi->tree));
	gtk_container_add(GTK_CONTAINER(vvpmi->widget),t);
	while(n){
		velocity_view_profile*vvp=(velocity_view_profile*)n->data;
		gtk_list_store_append(vvpmi->store,&iter);
		gtk_list_store_set(vvpmi->store,&iter,0,vvp->name,1,vvp,-1);
		n=n->next;
	}
	gtk_box_pack_start(GTK_BOX(vvpmi->widget),bb,FALSE,FALSE,6);
	gtk_box_set_spacing(GTK_BOX(bb),10);
	t=gtk_button_new_from_stock(GTK_STOCK_NEW);
	g_signal_connect(t,"clicked",G_CALLBACK(velocity_view_profile_manager_new_cb),vvpmi);
	gtk_container_add(GTK_CONTAINER(bb),t);
	t=gtk_button_new_from_stock(GTK_STOCK_PROPERTIES);
	g_signal_connect(t,"clicked",G_CALLBACK(velocity_view_profile_manager_properties_cb),vvpmi);
	gtk_container_add(GTK_CONTAINER(bb),t);
	t=gtk_button_new_from_stock(GTK_STOCK_DELETE);
	g_signal_connect(t,"clicked",G_CALLBACK(velocity_view_profile_manager_delete_cb),vvpmi);
	gtk_container_add(GTK_CONTAINER(bb),t);
	g_object_set_data(G_OBJECT(vvpmi->widget),"velocity_view_profile_manager_info",(gpointer)vvpmi);
	//TODO: Make on destroy callback, free vvpmi
	return(vvpmi->widget);
}
static void velocity_view_profile_manager_new(GtkOptionMenu*optionmenu){
	GtkWidget*vvpmw=velocity_view_profile_manager_widget_new();
	velocity_view_profile_manager_info*vvpmi=(velocity_view_profile_manager_info*)g_object_get_data(G_OBJECT(vvpmw),"velocity_view_profile_manager_info");
	GtkWidget*w=gtk_dialog_new_with_buttons(_("View profile manager"),NULL,GTK_DIALOG_MODAL,
		GTK_STOCK_CANCEL,GTK_RESPONSE_CANCEL,GTK_STOCK_OK,GTK_RESPONSE_OK,NULL);
	gboolean update=FALSE;
	gtk_widget_show_all(vvpmw);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(w)->vbox),vvpmw);
	switch(gtk_dialog_run(GTK_DIALOG(w))){
		case GTK_RESPONSE_OK:
			if(vvpmi->remove){
				GList*n=vvpmi->remove;
				while(n){
					velocity_view_profile_list_remove((velocity_view_profile*)n->data);
					update=TRUE;
					n=n->next;
				}
				g_list_free(vvpmi->remove);
			}
			if(vvpmi->add){
				GList*n=vvpmi->add;
				while(n){
					velocity_view_profile_list_add((velocity_view_profile*)n->data);
					update=TRUE;
					n=n->next;
				}
				g_list_free(vvpmi->add);
			}
			break;
		case GTK_RESPONSE_CANCEL:
		default:
			if(vvpmi->remove)g_list_free(vvpmi->remove);
			if(vvpmi->add){
				GList*n=vvpmi->add;
				while(n){
					velocity_view_profile_free((velocity_view_profile*)n->data);
					n=n->next;
				}
				g_list_free(vvpmi->add);
			}
			break;
	}
	if(update)velocity_view_profile_option_menu_update(optionmenu,gtk_widget_get_name(GTK_WIDGET(optionmenu)));
	g_object_unref(vvpmi->store);
	gtk_widget_destroy(w);
	g_free(vvpmi);
}
static void velocity_preferences_view_profile_manager_cb(GtkWidget*widget,gpointer user_data){
	velocity_view_profile_manager_new(GTK_OPTION_MENU(user_data));
}
static void velocity_preferences_view_profile_option_menu_changed_cb(GtkWidget*widget,gpointer user_data){
	velocity_view_profile*vvp=velocity_view_profile_option_menu_get_profile(GTK_OPTION_MENU(widget));
	if(vvp)gtk_widget_set_name(widget,vvp->name);
}
GtkWidget*velocity_view_profile_option_menu_new(const gchar*selected){
	GtkWidget*w=gtk_option_menu_new();
	g_signal_connect(w,"changed",G_CALLBACK(velocity_preferences_view_profile_option_menu_changed_cb),NULL);
	velocity_view_profile_option_menu_update(GTK_OPTION_MENU(w),selected);
	gtk_widget_show(w);
	return(w);
}
void velocity_view_profile_option_menu_update(GtkOptionMenu*om,const gchar*selected){
	GtkWidget*m=gtk_option_menu_get_menu(om);
	GtkWidget*mi;
	GList*n=velocity_view_profile_list();
	guint i=0;
	gboolean fakeselected=FALSE;
	VELOCITY_MSG("selected(%s)",selected);
	if(m)gtk_widget_destroy(m);
	m=gtk_menu_new();
	gtk_option_menu_set_menu(om,m);
	while(n){
		if(n->data){
			velocity_view_profile*p=(velocity_view_profile*)n->data;
			GtkWidget*mi=gtk_menu_item_new_with_label(p->name);
			gtk_widget_set_name(mi,g_strdup(p->name));
			gtk_menu_shell_append(GTK_MENU_SHELL(m),mi);
			gtk_widget_show(mi);
			if((selected)&&(strcmp(selected,p->name)==0))gtk_option_menu_set_history(om,i);
			i++;
		}
		n=n->next;
	}
	mi=gtk_menu_item_new();
	gtk_widget_set_name(mi,"VELOCITY_INTERNAL");
	gtk_menu_shell_append(GTK_MENU_SHELL(m),mi);
	gtk_widget_show(mi);
	mi=gtk_menu_item_new_with_label(_("More..."));
	gtk_widget_set_name(mi,"VELOCITY_INTERNAL");
	g_signal_connect(mi,"activate",G_CALLBACK(velocity_preferences_view_profile_manager_cb),om);
	gtk_menu_shell_append(GTK_MENU_SHELL(m),mi);
	gtk_widget_show(mi);
	if(selected)gtk_widget_set_name(GTK_WIDGET(om),g_strdup(selected));
}
velocity_view_profile*velocity_view_profile_option_menu_get_profile(GtkOptionMenu*om){
	gint history=gtk_option_menu_get_history(om);
	if((history>=0)&&(history<=g_list_length(velocity_view_profile_list())))return((velocity_view_profile*)g_list_nth_data(velocity_view_profile_list(),history));
	return(NULL);
}
