/*
 * Velocity - A GNOME File Manager
 *
 *
 * Copyright (C) (2002) (Kyle Davis) <kyled@blueriver.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "velocity-preferences-window.h"
#include "velocity-preferences-util.h"
#include "velocity-module-private.h"
#include "velocity-factories.h"
#include "callbacks.h"

static GtkWidget*velocity_preferences_general(velocity_preferences_dialog*vprefsdlg,const gchar*vuri){
	GtkWidget*b=gtk_vbox_new(FALSE,12);
	GtkWidget*t=gtk_check_button_new_with_label(_("Single-click icon activation"));
	GtkWidget*h=gtk_hbox_new(FALSE,12);
	GtkAdjustment*a=GTK_ADJUSTMENT(gtk_adjustment_new((gfloat)vprefs->view->tooltip_delay,0,10000,100,0,0));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(t),vprefs->view->single_click);
	g_object_set_data(G_OBJECT(b),"single_click",(gpointer)t);
	gtk_box_pack_start(GTK_BOX(b),t,FALSE,FALSE,0);
	t=gtk_check_button_new_with_label(_("Open folders in new window by default"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(t),!vprefs->view->dirs_in_same_win);	/*like Yoda speaking we are*/
	g_object_set_data(G_OBJECT(b),"dirs_in_same_win",(gpointer)t);
	gtk_box_pack_start(GTK_BOX(b),t,FALSE,FALSE,0);
	t=gtk_check_button_new_with_label(_("Automatic image preview"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(t),vprefs->view->preview_images);
	g_object_set_data(G_OBJECT(b),"preview_images",(gpointer)t);
	gtk_box_pack_start(GTK_BOX(b),t,FALSE,FALSE,0);
	t=gtk_label_new(_("Tooltip delay ms (0 means disabled):"));
	gtk_box_pack_start(GTK_BOX(h),t,TRUE,FALSE,0);
	t=gtk_hscale_new(a);
	gtk_scale_set_draw_value(GTK_SCALE(t),TRUE);
	gtk_scale_set_digits(GTK_SCALE(t),0);
	g_object_set_data(G_OBJECT(b),"tooltip_delay",(gpointer)a);
	gtk_box_pack_start(GTK_BOX(h),t,TRUE,TRUE,0);
	gtk_box_pack_start(GTK_BOX(b),h,FALSE,FALSE,0);
	gtk_widget_show_all(b);
	return(b);
}
static void velocity_preferences_general_free(velocity_preferences_dialog*vprefsdlg,GtkWidget*dlgcomponent,gint gtkresponse){
	GtkWidget*single_click=GTK_WIDGET(g_object_get_data(G_OBJECT(dlgcomponent),"single_click"));
	GtkWidget*dirs_in_same_win=GTK_WIDGET(g_object_get_data(G_OBJECT(dlgcomponent),"dirs_in_same_win"));
	GtkAdjustment*tooltip_delay=GTK_ADJUSTMENT(g_object_get_data(G_OBJECT(dlgcomponent),"tooltip_delay"));
	GtkWidget*preview_images=GTK_WIDGET(g_object_get_data(G_OBJECT(dlgcomponent),"preview_images"));
	GList*n=velocity_windowman_list();
	switch(gtkresponse){
		case GTK_RESPONSE_OK:
			vprefs->desktop->single_click=vprefs->view->single_click=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(single_click));
			vprefs->view->dirs_in_same_win=!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dirs_in_same_win));	/*like Yoda speaking we are*/
			vprefs->desktop->tooltip_delay=vprefs->view->tooltip_delay=(guint)tooltip_delay->value;
			vprefs->desktop->preview_images=vprefs->view->preview_images=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(preview_images));
			while(n){
				if(n->data){
					velocity_window*vwin=(velocity_window*)n->data;
					vwin->view->prefs->single_click=vprefs->view->single_click;
					vwin->view->prefs->dirs_in_same_win=vprefs->view->dirs_in_same_win;
					vwin->view->prefs->tooltip_delay=vprefs->view->tooltip_delay;
					vwin->view->prefs->preview_images=vprefs->view->preview_images;
					if(vwin->view->prefs->preview_images)velocity_view_soft_refresh(vwin->view,TRUE);
				}
				n=n->next;
			}
			if(velocity_windowman_desktop()){
				velocity_view*desktop=velocity_windowman_desktop();
				desktop->prefs->single_click=vprefs->desktop->single_click;
				/*The desktop should always open folders in new windows...so that doesn't apply here*/
				desktop->prefs->tooltip_delay=vprefs->desktop->tooltip_delay;
				desktop->prefs->preview_images=vprefs->desktop->preview_images;
				if(desktop->prefs->preview_images)velocity_view_soft_refresh(desktop,TRUE);
			}
			velocity_preferences_sync();
			break;
		case GTK_RESPONSE_CANCEL:
		default:
			break;
	}
}

static GtkWidget*velocity_preferences_lookandfeel_profiles(velocity_preferences_dialog*vprefsdlg,const gchar*vuri){
	GtkWidget*w=velocity_view_profile_manager_widget_new();
	gtk_widget_show_all(w);
	return(w);
}
static void velocity_preferences_lookandfeel_profiles_free(velocity_preferences_dialog*vprefsdlg,GtkWidget*dlgcomponent,gint gtkresponse){
	velocity_view_profile_manager_info*vvpmi=(velocity_view_profile_manager_info*)g_object_get_data(G_OBJECT(dlgcomponent),"velocity_view_profile_manager_info");
	gboolean update=FALSE;
	switch(gtkresponse){
		case GTK_RESPONSE_OK:
			if(vvpmi->remove){
				GList*n=vvpmi->remove;
				while(n){
					velocity_view_profile_list_remove((velocity_view_profile*)n->data);
					update=TRUE;
					n=n->next;
				}
				g_list_free(vvpmi->remove);
			}
			if(vvpmi->add){
				GList*n=vvpmi->add;
				while(n){
					velocity_view_profile_list_add((velocity_view_profile*)n->data);
					update=TRUE;
					n=n->next;
				}
				g_list_free(vvpmi->add);
			}
			break;
		case GTK_RESPONSE_CANCEL:
		default:
			if(vvpmi->remove)g_list_free(vvpmi->remove);
			if(vvpmi->add){
				GList*n=vvpmi->add;
				while(n){
					velocity_view_profile_free((velocity_view_profile*)n->data);
					n=n->next;
				}
				g_list_free(vvpmi->add);
			}
			break;
	}
	if(update){
		GList*n=velocity_windowman_list();
		while(n){
			velocity_window*vwin=(velocity_window*)n->data;
			velocity_view_profile_option_menu_update(GTK_OPTION_MENU(vwin->optionmenu),gtk_widget_get_name(GTK_WIDGET(vwin->optionmenu)));
			n=n->next;
		}
	}
	g_object_unref(vvpmi->store);
	g_free(vvpmi);
}

static GtkWidget*velocity_preferences_lookandfeel_tree(velocity_preferences_dialog*vprefsdlg,const gchar*vuri){
	GtkWidget*b=gtk_vbox_new(FALSE,12);
	GtkWidget*t=gtk_check_button_new_with_label(_("Keep tree in sync with current location"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(t),vprefs->sync_tree);
	g_object_set_data(G_OBJECT(b),"sync_tree",(gpointer)t);
	gtk_box_pack_start(GTK_BOX(b),t,FALSE,FALSE,0);
	gtk_widget_show_all(b);
	return(b);
}
static void velocity_preferences_lookandfeel_tree_free(velocity_preferences_dialog*vprefsdlg,GtkWidget*dlgcomponent,gint gtkresponse){
	GtkWidget*sync_tree=GTK_WIDGET(g_object_get_data(G_OBJECT(dlgcomponent),"sync_tree"));
	switch(gtkresponse){
		case GTK_RESPONSE_OK:
			vprefs->sync_tree=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(sync_tree));
			velocity_preferences_sync();
			break;
		case GTK_RESPONSE_CANCEL:
		default:
			break;
	}
}

static GtkWidget*velocity_preferences_lookandfeel_desktop(velocity_preferences_dialog*vprefsdlg,const gchar*vuri){
	GtkWidget*b=gtk_vbox_new(FALSE,12);
	GtkWidget*t=gtk_check_button_new_with_label(_("Let Velocity handle the desktop"));
	GtkWidget*f=gtk_frame_new(_("Desktop preferences"));
	GtkWidget*b2=gtk_vbox_new(FALSE,12);
	GtkWidget*h=gtk_hbox_new(TRUE,12);
	GtkWidget*o=velocity_view_profile_option_menu_new(vprefs->desktop->profile->name);
	GtkWidget*t2=gtk_menu_item_new();
	GList*n=velocity_view_profile_list();
	guint i=0;
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(t),vprefs->handle_desktop);
	g_object_set_data(G_OBJECT(b),"handle_desktop",(gpointer)t);
	gtk_box_pack_start(GTK_BOX(b),t,FALSE,FALSE,0);
	gtk_box_pack_start(GTK_BOX(b),f,FALSE,FALSE,0);
	gtk_container_add(GTK_CONTAINER(f),b2);
	t=gtk_label_new(_("Preferred view profile:"));
	gtk_box_pack_start(GTK_BOX(h),t,TRUE,FALSE,0);
	g_object_set_data(G_OBJECT(b),"view_profile",(gpointer)o);
	gtk_box_pack_start(GTK_BOX(h),o,FALSE,FALSE,0);
	gtk_box_pack_start(GTK_BOX(b2),h,FALSE,FALSE,0);
	h=gtk_hbox_new(TRUE,12);
	t=gtk_label_new(_("Icon/Tooltip font:"));
	gtk_box_pack_start(GTK_BOX(h),t,FALSE,FALSE,0);
	t=gnome_font_picker_new();
	gnome_font_picker_set_title(GNOME_FONT_PICKER(t),_("Icon/Tooltip font:"));
	gnome_font_picker_set_mode(GNOME_FONT_PICKER(t),GNOME_FONT_PICKER_MODE_FONT_INFO);
	gnome_font_picker_set_font_name(GNOME_FONT_PICKER(t),vprefs->desktop->font_str);
	g_object_set_data(G_OBJECT(b),"font",(gpointer)t);
	gtk_box_pack_start(GTK_BOX(h),t,FALSE,FALSE,0);
	gtk_box_pack_start(GTK_BOX(b2),h,FALSE,FALSE,0);
	gtk_widget_show_all(b);
	return(b);
}
static void velocity_preferences_nuke_icons_foreach_view(velocity_vfs_obj*vfsobj,gpointer user_data){
	if(vfsobj->icon){
		velocity_icon_free(vfsobj->icon);
		vfsobj->icon=NULL;
	}
}
static void velocity_preferences_lookandfeel_desktop_free(velocity_preferences_dialog*vprefsdlg,GtkWidget*dlgcomponent,gint gtkresponse){
	GtkWidget*handle_desktop=GTK_WIDGET(g_object_get_data(G_OBJECT(dlgcomponent),"handle_desktop"));
	GnomeFontPicker*gfp=GNOME_FONT_PICKER(g_object_get_data(G_OBJECT(dlgcomponent),"font"));
	velocity_view_profile*vvp=velocity_view_profile_option_menu_get_profile(GTK_OPTION_MENU(g_object_get_data(G_OBJECT(dlgcomponent),"view_profile")));
	const char*font=gnome_font_picker_get_font_name(gfp);
	switch(gtkresponse){
		case GTK_RESPONSE_OK:
			if(font){
				if(vprefs->desktop->font_str)g_free(vprefs->desktop->font_str);
				vprefs->desktop->font_str=g_strdup(font);
				if(vprefs->desktop->font)pango_font_description_free(vprefs->desktop->font);
				vprefs->desktop->font=pango_font_description_from_string(vprefs->desktop->font_str);
			}
			if(vvp){
				if((vprefs->desktop->profile!=vvp)&&(velocity_windowman_desktop())){
					velocity_view_foreach(velocity_windowman_desktop(),(GFunc)velocity_preferences_nuke_icons_foreach_view,NULL);
				}
				vprefs->desktop->profile=vvp;
			}
			vprefs->handle_desktop=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(handle_desktop));
			if(vprefs->handle_desktop){
				velocity_windowman_desktop_enable();
				(velocity_windowman_desktop())->prefs->background=vprefs->desktop->background;
				velocity_view_soft_refresh(velocity_windowman_desktop(),TRUE);
			}else{
				velocity_windowman_desktop_disable();
				if(!velocity_windowman_list())velocity_windowman_new(NULL);
			}
			velocity_preferences_sync();
			break;
		case GTK_RESPONSE_CANCEL:
		default:
			break;
	}
}
static gint velocity_preferences_lookandfeel_view_animator_cb(gpointer user_data){
	if(user_data){
		GtkWidget*b=(GtkWidget*)user_data;
		if(GTK_WIDGET_VISIBLE(b)){
			velocity_icon*i=(velocity_icon*)g_object_get_data(G_OBJECT(b),"icon");
			switch(i->effect_type){
				case VELOCITY_ICON_ANIMATION_BOUNCE:
					velocity_icon_animate(i,4.0);
					break;
				case VELOCITY_ICON_ANIMATION_ROTATE:
					velocity_icon_animate(i,10.0);
					break;
				case VELOCITY_ICON_ANIMATION_PULSATE:
					velocity_icon_animate(i,0.05);
					break;
				case VELOCITY_ICON_ANIMATION_INVALID:
				default:
					break;
			}
		}
	}
	return(TRUE);
}
static void velocity_preferences_lookandfeel_view_animation_change_cb(GtkWidget*w,gpointer user_data){
	velocity_icon*i=(velocity_icon*)user_data;
	velocity_icon_animation_mode m=(velocity_icon_animation_mode)g_object_get_data(G_OBJECT(w),"value");
	velocity_icon_reset_animation(i);
	velocity_icon_set_animation_type(i,m);
}
static GtkWidget*velocity_preferences_lookandfeel_view(velocity_preferences_dialog*vprefsdlg,const gchar*vuri){
	GtkWidget*b=gtk_vbox_new(FALSE,12);
	GtkWidget*t=gtk_label_new(_("Preferred view profile:"));
	GtkWidget*h=gtk_hbox_new(TRUE,12);
	GnomeCanvas*gc=GNOME_CANVAS(gnome_canvas_new());
	GnomeCanvasGroup*gcg=GNOME_CANVAS_GROUP(gnome_canvas_item_new(gnome_canvas_root(GNOME_CANVAS(gc)),gnome_canvas_group_get_type(),
		"x",(gdouble)0,"y",(gdouble)0,NULL));
	velocity_icon*icon=velocity_icon_new(gcg,vprefs->view->profile->icon_size.x>>1,vprefs->view->profile->icon_size.y>>1,
		vprefs->view->profile->icon_size,vprefs->view->profile->icon_spacing,vprefs->view->profile->caption_placement);
	GSList*rg=NULL;
	GtkWidget*o=velocity_view_profile_option_menu_new(vprefs->view->profile->name);
	GtkWidget*t2=gtk_menu_item_new();
	GList*n=velocity_view_profile_list();
	guint i=0;
	gtk_box_pack_start(GTK_BOX(h),t,FALSE,FALSE,0);
	g_object_set_data(G_OBJECT(b),"view_profile",(gpointer)o);
	gtk_box_pack_start(GTK_BOX(h),o,FALSE,FALSE,0);
	gtk_box_pack_start(GTK_BOX(b),h,FALSE,FALSE,0);
	h=gtk_hbox_new(TRUE,12);
	t=gtk_label_new(_("Icon/Tooltip font:"));
	gtk_box_pack_start(GTK_BOX(h),t,FALSE,FALSE,0);
	t=gnome_font_picker_new();
	gnome_font_picker_set_title(GNOME_FONT_PICKER(t),_("Icon/Tooltip font:"));
	gnome_font_picker_set_mode(GNOME_FONT_PICKER(t),GNOME_FONT_PICKER_MODE_FONT_INFO);
	gnome_font_picker_set_font_name(GNOME_FONT_PICKER(t),vprefs->view->font_str);
	g_object_set_data(G_OBJECT(b),"font",(gpointer)t);
	gtk_box_pack_start(GTK_BOX(h),t,FALSE,FALSE,0);
	gtk_box_pack_start(GTK_BOX(b),h,FALSE,FALSE,0);
	t=gtk_label_new(_("Background color:"));
	h=gtk_hbox_new(TRUE,12);
	gtk_box_pack_start(GTK_BOX(h),t,FALSE,FALSE,0);
	t=gnome_color_picker_new();
	gnome_color_picker_set_title(GNOME_COLOR_PICKER(t),_("Background color:"));
	gnome_color_picker_set_i16(GNOME_COLOR_PICKER(t),vprefs->view->background.red,vprefs->view->background.green,vprefs->view->background.blue,0);
	gnome_color_picker_set_use_alpha(GNOME_COLOR_PICKER(t),FALSE);
	g_object_set_data(G_OBJECT(b),"background",(gpointer)t);
	gtk_box_pack_start(GTK_BOX(h),t,FALSE,FALSE,0);
	gtk_box_pack_start(GTK_BOX(b),h,FALSE,FALSE,0);
	t=gtk_frame_new(_("Icon activation animation (Desktop included)"));
	gtk_box_pack_start(GTK_BOX(b),t,TRUE,TRUE,0);
	h=gtk_hbox_new(FALSE,12);
	gtk_container_add(GTK_CONTAINER(t),h);
	t=gtk_vbox_new(TRUE,12);
	gtk_box_pack_start(GTK_BOX(h),t,FALSE,FALSE,8);
	gtk_box_pack_start(GTK_BOX(h),GTK_WIDGET(gc),TRUE,TRUE,4);
	h=t;
	t=gtk_radio_button_new_with_label(rg,_("Bounce"));
	GTK_TOGGLE_BUTTON(t)->active=(vprefs->view->animation_mode==VELOCITY_ICON_ANIMATION_BOUNCE);
	rg=g_slist_prepend(rg,(gpointer)t);
	g_object_set_data(G_OBJECT(t),"value",(gpointer)VELOCITY_ICON_ANIMATION_BOUNCE);
	g_signal_connect(t,"toggled",G_CALLBACK(velocity_preferences_lookandfeel_view_animation_change_cb),icon);
	gtk_box_pack_start(GTK_BOX(h),t,FALSE,FALSE,0);
	t=gtk_radio_button_new_with_label(rg,_("Rotate"));
	GTK_TOGGLE_BUTTON(t)->active=(vprefs->view->animation_mode==VELOCITY_ICON_ANIMATION_ROTATE);
	rg=g_slist_prepend(rg,(gpointer)t);
	g_object_set_data(G_OBJECT(t),"value",(gpointer)VELOCITY_ICON_ANIMATION_ROTATE);
	g_signal_connect(t,"toggled",G_CALLBACK(velocity_preferences_lookandfeel_view_animation_change_cb),icon);
	gtk_box_pack_start(GTK_BOX(h),t,FALSE,FALSE,0);
	t=gtk_radio_button_new_with_label(rg,_("Pulsate"));
	GTK_TOGGLE_BUTTON(t)->active=(vprefs->view->animation_mode==VELOCITY_ICON_ANIMATION_PULSATE);
	rg=g_slist_prepend(rg,(gpointer)t);
	g_object_set_data(G_OBJECT(t),"value",(gpointer)VELOCITY_ICON_ANIMATION_PULSATE);
	g_signal_connect(t,"toggled",G_CALLBACK(velocity_preferences_lookandfeel_view_animation_change_cb),icon);
	gtk_box_pack_start(GTK_BOX(h),t,FALSE,FALSE,0);
	gtk_widget_show_all(b);
	gnome_canvas_scroll_to(gc,0,0);
	{
		char*iuri=velocity_image_get_uri(VELOCITY_DEFAULT_ICON,64);
		if(iuri){
			GdkPixbuf*p=velocity_image_cache_get(iuri,vprefs->view->profile->icon_size.x,vprefs->view->profile->icon_size.y,NULL,VELOCITY_IMAGE_CACHE_PRIORITY_LOW);
			if(p){
				velocity_icon_set_icon(icon,p);
				g_object_unref(p);
			}
			g_free(iuri);
		}
		gtk_widget_set_background_color(GTK_WIDGET(gc),&vprefs->view->background);
		velocity_icon_set_font_colors(icon,
			gdk_color_to_rgba(vprefs->view->caption_fg,255),gdk_color_to_rgba(vprefs->view->caption_bg,192),
			gdk_color_to_rgba(vprefs->view->caption_pfg,255),gdk_color_to_rgba(vprefs->view->caption_pbg,192),
			gdk_color_to_rgba(vprefs->view->caption_hfg,255),gdk_color_to_rgba(vprefs->view->caption_hbg,192));
		velocity_icon_set_caption(icon,_("Test Icon"),vprefs->view->font,FALSE);
		velocity_icon_set_animation_type(icon,vprefs->view->animation_mode);
		g_object_set_data(G_OBJECT(b),"animation_timeout",GINT_TO_POINTER(gtk_timeout_add(50,velocity_preferences_lookandfeel_view_animator_cb,(gpointer)b)));
		g_object_set_data(G_OBJECT(b),"icon",(gpointer)icon);
	}
	return(b);
}
static void velocity_preferences_lookandfeel_view_free(velocity_preferences_dialog*vprefsdlg,GtkWidget*dlgcomponent,gint gtkresponse){
	GnomeColorPicker*gcp=GNOME_COLOR_PICKER(g_object_get_data(G_OBJECT(dlgcomponent),"background"));
	GnomeFontPicker*gfp=GNOME_FONT_PICKER(g_object_get_data(G_OBJECT(dlgcomponent),"font"));
	velocity_view_profile*vvp=velocity_view_profile_option_menu_get_profile(GTK_OPTION_MENU(g_object_get_data(G_OBJECT(dlgcomponent),"view_profile")));
	velocity_icon*i=(velocity_icon*)g_object_get_data(G_OBJECT(dlgcomponent),"icon");
	const char*font=gnome_font_picker_get_font_name(gfp);
	GList*n=velocity_windowman_list();
	gtk_timeout_remove(GPOINTER_TO_INT(g_object_get_data(G_OBJECT(dlgcomponent),"animation_timeout")));
	switch(gtkresponse){
		case GTK_RESPONSE_OK:
			gnome_color_picker_get_i16(GNOME_COLOR_PICKER(gcp),&vprefs->view->background.red,&vprefs->view->background.green,&vprefs->view->background.blue,NULL);
			if(font){
				if(vprefs->view->font_str)g_free(vprefs->view->font_str);
				vprefs->view->font_str=g_strdup(font);
				if(vprefs->view->font)pango_font_description_free(vprefs->view->font);
				vprefs->view->font=pango_font_description_from_string(vprefs->view->font_str);
			}
			if(vvp)vprefs->view->profile=vvp;
			if(i){
				vprefs->view->animation_mode=i->effect_type;
				velocity_icon_free(i);
			}
			velocity_preferences_sync();
			while(n){
				if(n->data){
					velocity_window*vwin=(velocity_window*)n->data;
					velocity_view_set_background_color(vwin->view,&vprefs->view->background);
					vwin->view->prefs->animation_mode=vprefs->view->animation_mode;
					if(vvp){
						if(vwin->view->prefs->profile!=vvp)velocity_view_foreach(vwin->view,(GFunc)velocity_preferences_nuke_icons_foreach_view,NULL);
						vwin->view->prefs->profile=vvp;
					}
					velocity_view_soft_refresh(vwin->view,vwin->view->prefs->preview_images);
				}
				n=n->next;
			}
			if(velocity_windowman_desktop()){
				velocity_view*desktop=velocity_windowman_desktop();
				desktop->prefs->animation_mode=vprefs->view->animation_mode;
			}
			break;
		case GTK_RESPONSE_CANCEL:
		default:
			break;
	}
}
static void velocity_preferences_plugins_loader_check(GtkCellRendererToggle*cell,gchar*pathstr,gpointer user_data){
	GtkTreeModel*tm=(GtkTreeModel*)user_data;
	GtkTreeIter iter;
	GtkTreePath*tp=gtk_tree_path_new_from_string(pathstr);
	gboolean toggle;
	velocity_preferences_dialog*vprefsdlg;
	gchar*uri;
	gchar*olabel;
	gchar*label;
	gtk_tree_model_get_iter(tm,&iter,tp);
	gtk_tree_model_get(tm,&iter,0,&toggle,1,&olabel,2,&uri,3,&vprefsdlg,-1);
	toggle^=TRUE;
	if(!toggle){
		velocity_module*vmod=velocity_module_tracker_get(uri);
		if(vmod){
			char*furi=g_strconcat(N_("/Plugins/"),vmod->info->name,NULL);
			label=g_strdup(vmod->info->id);	//TODO:  It may be more consistant to use vfs_uri_tail here
			velocity_module_tracker_unregister_by_name(uri);
			velocity_tree_unregister_item(vprefsdlg->tree,furi);
			g_free(furi);
		}
	}else{
		velocity_module*vmod=velocity_module_tracker_register(uri);
		if(vmod){
			char*furi=g_strconcat(N_("/Plugins/"),vmod->info->name,NULL);
			velocity_tree_item*vti=velocity_tree_register_item(vprefsdlg->tree,furi);
			velocity_tree_item_set_data(vti,"velocity_module",(gpointer)vmod);
			label=g_strdup(vmod->info->name);
			g_free(furi);
		}else{
			GtkWidget*dlg=gtk_message_dialog_new(GTK_WINDOW(vprefsdlg->widget),GTK_DIALOG_MODAL,GTK_MESSAGE_ERROR,GTK_BUTTONS_CLOSE,
				_("There was an error loading the module/plugin.\n"
				"Please ensure that it is installed properly."));
			VELOCITY_ERR("Unable to load module (%s)",olabel);
			gtk_dialog_run(GTK_DIALOG(dlg));
			gtk_widget_destroy(dlg);
			label=g_strdup(olabel);
			toggle^=TRUE;
		}
	}
	gtk_list_store_set(GTK_LIST_STORE(tm),&iter,0,toggle,1,label,-1);
	g_free(olabel);
	gtk_tree_path_free(tp);
}
static GtkWidget*velocity_preferences_plugins(velocity_preferences_dialog*vprefsdlg,const gchar*vuri){
	GtkWidget*b=gtk_vbox_new(FALSE,12);
	GtkWidget*f=gtk_frame_new(_("Plugin list"));
	GtkWidget*s=gtk_scrolled_window_new(NULL,NULL);
	GtkListStore*ls=gtk_list_store_new(4,G_TYPE_BOOLEAN,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_POINTER);/*enabled,label,uri,vprefsdlg*/
	GtkCellRenderer*lr=gtk_cell_renderer_toggle_new();
	GtkTreeViewColumn*lc=gtk_tree_view_column_new_with_attributes(_("Enabled"),lr,"active",0,NULL);
	GtkTreeView*l=GTK_TREE_VIEW(gtk_tree_view_new_with_model(GTK_TREE_MODEL(ls)));
	GtkWidget*bb=gtk_hbox_new(TRUE,12);
	GtkWidget*t=gtk_button_new_with_mnemonic(_("C_onfigure"));
	char*puri[]={PACKAGE_PLUGINS_DIR GNOME_VFS_URI_PATH_STR,vprefs->locations->plugins,NULL};
	int d=0;
	g_signal_connect(lr,"toggled",G_CALLBACK(velocity_preferences_plugins_loader_check),gtk_tree_view_get_model(l));
	gtk_tree_view_append_column(l,lc);
	lr=gtk_cell_renderer_text_new();
	lc=gtk_tree_view_column_new_with_attributes(_("Plugin"),lr,"text",1,NULL);
	gtk_tree_view_append_column(l,lc);
	gtk_tree_view_set_rules_hint(l,TRUE);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(s),GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(s),GTK_WIDGET(l));
	gtk_container_add(GTK_CONTAINER(f),s);
	gtk_box_pack_start(GTK_BOX(b),f,TRUE,TRUE,0);
	while(puri[d]){
		GList*files=NULL;
		VELOCITY_MSG("Reading plugins from:  %s",puri[d]);
		if(gnome_vfs_directory_list_load(&files,puri[d],GNOME_VFS_FILE_INFO_GET_MIME_TYPE|GNOME_VFS_FILE_INFO_FORCE_FAST_MIME_TYPE|GNOME_VFS_FILE_INFO_FOLLOW_LINKS)==GNOME_VFS_OK){
			GList*n=files;
			while(n){
				if(n->data){
					GnomeVFSFileInfo*vfsinfo=(GnomeVFSFileInfo*)n->data;
					if(strcmp(vfsinfo->mime_type,"application/x-shared-library")==0){
						GtkTreeIter iter;
						char*uri=g_strconcat(puri[d],vfsinfo->name,NULL);
						velocity_module*vmod=velocity_module_tracker_get(uri);
						gtk_list_store_append(ls,&iter);
						gtk_list_store_set(ls,&iter,
							0,(vmod!=NULL),
							1,g_strdup((vmod)?vmod->info->name:vfsinfo->name),
							2,uri,
							3,vprefsdlg,
							-1);
					}
					gnome_vfs_file_info_unref(vfsinfo);
				}
				n=n->next;
			}
		}else{
			VELOCITY_ERR("An error occured trying to read %s\n",puri[d]);
		}
		d++;
	}
	g_object_unref(ls);
	gtk_box_pack_start(GTK_BOX(bb),t,FALSE,FALSE,2);
	t=gtk_button_new_with_mnemonic(_("_About"));
	gtk_box_pack_start(GTK_BOX(bb),t,FALSE,FALSE,2);
	gtk_box_pack_start(GTK_BOX(b),bb,FALSE,TRUE,0);
	gtk_widget_show_all(b);
	return(b);
}
static GtkWidget*velocity_preferences_main(velocity_preferences_dialog*vprefsdlg,const gchar*vuri){
	GtkWidget*b=gtk_vbox_new(FALSE,12);
	GtkWidget*i=gtk_image_new_from_file(PACKAGE_PIXMAPS_DIR G_DIR_SEPARATOR_S VELOCITY_ICON);
	GtkWidget*l=gtk_label_new(_("This is the main preferences dialog for Velocity.\n\n"
		"You can customize the way Velocity\nlooks and behaves from here."));
	gtk_label_set_line_wrap(GTK_LABEL(l),TRUE);
	gtk_label_set_selectable(GTK_LABEL(l),TRUE);
	gtk_container_add(GTK_CONTAINER(b),i);
	gtk_container_add(GTK_CONTAINER(b),l);
	gtk_widget_show_all(b);
	return(b);
}
static GtkWidget*velocity_preferences_advanced(velocity_preferences_dialog*vprefsdlg,const gchar*vuri){
	GtkWidget*b=gtk_vbox_new(FALSE,12);
	GtkWidget*h=gtk_hbox_new(FALSE,12);
	GtkWidget*t=gtk_label_new(_("Icon selector type:"));
	gchar*s[]={_("Circle"),_("Square"),
		_("Highlight"),_("Image Overlay"),NULL};
	GtkWidget*som=velocity_option_menu_factory(s);
	gtk_option_menu_set_history(GTK_OPTION_MENU(som),vprefs->selector_type-1);
	g_object_set_data(G_OBJECT(b),"selector_type",(gpointer)som);
	gtk_container_add(GTK_CONTAINER(h),t);
	gtk_container_add(GTK_CONTAINER(h),som);
	gtk_box_pack_start(GTK_BOX(b),h,FALSE,TRUE,12);
	gtk_widget_show_all(b);
	return(b);
}
static void velocity_preferences_advanced_foreach_view(velocity_vfs_obj*vfsobj,GtkWidget*dlgcomponent){
	if(vfsobj->icon)velocity_icon_set_selector_type(vfsobj->icon,vprefs->selector_type);
}
static void velocity_preferences_advanced_foreach(velocity_view*vview,GtkWidget*dlgcomponent){
	velocity_view_foreach(vview,(GFunc)velocity_preferences_advanced_foreach_view,dlgcomponent);
}
static void velocity_preferences_advanced_free(velocity_preferences_dialog*vprefsdlg,GtkWidget*dlgcomponent,gint gtkresponse){
	GtkWidget*som=GTK_WIDGET(g_object_get_data(G_OBJECT(dlgcomponent),"selector_type"));
	guint ns=gtk_option_menu_get_history(GTK_OPTION_MENU(som))+1;
	switch(gtkresponse){
		case GTK_RESPONSE_OK:
			if(vprefs->selector_type!=ns){
				vprefs->selector_type=ns;
				velocity_windowman_foreach(TRUE,(GFunc)velocity_preferences_advanced_foreach,dlgcomponent);
			}
			break;
		case GTK_RESPONSE_CANCEL:
		default:
			break;
	}
}

/*This is to be a set of URI (NOTE: can be fake URI's),velocity_preferences_dialog_component,
	velocity_preferences_dialog_component_free,
	expand_and_collapse_img_uri*/
static void*prefs_items[]={N_("/"),velocity_preferences_main,NULL,PACKAGE_PIXMAPS_DIR G_DIR_SEPARATOR_S VELOCITY_ICON,
	N_("/General/"),velocity_preferences_general,
		velocity_preferences_general_free,PACKAGE_PIXMAPS_DIR G_DIR_SEPARATOR_S VELOCITY_PREFS_BEHAVIOR,
	N_("/Look & Feel/"),NULL,NULL,PACKAGE_PIXMAPS_DIR G_DIR_SEPARATOR_S VELOCITY_PREFS_THEMES,
	N_("/Look & Feel/View Profiles/"),velocity_preferences_lookandfeel_profiles,
		velocity_preferences_lookandfeel_profiles_free,PACKAGE_PIXMAPS_DIR G_DIR_SEPARATOR_S VELOCITY_PREFS_BIN,
	N_("/Look & Feel/View/"),velocity_preferences_lookandfeel_view,
		velocity_preferences_lookandfeel_view_free,PACKAGE_PIXMAPS_DIR G_DIR_SEPARATOR_S VELOCITY_PREFS_BIN,
	N_("/Look & Feel/Desktop/"),velocity_preferences_lookandfeel_desktop,
		velocity_preferences_lookandfeel_desktop_free,PACKAGE_PIXMAPS_DIR G_DIR_SEPARATOR_S VELOCITY_PREFS_DESKTOP,
	N_("/Look & Feel/Tree/"),velocity_preferences_lookandfeel_tree,
		velocity_preferences_lookandfeel_tree_free,PACKAGE_PIXMAPS_DIR G_DIR_SEPARATOR_S VELOCITY_PREFS_BIN,
	N_("/Look & Feel/Toolbar/"),NULL,NULL,PACKAGE_PIXMAPS_DIR G_DIR_SEPARATOR_S VELOCITY_PREFS_BIN,
	N_("/Advanced/"),velocity_preferences_advanced,
		velocity_preferences_advanced_free,PACKAGE_PIXMAPS_DIR G_DIR_SEPARATOR_S VELOCITY_PREFS_ADVANCED,
	N_("/Plugins/"),velocity_preferences_plugins,NULL,PACKAGE_PIXMAPS_DIR G_DIR_SEPARATOR_S VELOCITY_PREFS_PUZZLE,
	NULL};
static void*velocity_preferences_tree_item_modfunc(velocity_tree_item*vti,velocity_tree_item_mod_type modtype){
	if(modtype==VELOCITY_TREE_ITEM_MOD_SET_ICONS){
		if((strlen(vti->uri)>9)&&(strncmp(vti->uri,N_("/Plugins/"),9)==0)){
			vti->expand_file=g_strdup(PACKAGE_PIXMAPS_DIR G_DIR_SEPARATOR_S VELOCITY_PREFS_PUZZLE_PIECE);
			vti->collapse_file=g_strdup(PACKAGE_PIXMAPS_DIR G_DIR_SEPARATOR_S VELOCITY_PREFS_PUZZLE_PIECE);
		}else{
			guint i=0;
			while(prefs_items[i]){
				if((prefs_items[i+3])&&(strcmp(vti->uri,prefs_items[i])==0)){
					vti->expand_file=g_strdup((char*)prefs_items[i+3]);
					vti->collapse_file=g_strdup(vti->expand_file);
				}
				i+=4;
			}
		}
		vti->size.x=vti->size.y=24;
	}else if(modtype==VELOCITY_TREE_ITEM_MOD_GET_LABEL){
		if(velocity_vfs_uris_match(vti->uri,"/"))return(g_strdup(_("Preferences")));
	}
	return(NULL);
}
static void velocity_preferences_tree_cb(velocity_tree_item*vti,gpointer user_data){
	velocity_preferences_dialog*vprefsdlg=(velocity_preferences_dialog*)user_data;
	GtkWidget*w=NULL;
	gboolean unref=FALSE;
	if(vprefsdlg->active){
		gtk_widget_ref(vprefsdlg->active);
		gtk_container_remove(GTK_CONTAINER(vprefsdlg->pane),vprefsdlg->active);
	}
	//TODO:  Eventually try to devise a better method than this string lookup...
	if((strlen(vti->uri)>9)&&(strncmp(vti->uri,"/Plugins/",9)==0)){
		velocity_module*vmod=(velocity_module*)velocity_tree_item_get_data(vti,"velocity_module");
		if(vmod){
			if(vmod->preferences_new){
				w=(GtkWidget*)velocity_tree_item_get_data(vti,"preferences_component");
				if(!w){
					w=vmod->preferences_new(vprefsdlg,vti->uri);
					if(w){
						velocity_tree_item_set_data(vti,"preferences_component",(gpointer)w);
					}else{
						VELOCITY_ERR("Object at %s was unable to return a component!!",vti->uri);
					}
				}else{
					VELOCITY_MSG("Component for this object has already been created");
					unref=TRUE;
				}
			}
		}
	}else{
		velocity_preferences_dialog_component component_func=(velocity_preferences_dialog_component)velocity_tree_item_get_data(vti,"preferences_component_new");
		if(component_func){
			w=(GtkWidget*)velocity_tree_item_get_data(vti,"preferences_component");
			if(!w){
				w=component_func(vprefsdlg,vti->uri);
				if(w){
					velocity_tree_item_set_data(vti,"preferences_component",(gpointer)w);
				}else{
					VELOCITY_ERR("Object at %s was unable to return a component!!",vti->uri);
				}
			}else{
				VELOCITY_MSG("Component for this object has already been created");
				unref=TRUE;
			}
		}
	}
	if(!w)w=vprefsdlg->generic;
	vprefsdlg->active=w;
	if(GTK_IS_CONTAINER(w))gtk_container_set_border_width(GTK_CONTAINER(w),4); /*This will make things seem a bit nicer*/
	gtk_paned_add2(GTK_PANED(vprefsdlg->pane),w);
	if(unref)gtk_widget_unref(w);
	gtk_widget_show(w);
}
static void velocity_preferences_free_all_recurse(velocity_preferences_dialog*vprefsdlg,gint gtkresponse,velocity_tree_item*pvti){
	if(pvti){
		GList*n=pvti->children;
		while(n){
			if(n->data){
				velocity_tree_item*vti=(velocity_tree_item*)n->data;
				GtkWidget*w=(GtkWidget*)velocity_tree_item_get_data(vti,"preferences_component");
				if(vti->children)velocity_preferences_free_all_recurse(vprefsdlg,gtkresponse,vti);
				if(w){
					if((strlen(vti->uri)>9)&&(strncmp(vti->uri,"/Plugins/",9)==0)){
						velocity_module*vmod=(velocity_module*)velocity_tree_item_get_data(vti,"velocity_module");
						if(vmod){
							if(vmod->preferences_free)vmod->preferences_free(vprefsdlg,w,gtkresponse);
						}
					}else{
						velocity_preferences_dialog_component_free component_func=(velocity_preferences_dialog_component_free)velocity_tree_item_get_data(vti,"preferences_component_free");
						if(component_func)component_func(vprefsdlg,w,gtkresponse);
						if(w)gtk_widget_destroy(w);
					}
				}
			}
			n=n->next;
		}
	}
}
static void velocity_preferences_response_cb(GtkDialog*dialog,gint response,gpointer user_data){
	if(response!=GTK_RESPONSE_HELP){	//TODO:  Finish/do help....
		velocity_preferences_dialog*vprefsdlg=(velocity_preferences_dialog*)user_data;
		velocity_preferences_free_all_recurse(vprefsdlg,response,vprefsdlg->tree->root);
		velocity_tree_free(vprefsdlg->tree);
		gtk_widget_destroy(GTK_WIDGET(vprefsdlg->widget));
		g_free(vprefsdlg);
	}
}
void velocity_preferences_window_new(GtkWidget*parent){
	guint i=0;
	GList*n=velocity_module_tracker_get_list();
	velocity_preferences_dialog*vprefsdlg=g_new0(velocity_preferences_dialog,1);
	velocity_tree_item*rvti=NULL;
	vprefsdlg->widget=gtk_dialog_new_with_buttons(_("Preferences"),GTK_WINDOW(parent),GTK_DIALOG_MODAL,
		GTK_STOCK_HELP,GTK_RESPONSE_HELP,GTK_STOCK_CANCEL,GTK_RESPONSE_CANCEL,GTK_STOCK_OK,GTK_RESPONSE_OK,NULL);
	vprefsdlg->pane=gtk_hpaned_new();
	vprefsdlg->active=vprefsdlg->generic=gtk_label_new(_("There are no settings for the item you have selected."));
	gtk_label_set_line_wrap(GTK_LABEL(vprefsdlg->generic),TRUE);
	gtk_paned_add2(GTK_PANED(vprefsdlg->pane),vprefsdlg->generic);
	gtk_window_set_position(GTK_WINDOW(vprefsdlg->widget),GTK_WIN_POS_CENTER);
	gtk_widget_set_size_request(GTK_WIDGET(vprefsdlg->widget),540,440);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(vprefsdlg->widget)->vbox),vprefsdlg->pane,TRUE,TRUE,0);
	vprefsdlg->tree=velocity_tree_new(NULL,(velocity_tree_item_mod_func)velocity_preferences_tree_item_modfunc,
		(velocity_tree_item_select_func)velocity_preferences_tree_cb,(gpointer)vprefsdlg);
	gtk_paned_add1(GTK_PANED(vprefsdlg->pane),vprefsdlg->tree->widget);
	gtk_paned_set_position(GTK_PANED(vprefsdlg->pane),200);
	g_signal_connect(vprefsdlg->widget,"response",G_CALLBACK(velocity_preferences_response_cb),vprefsdlg);
	gtk_widget_show_all(vprefsdlg->widget);
	while(prefs_items[i]){
		velocity_tree_item*vti=velocity_tree_register_item(vprefsdlg->tree,(gchar*)prefs_items[i]);
		if(!rvti)rvti=vti;
		velocity_tree_item_set_data(vti,"preferences_component_new",(gpointer)prefs_items[i+1]);
		velocity_tree_item_set_data(vti,"preferences_component_free",(gpointer)prefs_items[i+2]);
		i+=4;
	}
	while(n){
		if(n->data){
			velocity_module*vmod=(velocity_module*)n->data;
			if(velocity_module_is_dynamic(vmod)){
				char*furi=g_strconcat(N_("/Plugins/"),vmod->info->name,NULL);
				velocity_tree_item*vti=velocity_tree_register_item(vprefsdlg->tree,furi);
				if(vti)velocity_tree_item_set_data(vti,"velocity_module",(gpointer)vmod);
				g_free(furi);
			}
		}
		n=n->next;
	}
	gtk_tree_selection_select_iter(gtk_tree_view_get_selection(GTK_TREE_VIEW(vprefsdlg->tree->tree)),&rvti->iter);
	vprefsdlg->tree->select_func(rvti,(gpointer)vprefsdlg);
}
