/*
 * Velocity - A GNOME File Manager
 *
 *
 * Copyright (C) (2002) (Kyle Davis) <kyled@blueriver.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <sys/stat.h>
#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <glib.h>
#include <pango/pango.h>
#include "velocity-preferences.h"
#include "velocity-vfs.h"
#include "velocity-icons.h"
#include "velocity-module.h"

#define VELOCITY_PREFERENCES_VIEW_PROFILE_CONFIG	"/Velocity/view_profiles/"

velocity_preferences*vprefs=NULL;
static GList*velocity_view_profiles=NULL;

void velocity_preferences_init(){
	char*t=NULL;
	if(!vprefs){
		vprefs=g_new0(velocity_preferences,1);
		vprefs->window=g_new(velocity_window_preferences,1);
		vprefs->locations=g_new(velocity_locations_preferences,1);
		vprefs->view=g_new(velocity_view_preferences,1);
		vprefs->view->profile=g_new(velocity_view_profile,1);
		vprefs->desktop=g_new(velocity_view_preferences,1);
		vprefs->desktop->profile=g_new(velocity_view_profile,1);
		vprefs->trash=g_new(velocity_trash_preferences,1);
		vprefs->disk_usage=g_new(velocity_disk_usage_preferences,1);
		vprefs->plugin=g_new(velocity_plugin_preferences,1);
		vprefs->ext_usage_stats=NULL;
		/*config dir creation ideas borrowed and modified from xmms*/
		/*TODO: Trash and Desktop directory shouldn't be hard coded, should be part of prefs */
		vprefs->locations->home=velocity_vfs_ensure_dir_uri(g_get_home_dir());
		vprefs->locations->trash=g_strconcat(vprefs->locations->home,".Trash/",NULL);
		mkdir(vprefs->locations->trash,S_IRWXU|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);
		vprefs->locations->desktop=g_strconcat(vprefs->locations->home,"Desktop/",NULL);
		mkdir(vprefs->locations->desktop,S_IRWXU|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);
/*{
		GnomeVFSURI*tmp;
		if(gnome_vfs_find_directory(NULL,GNOME_VFS_DIRECTORY_KIND_DESKTOP,&tmp,TRUE,TRUE,0777)==GNOME_VFS_OK){
			VELOCITY_MSG("DESKTOP TEST(%s)",tmp->text);
			gnome_vfs_uri_unref(tmp);
		}
		if(gnome_vfs_find_directory(NULL,GNOME_VFS_DIRECTORY_KIND_TRASH,&tmp,TRUE,TRUE,0777)==GNOME_VFS_OK){
			VELOCITY_MSG("TRASH TEST(%s)",tmp->text);
			gnome_vfs_uri_unref(tmp);
		}
}*/
		
		vprefs->prefix=g_strconcat(vprefs->locations->home,".velocity/",NULL);
		mkdir(vprefs->prefix,S_IRWXU|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);
		vprefs->locations->scripts=g_strconcat(vprefs->prefix,VELOCITY_PREFERENCES_USER_SCRIPTS,NULL);
		mkdir(vprefs->locations->scripts,S_IRWXU|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);
		vprefs->locations->templates=g_strconcat(vprefs->prefix,VELOCITY_PREFERENCES_USER_TEMPLATES,NULL);
		mkdir(vprefs->locations->templates,S_IRWXU|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);
		vprefs->locations->plugins=g_strconcat(vprefs->prefix,VELOCITY_PREFERENCES_USER_PLUGINS,NULL);
		mkdir(vprefs->locations->plugins,S_IRWXU|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);
		vprefs->locations->sendto=g_strconcat(vprefs->prefix,VELOCITY_PREFERENCES_USER_SENDTO,NULL);
		mkdir(vprefs->locations->sendto,S_IRWXU|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);
	}
	
	/* Hard-coded defaults start here*/
	vprefs->locations->terminal="gnome-terminal";
	//TODO: exec_option for terminal (-x) (currently hardcoded)??

	/* Load preferences*/
	gnome_config_sync();	/*For some reason you must sync before trying to read from the config*/
	vprefs->handle_desktop=gnome_config_get_bool("/Velocity/general/handle_desktop=false");
	vprefs->sync_tree=gnome_config_get_bool("/Velocity/general/sync_tree=true");
	vprefs->selector_type=gnome_config_get_int("/Velocity/general/selector_type=3");	/*VELOCITY_ICON_SELECTION_PRELIGHT*/
	vprefs->window->size.x=gnome_config_get_int("/Velocity/window/width=660");
	vprefs->window->size.y=gnome_config_get_int("/Velocity/window/height=460");
	vprefs->window->toolbar_size=gnome_config_get_int("/Velocity/window/toolbar_size=24");
	vprefs->window->treeview_size=gnome_config_get_int("/Velocity/window/treeview_size=200");
	t=gnome_config_get_string("/Velocity/view/profile");
	velocity_preferences_view_profile_get(vprefs->view->profile,t);
	if(t)g_free(t);
	vprefs->trash->is_delete=gnome_config_get_bool("/Velocity/trash/is_delete=false");
	vprefs->trash->ask=gnome_config_get_bool("/Velocity/trash/ask=true");
	vprefs->view->dirs_first=gnome_config_get_bool("/Velocity/view/dirs_first=true");
	vprefs->view->filter=gnome_config_get_int("/Velocity/view/filter=1");
	vprefs->desktop->sorting=vprefs->view->sorting=gnome_config_get_int("/Velocity/view/sorting=1");
	t=gnome_config_get_string("/Velocity/view/background=#FFFFFF");
	gdk_color_parse(t,&vprefs->view->background);
	g_free(t);
	t=gnome_config_get_string("/Velocity/view/caption_fg=#000000");
	gdk_color_parse(t,&vprefs->view->caption_fg);
	g_free(t);
	t=gnome_config_get_string("/Velocity/view/caption_bg=#dddddd");
	gdk_color_parse(t,&vprefs->view->caption_bg);
	g_free(t);
	t=gnome_config_get_string("/Velocity/view/caption_pfg=#0000AA");
	gdk_color_parse(t,&vprefs->view->caption_pfg);
	g_free(t);
	t=gnome_config_get_string("/Velocity/view/caption_pbg=#ddddee");
	gdk_color_parse(t,&vprefs->view->caption_pbg);
	g_free(t);
	t=gnome_config_get_string("/Velocity/view/caption_hfg=#4444FF");
	gdk_color_parse(t,&vprefs->view->caption_hfg);
	g_free(t);
	t=gnome_config_get_string("/Velocity/view/caption_hbg=#ddddff");
	gdk_color_parse(t,&vprefs->view->caption_hbg);
	g_free(t);
	vprefs->desktop->tooltip_delay=vprefs->view->tooltip_delay=gnome_config_get_int("/Velocity/view/tooltip_delay=2500");
	vprefs->desktop->animation_mode=vprefs->view->animation_mode=gnome_config_get_int("/Velocity/view/animation_mode=2");
	vprefs->desktop->single_click=vprefs->view->single_click=gnome_config_get_bool("/Velocity/view/single_click=false");
	vprefs->desktop->preview_images=vprefs->view->preview_images=gnome_config_get_bool("/Velocity/view/preview_images=true");
	vprefs->view->font_str=gnome_config_get_string("/Velocity/view/font_str=Sans 8");
	vprefs->view->font=pango_font_description_from_string(vprefs->view->font_str);
	vprefs->view->dirs_in_same_win=gnome_config_get_bool("/Velocity/view/dirs_in_same_win=true");
	vprefs->desktop->dirs_first=TRUE;	//TODO:  This won't even matter once you get icon position saving done
	vprefs->desktop->filter=VELOCITY_VFS_FILTER_NONE;
	vprefs->desktop->can_open=TRUE;
	vprefs->desktop->can_drag=TRUE;
	vprefs->desktop->can_lasso=TRUE;
	vprefs->desktop->dirs_in_same_win=FALSE;
	vprefs->desktop->font_str=gnome_config_get_string("/Velocity/desktop/font_str=Sans 10");
	vprefs->desktop->font=pango_font_description_from_string(vprefs->desktop->font_str);
	t=gnome_config_get_string("/Velocity/desktop/profile");
	velocity_preferences_view_profile_get(vprefs->desktop->profile,t);
	if(t)g_free(t);
	t=gnome_config_get_string("/Velocity/desktop/background=#000000");
	gdk_color_parse(t,&vprefs->desktop->background);
	g_free(t);
	t=gnome_config_get_string("/Velocity/desktop/caption_fg=#ffffff");
	gdk_color_parse(t,&vprefs->desktop->caption_fg);
	g_free(t);
	t=gnome_config_get_string("/Velocity/desktop/caption_bg=#444444");
	gdk_color_parse(t,&vprefs->desktop->caption_bg);
	g_free(t);
	t=gnome_config_get_string("/Velocity/desktop/caption_pfg=#aaccee");
	gdk_color_parse(t,&vprefs->desktop->caption_pfg);
	g_free(t);
	t=gnome_config_get_string("/Velocity/desktop/caption_pbg=#222244");
	gdk_color_parse(t,&vprefs->desktop->caption_pbg);
	g_free(t);
	t=gnome_config_get_string("/Velocity/desktop/caption_hfg=#cceeff");
	gdk_color_parse(t,&vprefs->desktop->caption_hfg);
	g_free(t);
	t=gnome_config_get_string("/Velocity/desktop/caption_hbg=#222266");
	gdk_color_parse(t,&vprefs->desktop->caption_hbg);
	g_free(t);
	t=gnome_config_get_string(VELOCITY_PREFERENCES_VIEW_PROFILE_CONFIG "profile_list=Icons,List,Desktop");
{
	char**vl=g_strsplit(t,",",0);
	guint i=0;
	while(vl[i]){
		velocity_view_profile*p=g_new(velocity_view_profile,1);
		velocity_preferences_view_profile_get(p,vl[i]);
		VELOCITY_MSG("view profile (%s) added",p->name);
		velocity_view_profile_list_add(p);
		i++;
	}
	g_strfreev(vl);
}
	g_free(t);
}
GList*velocity_view_profile_list(){
	return(velocity_view_profiles);
}
velocity_view_profile*velocity_view_profile_list_new(const char*name,GtkAnchorType draw_start,GtkAnchorType draw_direction,
	GtkAnchorType caption_placement,velocity_view_caption_type caption_type,gboolean caption_wrap,
	GdkPoint icon_size,GdkPoint icon_spacing){
	velocity_view_profile*p=g_new(velocity_view_profile,1);
	p->name=g_strdup(name);
	p->draw_start=draw_start;
	p->draw_direction=draw_direction;
	p->caption_placement=caption_placement;
	p->caption_type=caption_type;
	p->caption_wrap=caption_wrap;
	p->icon_size.x=icon_size.x;
	p->icon_size.y=icon_size.y;
	p->icon_spacing.x=icon_spacing.x;
	p->icon_spacing.y=icon_spacing.y;
	velocity_view_profile_list_add(p);
	return(p);
}
void velocity_view_profile_list_add(velocity_view_profile*profile){
	if(profile)velocity_view_profiles=g_list_append(velocity_view_profiles,(gpointer)profile);
}
void velocity_view_profile_free(velocity_view_profile*profile){
	if(profile){
		if(profile->name)g_free(profile->name);
		g_free(profile);
	}
}
void velocity_view_profile_list_remove(velocity_view_profile*profile){
	if((velocity_view_profiles)&&(profile)){
		velocity_view_profiles=g_list_remove(velocity_view_profiles,(gconstpointer)profile);
		velocity_view_profile_free(profile);
	}
}
void velocity_preferences_view_profile_set(velocity_view_profile*profile){
	if((profile)&&(profile->name)){
		char*cs=g_strdup_printf(VELOCITY_PREFERENCES_VIEW_PROFILE_CONFIG "%s",profile->name);
		char*pcs=g_strdup_printf("%i,%i,%i,%i,%i,%i,%i,%i,%i",
			profile->draw_start,profile->draw_direction,
			profile->caption_placement,profile->caption_type,profile->caption_wrap,
			profile->icon_size.x,profile->icon_size.y,
			profile->icon_spacing.x,profile->icon_spacing.y);
		gnome_config_set_string(cs,pcs);
		gnome_config_sync();
		g_free(pcs);
		g_free(cs);
	}
}
void velocity_preferences_view_profile_get(velocity_view_profile*profile,const char*profile_name){
	char*cs=(profile_name!=NULL)?g_strdup_printf(VELOCITY_PREFERENCES_VIEW_PROFILE_CONFIG "%s",profile_name):NULL;
	char*pp=(cs!=NULL)?gnome_config_get_string(cs):NULL;
	if(pp){
		char**pv=g_strsplit(pp,",",9);
		profile->name=g_strdup(profile_name);
		profile->draw_start=atoi(pv[0]);
		profile->draw_direction=atoi(pv[1]);
		profile->caption_placement=atoi(pv[2]);
		profile->caption_type=atoi(pv[3]);
		profile->caption_wrap=atoi(pv[4]);
		profile->icon_size.x=atoi(pv[5]);
		profile->icon_size.y=atoi(pv[6]);
		profile->icon_spacing.x=atoi(pv[7]);
		profile->icon_spacing.y=atoi(pv[8]);
		g_strfreev(pv);
		g_free(pp);
	}else{	/*On invalid profile load (or none exist) we just recreate/set default*/
		profile->name=g_strdup("Desktop");
		profile->draw_start=GTK_ANCHOR_NW;
		profile->draw_direction=GTK_ANCHOR_SOUTH;
		profile->caption_placement=GTK_ANCHOR_SOUTH;
		profile->caption_type=VELOCITY_VIEW_CAPTION_NAME;
		profile->caption_wrap=TRUE;
		profile->icon_size.x=64;
		profile->icon_size.y=64;
		profile->icon_spacing.x=120;
		profile->icon_spacing.y=142;
		velocity_preferences_view_profile_set(profile);
		profile->name=g_strdup("List");
		profile->draw_start=GTK_ANCHOR_NW;
		profile->draw_direction=GTK_ANCHOR_SOUTH;
		profile->caption_placement=GTK_ANCHOR_EAST;
		profile->caption_type=VELOCITY_VIEW_CAPTION_NAME;
		profile->caption_wrap=FALSE;
		profile->icon_size.x=24;
		profile->icon_size.y=24;
		profile->icon_spacing.x=160;
		profile->icon_spacing.y=25;
		velocity_preferences_view_profile_set(profile);
		profile->name=g_strdup("Icons");
		profile->draw_start=GTK_ANCHOR_NW;
		profile->draw_direction=GTK_ANCHOR_EAST;
		profile->caption_placement=GTK_ANCHOR_SOUTH;
		profile->caption_type=VELOCITY_VIEW_CAPTION_NAME;
		profile->caption_wrap=TRUE;
		profile->icon_size.x=64;
		profile->icon_size.y=64;
		profile->icon_spacing.x=80;
		profile->icon_spacing.y=96;
		velocity_preferences_view_profile_set(profile);
	}
	if(cs)g_free(cs);
}
/*sync to config file (in vprefs struct)*/
void velocity_preferences_sync(){
	char*t=g_strdup_printf("#%06x",(guint)(gdk_color_to_rgba(vprefs->view->background,0)>>8));
	gnome_config_set_bool("/Velocity/general/handle_desktop",vprefs->handle_desktop);
	gnome_config_set_int("/Velocity/general/selector_type",vprefs->selector_type);
	gnome_config_set_int("/Velocity/window/width",vprefs->window->size.x);
	gnome_config_set_int("/Velocity/window/height",vprefs->window->size.y);
	gnome_config_set_int("/Velocity/window/toolbar_size",vprefs->window->toolbar_size);
	gnome_config_set_int("/Velocity/window/treeview_size",vprefs->window->treeview_size);
	gnome_config_set_bool("/Velocity/trash/is_delete",vprefs->trash->is_delete);
	gnome_config_set_bool("/Velocity/trash/ask",vprefs->trash->ask);
	gnome_config_set_string("/Velocity/view/profile",vprefs->view->profile->name);
	velocity_preferences_view_profile_set(vprefs->view->profile);
	gnome_config_set_bool("/Velocity/view/dirs_first",vprefs->view->dirs_first);
	gnome_config_set_int("/Velocity/view/filter",vprefs->view->filter);
	gnome_config_set_int("/Velocity/view/sorting",vprefs->view->sorting);
	gnome_config_set_string("/Velocity/view/background",t);
	g_free(t);
	t=g_strdup_printf("#%06x",(guint)(gdk_color_to_rgba(vprefs->view->caption_fg,0)>>8));
	gnome_config_set_string("/Velocity/view/caption_fg",t);
	g_free(t);
	t=g_strdup_printf("#%06x",(guint)(gdk_color_to_rgba(vprefs->view->caption_bg,0)>>8));
	gnome_config_set_string("/Velocity/view/caption_bg",t);
	g_free(t);
	t=g_strdup_printf("#%06x",(guint)(gdk_color_to_rgba(vprefs->view->caption_pfg,0)>>8));
	gnome_config_set_string("/Velocity/view/caption_pfg",t);
	g_free(t);
	t=g_strdup_printf("#%06x",(guint)(gdk_color_to_rgba(vprefs->view->caption_pbg,0)>>8));
	gnome_config_set_string("/Velocity/view/caption_pbg",t);
	g_free(t);
	t=g_strdup_printf("#%06x",(guint)(gdk_color_to_rgba(vprefs->view->caption_hfg,0)>>8));
	gnome_config_set_string("/Velocity/view/caption_hfg",t);
	g_free(t);
	t=g_strdup_printf("#%06x",(guint)(gdk_color_to_rgba(vprefs->view->caption_hbg,0)>>8));
	gnome_config_set_string("/Velocity/view/caption_hbg",t);
	g_free(t);
	gnome_config_set_string("/Velocity/view/font_str",vprefs->view->font_str);
	gnome_config_set_int("/Velocity/view/tooltip_delay",vprefs->view->tooltip_delay);
	gnome_config_set_int("/Velocity/view/animation_mode",vprefs->view->animation_mode);
	gnome_config_set_bool("/Velocity/view/dirs_in_same_win",vprefs->view->dirs_in_same_win);
	gnome_config_set_bool("/Velocity/view/single_click",vprefs->view->single_click);
	gnome_config_set_bool("/Velocity/view/preview_images",vprefs->view->preview_images);
	gnome_config_set_string("/Velocity/desktop/profile",vprefs->desktop->profile->name);
	velocity_preferences_view_profile_set(vprefs->desktop->profile);
	gnome_config_set_string("/Velocity/desktop/font_str",vprefs->desktop->font_str);
	t=g_strdup_printf("#%06x",(guint)(gdk_color_to_rgba(vprefs->desktop->background,0)>>8));
	gnome_config_set_string("/Velocity/desktop/background",t);
	g_free(t);
	t=g_strdup_printf("#%06x",(guint)(gdk_color_to_rgba(vprefs->desktop->caption_fg,0)>>8));
	gnome_config_set_string("/Velocity/desktop/caption_fg",t);
	g_free(t);
	t=g_strdup_printf("#%06x",(guint)(gdk_color_to_rgba(vprefs->desktop->caption_bg,0)>>8));
	gnome_config_set_string("/Velocity/desktop/caption_bg",t);
	g_free(t);
	t=g_strdup_printf("#%06x",(guint)(gdk_color_to_rgba(vprefs->desktop->caption_pfg,0)>>8));
	gnome_config_set_string("/Velocity/desktop/caption_pfg",t);
	g_free(t);
	t=g_strdup_printf("#%06x",(guint)(gdk_color_to_rgba(vprefs->desktop->caption_pbg,0)>>8));
	gnome_config_set_string("/Velocity/desktop/caption_pbg",t);
	g_free(t);
	t=g_strdup_printf("#%06x",(guint)(gdk_color_to_rgba(vprefs->desktop->caption_hfg,0)>>8));
	gnome_config_set_string("/Velocity/desktop/caption_hfg",t);
	g_free(t);
	t=g_strdup_printf("#%06x",(guint)(gdk_color_to_rgba(vprefs->desktop->caption_hbg,0)>>8));
	gnome_config_set_string("/Velocity/desktop/caption_hbg",t);
	g_free(t);
	{
		char*l=NULL;
		GList*n=velocity_view_profiles;
		while(n){
			if(n->data){
				velocity_view_profile*p=(velocity_view_profile*)n->data;
				velocity_preferences_view_profile_set(p);
				if(l){
					char*t=l;
					l=g_strconcat(t,",",p->name,NULL);
					g_free(t);
				}else{
					l=g_strdup(p->name);
				}
			}
			n=n->next;
		}
		if(l){
			gnome_config_set_string(VELOCITY_PREFERENCES_VIEW_PROFILE_CONFIG "profile_list",l);
			g_free(l);
		}
	}
	gnome_config_sync();
}

void velocity_preferences_shutdown(){
	if(vprefs){
		velocity_preferences_sync();
		g_free(vprefs->plugin);
		g_free(vprefs->disk_usage);
		g_free(vprefs->trash);
		g_free(vprefs->view);
		g_free(vprefs->locations);
		g_free(vprefs);
		vprefs=NULL;
	}
}
