/*
 * Velocity - A GNOME File Manager
 *
 *
 * Copyright (C) (2002) (Kyle Davis) <kyled@blueriver.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef VELOCITY_PREFERENCES_H
#define VELOCITY_PREFERENCES_H

#include <glib.h>
#include "velocity-sys.h"
#include "velocity-icon.h"
#include "velocity-vfs.h"

typedef struct{
	GdkPoint size;	/*For saving width and height of our window*/
	//TODO:  Save gutter
	//TODO:  Save toolbar/statusbar/locationbar
	//TODO:  Save icon view
	GtkToolbarStyle toolbar_style;
	guint toolbar_size;
	guint treeview_size;
}velocity_window_preferences;

typedef enum{VELOCITY_VIEW_SORT_NONE=0,
	VELOCITY_VIEW_SORT_NAME,
	VELOCITY_VIEW_SORT_TYPE,
	VELOCITY_VIEW_SORT_SIZE,
	VELOCITY_VIEW_SORT_ACCESSED,
	VELOCITY_VIEW_SORT_MODIFIED,
	VELOCITY_VIEW_SORT_CREATED,
	VELOCITY_VIEW_SORT_NUM}velocity_view_sort_method;

typedef enum{VELOCITY_VIEW_CAPTION_INVALID=NULL,
	VELOCITY_VIEW_CAPTION_NAME,
	VELOCITY_VIEW_CAPTION_NAME_SIZE,
	VELOCITY_VIEW_CAPTION_NAME_TYPE,
	VELOCITY_VIEW_CAPTION_NAME_TYPE_SIZE,
	VELOCITY_VIEW_CAPTION_NAME_SIZE_TYPE,
	VELOCITY_VIEW_CAPTION_NUM}velocity_view_caption_type;

typedef struct{
	char*name;
	GtkAnchorType draw_start;	/*Must be a corner*/
	GtkAnchorType draw_direction;	/*Must NOT be a corner*/
	GtkAnchorType caption_placement;	/*Must NOT be a corner*/	//TODO:  Finish this...only East & South are done
	velocity_view_caption_type caption_type;
	gboolean caption_wrap;
	GdkPoint icon_size;
	GdkPoint icon_spacing;
}velocity_view_profile;

typedef struct{
	gboolean dirs_first;
	velocity_vfs_filter_flags filter;	/*Used for filtering out Hidden and Backup files*/
	velocity_view_sort_method sorting; /*the sorthing method to be used*/
	/*View Profile*/
	velocity_view_profile*profile;
	/*Font stuff*/
	gchar*font_str;
	PangoFontDescription*font;
	/*Misc. preferences*/
	guint tooltip_delay;	/*Delay (in ms) for tooltip.  0 means off _not_ instantaneous.*/
	GdkColor background;	/*Background color*/
	GdkColor caption_fg;	/*normal caption foreground*/
	GdkColor caption_bg;	/*normal caption background*/
	GdkColor caption_pfg;	/*prelit caption foreground*/
	GdkColor caption_pbg;	/*prelit caption background*/
	GdkColor caption_hfg;	/*highlighted caption foreground*/
	GdkColor caption_hbg;	/*highlighted caption background*/
	velocity_icon_animation_mode animation_mode;
	gboolean dirs_in_same_win;	/*If opening a directory should cause an open new window request*/
	gboolean single_click;	/*If we should use single clicks to open rather than double clicks*/
	gboolean can_open;	/*If the view is allowed to try to open things on double click (single click if the above is set)*/
	gboolean can_drag;	/*If files can be dragged around*/
	gboolean can_lasso;	/*If the lasso can be used*/
	gboolean can_cancel;	/*If file loading can be cancelled via the Escape key*/
	gboolean preview_images;
	gboolean preview_animations;
	int max_animation_size;	//TODO:  ALL SIZES ARE IN KB
	int max_image_size;	//TODO:  make these work....
//	int cache_size;
//	int cache_to_disk_size;
}velocity_view_preferences;

#define VELOCITY_PREFERENCES_USER_SCRIPTS	"scripts/"
#define VELOCITY_PREFERENCES_USER_TEMPLATES	"templates/"
#define VELOCITY_PREFERENCES_USER_PLUGINS	"plugins/"
#define VELOCITY_PREFERENCES_USER_SENDTO "sendto/"

typedef struct{
	char*home;
	char*trash;
	char*desktop;
	char*terminal;
	/*These are set by the velocity_preferences_init() function and used all throughout*/
	char*config;
	char*scripts;
	char*templates;
	char*plugins;
	char*sendto;
}velocity_locations_preferences;

typedef struct{
	gboolean is_delete; /*some people hate the trash idea....*/
	gboolean ask;	/*Use "Are you sure..." dialog*/
	gulong maxsize;
}velocity_trash_preferences;

typedef struct{ /*memory usage too...*/
	gulong min_free_space;
	gulong cache_size;
	gulong cache_to_disk_size;
}velocity_disk_usage_preferences;

typedef struct{
	/*TODO:  should this be a list of enabled or a list of disabled? (all on or off by default?)*/
	GList*enabled; /*this will be pointers to plugin/module structures*/
}velocity_plugin_preferences;

typedef enum{VELOCITY_EXT_NULL=NULL,
	VELOCITY_EXT_TEMPLATE,
	VELOCITY_EXT_SCRIPT,
	VELOCITY_EXT_MODULE}velocity_ext_type;
typedef struct{
	int stats; /* TODO:  When adding to stats, if we reach the LIMIT (set later) then divide/bitshift all things as to keep the same "proportions" */
	velocity_ext_type type;
	char*filename; /* we will later compare using both the filename and type to be sure that we have the correct one */
}velocity_ext_usage_obj;

typedef struct{
	char*prefix; /*URI path to prefs directory*/
	gboolean handle_desktop;
	gboolean sync_tree; /*if velocity_tree selection is changed when we go somewhere....*/
	gboolean menu_stats; /*stats on which items in the dynamic menus (open with/scripts/plugin stuff) are used most.  used for reordering them by usage.*/

	velocity_icon_selector_type selector_type;	/*Selector type for the view's, global for simplicity*/
	velocity_locations_preferences*locations;
	velocity_window_preferences*window;
	velocity_view_preferences*view;
	velocity_view_preferences*desktop;
	velocity_trash_preferences*trash;
	velocity_disk_usage_preferences*disk_usage;
	velocity_plugin_preferences*plugin;
	GList*ext_usage_stats; /* GList of velocity_ext_usage_obj's */
	/*	this will let us keep an list of "Open With","Scripts","Plugins" and anything else for the user's convienence of which they use the most often*/
}velocity_preferences;

extern velocity_preferences*vprefs;

/*FUNCTIONS*/
void velocity_preferences_init(void);	/*Start preferences subsystem*/
GList*velocity_view_profile_list();
velocity_view_profile*velocity_view_profile_list_new(const char*name,GtkAnchorType draw_start,GtkAnchorType draw_direction,
	GtkAnchorType caption_placement,velocity_view_caption_type caption_type,gboolean caption_wrap,
	GdkPoint icon_size,GdkPoint icon_spacing);
void velocity_view_profile_list_add(velocity_view_profile*profile);
void velocity_view_profile_free(velocity_view_profile*profile);
void velocity_view_profile_list_remove(velocity_view_profile*profile);
void velocity_preferences_view_profile_get(velocity_view_profile*profile,const char*profile_name);
void velocity_preferences_view_profile_set(velocity_view_profile*profile);
void velocity_preferences_sync(void);	/*Forcibly sync preferences struct to hard drive*/
void velocity_preferences_shutdown(void);	/*Shutdown preferences subsystem*/
#endif
