/*
 * Velocity - A GNOME File Manager
 *
 *
 * Copyright (C) (2002) (Kyle Davis) <kyled@blueriver.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "velocity-sys.h"
#include "velocity-script.h"

velocity_script*velocity_script_new_from_file(const char*script){
	int a=0;
	char**tmp=NULL;
	velocity_script*vscript=g_new0(velocity_script,1);
	g_assert(vscript!=NULL);
	vscript->refcount=1;
	vscript->file=g_strdup(script);
	tmp=script_get_info_array(script,VELOCITY_SCRIPT_GET_INFO);
	while(tmp[a]){
		switch(a){
			case 0: /*version*/
				vscript->version=g_strdup(tmp[a]);
				break;
			case 1: /*name*/
				vscript->name=g_strdup(tmp[a]);
				break;
			case 2: /*authors*/
				vscript->authors=g_strsplit(tmp[a],",",0);
				break;
			case 3: /*about*/
				vscript->about=g_strdup(tmp[a]);
				break;
			case 4: /*mime_types*/
				vscript->mime_types=g_strsplit(tmp[a],",",0);
				break;
         case 5: /*dependancies*/
            vscript->dependancies=g_strsplit(tmp[a],",",0);
				vscript->checked_dependancies=velocity_dependancy_vector_check(vscript->dependancies,&vscript->have_dependancies);
            break;
			case 6: /*needs_terminal*/
				if((tmp[a])&&(strlen(tmp[a])>0)){
					vscript->needs_terminal=TRUE;
				}else{
					vscript->needs_terminal=FALSE;
				}
				break;
			case 7: /*multiple_files*/
				if((tmp[a])&&(strlen(tmp[a])>0)){
					vscript->multiple_files=TRUE;
				}else{
					vscript->multiple_files=FALSE;
				}
				break;
			case 8: /*local_only*/
				if((tmp[a])&&(strlen(tmp[a])>0)){
					vscript->local_only=TRUE;
				}else{
					vscript->local_only=FALSE;
				}
				break;
		}
		a++;
	}
	g_strfreev(tmp);
	return(vscript);
}

velocity_script*velocity_script_dup(velocity_script*vscript){
	vscript->refcount++;
	return(vscript);
}

void velocity_script_free(velocity_script*vscript){
	if(vscript){
		vscript->refcount--;
		if(vscript->refcount==0){
			if(vscript->file)g_free(vscript->file);
			if(vscript->name)g_free(vscript->name);
			if(vscript->version)g_free(vscript->version);
			if(vscript->authors)g_strfreev(vscript->authors);
			if(vscript->about)g_free(vscript->about);
			if(vscript->mime_types)g_strfreev(vscript->mime_types);
			g_free(vscript);
		}
	}
}
