/*
 * Velocity - A GNOME File Manager
 *
 *
 * Copyright (C) (2002) (Kyle Davis) <kyled@blueriver.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

/*
 *  This contains most of the system related functions
 */

#include <sys/stat.h>
#include <glib.h>

#include "velocity-sys.h"

gchar*get_command_output(const gchar*command){	//TODO:  Probably need to do more error checking
	gchar*stdo;
	if(g_spawn_command_line_sync(command,&stdo,NULL,NULL,NULL))return(stdo);
	return(NULL);
}
gboolean get_str_bool(const char*str){
	if(strcasecmp(str,"yes")==0)return(TRUE);
	if(strcasecmp(str,"true")==0)return(TRUE);
	if(strcasecmp(str,"on")==0)return(TRUE);
	if(atoi(str)==1)return(TRUE);
	if(strcasecmp(str,"no")==0)return(FALSE);
	if(strcasecmp(str,"false")==0)return(FALSE);
	if(strcasecmp(str,"off")==0)return(FALSE);
	return(FALSE);
}
int get_str_ranged_int(const char*str,const int lowend,const int highend,const int failsafe){
	int ret=atoi(str);
	if((ret<lowend)||(ret>highend)){
		ret=failsafe;
	}
	return(ret);
}
char*script_get_info_var(const char*script,const char*opt){
	char*cmd=g_strdup_printf("\"%s\" \"%s\"",script,opt);
	char*retval=get_command_output(cmd);
	g_free(cmd);
	if(retval){
		if(strlen(retval)==0){
			g_free(retval);
			retval=NULL;
		}
	}
	return(retval);
}
char**script_get_info_array(const char*script,const char*opt){
	char*cmd=g_strdup_printf("\"%s\" \"%s\"",script,opt);
	char*buf=get_command_output(cmd);
	char**retval=NULL;
	if(buf){
		retval=g_strsplit(buf,"\n",0);
	}else{
		retval=NULL;
	}
	g_free(buf);
	g_free(cmd);
	return(retval);
}
gboolean script_get_info_bool(const char*script,const char*opt){
	char*ret=script_get_info_var(script,opt);
	if(!ret) return(FALSE);
	g_free(ret);
	return(TRUE);
}
gulong gdk_color_to_rgba(GdkColor color,guchar alpha){
	guchar r=color.red>>8;
	guchar g=color.green>>8;
	guchar b=color.blue>>8;
	return((r<<24)|(g<<16)|(b<<8)|(alpha));
}
gboolean velocity_mime_type_vector_check(gchar**mime_types,const gchar*mime_type){
	/* MIME-TYPE LIST SPECIFICATION
	 * 
	 * If "internal/all" is found as the first item in the list then all following MIME types
	 * represent types that should be excluded (thereby inversing the list from being a list of
	 * supported MIME types to being a list of unsupported MIME types)
	 *
	 * Super-types are also supported and honored.
	 */
	gboolean inverse=FALSE;
	guint i=0;
	if((mime_types[0])&&(strcmp(mime_types[0],"internal/all")==0)){
		inverse=TRUE;
		i++;
	}
	while(mime_types[i]){
		if(strncmp(mime_types[i],mime_type,strlen(mime_types[i]))==0)return(TRUE^inverse);
		i++;
	}
	return(FALSE^inverse);
}
gchar**velocity_dependancy_vector_check(gchar**dependancies,gboolean*passed){
	/* Given a vector of program names to search for in the path this will check which ones can
	 * are found on the local system.  Also in the list '|' can be used meaning OR between
	 * program names.
	 * 
	 * A vector will be returned with the full locations of said programs if they are found.
	 * If they are not found the program name will be replaced with VELOCITY_DEPENDANCY_VECTOR_ERROR.
	 *
	 * Example:
	 *		"swriter|abiword|gedit|gvim","basename","perl","BOGUS",NULL
	 *			will return something like this:  "/usr/bin/abiword","/usr/bin/basename","/usr/bin/perl",VELOCITY_DEPENDANCY_VECTOR_ERROR,NULL
	 */
	gchar**ret=NULL;
	gchar*cdeps=NULL; /*checked deps list to be return (once in array form)*/
	guint i=0;	/*Main dependancy array iterator*/
	*passed=TRUE;
	while(dependancies[i]){
		gchar**sdeps=g_strsplit(dependancies[i],"|",0);	/*Sub dependancies (a succesful one of these will satisfy this dependancy)*/
		guint si=0;
		char*r=NULL;
		while((sdeps[si])&&(!r)){
			/*a slight hack, we could use GnomeVFS, but we don't want the dependancy here, so this should work for now*/
			r=g_find_program_in_path(sdeps[si]);
			si++;
		}
		/*This dependancy failed*/
		if(!r){
			r=g_strdup(VELOCITY_DEPENDANCY_VECTOR_ERROR);
			*passed=FALSE;
		}
		if(cdeps){
			cdeps=g_strconcat(cdeps,",",g_strdup(r),NULL);
		}else{
			cdeps=g_strdup(r);
		}
		g_free(r);
		i++;
	}
	ret=g_strsplit(cdeps,",",0);
	g_free(cdeps);
	return(ret);
}
