/*
 * Velocity - A GNOME File Manager
 *
 *
 * Copyright (C) (2002) (Kyle Davis) <kyled@blueriver.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef VELOCITY_SYS_H
#define VELOCITY_SYS_H

#include <sys/types.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <glib.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>

#define VELOCITY_ERR(...)	fprintf(stderr,G_STRLOC ":  "); fprintf(stderr,__VA_ARGS__); fprintf(stderr,"\n");
#ifdef VELOCITY_DEBUG
#define VELOCITY_MSG(...)	VELOCITY_ERR(__VA_ARGS__)
#else
#define VELOCITY_MSG(...)
#endif

#define VELOCITY_DEPENDANCY_VECTOR_ERROR "VELOCITY_DEPENDANCY_VECTOR_ERROR"

/*convenience functions that are nice for certian situations*/
gchar*get_command_output(const gchar*command);


/*Core module, script, and template support functions*/

/* MIME-TYPE LIST SPECIFICATION
 * 
 * If "internal/all" is found as the first item in the list then all following MIME types
 * represent types that should be excluded (thereby inversing the list from being a list of
 * supported MIME types to being a list of unsupported MIME types)
 *
 * Super-types are also supported and honored.
 */
gboolean velocity_mime_type_vector_check(gchar**mime_types,const gchar*mime_type);
/* Given a vector of program names to search for in the path this will check which ones can
 * are found on the local system.  Also in the list '|' can be used meaning OR between
 * program names.
 * 
 * A vector will be returned with the full locations of said programs if they are found.
 * If they are not found the program name will be replaced with VELOCITY_DEPENDANCY_VECTOR_ERROR.
 *
 * Example:
 *		"swriter|abiword|gedit|gvim","basename","perl","BOGUS",NULL
 *			will return something like this:  "/usr/bin/abiword","/usr/bin/basename","/usr/bin/perl",VELOCITY_DEPENDANCY_VECTOR_ERROR,NULL
 */
gchar**velocity_dependancy_vector_check(gchar**dependancies,gboolean*passed);


/*these were mainly to make the code nicer...*/
gboolean get_str_bool(const char*str);
int get_str_ranged_int(const char*str,const int lowend,const int highend,const int failsafe);
char*script_get_info_var(const char*script,const char*opt);
char**script_get_info_array(const char*script,const char*opt);
gboolean script_get_info_bool(const char*script,const char*opt);
gulong gdk_color_to_rgba(GdkColor color,guchar alpha);

#endif
