/*
 * Velocity - A GNOME File Manager
 *
 *
 * Copyright (C) (2002) (Kyle Davis) <kyled@blueriver.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <string.h>
#include <glib.h>
#include "velocity-template.h"
#include "velocity-sys.h"

velocity_template*velocity_template_new_from_file(const gchar*script){
	int a=0;
	char**tmp=NULL;
	velocity_template*vtemplate=g_new0(velocity_template,1);
	vtemplate->refcount=1;
	vtemplate->file=g_strdup(script);
	tmp=script_get_info_array(script,VELOCITY_TEMPLATE_GET_INFO);
	while(tmp[a]){
		switch(a){  /* handle which line is which */
			case 0: /*version*/
				vtemplate->version=g_strdup(tmp[a]);
				break;
			case 1: /*name*/
				vtemplate->name=g_strdup(tmp[a]);
				break;
			case 2: /*authors*/
				vtemplate->authors=g_strsplit(tmp[a],",",0);
				break;
			case 3: /*about*/
				vtemplate->about=g_strdup(tmp[a]);
				break;
			case 4: /*local_only*/
				if(strlen(tmp[a])>0){
					vtemplate->local_only=TRUE;
				}else{
					vtemplate->local_only=FALSE;
				}
				break;
			case 5: /*mime_type*/
				vtemplate->mime_type=g_strdup(tmp[a]);
				break;
			case 6: /*dependancies*/
				vtemplate->dependancies=g_strsplit(tmp[a],",",0);
				vtemplate->checked_dependancies=velocity_dependancy_vector_check(vtemplate->dependancies,&vtemplate->have_dependancies);
				break;
			case 7: /*default desired name*/
				vtemplate->desired=g_strdup(tmp[a]);
				break;
		}
		a++;
	}
	g_strfreev(tmp);
	return(vtemplate);
}
velocity_template*velocity_template_new(const gchar*name,const gchar*version,const gchar*mime_type,const gchar*desired,const gchar*about,gboolean local_only,velocity_template_activate_cb activate){
	velocity_template*vtemplate=g_new0(velocity_template,1);
	vtemplate->refcount=1;
	vtemplate->name=g_strdup(name);
	vtemplate->version=g_strdup(version);
	vtemplate->mime_type=g_strdup(mime_type);
	vtemplate->desired=g_strdup(desired);
	vtemplate->about=g_strdup(about);
	vtemplate->local_only=local_only;
	vtemplate->activate=activate;
	return(vtemplate);
}
velocity_template*velocity_template_dup(velocity_template*vtemplate){
	vtemplate->refcount++;
	return(vtemplate);
}
void velocity_template_free(velocity_template*vtemplate){
	if(vtemplate){
		vtemplate->refcount--;
		if(vtemplate->refcount==0){
			if(vtemplate->file)g_free(vtemplate->file);
			if(vtemplate->name)g_free(vtemplate->name);
			if(vtemplate->version)g_free(vtemplate->version);
			if(vtemplate->authors)g_strfreev(vtemplate->authors);
			if(vtemplate->about)g_free(vtemplate->about);
			if(vtemplate->dependancies)g_strfreev(vtemplate->dependancies);
			if(vtemplate->checked_dependancies)g_strfreev(vtemplate->checked_dependancies);
			if(vtemplate->mime_type)g_free(vtemplate->mime_type);
			if(vtemplate->desired)g_free(vtemplate->desired);
			g_free(vtemplate);
		}
	}
}
