/*
 * Velocity - A GNOME File Manager
 *
 *
 * Copyright (C) (2002) (Kyle Davis) <kyled@blueriver.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef VELOCITY_TEMPLATE_H
#define VELOCITY_TEMPLATE_H

#include <glib.h>

#define VELOCITY_TEMPLATE_GET_INFO	"--get-info"
#define VELOCITY_TEMPLATE_FLAG_OVERIDE_DESIRED	"--overide-desired"
#define VELOCITY_TEMPLATE_FLAG_EXECUTE "--execute"

/*This is for static templates.  Return TRUE and create file if uri/name are valid, FALSE if not*/
typedef gboolean(*velocity_template_activate_cb)(const gchar*uri,const gchar*name);
typedef struct{
	gchar*file; /*full path to our actual template, NULL denotes a static module*/
	gchar*name;	/*name of template*/
	gchar*version;
	gchar**authors;
	gchar*about;  /*this can be multilined, but we don't need to break it into a vector...*/
	gchar*mime_type;
	gchar**dependancies;
	gchar**checked_dependancies;	/*We'll keep the checked version here as well...*/
	gboolean have_dependancies;
	gchar*desired;	/*desired filename from template*/
	gboolean local_only;
	velocity_template_activate_cb activate;	/*For static templates only*/
	gpointer user_data;
	guint refcount;
}velocity_template;

velocity_template*velocity_template_new_from_file(const gchar*script);
velocity_template*velocity_template_new(const gchar*name,const gchar*version,const gchar*mime_type,const gchar*desired,const gchar*about,gboolean local_only,velocity_template_activate_cb activate);
velocity_template*velocity_template_dup(velocity_template*vtemplate);
void velocity_template_free(velocity_template*vtemplate);

#endif
