/*
 * Velocity - A GNOME File Manager
 *
 *
 * Copyright (C) (2002) (Kyle Davis) <kyled@blueriver.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <config.h>
#include <libgnomeui/gnome-vfs-util.h>
#include <libgnomeui/gnome-thumbnail.h>

#include "velocity-sys.h"
#include "velocity-thumbnail.h"
#include "velocity-module.h"
#include "velocity-factories.h"
#include "velocity-windowman.h"
#include "velocity-icons.h"

#define VELOCITY_THUMBNAIL_SIZE	GNOME_THUMBNAIL_SIZE_NORMAL

static GList*velocity_thumbnails=NULL;
static GList*velocity_thumbnails_queue=NULL;
static guint velocity_thumbnails_count=0;
static guint velocity_thumbnails_queue_count=0;
static GnomeThumbnailFactory*gtf=NULL;

static void velocity_thumbnail_init(){
	gtf=gnome_thumbnail_factory_new(VELOCITY_THUMBNAIL_SIZE);	
}
static void velocity_async_get_image_info_cb(GdkPixbufLoader*loader,gint width,gint height,velocity_image_info*info){
	info->width=width;
	info->height=height;
}
static void velocity_async_get_image_info_closed_cb(GdkPixbufLoader*loader,velocity_image_info*info){
}
static velocity_image_info*velocity_async_get_image_info(const char*uri){
	velocity_image_info*info=g_new(velocity_image_info,1);
	char*turi=gnome_thumbnail_path_for_uri(uri,VELOCITY_THUMBNAIL_SIZE);
	char*iuri=g_strconcat(turi,".info",NULL);
	char*wpath=g_strconcat("=",iuri,"=/Info/width",NULL);
	char*hpath=g_strconcat("=",iuri,"=/Info/height",NULL);
	if(g_file_test(iuri,G_FILE_TEST_EXISTS)){
		info->width=gnome_config_get_int(wpath);
		info->height=gnome_config_get_int(hpath);
	}else{
		GdkPixbufLoader*loader=gdk_pixbuf_loader_new();
		GnomeVFSHandle*h;
		info->width=info->height=0;
		g_signal_connect(loader,"size-prepared",G_CALLBACK(velocity_async_get_image_info_cb),info);
		g_signal_connect(loader,"closed",G_CALLBACK(velocity_async_get_image_info_closed_cb),info);
		if(gnome_vfs_open(&h,uri,GNOME_VFS_OPEN_READ)==GNOME_VFS_OK){
			GnomeVFSFileInfo*fi=gnome_vfs_file_info_new();
			if(gnome_vfs_get_file_info_from_handle(h,fi,GNOME_VFS_FILE_INFO_DEFAULT)==GNOME_VFS_OK){
				guchar*buf=g_malloc(fi->io_block_size);
				GnomeVFSFileSize read;
				guint total=0;
				while((total<32768)&&(gnome_vfs_read(h,buf,fi->io_block_size,&read)==GNOME_VFS_OK)&&(gdk_pixbuf_loader_write(loader,buf,read,NULL))){
					/*We count total KB read, if not found within the first 32KB, screw it... (for now)*/
					total+=read;
				}
				g_free(buf);
				gnome_vfs_close(h);
			}
			gnome_vfs_file_info_unref(fi);
		}
		gdk_pixbuf_loader_close(loader,NULL);
		g_object_unref(loader);
		if(info->width+info->height==0){
			g_free(info);
			info=NULL;
		}else{
			gnome_config_set_int(wpath,info->width);
			gnome_config_set_int(hpath,info->height);
			gnome_config_sync();
		}
	}
	g_free(wpath);
	g_free(hpath);
	g_free(turi);
	g_free(iuri);
	return(info);
}
//HACK!!! See below...
static void _velocity_thumbnail_load_cb(GnomeGdkPixbufAsyncHandle*handle,GnomeVFSResult error,GdkPixbuf*pixbuf,gpointer user_data){
	velocity_thumbnail*vap=(velocity_thumbnail*)user_data;
	velocity_image_info*info=NULL;
	GdkPixbuf*p=NULL;
	char*uri=velocity_vfs_get_uri_unescaped(vap->vfsobj);
	if(error!=GNOME_VFS_ERROR_CANCELLED){
		if(vap->thumbnail){
			p=gnome_thumbnail_factory_generate_thumbnail(gtf,uri,vap->vfsobj->info->mime_type);
			if(p){
				gnome_thumbnail_factory_save_thumbnail(gtf,p,uri,vap->vfsobj->info->mtime);
			}else{
				gnome_thumbnail_factory_create_failed_thumbnail(gtf,uri,vap->vfsobj->info->mtime);
			}
		}else{
			p=(error!=GNOME_VFS_OK)?NULL:pixbuf;
		}
	}
	if(p)info=velocity_async_get_image_info(uri);
	if(vap->done)vap->done(vap->vfsobj,p,info,vap->user_data);
	if(info)g_free(info);
	g_free(uri);
}
static void _velocity_thumbnail_done_cb(GnomeGdkPixbufAsyncHandle*handle,gpointer user_data){
	velocity_thumbnail*vap=(velocity_thumbnail*)user_data;
	vap->handle=NULL;
	velocity_thumbnail_free(vap);
}
//HACK!!!
/*This is a very big hack so it deserves a fairly big explanation.
If a thumbnail is found then it is loaded using the normal gnome async pixbuf API,
but if not, we force async pixbuf to attempt to load "/" which will fail, but it will
fail while in a thread, and we can use that thread to asyncly use the (sync only)
gnome_thumbnail API to load and thumbnail our image.	Hope this makes sense in a few months...*/
static gboolean _velocity_thumbnail_start(velocity_thumbnail*vap){
	vap->thumbnail=TRUE;
	velocity_thumbnails_count++;
	velocity_thumbnails=g_list_prepend(velocity_thumbnails,vap);
	vap->handle=gnome_gdk_pixbuf_new_from_uri_async("/",(GnomeGdkPixbufLoadCallback)_velocity_thumbnail_load_cb,
		(GnomeGdkPixbufDoneCallback)_velocity_thumbnail_done_cb,(gpointer)vap);
	return(TRUE);
}
void velocity_thumbnail_new(velocity_vfs_obj*vfsobj,velocity_thumbnail_done done,gpointer user_data){
	velocity_thumbnail*vap=g_new0(velocity_thumbnail,1);
	g_assert(vap!=NULL);
	if(vfsobj){
		vap->vfsobj=vfsobj;
		vap->type=VELOCITY_THUMBNAIL_NORMAL;
		vap->done=done;
		vap->user_data=user_data;
		if(velocity_thumbnails_count<VELOCITY_THUMBNAIL_MAX){
			_velocity_thumbnail_start(vap);
		}else{
			velocity_thumbnails_queue_count++;
			velocity_thumbnails_queue=g_list_prepend(velocity_thumbnails_queue,vap);
		}
	}
}
gboolean velocity_thumbnail_can_thumbnail(velocity_vfs_obj*vfsobj){
	char*uri=velocity_vfs_get_uri(vfsobj);
	gboolean r;
	/*XCF's don't preview yet and we use gnome_thumbnail_factory to check the rest*/
	r=((!strcmp(vfsobj->info->mime_type,"image/x-xcf")==0)&&
	((g_str_has_prefix(vfsobj->info->mime_type,"image/"))||(gnome_thumbnail_factory_can_thumbnail(gtf,uri,vfsobj->info->mime_type,vfsobj->info->mtime))));
	g_free(uri);
	return(r);
}
/*This will also cancel too*/
void velocity_thumbnail_free(velocity_thumbnail*vap){
	if(vap){
		if(vap->handle){
			gnome_gdk_pixbuf_new_from_uri_cancel(vap->handle);
			vap->handle=NULL;
			_velocity_thumbnail_load_cb(NULL,GNOME_VFS_ERROR_CANCELLED,NULL,(gpointer)vap);
			return;
		}
		velocity_thumbnails=g_list_remove(velocity_thumbnails,(gpointer)vap);
//TODO: Whats wrong with this....
//		g_free(vap);
		vap=NULL;
		velocity_thumbnails_count--;
		if(velocity_thumbnails_count<=0){
			velocity_thumbnails=NULL;
			velocity_thumbnails_count=0;
		}
velocity_thumbnail_free_requeue:
		if(velocity_thumbnails_queue_count>0){
			GList*n=g_list_last(velocity_thumbnails_queue);
			vap=(velocity_thumbnail*)n->data;
			velocity_thumbnails_queue_count--;
			velocity_thumbnails_queue=g_list_remove(velocity_thumbnails_queue,n->data);
			if(velocity_thumbnails_queue_count<=0){
				velocity_thumbnails_queue=NULL;
				velocity_thumbnails_queue_count=0;
			}
			if((!_velocity_thumbnail_start(vap))&&(velocity_thumbnails_queue_count>0))goto velocity_thumbnail_free_requeue;
		}
	}
}
void velocity_thumbnail_free_all_by_data(gpointer data){
velocity_thumbnail_free_all_by_data_jmp:;
	if(velocity_thumbnails_count>0){
		GList*n=velocity_thumbnails;
		while(n){
			velocity_thumbnail*vap=(velocity_thumbnail*)n->data;
			if((!data)||(vap->user_data==data)){
				velocity_thumbnail_free(vap);
				vap=NULL;
				goto velocity_thumbnail_free_all_by_data_jmp;
			}
			n=n->next;
		}
	}
	if(velocity_thumbnails_queue_count>0){
		GList*n=velocity_thumbnails_queue;
		while(n){
			velocity_thumbnail*vap=(velocity_thumbnail*)n->data;
			if((!data)||(vap->user_data==data)){
				goto velocity_thumbnail_free_all_by_data_jmp;
			}
			n=n->next;
		}
	}
}
void velocity_thumbnail_free_all(){
	velocity_thumbnail_free_all_by_data(NULL);
}
void velocity_image_vfs_ext_destroy(velocity_image_vfs_ext*ext){
	g_free(ext);
}

/*MODULE STUFF*/
static GtkWidget*velocity_image_module_page1_new(velocity_vfs_obj*vfsobj){
	velocity_image_vfs_ext*p=(velocity_image_vfs_ext*)velocity_vfs_obj_ext_get_data(vfsobj,VELOCITY_STATIC_MODULE_INFO_VAR(image).id);
	if(p){
		char*t=g_strdup_printf(_("Image Width:  %i\nImage Height:  %i\n"),p->width,p->height);
		GtkWidget*w=gtk_label_new(t);
		g_free(t);
		return(w);
	}
	return(NULL);
}
static void velocity_image_module_page1_ok(GtkWidget*page1){
	VELOCITY_MSG("OK not done");
}
static void velocity_image_module_page1_cancel(GtkWidget*page1){
	VELOCITY_MSG("Cancel not done");
}
static void velocity_image_icon_register_with_view(velocity_view*vview,velocity_vfs_obj*vfsobj){
	velocity_image_vfs_ext*ext=(velocity_image_vfs_ext*)velocity_vfs_obj_ext_get_data(vfsobj,VELOCITY_STATIC_MODULE_INFO_VAR(image).id);
	/*For now we only gather info on images*/
 	if(g_str_has_prefix(vfsobj->info->mime_type,"image/")){
		if(!ext){
			char*uri=velocity_vfs_get_uri_unescaped(vfsobj);
			velocity_image_info*info=velocity_async_get_image_info(uri);
			if(info){
				ext=g_new0(velocity_image_vfs_ext,1);
				ext->width=info->width;
				ext->height=info->height;
				velocity_vfs_obj_ext_add(vfsobj,VELOCITY_STATIC_MODULE_INFO_VAR(image).id,
					(gpointer)ext,(velocity_vfs_obj_ext_destroy)velocity_image_vfs_ext_destroy);
				g_free(info);
			}
			g_free(uri);
		}
		if(ext){
			/*Remove any possible old info*/
			velocity_tooltip_remove(vfsobj,"image_width");
			velocity_tooltip_remove(vfsobj,"image_height");
			/*Install tooltip info*/
			velocity_tooltip_add(vfsobj,"image_width",_("Image Width"),VELOCITY_TOOLTIP_ITEM_TYPE_INT,GINT_TO_POINTER(ext->width));
			velocity_tooltip_add(vfsobj,"image_height",_("Image Height"),VELOCITY_TOOLTIP_ITEM_TYPE_INT,GINT_TO_POINTER(ext->height));
		}
	}
	vview->read.images++;
	if(vview->read.objects+vview->read.images==vview->total.objects+vview->total.images){
		if(vview->on_status_event)vview->on_status_event(vview,VELOCITY_VIEW_STATUS_EVENT_DONE,vview->clocation,GNOME_VFS_OK,vview->user_data);
	}
}
static void velocity_image_icon_done_cb(velocity_vfs_obj*vfsobj,GdkPixbuf*pixbuf,velocity_image_info*info,gpointer user_data){
	velocity_view*vview=(velocity_view*)user_data;
	if(vfsobj){
		if(!pixbuf){	/*This shouldn't really happen, but just in case somebody has a broken image*/
			char*iconuri=velocity_image_get_uri(VELOCITY_DEFAULT_ICON,MAX(vview->prefs->profile->icon_size.x,vview->prefs->profile->icon_size.y));
			if(iconuri){
				pixbuf=velocity_image_cache_get(iconuri,vview->prefs->profile->icon_size.x,vview->prefs->profile->icon_size.y,NULL,VELOCITY_IMAGE_CACHE_PRIORITY_HIGH);
				g_free(iconuri);
			}
		}else{
			if(info){	/*If we are just now thumbnailing then this is correct information*/
				velocity_image_vfs_ext*ext=g_new0(velocity_image_vfs_ext,1);
				ext->width=info->width;
				ext->height=info->height;
				velocity_vfs_obj_ext_add(vfsobj,VELOCITY_STATIC_MODULE_INFO_VAR(image).id,
					(gpointer)ext,(velocity_vfs_obj_ext_destroy)velocity_image_vfs_ext_destroy);
			}
		}
		if(pixbuf)velocity_icon_set_icon(vfsobj->icon,pixbuf);
		velocity_image_icon_register_with_view(vview,vfsobj);
	}
}
static void velocity_image_rethumbnail_cb(GtkMenuItem*menuitem,gpointer user_data){
	velocity_view*vview=(velocity_view*)g_object_get_data(G_OBJECT(menuitem),"velocity_view");	/*Always set by the factories for convenience*/
	if(vview){
		GList*n=vview->selection->items;
		while(n){
			if(n->data){
				velocity_vfs_obj*vfsobj=(velocity_vfs_obj*)n->data;
				char*uri=velocity_vfs_get_uri_unescaped(vfsobj);
				char*md5=gnome_thumbnail_md5(uri);
				char*fail_uri=g_strconcat(g_get_home_dir(),"/.thumbnails/fail/gnome-thumbnail-factory/",md5,".png",NULL);
				gnome_vfs_unlink(fail_uri);
				velocity_thumbnail_new(vfsobj,(velocity_thumbnail_done)velocity_image_icon_done_cb,(gpointer)vview);
				g_free(fail_uri);
				g_free(md5);
				g_free(uri);
			}
			n=n->next;
		}
	}
}
static void velocity_image_set_background_cb(GtkMenuItem*menuitem,gpointer user_data){
	velocity_view*vview=(velocity_view*)g_object_get_data(G_OBJECT(menuitem),"velocity_view");	/*Always set by the factories for convenience*/
	if(vview){
		if(!velocity_clipboard_has_multiple(vview->selection)){
			velocity_view*desktop=velocity_windowman_desktop();
			if(desktop){
				velocity_vfs_obj*vfsobj=(velocity_vfs_obj*)vview->selection->items->data;
				char*uri=velocity_vfs_get_uri(vfsobj);
				velocity_view_set_background(desktop,uri);
				gnome_config_set_string("/Background/Default/wallpaper",uri);
				gnome_config_sync();
				g_free(uri);
			}
		}
	}
}
static void velocity_image_view_cb(GtkMenuItem*menuitem,gpointer user_data){
	velocity_view*vview=(velocity_view*)g_object_get_data(G_OBJECT(menuitem),"velocity_view");	/*Always set by the factories for convenience*/
	if(vview){
		velocity_window*vwin=(velocity_window*)vview->user_data;
		velocity_vfs_obj*vfsobj=(velocity_vfs_obj*)vview->selection->items->data;
		char*furi=velocity_vfs_get_uri(vfsobj);
		char*uri=g_strconcat(VELOCITY_MODULE_URI_PREFIX "image:",furi,NULL);
		if(!vwin){
			velocity_windowman_new(uri);
		}else{
			velocity_window_set_location(vwin,uri);
		}
		g_free(uri);
		g_free(furi);
	}
}
static void velocity_image_preview_click_cb(GtkMenuItem*menuitem,gpointer user_data){
	velocity_view*vview=(velocity_view*)user_data;
	if(vview){
		velocity_vfs_obj*vfsobj=(velocity_vfs_obj*)vview->selection->items->data;
		velocity_view_open(vview,vfsobj);
	}
}
static void velocity_image_preview_cb(GtkMenuItem*menuitem,gpointer user_data){
	velocity_view*vview=(velocity_view*)g_object_get_data(G_OBJECT(menuitem),"velocity_view");  /*Always set by the factories for convenience*/
	if(vview){
		GtkWidget*menu=gtk_menu_item_get_submenu(menuitem);
		if(!g_object_get_data(G_OBJECT(menu),"image")){	//TODO:  make this async...
			velocity_vfs_obj*vfsobj=(velocity_vfs_obj*)vview->selection->items->data;
			char*uri=velocity_vfs_get_uri_unescaped(vfsobj);
			if(uri){
				velocity_image_info*info;
				GdkPixbuf*p=velocity_image_cache_get(uri,256,256,&info,VELOCITY_IMAGE_CACHE_PRIORITY_LOW);
				if(p){
					char*szstr=gnome_vfs_format_file_size_for_display(vfsobj->info->size);
					char*name=g_strdup_printf(_("<b>Name:</b>  %s\n<b>Dimensions:</b>  %i x %i\n<b>File size:</b>  %s"),
						vfsobj->info->name,info->width,info->height,szstr);
					GtkWidget*mi=gtk_menu_item_new();
					GtkWidget*v=gtk_vbox_new(FALSE,12);
					GtkWidget*i=gtk_image_new_from_pixbuf(p);
					GtkWidget*t=gtk_label_new(name);
					gtk_label_set_use_markup(GTK_LABEL(t),TRUE);
					gtk_box_pack_start(GTK_BOX(v),i,TRUE,TRUE,0);
					gtk_box_pack_start(GTK_BOX(v),t,FALSE,FALSE,0);
					gtk_widget_show_all(v);
					gtk_container_add(GTK_CONTAINER(mi),v);
					gtk_menu_shell_append(GTK_MENU_SHELL(menu),mi);
					gtk_widget_show(mi);
					g_signal_connect(mi,"activate",G_CALLBACK(velocity_image_preview_click_cb),vview);
					g_object_set_data(G_OBJECT(menu),"image",(gpointer)mi);
					g_object_unref(p);
					g_free(name);
					g_free(szstr);
				}
			}
		}
	}
}
//TODO: Localisation needed...somehow...
gpointer velocity_image_module_gui_ext_context[]={VELOCITY_MENU_FACTORY_SUBMENU,NULL,NULL,N_("Preview"),VELOCITY_FACTORY_TRUE,velocity_image_preview_cb,NULL,
	N_("View"),VELOCITY_FACTORY_TRUE,velocity_image_view_cb,NULL,
	NULL};
gpointer velocity_image_module_gui_ext_addons_thm[]={N_("Re-thumbnail"),VELOCITY_FACTORY_TRUE,velocity_image_rethumbnail_cb,NULL,
	NULL};
gpointer velocity_image_module_gui_ext_addons_bg[]={N_("Set as Background"),VELOCITY_FACTORY_TRUE,velocity_image_set_background_cb,NULL,
	VELOCITY_MENU_FACTORY_EMBEDDED_FACTORY,velocity_image_module_gui_ext_addons_thm,
	NULL};
VELOCITY_STATIC_MODULE_INFO_DEFINE(image);
VELOCITY_STATIC_MODULE_GUI_DEFINE(image);
velocity_module_gui_ext_page velocity_image_module_page1;

velocity_module_return velocity_image_init(void){
	VELOCITY_STATIC_MODULE_INFO(image,_("Image loader"),
		_("Handles loading, previewing, and viewing of images."),"http://velocity.sourceforge.net/");
	VELOCITY_STATIC_MODULE_VERSION(image,1,0,0);
	VELOCITY_STATIC_MODULE_AUTHORS(image,"Kyle Davis");
	VELOCITY_STATIC_MODULE_MIME_TYPES(image,"internal/all");
	velocity_image_module_page1.page_title=_("Info");
	velocity_image_module_page1.page_new=(velocity_module_gui_ext_page_new)velocity_image_module_page1_new;
	velocity_image_module_page1.page_ok=(velocity_module_ext_callback)velocity_image_module_page1_ok;
	velocity_image_module_page1.page_cancel=(velocity_module_ext_callback)velocity_image_module_page1_cancel;
	VELOCITY_STATIC_MODULE_GUI_VAR(image).pages=g_list_prepend(NULL,&velocity_image_module_page1);
	velocity_thumbnail_init();
	return(VELOCITY_MODULE_OK);
}
void velocity_image_about(void){
	VELOCITY_MSG("an about box goes here...");
}
void velocity_image_unload(void){
	velocity_thumbnail_free_all();
}
void*velocity_image_on_view_behavior_event(velocity_view*vview,velocity_view_behavior_event vevent,GdkEvent*event,velocity_vfs_obj*vfsobj,gpointer user_data){
	switch(vevent){
		case VELOCITY_VIEW_BEHAVIOR_EVENT_CONTEXT:
			if(velocity_thumbnail_can_thumbnail(vfsobj)){
				if((!velocity_clipboard_has_multiple(vview->selection))&&
					(g_str_has_prefix(vfsobj->info->mime_type,"image/"))){
					VELOCITY_STATIC_MODULE_GUI_VAR(image).context=(gpointer*)velocity_image_module_gui_ext_context;
					VELOCITY_STATIC_MODULE_GUI_VAR(image).addons=(gpointer*)velocity_image_module_gui_ext_addons_bg;
				}else{
					VELOCITY_STATIC_MODULE_GUI_VAR(image).context=NULL;
					VELOCITY_STATIC_MODULE_GUI_VAR(image).addons=(gpointer*)velocity_image_module_gui_ext_addons_thm;
				}
			}else{
				VELOCITY_STATIC_MODULE_GUI_VAR(image).context=NULL;
				VELOCITY_STATIC_MODULE_GUI_VAR(image).addons=NULL;
			}
			break;
		case VELOCITY_VIEW_BEHAVIOR_EVENT_BACKGROUND_CONTEXT:
			VELOCITY_MSG("Called");
			VELOCITY_STATIC_MODULE_GUI_VAR(image).context=NULL;
			VELOCITY_STATIC_MODULE_GUI_VAR(image).addons=NULL;
			break;
		case VELOCITY_VIEW_BEHAVIOR_EVENT_OBJECT_GET_ICON:
			/*If its an image file, attempt to use it as our icon unless preview_images is false.*/
			if(vview->prefs->preview_images){
				if(velocity_thumbnail_can_thumbnail(vfsobj)){
					gboolean cont=TRUE;
					char*uri=velocity_vfs_get_uri_unescaped(vfsobj); 
					char*iconuri=NULL;
					gboolean async=TRUE;
					if(gnome_thumbnail_factory_has_valid_failed_thumbnail(gtf,uri,vfsobj->info->mtime)){
						iconuri=velocity_image_get_uri(VELOCITY_DEFAULT_ICON,MAX(vview->prefs->profile->icon_size.x,vview->prefs->profile->icon_size.y));
					}else{
						/*Check against the 4 different URI formats in the order they are most used ([non-]prefixed + [un]escaped)*/
						iconuri=gnome_thumbnail_factory_lookup(gtf,uri,vfsobj->info->mtime);
						if(!iconuri){
							char*euri=velocity_vfs_get_uri(vfsobj);
							if(vfsobj->prefix[0]==GNOME_VFS_URI_PATH_CHR){
								if(!iconuri){
									char*tmp=g_strconcat("file://",uri,NULL);
									iconuri=gnome_thumbnail_factory_lookup(gtf,tmp,vfsobj->info->mtime);
									if(!iconuri){
										g_free(tmp);
										tmp=g_strconcat("file://",euri,NULL);
										iconuri=gnome_thumbnail_factory_lookup(gtf,tmp,vfsobj->info->mtime);
									}
									g_free(tmp);
								}
							}
							if(!iconuri)
								iconuri=gnome_thumbnail_factory_lookup(gtf,euri,vfsobj->info->mtime);
							g_free(euri);
						}
					}
					if(iconuri){
						async=FALSE;
					}else{
						iconuri=velocity_image_get_uri(VELOCITY_LOADING_ICON,MAX(vview->prefs->profile->icon_size.x,vview->prefs->profile->icon_size.y));
					}
					g_free(uri);
					if(iconuri){
						GdkPixbuf*icon=velocity_image_cache_get(iconuri,vview->prefs->profile->icon_size.x,vview->prefs->profile->icon_size.y,NULL,VELOCITY_IMAGE_CACHE_PRIORITY_HIGH);
						if(icon){
							vview->total.images++;
							if(async){
								velocity_thumbnail_new(vfsobj,(velocity_thumbnail_done)velocity_image_icon_done_cb,(gpointer)vview);
							}else{
								velocity_image_icon_register_with_view(vview,vfsobj);
							}
							g_free(iconuri);
							return(icon);
						}
						g_free(iconuri);
					}
				}
			}
			break;
		default:
			break;
	}
	return(NULL);
}
GtkWidget*velocity_image_viewer_new(velocity_window*vwin,const char*uri){
	GdkPixbuf*p=velocity_image_cache_get(uri,0,0,NULL,VELOCITY_IMAGE_CACHE_PRIORITY_NONE);
	if(p){
		GtkWidget*w=gtk_scrolled_window_new(NULL,NULL);
		GtkWidget*i=gtk_image_new_from_pixbuf(p);
		g_object_unref(p);
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(w),GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
		gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(w),i);
		gtk_widget_show_all(w);
		return(w);
	}
	return(NULL);
}
void velocity_image_viewer_free(GtkWidget*viewer){
	/*Nothing needs to be done here actually... (for now?)*/
}
