/*
 * Velocity - A GNOME File Manager
 *
 *
 * Copyright (C) (2002) (Kyle Davis) <kyled@blueriver.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef VELOCITY_THUMBNAIL_H
#define VELOCITY_THUMBNAIL_H

#include <config.h>
#include <glib.h>
#include <gdk-pixbuf/gdk-pixbuf-loader.h>
#include "velocity-image.h"
#include "velocity-vfs.h"
#include "velocity-module.h"

/*Maximum number of async thumbnails to attempt to load at the same time, others will be queued*/
#define VELOCITY_THUMBNAIL_MAX	4

extern GList*velocity_thumbnails;

typedef enum{VELOCITY_THUMBNAIL_INVALID,
	VELOCITY_THUMBNAIL_NORMAL, /*Normal gdk_pixbuf_loader_get_pixbuf if gdk_pixbuf worked*/
	VELOCITY_THUMBNAIL_SVG, /*uses the librsvg loading scheme inside of a GnomeVFS thread*/
	VELOCITY_THUMBNAIL_NUM}velocity_thumbnail_type;

typedef void(*velocity_thumbnail_done)(velocity_vfs_obj*vfsobj,GdkPixbuf*pixbuf,velocity_image_info*info,gpointer user_data);

typedef struct{
	velocity_vfs_obj*vfsobj;
	GnomeGdkPixbufAsyncHandle*handle;
	velocity_thumbnail_type type;
	velocity_thumbnail_done done;
	gpointer user_data;
	gboolean thumbnail;
	/*TODO:  error callback?*/
}velocity_thumbnail;

/*Create a new async pixbuf object and start reading in the vfsobj)*/
void velocity_thumbnail_new(velocity_vfs_obj*vfsobj,velocity_thumbnail_done done,gpointer user_data);
gboolean velocity_thumbnail_can_thumbnail(velocity_vfs_obj*vfsobj);
/*This will also cancel too*/
void velocity_thumbnail_free(velocity_thumbnail*vap);
void velocity_thumbnail_free_all(void);
void velocity_thumbnail_free_all_by_data(gpointer user_data);

/*VFS Extentions*/
typedef struct{
	guint width;
	guint height;
	//TODO:  color mode, channels, etc.
}velocity_image_vfs_ext;
void velocity_image_vfs_ext_destroy(velocity_image_vfs_ext*ext);

/*Static module support*/
extern VELOCITY_STATIC_MODULE_INFO_DEFINE(image);
extern VELOCITY_STATIC_MODULE_GUI_DEFINE(image);
velocity_module_return velocity_image_init(void);
void velocity_image_about(void);
void velocity_image_unload(void);
void*velocity_image_on_view_behavior_event(velocity_view*vview,velocity_view_behavior_event vevent,GdkEvent*event,velocity_vfs_obj*vfsobj,gpointer user_data);
GtkWidget*velocity_image_viewer_new(velocity_window*vwin,const char*uri);
void velocity_image_viewer_free(GtkWidget*viewer);
#endif
