/*
 * Velocity - A GNOME File Manager
 *
 *
 * Copyright (C) (2002) (Kyle Davis) <kyled@blueriver.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
                                                                                                                               
#include <stdio.h>
#include <stdlib.h>
#include <gtk/gtk.h>

#include "velocity-vfs.h"
#include "velocity-sys.h"
#include "velocity-tree.h"
#include "velocity-image.h"

/*treestore data*/
enum{COLLAPSE_PIXBUF=0,
	EXPAND_PIXBUF,
	LABEL,
	DATA,
	NUM_COLUMNS};

velocity_tree_item*velocity_tree_item_new(velocity_tree*vt,velocity_tree_item*parent,const gchar*uri,const gchar*mime_type){
	velocity_tree_item*rp=((parent)?parent:vt->root);
	velocity_tree_item*vti=g_new0(velocity_tree_item,1);
	vti->uri=g_strdup(uri);
	vti->mime_type=g_strdup(mime_type);
	vti->tree=vt;
	vti->parent=parent;
	gtk_tree_store_append(vt->store,&vti->iter,(rp==vt->root)?NULL:&rp->iter);
	gtk_tree_store_set(vt->store,&vti->iter,DATA,vti,-1);	/*This way we can get our struct from the iter*/
	velocity_tree_item_update(vti);
	rp->children=g_list_prepend(rp->children,(gpointer)vti);
	return(vti);
}
void velocity_tree_item_update(velocity_tree_item*vti){
	gboolean update=((vti->collapse)||(vti->expand));
	char*ocf=vti->collapse_file;
	char*oef=vti->expand_file;
	vti->collapse_file=NULL;
	vti->expand_file=NULL;
	/*Setup the icons*/
	if(vti->tree->modfunc){
		if(vti->label)g_free(vti->label);
		vti->label=NULL;
		vti->label=vti->tree->modfunc(vti,VELOCITY_TREE_ITEM_MOD_GET_LABEL);
	}
	if(!vti->label){
		char*t=velocity_vfs_uri_get_tail(vti->uri);
		vti->label=g_locale_to_utf8(t,-1,NULL,NULL,NULL);
		g_free(t);
	}
	if(vti->tree->modfunc)vti->tree->modfunc(vti,VELOCITY_TREE_ITEM_MOD_SET_ICONS);
	if(vti->expand_file){
		if(((update)&&(strcmp(oef,vti->expand_file)!=0))||(!update)){
			GdkPixbuf*p=velocity_image_cache_get(vti->expand_file,vti->size.x,vti->size.y,NULL,VELOCITY_IMAGE_CACHE_PRIORITY_HIGH);
			if(p){
				if(vti->expand)g_object_unref(vti->expand);
				vti->expand=p;
			}
		}
	}
	if(vti->collapse_file){
		if(((update)&&(strcmp(ocf,vti->collapse_file)!=0))||(!update)){
			GdkPixbuf*p=velocity_image_cache_get(vti->collapse_file,vti->size.x,vti->size.y,NULL,VELOCITY_IMAGE_CACHE_PRIORITY_HIGH);
			if(p){
				if(vti->collapse)g_object_unref(vti->collapse);
				vti->collapse=p;
			}
		}
	}
	if((vti->expand)&&(vti->collapse))gtk_tree_store_set(vti->tree->store,&vti->iter,COLLAPSE_PIXBUF,vti->collapse,EXPAND_PIXBUF,vti->expand,-1);
	if(vti->label)gtk_tree_store_set(vti->tree->store,&vti->iter,LABEL,vti->label,-1);
	if(oef)g_free(oef);
	if(ocf)g_free(ocf);
}

//TODO:  Rethink/Nuke this stuff...
typedef struct{
	const gchar*id;
	gpointer data;
}velocity_tree_item_data;
void velocity_tree_item_set_data(velocity_tree_item*vti,const gchar*id,gpointer data){
	gpointer d=velocity_tree_item_get_data(vti,id);
	if(d){
		d=data;
	}else{
		velocity_tree_item_data*vtid=g_new(velocity_tree_item_data,1);
		vtid->id=id;
		vtid->data=data;
		vti->data=g_list_prepend(vti->data,(gpointer)vtid);
	}
}
gpointer*velocity_tree_item_get_data(velocity_tree_item*vti,const gchar*id){
	GList*n=vti->data;
	GList*nl=NULL;
	while(n){
		velocity_tree_item_data*vtid=(velocity_tree_item_data*)n->data;
		if(vtid->id==id)return(vtid->data);
		n=n->next;
	}
	return(NULL);
}
static void velocity_tree_item_free_recurse(velocity_tree_item*vti){
	if(vti){
		velocity_tree_item*pvti=(velocity_tree_item*)vti->parent;
		GList*n=vti->children;
		while(n){
			velocity_tree_item_free_recurse(n->data);
			n=n->next;
		}
		gtk_tree_store_remove(vti->tree->store,&vti->iter);
		pvti->children=g_list_remove(pvti->children,(gpointer)vti);
		if(vti->label)g_free(vti->label);
		if(vti->collapse_file)g_free(vti->collapse_file);
		if(vti->expand_file)g_free(vti->expand_file);
		if(vti->data)g_list_free(vti->data);
		g_free(vti->mime_type);
		g_free(vti->uri);
		g_free(vti);
	}
}
void velocity_tree_item_free(velocity_tree_item*vti){
	velocity_tree_item_free_recurse(vti);
}

static void velocity_tree_add_columns(GtkTreeView*treeview){
	GtkCellRenderer*renderer=gtk_cell_renderer_pixbuf_new();
	GtkTreeViewColumn*column=gtk_tree_view_column_new();
	gtk_tree_view_column_pack_start(column,renderer,FALSE);
	gtk_tree_view_column_set_attributes(column,renderer,
		"pixbuf",COLLAPSE_PIXBUF,
		"pixbuf-expander-closed",COLLAPSE_PIXBUF,
		"pixbuf-expander-open",EXPAND_PIXBUF,
		NULL);
	renderer=gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(column,renderer,TRUE);
	gtk_tree_view_column_set_attributes(column,renderer,"text",LABEL,NULL);
	gtk_tree_view_append_column(treeview,column);
}
static void velocity_tree_item_select_cb(GtkTreeSelection*selection,gpointer user_data){
	GtkTreeView*treeview=gtk_tree_selection_get_tree_view(selection);
	GtkTreeModel*model;
	GtkTreeIter iter;
	if(gtk_tree_selection_get_selected(selection,&model,&iter)){
		velocity_tree_item*vti=NULL;
		gtk_tree_model_get(model,&iter,DATA,&vti,-1);
		if(!vti->tree->first_select){	/*really annoyed*/
			if((vti)&&(vti->tree->select_func))vti->tree->select_func(vti,user_data);
		}
		vti->tree->first_select=FALSE;
	}
}
static gint velocity_tree_sort_cmp(GtkTreeModel*model,GtkTreeIter*a,GtkTreeIter*b,GCompareFunc cmp){
	velocity_tree_item*vtia=NULL;
	velocity_tree_item*vtib=NULL;
	gtk_tree_model_get(model,a,DATA,&vtia,-1);
	gtk_tree_model_get(model,b,DATA,&vtib,-1);
	if((vtia->label)&&(vtib->label))return(cmp(vtia->label,vtib->label));
	return(0);
}
velocity_tree*velocity_tree_new(GCompareFunc sort_method,velocity_tree_item_mod_func modfunc,velocity_tree_item_select_func select_func,gpointer select_data){
	velocity_tree*vt=g_new0(velocity_tree,1);
	vt->root=g_new0(velocity_tree_item,1);
	vt->store=gtk_tree_store_new(NUM_COLUMNS,GDK_TYPE_PIXBUF,GDK_TYPE_PIXBUF,G_TYPE_STRING,G_TYPE_POINTER);
	vt->sort_method=sort_method;
	vt->modfunc=modfunc;
	vt->first_select=TRUE;	/*annoyed annoyed annoyed*/
	vt->widget=gtk_scrolled_window_new(NULL,NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(vt->widget),GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
	/*create tree view*/
	vt->tree=gtk_tree_view_new_with_model(GTK_TREE_MODEL(vt->store));
	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(vt->tree),FALSE);
	velocity_tree_add_columns(GTK_TREE_VIEW(vt->tree));
	if(vt->sort_method){
		gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(vt->store),LABEL,(GtkTreeIterCompareFunc)velocity_tree_sort_cmp,vt->sort_method,NULL);
		gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(vt->store),LABEL,GTK_SORT_ASCENDING);
	}
	if(select_func){
		GtkTreeSelection*selection=gtk_tree_view_get_selection(GTK_TREE_VIEW(vt->tree));
		gtk_tree_selection_set_mode(selection,GTK_SELECTION_BROWSE);
		vt->select_func=select_func;
		g_signal_connect(selection,"changed",G_CALLBACK(velocity_tree_item_select_cb),select_data);
	}
	gtk_container_add(GTK_CONTAINER(vt->widget),GTK_WIDGET(vt->tree));
	gtk_widget_show_all(vt->widget);
	return(vt);
}
void velocity_tree_free(velocity_tree*vt){
	g_object_unref(vt->store);
	gtk_widget_destroy(vt->tree);
	gtk_widget_destroy(vt->widget);
	g_free(vt);
}
static velocity_tree_item*velocity_tree_item_find(velocity_tree_item*pvti,const gchar*uri){
	if(pvti){
		GList*n=pvti->children;
		while(n){
			if(n->data){
				velocity_tree_item*vti=(velocity_tree_item*)n->data;
				if(velocity_vfs_uris_match(vti->uri,uri))return(vti);
				if(vti->children){
					vti=velocity_tree_item_find(vti,uri);
					if(vti)return(vti);
				}
			}
			n=n->next;
		}
	}
	return(NULL);
}
velocity_tree_item*velocity_tree_find_item(velocity_tree*vt,const gchar*uri){
	return(velocity_tree_item_find(vt->root,uri));
}
velocity_tree_item*velocity_tree_register_item(velocity_tree*vt,const gchar*uri){
	GList*pl=velocity_vfs_get_uri_parents_reversed(uri);
	GList*n=pl;
	velocity_tree_item*st=(velocity_tree_item*)vt->root;
	velocity_tree_item*vti=velocity_tree_item_find(st,(gchar*)n->data);
	while(n){
		if(!vti){
			GtkTreePath*path;
			GtkTreeSelection*selection=gtk_tree_view_get_selection(GTK_TREE_VIEW(vt->tree));
			char*mt=gnome_vfs_get_mime_type((gchar*)n->data);
			st=velocity_tree_item_new(vt,st,(gchar*)n->data,mt);
			path=gtk_tree_model_get_path(GTK_TREE_MODEL(vt->store),&st->iter);
			gtk_tree_view_expand_to_path(GTK_TREE_VIEW(vt->tree),path);
			gtk_tree_path_free(path);
			if(mt)g_free(mt);
			if(!n->next)vti=st;
		}else{
			velocity_tree_item_update(vti);
			st=vti;
			if(n->next)vti=velocity_tree_item_find(st,(gchar*)n->next->data);
		}
		n=n->next;
	}
	gnome_vfs_list_deep_free(pl);
	return(vti);
}
void velocity_tree_unregister_item(velocity_tree*vt,const gchar*uri){
	char*furi=velocity_vfs_ensure_dir_uri(uri);
	velocity_tree_item*vti=velocity_tree_find_item(vt,furi);
	if(vti)velocity_tree_item_free(vti);
	g_free(furi);
}
