/*
 * Velocity - A GNOME File Manager
 *
 *
 * Copyright (C) (2002) (Kyle Davis) <kyled@blueriver.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef VELOCITY_TREE_H
#define VELOCITY_TREE_H

#include <stdio.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include "velocity-image.h"

/*This can be used to modify default look and feel of objects within the tree*/
typedef enum{VELOCITY_TREE_ITEM_MOD_INVALID=NULL,
	VELOCITY_TREE_ITEM_MOD_SET_ICONS,    /*should set size and expand/collapse file uris, return is ignored*/
	VELOCITY_TREE_ITEM_MOD_GET_LABEL, /*should return an alloc'd, NULL terminated string containing user-friendly name*/
	VELOCITY_TREE_ITEM_MOD_NUM}velocity_tree_item_mod_type;
typedef void*(*velocity_tree_item_mod_func)(void*your_velocity_tree_item,velocity_tree_item_mod_type modtype);
typedef void*(*velocity_tree_item_select_func)(void*your_velocity_tree_item,gpointer user_data);
typedef struct{
	GtkWidget*widget;
	GtkWidget*tree;
	GtkTreeStore*store;
	gpointer root;  /*This is the root velocity_tree_item*/
	GCompareFunc sort_method;	/*Define sorting method, NULL for unsorted*/
	velocity_tree_item_mod_func modfunc;
	velocity_tree_item_select_func select_func;
	gboolean first_select;	/*This is really annoying....*/
}velocity_tree;

typedef struct{
	gchar*uri;
	gchar*mime_type;
	gchar*label;
	velocity_tree*tree;
	void*parent;	/*this is a velocity_tree_item*/
	GtkTreeIter iter;
	GdkPixbuf*collapse;
	GdkPixbuf*expand;
	gchar*collapse_file;
	gchar*expand_file;
	GdkPoint size;
	GList*data;	/*TODO: A better idea would probably be to make this into a full GObject* and use g_object_*_data functions*/
	GList*children;	/*G_PTR_ARRAY's may make more sense and be faster here*/
}velocity_tree_item;

/*Tree Item functions*/
velocity_tree_item*velocity_tree_item_new(velocity_tree*tree,velocity_tree_item*parent,const gchar*uri,const gchar*mime_type);
void velocity_tree_item_update(velocity_tree_item*vti);
void velocity_tree_item_free(velocity_tree_item*vti);
gpointer*velocity_tree_item_get_data(velocity_tree_item*vti,const gchar*id);
void velocity_tree_item_set_data(velocity_tree_item*vti,const gchar*id,gpointer data);

/*Tree functions*/
velocity_tree*velocity_tree_new(GCompareFunc sort_method,velocity_tree_item_mod_func modfunc,velocity_tree_item_select_func select_func,gpointer select_data);
void velocity_tree_free(velocity_tree*vt);
velocity_tree_item*velocity_tree_find_item(velocity_tree*vt,const gchar*uri);
velocity_tree_item*velocity_tree_register_item(velocity_tree*vt,const gchar*uri);
void velocity_tree_unregister_item(velocity_tree*vt,const gchar*uri);

#endif

