/*
 * Velocity - A GNOME File Manager
 *
 *
 * Copyright (C) (2002) (Kyle Davis) <kyled@blueriver.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <libgnomevfs/gnome-vfs.h>
#include "velocity-vfs-object.h"
#include "velocity-icon.h"

velocity_vfs_obj*velocity_vfs_obj_new(const char*uri,GnomeVFSFileInfo*info,velocity_icon*icon,gpointer data){
	velocity_vfs_obj*vfsobj=g_new(velocity_vfs_obj,1);
	g_assert(vfsobj!=NULL);
	gnome_vfs_file_info_ref(info);
	vfsobj->prefix=gnome_vfs_unescape_string(uri,GNOME_VFS_URI_PATH_STR);
	vfsobj->info=info;
	vfsobj->icon=icon;
	vfsobj->data=data;
	vfsobj->extensions=NULL;
	vfsobj->refcount=1;
	return(vfsobj);
}
velocity_vfs_obj*velocity_vfs_obj_dup(velocity_vfs_obj*vfsobj){
	/*I love reference counting*/
	gnome_vfs_file_info_ref(vfsobj->info);
	vfsobj->refcount++;
	return(vfsobj);
}
void velocity_vfs_obj_ext_add(velocity_vfs_obj*vfsobj,const char*id,gpointer data,velocity_vfs_obj_ext_destroy destroyucb){
	velocity_vfs_obj_ext*ext=g_new0(velocity_vfs_obj_ext,1);
	g_assert(ext!=NULL);
	ext->id=g_strdup(id);
	ext->data=data;
	ext->destroyucb=destroyucb;
	vfsobj->extensions=g_list_prepend(vfsobj->extensions,ext);
}
void velocity_vfs_obj_ext_remove(velocity_vfs_obj*vfsobj,const char*id){	/*NOTE: This _WILL_ change the extensions list*/
	GList*n=vfsobj->extensions;
	GList*nl=NULL;
	while(n){
		velocity_vfs_obj_ext*ext=(velocity_vfs_obj_ext*)n->data;
		if(ext){
			if(strcmp(ext->id,id)==0){
				if(ext->destroyucb){
					ext->destroyucb(ext->data);
				}
				g_free(ext->id);
				g_free(ext);
			}else{
				nl=g_list_prepend(nl,n->data);
			}
		}
		n=n->next;
	}
	if(vfsobj->extensions)g_list_free(vfsobj->extensions);
	vfsobj->extensions=nl;
}
gpointer*velocity_vfs_obj_ext_get_data(velocity_vfs_obj*vfsobj,const char*id){
	GList*n=vfsobj->extensions;
	while(n){
		velocity_vfs_obj_ext*ext=(velocity_vfs_obj_ext*)n->data;
		if(strcmp(ext->id,id)==0)return(ext->data);
		n=n->next;
	}
	return(NULL);
}
gboolean velocity_vfs_obj_ext_set_data(velocity_vfs_obj*vfsobj,const char*id,gpointer data){
	GList*n=vfsobj->extensions;
	while(n){
		velocity_vfs_obj_ext*ext=(velocity_vfs_obj_ext*)n->data;
		if(strcmp(ext->id,id)==0){
			ext->data=data;
			return(TRUE);
		}
		n=n->next;
	}
	return(FALSE);
}
void velocity_vfs_obj_free(velocity_vfs_obj*vfsobj){
	if(vfsobj){
		vfsobj->refcount--;
		if(vfsobj->info)gnome_vfs_file_info_unref(vfsobj->info);
		if(vfsobj->refcount==0){
			GList*n=vfsobj->extensions;
			while(n){
				velocity_vfs_obj_ext*ext=(velocity_vfs_obj_ext*)n->data;
				if(ext){
					velocity_vfs_obj_ext_remove(vfsobj,ext->id);	/*This is okay because we're doing this in order, so we won't have wasted loops*/
					n=vfsobj->extensions;
				}
			}
			if(vfsobj->prefix)g_free(vfsobj->prefix);
			if(vfsobj->icon)velocity_icon_free(vfsobj->icon);
			vfsobj->prefix=NULL;
			vfsobj->info=NULL;
			vfsobj->icon=NULL;
			g_free(vfsobj);
		}
	}
}
void velocity_vfs_obj_destroy(velocity_vfs_obj*vfsobj){
	if(vfsobj){
		vfsobj->refcount=1;
		if(vfsobj->info)vfsobj->info->refcount=1;
		velocity_vfs_obj_free(vfsobj);
	}
}
