/*
 * Velocity - A GNOME File Manager
 *
 *
 * Copyright (C) (2002) (Kyle Davis) <kyled@blueriver.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

//TODO:  Use velocity_vfs_object's in this...that will be reason to improve the velocity_vfs_object's
//TODO:  This will control async moves and such...the window and all...
//TODO:  functions should get passed the velocity_view object

/*Testing functions*/
//TODO:  These functions should check against the current user's permissions also

#include <config.h>
#include <unistd.h>
#include <sys/types.h>
#include <glib.h>
#include <libgnome/gnome-desktop-item.h>
#include "velocity-vfs.h"
#include "velocity-vfs-object.h"
#include "velocity-sys.h"
#include "velocity-preferences.h"
#include "velocity-image.h"
#include "velocity-icons.h"
#include "velocity-icon.h"
//#include "interface.h"

static void velocity_vfs_progress_cancel_cb(GtkWidget*w,gpointer user_data);
static void velocity_vfs_xfer_cancel_cb(velocity_vfs_progress*p,gpointer user_data);
static int velocity_vfs_xfer_sync_cb(GnomeVFSXferProgressInfo*progress_info,gpointer user_data);
static int velocity_vfs_xfer_handler_cb(GnomeVFSAsyncHandle*handle,GnomeVFSXferProgressInfo*progress_info,gpointer user_data);
static int velocity_vfs_xfer_handle_ok(const GnomeVFSXferProgressInfo*progress_info,velocity_vfs_xfer_info*xferinfo);
static int velocity_vfs_xfer_handle_error(const GnomeVFSXferProgressInfo*progress_info,velocity_vfs_xfer_info*transfer_info);
static int velocity_vfs_xfer_handle_duplicate(GnomeVFSXferProgressInfo*progress_info,velocity_vfs_xfer_info *transfer_info);
static int velocity_vfs_xfer_handle_overwrite(const GnomeVFSXferProgressInfo*progress_info,velocity_vfs_xfer_info*transfer_info);

typedef struct{
	GSList*monitors;	/*List of monitors*/
	GSList*monitor_events;	/*List of events that have not been taken care of yet*/
	guint monitor_idler;	/*Our idle handle.  This is where the magic happens*/
}velocity_vfs_sysdata;
static velocity_vfs_sysdata*velocity_vfs_sys=NULL;

/*Initialize the Velocity VFS system*/
void velocity_vfs_init(){
	if(!velocity_vfs_sys){
		g_assert(gnome_vfs_init());
		velocity_vfs_sys=g_new0(velocity_vfs_sysdata,1);
	}
}
void velocity_vfs_shutdown(){
	if(velocity_vfs_sys){
		GSList*n=velocity_vfs_sys->monitors;
		while(n){
			if(n->data){
				velocity_vfs_monitor*vfsmon=(velocity_vfs_monitor*)n->data;
				velocity_vfs_monitor_free(vfsmon);
				/*The list changed so we need to start over.  This is okay because the beginning of the new list is also the next item in question anyhow...*/
				n=velocity_vfs_sys->monitors;
				continue;
			}
			n=n->next;
		}
		if(velocity_vfs_sys->monitors)g_slist_free(velocity_vfs_sys->monitors);
		g_free(velocity_vfs_sys);
		VELOCITY_MSG("FIXME:  if you stop an async pixbuf/thumbnail operation prematurely it will cause gnome_vfs_shutdown to lock on exit (code commented out)");
		//gnome_vfs_shutdown();
	}
}
char*velocity_vfs_uri_from_input(const char*uri){
	char*r=NULL;
	if(uri){
		char*tmp=g_strstrip(g_strdup(uri));
		if(tmp){
			switch(tmp[0]){
				case GNOME_VFS_URI_PATH_CHR:
					r=gnome_vfs_get_uri_from_local_path(tmp);
					break;
				case '~':
					r=gnome_vfs_expand_initial_tilde(tmp);
					break;
				default:
					if((tmp[0]=='/')||(strchr(tmp,':'))){
						r=g_strdup(tmp);
					}else{
						r=gnome_vfs_make_uri_full_from_relative(g_get_current_dir(),tmp);
					}
					break;
			}
			g_free(tmp);
		}
	}
	return(r);
}
char*velocity_vfs_uri_reformat(const char*uri){
	return(g_str_has_prefix(uri,"file://")?g_strdup(uri+7):g_strdup(uri));
}
char*velocity_vfs_uri_get_tail(const char*uri){
	GnomeVFSURI*curi=NULL;
	char*ret=NULL;
	g_return_val_if_fail(uri!=NULL,NULL);
	curi=gnome_vfs_uri_new(uri);
	ret=gnome_vfs_uri_extract_short_name(curi);
	gnome_vfs_uri_unref(curi);
	return(ret);
}
char*velocity_vfs_ensure_dir_uri(const char*uri){
	g_return_val_if_fail(uri!=NULL,NULL);
	return((uri[strlen(uri)-1]==GNOME_VFS_URI_PATH_CHR)?g_strdup(uri):g_strconcat(uri,GNOME_VFS_URI_PATH_STR,NULL));
}
char*velocity_vfs_get_uri_unescaped(velocity_vfs_obj*vfsobj){
	/*We shouldn't need the "file://" since it should always be assumed as the default anyways*/
	char*uri=g_strconcat(vfsobj->prefix,vfsobj->info->name,NULL);
	if(velocity_vfs_is_directory(vfsobj)){
		char*tmp=uri;
		uri=velocity_vfs_ensure_dir_uri(tmp);
		g_free(tmp);
	}
	if(g_str_has_prefix(vfsobj->prefix,"file://")){	/*Local files should never have a prepending file://, some things complain*/
		char*tmp=uri;
		uri=g_strdup(tmp+7);
		g_free(tmp);
	}
	return(uri);
}
char*velocity_vfs_get_uri(velocity_vfs_obj*vfsobj){
	char*uri=velocity_vfs_get_uri_unescaped(vfsobj);
	if(uri){
		char*tmp=uri;
		uri=gnome_vfs_escape_host_and_path_string(tmp);
		g_free(tmp);
	}
	return(uri);
}
char*velocity_vfs_get_human_readable(const char*uri){
	char*lt[]={
		VELOCITY_VFS_LOCAL_URI,_("Local Computer"),
		VELOCITY_VFS_START_HERE_URI,_("Start Here"),
		VELOCITY_VFS_APPLICATIONS_URI,_("Applications"),
		VELOCITY_VFS_APPLICATIONS_ALL_USERS_URI,_("All User Applications"),
		VELOCITY_VFS_ALL_APPLICATIONS_URI,_("All Applications"),
		VELOCITY_VFS_PREFERENCES_URI,_("System Preferences"),
		VELOCITY_VFS_PREFERENCES_ALL_USERS_URI,_("All User Preferences"),
		VELOCITY_VFS_ALL_PREFERENCES_URI,_("All Preferences"),
		VELOCITY_VFS_SYSTEM_SETTINGS_URI,_("System Settings"),
		VELOCITY_VFS_SERVER_SETTINGS_URI,_("Server Settings"),
		VELOCITY_VFS_NETWORK_URI,_("Network"),
		VELOCITY_VFS_SAMBA_URI,_("Windows Network Shares"),
		VELOCITY_VFS_FONTS_URI,_("Fonts"),
		VELOCITY_VFS_BURN_URI,_("Burn"),
		NULL};
	guint i=0;
	while(lt[i]){
		if(velocity_vfs_uris_match(uri,lt[i]))return(g_strdup(lt[i+1]));
		i+=2;
	}
	return(velocity_vfs_uri_get_tail(uri));
}
static gchar*velocity_vfs_uri_get_matchable(const char*uri){
	/*NOTE:  We do all this hacking because GnomeVFS gets it wrong if you compare:
	 * a) "/path/to/file/" to "/path/to/file"
	 * b) "method://" to "method:///"
	 *
	 * So as a solution to the problem we strip all trailing delimeters
	 * from our URI's (and put them back when done to avoid memory leaks.
	 */
	int l=strlen(uri)-1;
	if(l>0){
		int n=0;
		while(uri[l-n]==GNOME_VFS_URI_PATH_CHR){
			n++;
		}
		return(g_strndup(uri,l+1-n));
	}
	return(g_strdup(uri));
}
gboolean velocity_vfs_uris_match(const char*uri1,const char*uri2){
	char*turi1=velocity_vfs_uri_get_matchable(uri1);
	char*turi2=velocity_vfs_uri_get_matchable(uri2);
	gboolean r;
	r=gnome_vfs_uris_match(turi1,turi2);
	g_free(turi2);
	g_free(turi1);
	return(r);
}
gboolean velocity_vfs_uri_has_parent(const char*uri){
	GnomeVFSURI*curi=gnome_vfs_uri_new(uri);
	gboolean ret=gnome_vfs_uri_has_parent(curi);
	gnome_vfs_uri_unref(curi);
	return(ret);
}
char*velocity_vfs_get_uri_parent(const char*uri){
	/*use GNOME VFS to do this for us :)*/
	GnomeVFSURI*cguri=gnome_vfs_uri_new(uri);
	if(gnome_vfs_uri_has_parent(cguri)){
		GnomeVFSURI*pguri=gnome_vfs_uri_get_parent(cguri);
		char*puri=gnome_vfs_uri_to_string(pguri,GNOME_VFS_URI_HIDE_USER_NAME|GNOME_VFS_URI_HIDE_PASSWORD);
		char*furi=velocity_vfs_uri_reformat(puri);
		char*duri=velocity_vfs_ensure_dir_uri(furi);
		gnome_vfs_uri_unref(pguri);
		gnome_vfs_uri_unref(cguri);
		g_free(puri);
		g_free(furi);
		return(duri);
	}
	gnome_vfs_uri_unref(cguri);
	return(NULL);
}
GList*velocity_vfs_get_uri_parents_reversed(const char*uri){
	GList*plist=NULL;
	char*parent=g_strdup(uri);
	while(parent){
		plist=g_list_prepend(plist,(gpointer)parent);
		parent=velocity_vfs_get_uri_parent(parent);
	}
	return(plist);
}
GList*velocity_vfs_get_uri_parents(const char*uri){
	GList*plist=velocity_vfs_get_uri_parents_reversed(uri);
	return(g_list_reverse(plist));
}
gboolean velocity_vfs_is_owner(velocity_vfs_obj*vfsobj){
	return((vfsobj->info->uid==getuid()));
}
gboolean velocity_vfs_is_in_group(velocity_vfs_obj*vfsobj){
	return((vfsobj->info->gid==getgid()));
}
gboolean velocity_vfs_is_local(velocity_vfs_obj*vfsobj){
	return(GNOME_VFS_FILE_INFO_LOCAL(vfsobj->info));
}
gboolean velocity_vfs_is_readable(velocity_vfs_obj*vfsobj){
	if(!(vfsobj->info->valid_fields&GNOME_VFS_FILE_INFO_FIELDS_PERMISSIONS))return(TRUE);
	if(getuid()==0){
		return(TRUE);
	}else{
		gboolean isowner=velocity_vfs_is_owner(vfsobj);
		gboolean ingroup=velocity_vfs_is_in_group(vfsobj);
		if(((isowner)&&(vfsobj->info->permissions&GNOME_VFS_PERM_USER_READ))||
			((ingroup)&&(vfsobj->info->permissions&GNOME_VFS_PERM_GROUP_READ))||
			((!isowner)&&(!ingroup)&&(vfsobj->info->permissions&GNOME_VFS_PERM_OTHER_READ))){
			/* *NIX Filesystems require that users have the exec perm to read a directory's contents*/
			return((!velocity_vfs_is_directory(vfsobj))?TRUE:velocity_vfs_is_executable(vfsobj));
		}
	}
	return(FALSE);
}
gboolean velocity_vfs_is_writable(velocity_vfs_obj*vfsobj){
	if(!(vfsobj->info->valid_fields&GNOME_VFS_FILE_INFO_FIELDS_PERMISSIONS))return(TRUE);
	if(strncmp(vfsobj->prefix,VELOCITY_VFS_APPLICATIONS_URI,strlen(VELOCITY_VFS_APPLICATIONS_URI))==0){
		return(TRUE);
	}else{
		gboolean isowner=velocity_vfs_is_owner(vfsobj);
		gboolean ingroup=velocity_vfs_is_in_group(vfsobj);
		return(((isowner)&&(vfsobj->info->permissions&GNOME_VFS_PERM_USER_WRITE))||
			((ingroup)&&(vfsobj->info->permissions&GNOME_VFS_PERM_GROUP_WRITE))||
			((!isowner)&&(!ingroup)&&(vfsobj->info->permissions&GNOME_VFS_PERM_OTHER_WRITE)));
	}
}
gboolean velocity_vfs_is_executable(velocity_vfs_obj*vfsobj){
	if(!(vfsobj->info->valid_fields&GNOME_VFS_FILE_INFO_FIELDS_PERMISSIONS)){
		return(TRUE);
	}else{
		gboolean isowner=velocity_vfs_is_owner(vfsobj);
		gboolean ingroup=velocity_vfs_is_in_group(vfsobj);
		return(((isowner)&&(vfsobj->info->permissions&GNOME_VFS_PERM_USER_EXEC))||
			((ingroup)&&(vfsobj->info->permissions&GNOME_VFS_PERM_GROUP_EXEC))||
			((!isowner)&&(!ingroup)&&(vfsobj->info->permissions&GNOME_VFS_PERM_OTHER_EXEC)));
	}
}
static gboolean velocity_vfs_is_directory_info(GnomeVFSFileInfo*info){
	return(((info->valid_fields&GNOME_VFS_FILE_INFO_FIELDS_TYPE)&&(info->type&GNOME_VFS_FILE_TYPE_DIRECTORY))&&
		((!info->mime_type)||(strncasecmp("x-directory/",info->mime_type,12)==0)));
}
gboolean velocity_vfs_is_directory(velocity_vfs_obj*vfsobj){
	return(velocity_vfs_is_directory_info(vfsobj->info));
}
gboolean velocity_vfs_is_fifo(velocity_vfs_obj*vfsobj){
	return((vfsobj->info->valid_fields&GNOME_VFS_FILE_INFO_FIELDS_TYPE)&&(vfsobj->info->type&GNOME_VFS_FILE_TYPE_FIFO));
}
gboolean velocity_vfs_is_socket(velocity_vfs_obj*vfsobj){
	return((vfsobj->info->valid_fields&GNOME_VFS_FILE_INFO_FIELDS_TYPE)&&(vfsobj->info->type&GNOME_VFS_FILE_TYPE_SOCKET));
}
gboolean velocity_vfs_is_char_dev(velocity_vfs_obj*vfsobj){
	return((vfsobj->info->valid_fields&GNOME_VFS_FILE_INFO_FIELDS_TYPE)&&(vfsobj->info->type&GNOME_VFS_FILE_TYPE_CHARACTER_DEVICE));
}
gboolean velocity_vfs_is_block_dev(velocity_vfs_obj*vfsobj){
	return((vfsobj->info->valid_fields&GNOME_VFS_FILE_INFO_FIELDS_TYPE)&&(vfsobj->info->type&GNOME_VFS_FILE_TYPE_BLOCK_DEVICE));
}
gboolean velocity_vfs_is_symlink(velocity_vfs_obj*vfsobj){
	return(GNOME_VFS_FILE_INFO_SYMLINK(vfsobj->info));
}
gboolean velocity_vfs_is_home_dir_uri(const char*uri){
	return(velocity_vfs_uris_match(uri,vprefs->locations->home));
}
gboolean velocity_vfs_is_trash_dir_uri(const char*uri){
	return(velocity_vfs_uris_match(uri,vprefs->locations->trash));
}
gboolean velocity_vfs_is_desktop_dir_uri(const char*uri){
	return(velocity_vfs_uris_match(uri,vprefs->locations->desktop));
}
gboolean velocity_vfs_is_home_dir(velocity_vfs_obj*vfsobj){
	char*uri=velocity_vfs_get_uri(vfsobj);
	gboolean ret=velocity_vfs_is_home_dir_uri(uri);
	g_free(uri);
	return(ret);
}
gboolean velocity_vfs_is_trash_dir(velocity_vfs_obj*vfsobj){
	char*uri=velocity_vfs_get_uri(vfsobj);
	gboolean ret=velocity_vfs_is_trash_dir_uri(uri);
	g_free(uri);
	return(ret);
}
gboolean velocity_vfs_is_desktop_dir(velocity_vfs_obj*vfsobj){
	char*uri=velocity_vfs_get_uri(vfsobj);
	gboolean ret=velocity_vfs_is_trash_dir_uri(uri);
	g_free(uri);
	return(ret);
}

GnomeVFSResult velocity_vfs_move(velocity_vfs_obj*vfsobj,const char*new){
	/*TODO:  Use GnomeVFS URI functions here*/
	char*orig=velocity_vfs_get_uri(vfsobj);
	char*origtext=velocity_vfs_uri_get_tail(orig);
	char*text=velocity_vfs_uri_get_tail(new);
	GnomeVFSResult ret=gnome_vfs_move(orig,new,FALSE);
	if(strcmp(origtext,text)!=0){
		if(vfsobj->icon)velocity_icon_set_caption(vfsobj->icon,text,vprefs->view->font,FALSE);
		g_free(vfsobj->info->name);
		vfsobj->info->name=g_strdup(text);
		gnome_vfs_set_file_info(new,vfsobj->info,GNOME_VFS_SET_FILE_INFO_NAME);
	}
	/*TODO:  Set the vfsobj->prefix (URI for directory)*/
	g_free(text);
	g_free(origtext);
	g_free(orig);
	return(ret);
}
GnomeVFSResult velocity_vfs_rename(velocity_vfs_obj*vfsobj,const char*newname){
	/*TODO:  Use GnomeVFS URI functions here*/
	GnomeVFSResult r;
	char*n=g_strconcat(vfsobj->prefix,newname,NULL);
	if(velocity_vfs_is_symlink(vfsobj)){
		char*o=velocity_vfs_get_uri(vfsobj);
		char*oldname=velocity_vfs_uri_get_tail(o);
		guint nl=strlen(n)-1;
		guint ol=strlen(o)-1;
		if(n[nl]=='/')n[nl]=(char)NULL;	/*I hate GnomeVFS right now....*/
		if(o[ol]=='/')o[ol]=(char)NULL;	/*I hate GnomeVFS right now....*/
		r=gnome_vfs_move(o,n,FALSE);
		if(strcmp(newname,oldname)!=0){
			if(vfsobj->icon)velocity_icon_set_caption(vfsobj->icon,newname,vprefs->view->font,FALSE);
			g_free(vfsobj->info->name);
			vfsobj->info->name=g_strdup(newname);
			gnome_vfs_set_file_info(n,vfsobj->info,GNOME_VFS_SET_FILE_INFO_NAME);
		}
		if(n[nl]==(char)NULL)n[nl]=1;	/*So it gets free'd properly*/
		if(o[ol]==(char)NULL)o[ol]=1;	/*So it gets free'd properly*/
	}else{
		r=velocity_vfs_move(vfsobj,n);
	}
	g_free(n);
	return(r);
}

gboolean velocity_vfs_can_open(velocity_vfs_obj*vfsobj){
	if((strcmp(vfsobj->info->mime_type,VELOCITY_VFS_MIME_TYPE_EXECUTABLE)==0)||(strcmp(vfsobj->info->mime_type,VELOCITY_VFS_MIME_TYPE_DIRECTORY)==0)){
		return(TRUE);
	}else{
		GnomeVFSMimeAction*act=gnome_vfs_mime_get_default_action(vfsobj->info->mime_type);
		if(act){
			gnome_vfs_mime_action_free(act);
			return(TRUE);
		}
	}
	return(FALSE);
}
char*velocity_vfs_get_mime_icon_uri(const char*mime_type,const char*uri,guint base_size){
	char*ret=NULL;
	if(uri){	/*Some hard coded method icons*/
		char*lt[]={VELOCITY_VFS_LOCAL_URI,VELOCITY_ROOT,
			VELOCITY_VFS_START_HERE_URI,"gnome-starthere",
			vprefs->locations->home,VELOCITY_HOME,
			vprefs->locations->trash,VELOCITY_TRASH,
			vprefs->locations->desktop,VELOCITY_DESKTOP,
			VELOCITY_VFS_SYSTEM_SETTINGS_URI,"gnome-system-config",
			VELOCITY_VFS_SERVER_SETTINGS_URI,"gnome-server-config",
			VELOCITY_VFS_NETWORK_URI,VELOCITY_NETWORK,
			VELOCITY_VFS_SAMBA_URI,"gnome-fs-smb",
			VELOCITY_VFS_FONTS_URI,GTK_STOCK_SELECT_FONT,
			VELOCITY_VFS_BURN_URI,"gnome-dev-cdrom",
			NULL};
		guint i=0;
		while((lt[i])&&(!ret)){
			if(velocity_vfs_uris_match(lt[i],uri)){
				ret=velocity_image_get_uri(lt[i+1],base_size);
			}
			i+=2;
		}
	}
	if(!ret){	/*if we haven't already found something*/
		if(strcasecmp(mime_type,VELOCITY_VFS_MIME_TYPE_VFOLDER)==0){	/*If we're dealing with a VFolder, try to see if it has any information*/
			char*duri=velocity_vfs_ensure_dir_uri(uri);
			char*furi=g_strconcat(duri,".directory",NULL);
			GnomeDesktopItem*di=gnome_desktop_item_new_from_uri(furi,GNOME_DESKTOP_ITEM_LOAD_ONLY_IF_EXISTS,NULL);
			if(di){
				if(gnome_desktop_item_attr_exists(di,GNOME_DESKTOP_ITEM_ICON)){
					const char*tmp=gnome_desktop_item_get_string(di,GNOME_DESKTOP_ITEM_ICON);
					ret=(tmp[0]==GNOME_VFS_URI_PATH_CHR)?g_strdup(tmp):velocity_image_get_uri(tmp,base_size);
					if((ret)&&(!g_file_test(ret,G_FILE_TEST_EXISTS))){
						ret=velocity_image_get_uri(VELOCITY_FOLDER,base_size);
					}
				}
				gnome_desktop_item_unref(di);
			}
			g_free(duri);
			g_free(furi);
		}
		if(!ret)ret=velocity_image_get_mime_type_uri(mime_type,base_size);
		if(!ret)ret=velocity_image_get_uri(VELOCITY_DEFAULT_ICON,base_size);
	}
	return(ret);
}
GnomeVFSFileSize velocity_vfs_get_free_space(char*uri){
	GnomeVFSURI*curi=gnome_vfs_uri_new(uri);
	GnomeVFSFileSize sz;
	gnome_vfs_get_volume_free_space(curi,&sz);
	gnome_vfs_uri_unref(curi);
	return(sz);
}
gboolean velocity_vfs_filter(GnomeVFSFileInfo*info,const char*shell_exp,velocity_vfs_filter_flags filter_flags){
	if((strcmp(info->name,".")==0)||(strcmp(info->name,"..")==0)||
		((filter_flags&VELOCITY_VFS_FILTER_HIDDEN)&&(info->name[0]=='.'))||
		((filter_flags&VELOCITY_VFS_FILTER_BACKUPS)&&(info->name[strlen(info->name)-1]=='~'))||
		((filter_flags&VELOCITY_VFS_FILTER_FILES)&&(!velocity_vfs_is_directory_info(info)))||
		((filter_flags&VELOCITY_VFS_FILTER_DIRS)&&(velocity_vfs_is_directory_info(info))))return(TRUE);
	if(shell_exp)return(!g_pattern_match_simple(shell_exp,info->name));
	return(FALSE);
}
GList*velocity_vfs_directory_list_load_recursive(const char*uri,GnomeVFSFileInfoOptions options){
	GList*r=NULL;
	GList*l=NULL;
	if(gnome_vfs_directory_list_load(&l,uri,options)==GNOME_VFS_OK){
		if(l){
			GList*n=l;
			while(n){
				if(n->data){
					GnomeVFSFileInfo*vfsinfo=(GnomeVFSFileInfo*)n->data;
					if((strcmp(vfsinfo->name,".")!=0)&&(strcmp(vfsinfo->name,"..")!=0)){
						if(vfsinfo->type&GNOME_VFS_FILE_TYPE_DIRECTORY){
							char*nuri=g_strconcat(uri,vfsinfo->name,NULL);
							GList*cr=velocity_vfs_directory_list_load_recursive(nuri,options);
							if(cr){
								GList*cn=cr;
								while(cn){
									r=g_list_prepend(r,cn->data);
									cn=cn->next;
								}
								g_list_free(cr);
							}
						}else{
							char*turi=velocity_vfs_ensure_dir_uri(uri);
							velocity_vfs_obj*vfsobj=velocity_vfs_obj_new(turi,gnome_vfs_file_info_dup(vfsinfo),NULL,NULL);
							r=g_list_prepend(r,(gpointer)vfsobj);
							g_free(turi);
						}
					}
					gnome_vfs_file_info_unref(vfsinfo);
				}
				n=n->next;
			}
			g_list_free(l);
		}
	}
	return(r);
}
GnomeVFSMimeApplication*velocity_vfs_mime_application_new_from_cmd(const char*cmd){
	GnomeVFSMimeApplication*r=NULL;
	GList*al=gnome_vfs_application_registry_get_applications(NULL);
	GList*n=al;
	char*bcmd=g_path_get_basename(cmd);
	while(n){
		const char*aid=(const char*)n->data;
		const char*tcmd=gnome_vfs_application_registry_peek_value(aid,GNOME_VFS_APPLICATION_REGISTRY_COMMAND);
		if(tcmd){
			if((strcmp(cmd,tcmd)==0)||(strcmp(bcmd,tcmd)==0)){
				r=gnome_vfs_application_registry_get_mime_application(aid);
				break;
			}
		}
		n=n->next;
	}
	g_list_free(al);
	g_free(bcmd);
	return(r);
}
GnomeVFSMimeApplication*velocity_vfs_mime_application_new(const char*cmd,const char*mime_type,gboolean requires_terminal,gboolean can_open_multiple_files){
	GnomeVFSMimeApplication*r=g_new0(GnomeVFSMimeApplication,1);
	char*bcmd=g_path_get_basename(cmd);
	r->command=g_strdup(cmd);
	r->name=g_strdup(bcmd);
	r->requires_terminal=requires_terminal;
	r->can_open_multiple_files=can_open_multiple_files;
	/* Can we use the app name as the id ?
	 * We know that there are no apps with the same
	 * command, so if the id is taken we are screwed
	 */
	if(!gnome_vfs_application_registry_get_mime_application(r->name)){
		r->id=g_strdup(r->name);
	}else{ /* fail safe to ensure a name */
		g_free(r->name);
		r->name=g_strdup_printf("Custom %s",mime_type);
	}
	/* If there is no id yet, make up a unique string */
	if(!r->id){
		uuid_t uuid;
		gchar uuid_str[100];
		uuid_generate(uuid);
		uuid_unparse(uuid,uuid_str);
		r->id=g_strdup(uuid_str);
	}
	gnome_vfs_application_registry_save_mime_application(r);
	gnome_vfs_application_registry_sync();	//TODO:  Possibly use the GnomeVFSResult from this...
	g_free(bcmd);
	return(r);
}

//TODO:  These two should check against supported URI types list too
void velocity_vfs_mime_application_open(GnomeVFSMimeApplication*mimeapp,char*uri){
	GList*uris=g_list_prepend(NULL,uri);
	velocity_vfs_mime_application_open_list(mimeapp,uris);
	g_list_free(uris);
}
void velocity_vfs_mime_application_open_list(GnomeVFSMimeApplication*mimeapp,GList*uris){
	guint l=g_list_length(uris);
	guint i=0;
	char**argv=(gchar**)g_malloc(sizeof(char*)*(l+1+((int)mimeapp->requires_terminal*2)));
	GList*n=uris;
	if(mimeapp->requires_terminal){
		argv[i++]=vprefs->locations->terminal;
		argv[i++]="-x";
	}
	argv[i++]=mimeapp->command;
	while(n){
		argv[i++]=(char*)n->data;
		n=n->next;
	}
	gnome_execute_async(NULL,l+1+((int)mimeapp->requires_terminal*2),argv);
	g_free(argv);
}

/*
GnomeVFSResult velocity_vfs_copy(velocity_vfs_obj*vfsobj,const char*duri)
GnomeVFSResult velocity_vfs_set_permissions(velocity_vfs_obj*vfsobj,GnomeVFSFilePermissions perms,gboolean recursive)

velocity_vfs_list_copy(GList*vfslist,const char*duri)
*/


void velocity_vfs_progress_operation_update(velocity_vfs_progress*p){
	char*tmp;
	if(p->files_done>p->total_files)p->files_done=p->total_files;	/*For some reason this actually happens...*/
	tmp=g_strdup_printf(_("%s...  (%lu of %lu)"),p->operation,p->files_done,p->total_files);
	gtk_label_set_text(GTK_LABEL(p->lblOperation),tmp);
	g_free(tmp);
}
void velocity_vfs_progress_status_update(velocity_vfs_progress*p,const char*current_file){
	char*tmp=g_strdup_printf(_("%s to %s"),current_file,(p->dest_prefix)?p->dest_prefix:"/dev/null");
	gtk_label_set_text(GTK_LABEL(p->lblStatus),tmp);
	g_free(tmp);
}
void velocity_vfs_progress_bar_update(velocity_vfs_progress*p){
	if((p->bytes_done>0)&&(p->total_bytes>0))gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(p->progress),(double)p->bytes_done/(double)p->total_bytes);
}
void velocity_vfs_progress_set_totals(velocity_vfs_progress*p,guint total_files,gulong total_bytes){
	p->total_files=total_files;
	p->total_bytes=total_bytes;
	velocity_vfs_progress_bar_update(p);
}
void velocity_vfs_progress_operation_set_string(velocity_vfs_progress*p,const char*operation_string){
	if(p->operation)g_free(p->operation);
	p->operation=g_strdup(operation_string);
	velocity_vfs_progress_operation_update(p);
}
void velocity_vfs_progress_update(velocity_vfs_progress*p){
	velocity_vfs_progress_operation_update(p);
	velocity_vfs_progress_bar_update(p);
}
static void velocity_vfs_progress_cancel_cb(GtkWidget*w,gpointer user_data){
	velocity_vfs_progress*p=(velocity_vfs_progress*)user_data;
	if(p->total_files!=p->files_done){
		if(p->on_cancel){
			p->on_cancel(p,p->user_data);
		}
	}
	velocity_vfs_progress_free(p);
}
static void velocity_vfs_xfer_cancel_cb(velocity_vfs_progress*p,gpointer user_data){
	velocity_vfs_xfer_info*xferinfo=(velocity_vfs_xfer_info*)user_data;
	xferinfo->cancelled=TRUE;
}
velocity_vfs_progress*velocity_vfs_progress_new(const char*operation,const char*src_prefix,const char*dest_prefix,guint total_files,gulong total_bytes,gpointer user_data){
	velocity_vfs_progress*pwin=g_new0(velocity_vfs_progress,1);
	GtkWidget*vbox;
	GtkWidget*lblNUKEME;
	GtkWidget*btnCancel;
	char*title=g_strdup_printf("%s...",operation);
	pwin->src_prefix=g_strdup(src_prefix);
	pwin->dest_prefix=g_strdup(dest_prefix);
	pwin->total_files=total_files;
	pwin->total_bytes=total_bytes;
	pwin->user_data=user_data;
	pwin->timer=g_timer_new();
	g_timer_start(pwin->timer);
	g_timer_reset(pwin->timer);

	pwin->widget=gtk_window_new(GTK_WINDOW_TOPLEVEL);
	g_signal_connect(pwin->widget,"delete_event",G_CALLBACK(velocity_vfs_progress_cancel_cb),pwin);
	gtk_window_set_title(GTK_WINDOW(pwin->widget),title);
	gtk_window_set_default_size(GTK_WINDOW(pwin->widget),320,200);
	
	vbox=gtk_vbox_new(FALSE,6);
	gtk_container_add(GTK_CONTAINER(pwin->widget),vbox);
	
	//TODO:  Nuke this and put an animation here or just nuke it either one....
	lblNUKEME=gtk_label_new(_("{This will be an animation representing the tasks taking place....eventually....}"));
	gtk_box_pack_start(GTK_BOX(vbox),lblNUKEME,TRUE,TRUE,0);
	gtk_label_set_line_wrap(GTK_LABEL(lblNUKEME),TRUE);

	pwin->lblOperation=gtk_label_new("");
	gtk_box_pack_start(GTK_BOX(vbox),pwin->lblOperation,FALSE,FALSE,6);
	gtk_misc_set_alignment(GTK_MISC(pwin->lblOperation),0.1,0.5);
	velocity_vfs_progress_operation_set_string(pwin,operation);

	pwin->lblStatus=gtk_label_new(_("Working..."));
	gtk_box_pack_start(GTK_BOX(vbox),pwin->lblStatus,FALSE,FALSE,6);
	
	pwin->progress=gtk_progress_bar_new();
	gtk_box_pack_start(GTK_BOX(vbox),pwin->progress,FALSE,FALSE,6);
	
	btnCancel=gtk_button_new_from_stock(GTK_STOCK_CANCEL);
	g_signal_connect(btnCancel,"clicked",G_CALLBACK(velocity_vfs_progress_cancel_cb),pwin);
	gtk_box_pack_start(GTK_BOX(vbox),btnCancel,FALSE,FALSE,6);
	gtk_container_set_border_width(GTK_CONTAINER(btnCancel),6);

	velocity_vfs_progress_set_totals(pwin,total_files,total_bytes);
	gtk_widget_show_all(pwin->widget);
	g_free(title);
	return(pwin);
}
void velocity_vfs_progress_free(velocity_vfs_progress*p){
	if(p){
		if(p->timer)g_timer_destroy(p->timer);
		if(p->operation)g_free(p->operation);
		if(p->src_prefix)g_free(p->src_prefix);
		if(p->dest_prefix)g_free(p->dest_prefix);
		if(p->widget)gtk_widget_destroy(p->widget);
		g_free(p);
	}
}
/*Some of this code was taken from Nautilus and then highly rewritten to suit Velocity*/
static int velocity_vfs_xfer_sync_cb(GnomeVFSXferProgressInfo*progress_info,gpointer user_data){
	static char*adduri=NULL;
	if(progress_info->status==GNOME_VFS_XFER_PROGRESS_STATUS_OK) {
		switch(progress_info->phase){
			case GNOME_VFS_XFER_PHASE_OPENTARGET:
				if(adduri){
					velocity_vfs_inform_event(adduri,GNOME_VFS_MONITOR_EVENT_CREATED);
					g_free(adduri);
				}
				adduri=g_strdup(progress_info->target_name);
				break;
			case GNOME_VFS_XFER_PHASE_MOVING:
				if((progress_info->source_name)&&(progress_info->target_name)){
					if(strcmp(progress_info->source_name,progress_info->target_name)!=0){
						char*tmp=gnome_vfs_unescape_string(progress_info->source_name,GNOME_VFS_URI_PATH_STR);
						velocity_vfs_inform_event(tmp,GNOME_VFS_MONITOR_EVENT_DELETED);
						velocity_vfs_inform_event(progress_info->target_name,GNOME_VFS_MONITOR_EVENT_CREATED);
						g_free(tmp);
					}
				}
				break;
			case GNOME_VFS_XFER_PHASE_DELETESOURCE:
				{
					char*tmp=gnome_vfs_unescape_string(progress_info->source_name,GNOME_VFS_URI_PATH_STR);
					velocity_vfs_inform_event(tmp,GNOME_VFS_MONITOR_EVENT_DELETED);
					g_free(tmp);
				}
				break;
			case GNOME_VFS_XFER_PHASE_COMPLETED:
				//TODO:  free the xferinfo here?
				//TODO:  This should probably be *EVENT_CHANGED
				if(adduri){
					velocity_vfs_inform_event(adduri,GNOME_VFS_MONITOR_EVENT_CREATED);
					g_free(adduri);
				}
				adduri=NULL;
				break;
			default:
				break;
		}
	}
	return(TRUE);
}
static int velocity_vfs_xfer_handle_ok(const GnomeVFSXferProgressInfo*progress_info,velocity_vfs_xfer_info*xferinfo){
	if(!xferinfo)return(0);
	if((xferinfo->cancelled)&&(progress_info->phase!=GNOME_VFS_XFER_PHASE_COMPLETED)){
		/* If cancelled, delete any partially copied files that are laying around and return. */
		if(progress_info->bytes_total!=progress_info->bytes_copied){
			GList*delete_me=g_list_prepend(NULL,progress_info->target_name);
			velocity_vfs_delete(delete_me,xferinfo->parent);
			g_list_free(delete_me);
		}
		if(xferinfo->handle)gnome_vfs_async_cancel(xferinfo->handle);
		return(0);
	}
	switch(progress_info->phase){
		case GNOME_VFS_XFER_PHASE_INITIAL:
			return(1);
		case GNOME_VFS_XFER_PHASE_COLLECTING:
			return(1);
		case GNOME_VFS_XFER_PHASE_READYTOGO:
			if(xferinfo->progress){
				velocity_vfs_progress_set_totals(xferinfo->progress,progress_info->files_total,progress_info->bytes_total);
				velocity_vfs_progress_update(xferinfo->progress);
			}
			return(1);
		case GNOME_VFS_XFER_PHASE_DELETESOURCE:
			{
				char*tmp=gnome_vfs_unescape_string(progress_info->source_name,GNOME_VFS_URI_PATH_STR);
				velocity_vfs_inform_event(tmp,GNOME_VFS_MONITOR_EVENT_DELETED);
				g_free(tmp);
			}
		case GNOME_VFS_XFER_PHASE_OPENSOURCE:
		case GNOME_VFS_XFER_PHASE_OPENTARGET:
				if(xferinfo->progress){
					char*tmp=velocity_vfs_uri_get_tail(progress_info->source_name);
					xferinfo->progress->files_done=progress_info->file_index;
					velocity_vfs_progress_status_update(xferinfo->progress,tmp);
					velocity_vfs_progress_operation_update(xferinfo->progress);
					g_free(tmp);
				}
		case GNOME_VFS_XFER_PHASE_MOVING:
		case GNOME_VFS_XFER_PHASE_COPYING:
			if(xferinfo->progress){
				xferinfo->progress->bytes_done=progress_info->total_bytes_copied;
				velocity_vfs_progress_update(xferinfo->progress);
			}
			return(1);
		case GNOME_VFS_XFER_PHASE_COMPLETED:
			if(xferinfo->on_done)xferinfo->on_done(xferinfo,xferinfo->user_data);
			if(xferinfo->progress)velocity_vfs_progress_free(xferinfo->progress);
			xferinfo->progress=NULL;
			return(1);
		default:
			return(1);
	}
}
typedef enum{VFS_ERROR_READ_ONLY,
	VFS_ERROR_NOT_READABLE,
	VFS_ERROR_NOT_WRITABLE,
	VFS_ERROR_NOT_ENOUGH_PERMISSIONS,
	VFS_ERROR_NO_SPACE,
	VFS_ERROR_OTHER}VelocityFileOperationsErrorKind;
typedef enum{VFS_ERROR_LOCATION_UNKNOWN,
	VFS_ERROR_LOCATION_SOURCE,
	VFS_ERROR_LOCATION_SOURCE_PARENT,
	VFS_ERROR_LOCATION_SOURCE_OR_PARENT,
	VFS_ERROR_LOCATION_TARGET}VelocityFileOperationsErrorLocation;

char*velocity_vfs_get_error_string(GnomeVFSResult result){
	switch(result){
		case GNOME_VFS_ERROR_NOT_FOUND:
			return(_("File not found."));
			break;
		case GNOME_VFS_ERROR_TOO_BIG:
			return(_("File too big!"));
			break;
		case GNOME_VFS_ERROR_INVALID_URI:
			return(_("Invalid URI."));
			break;
		case GNOME_VFS_ERROR_ACCESS_DENIED:
			return(_("Access denied."));
			break;
		case GNOME_VFS_ERROR_NOT_PERMITTED:
			return(_("Not permitted."));
			break;
		case GNOME_VFS_ERROR_HOST_NOT_FOUND:
			return(_("Host not found."));
			break;
		case GNOME_VFS_ERROR_INVALID_HOST_NAME:
			return(_("Invalid host name."));
			break;
		case GNOME_VFS_ERROR_HOST_HAS_NO_ADDRESS:
			return(_("Host has no address."));
			break;
		case GNOME_VFS_ERROR_LOGIN_FAILED:
			return(_("Login failed."));
			break;
		case GNOME_VFS_ERROR_TOO_MANY_LINKS:
			return(_("Too many links."));
			break;
		case GNOME_VFS_ERROR_NAME_TOO_LONG:
			return(_("Name is too long."));
			break;
		case GNOME_VFS_ERROR_SERVICE_NOT_AVAILABLE:
			return(_("Service not available."));
			break;
		case GNOME_VFS_ERROR_SERVICE_OBSOLETE:
			return(_("Service is obsolete."));
			break;
		default:
			break;
	}
	return(_("Unknown error!"));
}

static char*build_error_string(const char*source_name,const char*target_name,
	velocity_vfs_xfer_type operation_kind,VelocityFileOperationsErrorKind error_kind,
	VelocityFileOperationsErrorLocation error_location,GnomeVFSResult error){
	/*Avoid clever message composing here, just use brute force and
	* duplicate the different flavors of error messages for all the
	* possible permutations.
	* That way localizers have an easier time and can even rearrange the
	* order of the words in the messages easily.
	*/
	const char*error_string=NULL;
	char*result=NULL;
	if(error_location==VFS_ERROR_LOCATION_SOURCE_PARENT){
		switch(operation_kind){
		case VELOCITY_VFS_XFER_MOVE:
		case VELOCITY_VFS_XFER_MOVE_TO_TRASH:
			if(error_kind==VFS_ERROR_READ_ONLY){
				error_string=_("Error while moving.\n\n"
					"\"%s\" cannot be moved because it is on "
					"a read-only disk.");
			}
			break;
		case VELOCITY_VFS_XFER_DELETE:
		case VELOCITY_VFS_XFER_EMPTY_TRASH:
			switch(error_kind){
				case VFS_ERROR_NOT_ENOUGH_PERMISSIONS:
				case VFS_ERROR_NOT_WRITABLE:
					error_string=_("Error while deleting.\n\n"
						 "\"%s\" cannot be deleted because you do not have "
						 "permissions to modify its parent folder.");
					break;
				
				case VFS_ERROR_READ_ONLY:
					error_string=_("Error while deleting.\n\n"
						 "\"%s\" cannot be deleted because it is on "
						 "a read-only disk.");
					break;
				default:
					break;
			}
			break;
		default:
			g_assert_not_reached();
			break;
		}
		if(error_string){
			g_assert(source_name!=NULL);
			result=g_strdup_printf(error_string,source_name);
		}
	}else if(error_location==VFS_ERROR_LOCATION_SOURCE_OR_PARENT){
		g_assert(source_name!=NULL);
		/* FIXME: Would be better if we could distinguish source vs parent permissions
		 * better somehow. The GnomeVFS copy engine would have to do some snooping
		 * after the failure in this case.
		 */
		switch(operation_kind){
		case VELOCITY_VFS_XFER_MOVE:
			if(error_kind==VFS_ERROR_NOT_ENOUGH_PERMISSIONS){
				error_string=_("Error while moving.\n\n"
					 "\"%s\" cannot be moved because you do not have "
					 "permissions to change it or its parent folder.");
			}
			break;
		case VELOCITY_VFS_XFER_MOVE_TO_TRASH:
			if(error_kind==VFS_ERROR_NOT_ENOUGH_PERMISSIONS){
				error_string=_("Error while moving.\n\n"
					 "\"%s\" cannot be moved to the Trash because you do not have "
					 "permissions to change it or its parent folder.");
			}
			break;
		default:
			g_assert_not_reached();
			break;
		}
		if(error_string){
			g_assert(source_name!=NULL);
			result=g_strdup_printf(error_string,source_name);
		}
	}else if(error_location==VFS_ERROR_LOCATION_SOURCE){
		g_assert(source_name!=NULL);
		switch(operation_kind) {
			case VELOCITY_VFS_XFER_COPY:
			case VELOCITY_VFS_XFER_DUPLICATE:
				if(error_kind==VFS_ERROR_NOT_READABLE){
					error_string=_("Error while copying.\n\n"
						"\"%s\" cannot be copied because you do not have "
						"permissions to read it.");
				}
				break;
			default:
				g_assert_not_reached();
				break;
		}
		if(error_string){
			g_assert(source_name!=NULL);
			result=g_strdup_printf(error_string,source_name);
		}
	}else if(error_location==VFS_ERROR_LOCATION_TARGET){
		if(error_kind==VFS_ERROR_NO_SPACE){
			switch(operation_kind){
				case VELOCITY_VFS_XFER_COPY:
				case VELOCITY_VFS_XFER_DUPLICATE:
					error_string=_("Error while copying to \"%s\".\n\n"
						 "There is not enough space on the destination.");
					break;
				case VELOCITY_VFS_XFER_MOVE:
				case VELOCITY_VFS_XFER_MOVE_TO_TRASH:
					error_string=_("Error while moving to \"%s\".\n\n"
						 "There is not enough space on the destination.");
					break;
				case VELOCITY_VFS_XFER_LINK:
					error_string=_("Error while creating link in \"%s\".\n\n"
						 "There is not enough space on the destination.");
					break;
				default:
					g_assert_not_reached();
					break;
			}
		}else{
			switch(operation_kind){
				case VELOCITY_VFS_XFER_COPY:
				case VELOCITY_VFS_XFER_DUPLICATE:
					if(error_kind==VFS_ERROR_NOT_ENOUGH_PERMISSIONS){
						error_string=_("Error while copying to \"%s\".\n\n"
							 "You do not have permissions to write to "
							 "this folder.");
					}else if(error_kind==VFS_ERROR_NOT_WRITABLE){
						error_string=_("Error while copying to \"%s\".\n\n"
							 "The destination disk is read-only.");
					} 
					break;
				case VELOCITY_VFS_XFER_MOVE:
				case VELOCITY_VFS_XFER_MOVE_TO_TRASH:
					if(error_kind==VFS_ERROR_NOT_ENOUGH_PERMISSIONS){
						error_string=_("Error while moving items to \"%s\".\n\n"
							 "You do not have permissions to write to "
							 "this folder.");
					}else if(error_kind==VFS_ERROR_NOT_WRITABLE){
						error_string=_("Error while moving items to \"%s\".\n\n"
							 "The destination disk is read-only.");
					} 
					break;
				case VELOCITY_VFS_XFER_LINK:
					if(error_kind==VFS_ERROR_NOT_ENOUGH_PERMISSIONS){
						error_string=_("Error while creating links in \"%s\".\n\n"
							 "You do not have permissions to write to "
							 "this folder.");
					}else if(error_kind==VFS_ERROR_NOT_WRITABLE){
						error_string=_("Error while creating links in \"%s\".\n\n"
							 "The destination disk is read-only.");
					} 
					break;
				default:
					g_assert_not_reached();
					break;
			}
		}
		if(error_string){
			g_assert(target_name!=NULL);
			result=g_strdup_printf(error_string,target_name);
		}
	}
	if(!result){
		/* None of the specific error messages apply, use a catch-all
		 * generic error
		 */
		g_message("Please tell kyled@blueriver.net that you hit case %s while doing a file operation.",
			gnome_vfs_result_to_string(error));
		/* FIXMEs: we need to consider a single item
		 * move/copy and not offer to continue in that case
		 */
		if(source_name){
			switch(operation_kind){
				case VELOCITY_VFS_XFER_COPY:
				case VELOCITY_VFS_XFER_DUPLICATE:
					error_string=_("Error \"%s\" while copying \"%s\".\n\n"
						"Would you like to continue?");
					break;
				case VELOCITY_VFS_XFER_MOVE:
					error_string=_("Error \"%s\" while moving \"%s\".\n\n"
						"Would you like to continue?");
					break;
				case VELOCITY_VFS_XFER_LINK:
					error_string=_("Error \"%s\" while creating a link to \"%s\".\n\n"
						"Would you like to continue?");
					break;
				case VELOCITY_VFS_XFER_DELETE:
				case VELOCITY_VFS_XFER_EMPTY_TRASH:
				case VELOCITY_VFS_XFER_MOVE_TO_TRASH:
					error_string=_("Error \"%s\" while deleting \"%s\".\n\n"
						"Would you like to continue?");
					break;
				default:
					g_assert_not_reached();
					break;
			}
			result=g_strdup_printf(error_string,gnome_vfs_result_to_string(error),source_name);
		}else{
			switch(operation_kind){
				case VELOCITY_VFS_XFER_COPY:
				case VELOCITY_VFS_XFER_DUPLICATE:
					error_string=_("Error \"%s\" while copying.\n\n"
						"Would you like to continue?");
					break;
				case VELOCITY_VFS_XFER_MOVE:
					error_string=_("Error \"%s\" while moving.\n\n"
						"Would you like to continue?");
					break;
				case VELOCITY_VFS_XFER_LINK:
					error_string=_("Error \"%s\" while linking.\n\n"
						"Would you like to continue?");
					break;
				case VELOCITY_VFS_XFER_DELETE:
					error_string=_("Error \"%s\" while deleting.\n\n"
						"Would you like to continue?");
				case VELOCITY_VFS_XFER_MOVE_TO_TRASH:
					error_string=_("Error \"%s\" while trashing.\n\n"
						"Would you like to continue?");
					break;
				case VELOCITY_VFS_XFER_EMPTY_TRASH:
					error_string=_("Error \"%s\" emptying trash.\n\n"
						"Would you like to continue?");
				default:
					g_assert_not_reached();
					break;
			}
			result=g_strdup_printf(error_string,gnome_vfs_result_to_string(error));
		}
	}
	return(result);
}

static int velocity_vfs_xfer_handle_error(const GnomeVFSXferProgressInfo*progress_info,velocity_vfs_xfer_info*transfer_info){
	int error_dialog_result;
	char*text;
	char*formatted_source_name;
	char*formatted_target_name;
	const char*dialog_title;
	VelocityFileOperationsErrorKind error_kind;
	VelocityFileOperationsErrorLocation error_location;
	GtkWidget*dlg;
	VELOCITY_MSG("error handling %s! (VFSResult: %s,Type: %i,Phase: %i,Status: %i)\n",progress_info->source_name,gnome_vfs_result_to_string(progress_info->vfs_status),transfer_info->type,progress_info->phase,progress_info->vfs_status);
	//return(GNOME_VFS_XFER_ERROR_ACTION_SKIP);
	switch(transfer_info->error_mode){
	case GNOME_VFS_XFER_ERROR_MODE_QUERY:
		// transfer error, prompt the user to continue or stop
		formatted_source_name = NULL;
		formatted_target_name = NULL;
		if(progress_info->source_name){
			formatted_source_name=velocity_vfs_uri_get_tail(progress_info->source_name);
		}
		if(progress_info->target_name){
			formatted_target_name=velocity_vfs_uri_get_tail(progress_info->target_name);
		}
		error_kind=VFS_ERROR_OTHER;
		error_location=VFS_ERROR_LOCATION_UNKNOWN;
		// Single out a few common error conditions for which we have custom-taylored error messages.
		if((progress_info->vfs_status==GNOME_VFS_ERROR_READ_ONLY_FILE_SYSTEM || progress_info->vfs_status==GNOME_VFS_ERROR_READ_ONLY)
				&& (transfer_info->type==VELOCITY_VFS_XFER_DELETE || transfer_info->type==VELOCITY_VFS_XFER_EMPTY_TRASH)){
			error_location=VFS_ERROR_LOCATION_SOURCE_PARENT;
			error_kind=VFS_ERROR_READ_ONLY;
		}else if(progress_info->vfs_status==GNOME_VFS_ERROR_ACCESS_DENIED &&
				(transfer_info->type==VELOCITY_VFS_XFER_DELETE || transfer_info->type==VELOCITY_VFS_XFER_EMPTY_TRASH)){
			error_location=VFS_ERROR_LOCATION_SOURCE_PARENT;
			error_kind=VFS_ERROR_NOT_ENOUGH_PERMISSIONS;
		}else if((progress_info->vfs_status==GNOME_VFS_ERROR_READ_ONLY_FILE_SYSTEM || progress_info->vfs_status==GNOME_VFS_ERROR_READ_ONLY)
				&& (transfer_info->type==VELOCITY_VFS_XFER_MOVE || transfer_info->type==VELOCITY_VFS_XFER_MOVE_TO_TRASH)
				&& progress_info->phase!=GNOME_VFS_XFER_CHECKING_DESTINATION){
			error_location=VFS_ERROR_LOCATION_SOURCE_PARENT;
			error_kind=VFS_ERROR_READ_ONLY;
		}else if(progress_info->vfs_status==GNOME_VFS_ERROR_ACCESS_DENIED && transfer_info->type==VELOCITY_VFS_XFER_MOVE &&
				progress_info->phase==GNOME_VFS_XFER_PHASE_OPENTARGET) {
			error_location=VFS_ERROR_LOCATION_TARGET;
			error_kind=VFS_ERROR_NOT_ENOUGH_PERMISSIONS;
		}else if(progress_info->vfs_status==GNOME_VFS_ERROR_ACCESS_DENIED && (transfer_info->type==VELOCITY_VFS_XFER_MOVE
				|| transfer_info->type==VELOCITY_VFS_XFER_MOVE_TO_TRASH) && progress_info->phase!=GNOME_VFS_XFER_CHECKING_DESTINATION){
			error_location=VFS_ERROR_LOCATION_SOURCE_OR_PARENT;
			error_kind=VFS_ERROR_NOT_ENOUGH_PERMISSIONS;
		}else if(progress_info->vfs_status==GNOME_VFS_ERROR_ACCESS_DENIED
			&& (transfer_info->type==VELOCITY_VFS_XFER_COPY
				|| transfer_info->type==VELOCITY_VFS_XFER_DUPLICATE)
			&& (progress_info->phase==GNOME_VFS_XFER_PHASE_OPENSOURCE
				|| progress_info->phase==GNOME_VFS_XFER_PHASE_COLLECTING
				|| progress_info->phase==GNOME_VFS_XFER_PHASE_INITIAL)) {
			error_location=VFS_ERROR_LOCATION_SOURCE;
			error_kind=VFS_ERROR_NOT_READABLE;
		}else if((progress_info->vfs_status==GNOME_VFS_ERROR_READ_ONLY_FILE_SYSTEM
				|| progress_info->vfs_status==GNOME_VFS_ERROR_READ_ONLY)
			&& progress_info->phase==GNOME_VFS_XFER_CHECKING_DESTINATION){
			error_location=VFS_ERROR_LOCATION_TARGET;
			error_kind=VFS_ERROR_NOT_WRITABLE;
		}else if(progress_info->vfs_status==GNOME_VFS_ERROR_ACCESS_DENIED
			&& progress_info->phase==GNOME_VFS_XFER_CHECKING_DESTINATION){
			error_location=VFS_ERROR_LOCATION_TARGET;
			error_kind=VFS_ERROR_NOT_ENOUGH_PERMISSIONS;
		}else if(progress_info->vfs_status==GNOME_VFS_ERROR_NO_SPACE){
			error_location=VFS_ERROR_LOCATION_TARGET;
			error_kind=VFS_ERROR_NO_SPACE;
		}
		text=build_error_string(formatted_source_name,formatted_target_name,transfer_info->type,error_kind,error_location,progress_info->vfs_status);
		if(error_location==VFS_ERROR_LOCATION_TARGET) {
			/*We can't continue, just tell the user.*/
			dlg=gtk_message_dialog_new((transfer_info->progress->widget)?GTK_WINDOW(transfer_info->progress->widget):NULL,GTK_DIALOG_MODAL,GTK_MESSAGE_ERROR,GTK_BUTTONS_CLOSE,text);
			gtk_dialog_run(GTK_DIALOG(dlg));
			gtk_widget_destroy(dlg);
			error_dialog_result=GNOME_VFS_XFER_ERROR_ACTION_ABORT;
		}else if((error_location==VFS_ERROR_LOCATION_SOURCE || error_location==VFS_ERROR_LOCATION_SOURCE_PARENT || error_location==VFS_ERROR_LOCATION_SOURCE_OR_PARENT)
			&&(error_kind==VFS_ERROR_NOT_ENOUGH_PERMISSIONS	|| error_kind==VFS_ERROR_NOT_READABLE)){
			/*The error could have happened on any of the files in the moved/copied/deleted hierarchy, we can probably continue. Allow the user to skip.*/
			dlg=gtk_message_dialog_new((transfer_info->progress->widget)?GTK_WINDOW(transfer_info->progress->widget):NULL,GTK_DIALOG_MODAL,GTK_MESSAGE_WARNING,GTK_BUTTONS_NONE,text);
			gtk_dialog_add_buttons(GTK_DIALOG(dlg),_("Skip"),0,_("Stop"),1,NULL);
			switch(gtk_dialog_run(GTK_DIALOG(dlg))){
				case 0:
					error_dialog_result=GNOME_VFS_XFER_ERROR_ACTION_SKIP;
					break;
				case 1:
					error_dialog_result=GNOME_VFS_XFER_ERROR_ACTION_ABORT;
					break;
				default:
					g_assert_not_reached();
					error_dialog_result=GNOME_VFS_XFER_ERROR_ACTION_ABORT;
			}
			gtk_widget_destroy(dlg);
		}else{
			/*Generic error, offer to retry and skip.*/
			dlg=gtk_message_dialog_new((transfer_info->progress->widget)?GTK_WINDOW(transfer_info->progress->widget):NULL,GTK_DIALOG_MODAL,GTK_MESSAGE_WARNING,GTK_BUTTONS_NONE,text);
			gtk_dialog_add_buttons(GTK_DIALOG(dlg),_("Skip"),0,_("Retry"),1,_("Stop"),2,NULL);
			switch(gtk_dialog_run(GTK_DIALOG(dlg))){
				case 0:
					error_dialog_result=GNOME_VFS_XFER_ERROR_ACTION_SKIP;
					break;
				case 1:
					error_dialog_result=GNOME_VFS_XFER_ERROR_ACTION_RETRY;
					break;
				case 2:
					error_dialog_result=GNOME_VFS_XFER_ERROR_ACTION_ABORT;
					break;
				default:
					g_assert_not_reached();
					error_dialog_result=GNOME_VFS_XFER_ERROR_ACTION_ABORT;
			}
			gtk_widget_destroy(dlg);
		}
		g_free(text);
		g_free(formatted_source_name);
		g_free(formatted_target_name);
		return(error_dialog_result);
	case GNOME_VFS_XFER_ERROR_MODE_ABORT:
	default:
		return(GNOME_VFS_XFER_ERROR_ACTION_ABORT);
	}
}

/* Localizers: 
 * Feel free to leave out the st, nd, rd and th suffix or
 * make some or all of them match.
 */
/* localizers: tag used to detect the first copy of a file */
const char untranslated_copy_duplicate_tag[] = N_(" (copy)");
/* localizers: tag used to detect the second copy of a file */
const char untranslated_another_copy_duplicate_tag[] = N_(" (another copy)");
/* localizers: tag used to detect the x1st copy of a file */
const char untranslated_st_copy_duplicate_tag[] = N_("st copy)");
/* localizers: tag used to detect the x2nd copy of a file */
const char untranslated_nd_copy_duplicate_tag[] = N_("nd copy)");
/* localizers: tag used to detect the x3rd copy of a file */
const char untranslated_rd_copy_duplicate_tag[] = N_("rd copy)");
/* localizers: tag used to detect the xxth copy of a file */
const char untranslated_th_copy_duplicate_tag[] = N_("th copy)");

#define COPY_DUPLICATE_TAG _(untranslated_copy_duplicate_tag)
#define ANOTHER_COPY_DUPLICATE_TAG _(untranslated_another_copy_duplicate_tag)
#define ST_COPY_DUPLICATE_TAG _(untranslated_st_copy_duplicate_tag)
#define ND_COPY_DUPLICATE_TAG _(untranslated_nd_copy_duplicate_tag)
#define RD_COPY_DUPLICATE_TAG _(untranslated_rd_copy_duplicate_tag)
#define TH_COPY_DUPLICATE_TAG _(untranslated_th_copy_duplicate_tag)

/* localizers: appended to first file copy */
static const char untranslated_first_copy_duplicate_format[] = N_("%s (copy)%s");
/* localizers: appended to second file copy */
static const char untranslated_second_copy_duplicate_format[] = N_("%s (another copy)%s");
/* localizers: appended to x1st file copy */
static const char untranslated_st_copy_duplicate_format[] = N_("%s (%dst copy)%s");
/* localizers: appended to x2nd file copy */
static const char untranslated_nd_copy_duplicate_format[] = N_("%s (%dnd copy)%s");
/* localizers: appended to x3rd file copy */
static const char untranslated_rd_copy_duplicate_format[] = N_("%s (%drd copy)%s");
/* localizers: appended to xxth file copy */
static const char untranslated_th_copy_duplicate_format[] = N_("%s (%dth copy)%s");

#define FIRST_COPY_DUPLICATE_FORMAT _(untranslated_first_copy_duplicate_format)
#define SECOND_COPY_DUPLICATE_FORMAT _(untranslated_second_copy_duplicate_format)
#define ST_COPY_DUPLICATE_FORMAT _(untranslated_st_copy_duplicate_format)
#define ND_COPY_DUPLICATE_FORMAT _(untranslated_nd_copy_duplicate_format)
#define RD_COPY_DUPLICATE_FORMAT _(untranslated_rd_copy_duplicate_format)
#define TH_COPY_DUPLICATE_FORMAT _(untranslated_th_copy_duplicate_format)

char *extract_string_until(const char *original, const char *until_substring){
	char *result;
	g_assert((int)strlen(original)>=until_substring-original);
	g_assert(until_substring-original>=0);
	result=g_malloc (until_substring-original+1);
	strncpy(result,original,until_substring-original);
	result[until_substring-original]='\0';
	return(result);
}

/* Dismantle a file name, separating the base name, the file suffix and removing any
 * (xxxcopy), etc. string. Figure out the count that corresponds to the given
 * (xxxcopy) substring.
 */
void parse_previous_duplicate_name(const char *name,char **name_base,const char **suffix,int *count){
	const char *tag;
	g_assert (name[0] != '\0');
	*suffix = strrchr (name + 1, '.');
	if (*suffix == NULL || (*suffix)[1] == '\0') {
		/* no suffix */
		*suffix = "";
	}

	tag = strstr (name, COPY_DUPLICATE_TAG);
	if (tag != NULL) {
		if (tag > *suffix) {
			/* handle case "foo. (copy)" */
			*suffix = "";
		}
		*name_base = extract_string_until (name, tag);
		*count = 1;
		return;
	}

	tag = strstr (name, ANOTHER_COPY_DUPLICATE_TAG);
	if (tag != NULL) {
		if (tag > *suffix) {
			/* handle case "foo. (another copy)" */
			*suffix = "";
		}
		*name_base = extract_string_until (name, tag);
		*count = 2;
		return;
	}

	/* Check to see if we got one of st, nd, rd, th. */
	tag = strstr (name, ST_COPY_DUPLICATE_TAG);
	if (tag == NULL) {
		tag = strstr (name, ND_COPY_DUPLICATE_TAG);
	}
	if (tag == NULL) {
		tag = strstr (name, RD_COPY_DUPLICATE_TAG);
	}
	if (tag == NULL) {
		tag = strstr (name, TH_COPY_DUPLICATE_TAG);
	}

	/* If we got one of st, nd, rd, th, fish out the duplicate number. */
	if (tag != NULL) {
		/* localizers: opening parentheses to match the "th copy)" string */
		tag = strstr (name, _(" ("));
		if (tag != NULL) {
			if (tag > *suffix) {
				/* handle case "foo. (22nd copy)" */
				*suffix = "";
			}
			*name_base = extract_string_until (name, tag);
			/* localizers: opening parentheses of the "th copy)" string */
			if (sscanf (tag, _(" (%d"), count) == 1) {
				if (*count < 1 || *count > 1000000) {
					/* keep the count within a reasonable range */
					*count = 0;
				}
				return;
			}
			*count = 0;
			return;
		}
	}

	*count = 0;
	if (**suffix != '\0') {
		*name_base = extract_string_until (name, *suffix);
	} else {
		*name_base = g_strdup (name);
	}
}

char *make_next_duplicate_name(const char *base, const char *suffix, int count){
        const char *format;
        char *result;
        if (count < 1) {
                g_warning ("bad count %d in get_duplicate_name", count);
                count = 1;
        }
        if (count <= 2) {
                /* Handle special cases for low numbers.
                 * Perhaps for some locales we will need to add more.
                 */
                switch (count) {
                default:
                        g_assert_not_reached ();
                        /* fall through */
                case 1:
                        format = FIRST_COPY_DUPLICATE_FORMAT;
                        break;
                case 2:
                        format = SECOND_COPY_DUPLICATE_FORMAT;
                        break;
                }
                result = g_strdup_printf (format, base, suffix);
        } else {
                /* Handle special cases for the first few numbers of each ten.
                 * For locales where getting this exactly right is difficult,
                 * these can just be made all the same as the general case below.
                 */
                switch (count % 10) {
                case 1:
                        format = ST_COPY_DUPLICATE_FORMAT;
                        break;
                case 2:
                        format = ND_COPY_DUPLICATE_FORMAT;
                        break;
                case 3:
                        format = RD_COPY_DUPLICATE_FORMAT;
                        break;
                default:
                        /* The general case. */
                        format = TH_COPY_DUPLICATE_FORMAT;
                        break;
                }
                result = g_strdup_printf (format, base, count, suffix);
        }
        return result;
}
char *get_duplicate_name (const char *name, int count_increment){
	char *result;
	char *name_base;
	const char *suffix;
	int count;
	parse_previous_duplicate_name (name, &name_base, &suffix, &count);
	result = make_next_duplicate_name (name_base, suffix, count + count_increment);
	g_free (name_base);
	return result;
}
char *get_next_duplicate_name(char *name, int count_increment){
	char *unescaped_name;
	char *unescaped_result;
	char *result;
	unescaped_name = gnome_vfs_unescape_string(name, GNOME_VFS_URI_PATH_STR);
	g_free(name);
	unescaped_result = get_duplicate_name(unescaped_name, count_increment);
	g_free(unescaped_name);
	result = gnome_vfs_escape_path_string(unescaped_result);
	g_free(unescaped_result);
	return result;
}
char*get_link_name(char *name, int count){
	char *result;
	char *unescaped_name;
	char *unescaped_result;
	const char *format;
	g_assert (name != NULL);
	unescaped_name = gnome_vfs_unescape_string (name, GNOME_VFS_URI_PATH_STR);
	g_free (name);
	if (count < 1) {
			 g_warning ("bad count in get_link_name");
			 count = 1;
	}
	if (count <= 2) {
			 /* Handle special cases for low numbers.
			  * Perhaps for some locales we will need to add more.
			  */
			 switch (count) {
			 default:
						g_assert_not_reached ();
						/* fall through */
			 case 1:
						/* appended to new link file */
						format = _("link to %s");
						break;
			 case 2:
						/* appended to new link file */
						format = _("another link to %s");
						break;
			 }
			 unescaped_result = g_strdup_printf (format, unescaped_name);
	} else {
			 /* Handle special cases for the first few numbers of each ten.
			  * For locales where getting this exactly right is difficult,
			  * these can just be made all the same as the general case below.
			  */
			 switch (count % 10) {
			 case 1:
						/* Localizers: Feel free to leave out the "st" suffix
						 * if there's no way to do that nicely for a
						 * particular language.
						 */
						format = _("%dst link to %s");
						break;
			 case 2:
						/* appended to new link file */
						format = _("%dnd link to %s");
						break;
			 case 3:
						/* appended to new link file */
						format = _("%drd link to %s");
						break;
			 default:
						/* appended to new link file */
						format = _("%dth link to %s");
						break;
			 }
			 unescaped_result = g_strdup_printf (format, count, unescaped_name);
	}
	result = gnome_vfs_escape_path_string (unescaped_result);
	g_free (unescaped_name);
	g_free (unescaped_result);
	return result;
}
static int velocity_vfs_xfer_handle_duplicate(GnomeVFSXferProgressInfo *progress_info,velocity_vfs_xfer_info *transfer_info){
	VELOCITY_MSG("THIS ISN'T DONE!");
	switch(transfer_info->type){
		case VELOCITY_VFS_XFER_LINK:
			progress_info->duplicate_name=get_link_name(progress_info->duplicate_name,progress_info->duplicate_count);
			break;
		case VELOCITY_VFS_XFER_COPY:
		case VELOCITY_VFS_XFER_MOVE_TO_TRASH:
			progress_info->duplicate_name=get_next_duplicate_name(progress_info->duplicate_name,progress_info->duplicate_count);
			break;
		default:
			/* For all other cases we use the name as-is. */
			break;
	}
	return(GNOME_VFS_XFER_ERROR_ACTION_SKIP);
}
static gboolean velocity_vfs_is_special_dir_uri(const char*uri){
	void*checklist[]={velocity_vfs_is_home_dir_uri,velocity_vfs_is_trash_dir_uri,velocity_vfs_is_desktop_dir_uri,NULL};
	GnomeVFSURI*guri=gnome_vfs_uri_new(uri);
	char*tmpuri=gnome_vfs_uri_to_string(guri,GNOME_VFS_URI_HIDE_NONE);
	gboolean ret=FALSE;
	guint f=0;
	while((checklist[f])&&(!ret)){
		gboolean(*func)(const char*uri)=checklist[f];
		if(func(uri))ret=TRUE;
		f++;
	}
	g_free(tmpuri);
	gnome_vfs_uri_unref(guri);
	return(ret);
}
static int velocity_vfs_xfer_handle_overwrite(const GnomeVFSXferProgressInfo*progress_info,velocity_vfs_xfer_info*transfer_info){
	char*text;
	char*formatted_name;
	/* Handle special case files such as Trash, mount links and home directory */
	VELOCITY_MSG("uri=%s",progress_info->target_name);
	if(velocity_vfs_is_special_dir_uri(progress_info->target_name)){
		GtkWidget*dlg;
		formatted_name=velocity_vfs_uri_get_tail(progress_info->target_name);
		if(transfer_info->type==VELOCITY_VFS_XFER_MOVE) {
			text=g_strdup_printf(_("\"%s\" could not be moved to the new location, because its name is already used for a special item that "
				"cannot be removed or replaced.\n\nIf you still want to move \"%s\", rename it and try again."),formatted_name,formatted_name);
		}else{
			text=g_strdup_printf(_("\"%s\" could not be copied to the new location, because its name is already used for a special item that "
				"cannot be removed or replaced.\n\nIf you still want to copy \"%s\", rename it and try again."),formatted_name,formatted_name);
		}
		dlg=gtk_message_dialog_new((transfer_info->progress->widget)?GTK_WINDOW(transfer_info->progress->widget):NULL,GTK_DIALOG_MODAL,GTK_MESSAGE_WARNING,GTK_BUTTONS_OK,text);
		gtk_dialog_run(GTK_DIALOG(dlg));
		gtk_widget_destroy(dlg);
		g_free(text);
		g_free(formatted_name);
		return(GNOME_VFS_XFER_OVERWRITE_ACTION_SKIP);
	}else{
		/* transfer conflict, prompt the user to replace or skip */
		formatted_name=velocity_vfs_uri_get_tail(progress_info->target_name);
		text=g_strdup_printf(_("File \"%s\" already exists.\n\nWould you like to replace it?"),formatted_name);
		g_free(formatted_name);
		if(progress_info->duplicate_count==1){
			/* we are going to only get one duplicate alert, don't offer Replace All */
			GtkWidget*dlg=gtk_message_dialog_new((transfer_info->progress->widget)?GTK_WINDOW(transfer_info->progress->widget):NULL,GTK_DIALOG_MODAL,GTK_MESSAGE_WARNING,GTK_BUTTONS_NONE,text);
			gint r;
			gtk_dialog_add_buttons(GTK_DIALOG(dlg),_("Replace"),0,_("Skip"),1,NULL);
			r=gtk_dialog_run(GTK_DIALOG(dlg));
			gtk_widget_destroy(dlg);
			switch(r){
				case 0:
					VELOCITY_MSG("Replace");
					return(GNOME_VFS_XFER_OVERWRITE_ACTION_REPLACE);
				case 1:
					VELOCITY_MSG("Skip");
					return(GNOME_VFS_XFER_OVERWRITE_ACTION_SKIP);
				default:
					g_assert_not_reached();
					/* fall through */
			}
		}else{
			GtkWidget*dlg=gtk_message_dialog_new((transfer_info->progress->widget)?GTK_WINDOW(transfer_info->progress->widget):NULL,GTK_DIALOG_MODAL,GTK_MESSAGE_WARNING,GTK_BUTTONS_NONE,text);
			gint r;
			gtk_dialog_add_buttons(GTK_DIALOG(dlg),_("Replace All"),0,_("Replace"),1,_("Skip"),2,NULL);
			r=gtk_dialog_run(GTK_DIALOG(dlg));
			gtk_widget_destroy(dlg);
			switch(r){
				case 0:
					VELOCITY_MSG("Replace All");
					return(GNOME_VFS_XFER_OVERWRITE_ACTION_REPLACE_ALL);
				case 1:
					VELOCITY_MSG("Replace");
					return(GNOME_VFS_XFER_OVERWRITE_ACTION_REPLACE);
				case 2:
					VELOCITY_MSG("Skip");
					return(GNOME_VFS_XFER_OVERWRITE_ACTION_SKIP);
				default:
					g_assert_not_reached();
					/* fall through */
			}
		}
	}
	return(1);
}
static int velocity_vfs_xfer_handler_cb(GnomeVFSAsyncHandle*handle,GnomeVFSXferProgressInfo*progress_info,gpointer user_data){
	velocity_vfs_xfer_info*xferinfo=(velocity_vfs_xfer_info*)user_data;
	switch(progress_info->status){
		case GNOME_VFS_XFER_PROGRESS_STATUS_OK:
			return(velocity_vfs_xfer_handle_ok(progress_info,xferinfo));
		case GNOME_VFS_XFER_PROGRESS_STATUS_VFSERROR:
			return(velocity_vfs_xfer_handle_error(progress_info,xferinfo));
		case GNOME_VFS_XFER_PROGRESS_STATUS_OVERWRITE:
			return(velocity_vfs_xfer_handle_overwrite(progress_info,xferinfo));
		case GNOME_VFS_XFER_PROGRESS_STATUS_DUPLICATE:
			return(velocity_vfs_xfer_handle_duplicate(progress_info,xferinfo));
		default:
			g_warning(_("Unknown GnomeVFSXferProgressStatus %d"),progress_info->status);
			return(0);
	}
}
void velocity_vfs_xfer(GtkWindow*window,GList*srcurilst,GList*desturilst,velocity_vfs_xfer_type action,gboolean show_progress_dialog,void*on_done,gpointer user_data){
	velocity_vfs_xfer_info*xferinfo=g_new0(velocity_vfs_xfer_info,1);
	GnomeVFSXferOptions xferoptions=GNOME_VFS_XFER_DEFAULT|GNOME_VFS_XFER_RECURSIVE;
	GnomeVFSURI*uri=NULL;
	char*op=NULL;
	char*srcprefix=NULL;
	char*destprefix=NULL;
	char*tmp=NULL;
	uri=(GnomeVFSURI*)srcurilst->data;
	xferinfo->local_src=gnome_vfs_uri_is_local(uri);
	tmp=velocity_vfs_get_uri_parent(uri->text);
	srcprefix=velocity_vfs_uri_reformat(tmp);
	g_free(tmp);
	uri=(GnomeVFSURI*)desturilst->data;
	xferinfo->local_dest=gnome_vfs_uri_is_local(uri);
	tmp=velocity_vfs_get_uri_parent(uri->text);
	destprefix=velocity_vfs_uri_reformat(tmp);
	g_free(tmp);
	switch(action){
		case VELOCITY_VFS_XFER_DELETE:
			op=_("Removing");
			xferoptions=GNOME_VFS_XFER_DELETE_ITEMS;
			break;
		case VELOCITY_VFS_XFER_MOVE:
			if((xferinfo->local_src)&&(!xferinfo->local_dest)){
				op=_("Uploading");
			}else if((!xferinfo->local_src)&&(xferinfo->local_dest)){
				op=_("Downloading");
			}else{
				op=_("Moving");
			}
			xferoptions|=GNOME_VFS_XFER_REMOVESOURCE;
			break;
		case VELOCITY_VFS_XFER_MOVE_TO_TRASH:
			op=_("Trashing");
			xferoptions|=GNOME_VFS_XFER_REMOVESOURCE;
			break;
		case VELOCITY_VFS_XFER_LINK:
			op=_("Symlinking");
			xferoptions|=GNOME_VFS_XFER_LINK_ITEMS;
			break;
		case VELOCITY_VFS_XFER_DUPLICATE:
			op=_("Duplicating");
			xferoptions|=GNOME_VFS_XFER_USE_UNIQUE_NAMES;
			break;
		case VELOCITY_VFS_XFER_COPY:
		default:
			if((xferinfo->local_src)&&(!xferinfo->local_dest)){
				op=_("Uploading");
			}else if((!xferinfo->local_src)&&(xferinfo->local_dest)){
				op=_("Downloading");
			}else{
				op=_("Copying");
			}
			break;
	}
	xferinfo->parent=window;
	xferinfo->type=action;
	xferinfo->error_mode=GNOME_VFS_XFER_ERROR_MODE_QUERY;
	xferinfo->overwrite_mode=GNOME_VFS_XFER_OVERWRITE_MODE_QUERY;
	xferinfo->user_data=user_data;
	xferinfo->srclist=srcurilst;
	xferinfo->destlist=desturilst;
	xferinfo->on_done=on_done;
	if(show_progress_dialog)xferinfo->progress=velocity_vfs_progress_new(op,srcprefix,destprefix,g_list_length(srcurilst),1,(gpointer)xferinfo);
	gnome_vfs_async_xfer(&xferinfo->handle,srcurilst,desturilst,xferoptions,
		GNOME_VFS_XFER_ERROR_MODE_QUERY,GNOME_VFS_XFER_OVERWRITE_MODE_QUERY,GNOME_VFS_PRIORITY_DEFAULT,
		velocity_vfs_xfer_handler_cb,(gpointer)xferinfo,velocity_vfs_xfer_sync_cb,(gpointer)xferinfo);
	g_free(srcprefix);
	g_free(destprefix);
}
int velocity_vfs_xfer_newfolder_cb(GnomeVFSAsyncHandle*handle,GnomeVFSXferProgressInfo*progress_info,gpointer data){
	velocity_vfs_xfer_new_folder_info*state=(velocity_vfs_xfer_new_folder_info*)data;
	char*tmp=NULL;
	if(progress_info->phase==GNOME_VFS_XFER_PHASE_COMPLETED){
		g_free(state);
		return(0);
	}
	switch(progress_info->status) {
		case GNOME_VFS_XFER_PROGRESS_STATUS_OK:
			/*TODO: UPDATE MONITORS HERE*/
			if(state->on_done){
				state->on_done(progress_info->target_name,state->user_data);
			}
			return(0);
		 case GNOME_VFS_XFER_PROGRESS_STATUS_DUPLICATE:
			tmp=progress_info->duplicate_name;
			if(progress_info->vfs_status==GNOME_VFS_ERROR_NAME_TOO_LONG){
				/* special case an 8.3 file system */
				progress_info->duplicate_name=g_strndup(tmp, 8);
				progress_info->duplicate_name[8]='\0';
				g_free(tmp);
				tmp=progress_info->duplicate_name;
				progress_info->duplicate_name=g_strdup_printf("%s.%d",progress_info->duplicate_name,progress_info->duplicate_count);
			} else {
				progress_info->duplicate_name=g_strdup_printf("%s%%20%d",progress_info->duplicate_name,progress_info->duplicate_count);
			}
			g_free(tmp);
			return(GNOME_VFS_XFER_ERROR_ACTION_SKIP);
		case GNOME_VFS_XFER_PROGRESS_STATUS_VFSERROR:
			switch(progress_info->vfs_status){
				case GNOME_VFS_ERROR_ACCESS_DENIED:
					tmp=_("Error creating new folder.\n\nYou do not have permissions to write to the destination.");
				case GNOME_VFS_ERROR_NO_SPACE:
					tmp=_("Error creating new folder.\n\nThere is no space on the destination.");
					break;
				default:
					tmp=g_strdup_printf (_("Error \"%s\" creating new folder."),gnome_vfs_result_to_string(progress_info->vfs_status));
			}
//eel_show_error_dialog(tmp,_("Error creating new folder"),GTK_WINDOW(state->pwindow));
			g_free(tmp);
			return(GNOME_VFS_XFER_ERROR_ACTION_ABORT);
		default:
			g_warning(_("Unknown GnomeVFSXferProgressStatus %d"),progress_info->status);
			return 0;
	}
}
void velocity_vfs_xfer_new_folder(GtkWindow*pwindow,const char*parent_dir,velocity_vfs_xfer_object_done_cb done_callback,gpointer user_data){
	GList*target_uri_list;
	GnomeVFSURI*uri;
	GnomeVFSURI*parent_uri;
	velocity_vfs_xfer_new_folder_info*state=g_new0(velocity_vfs_xfer_new_folder_info,1);
	state->parent=pwindow;
	state->on_done=done_callback;
	state->user_data=user_data;
	/* pass in the target directory and the new folder name as a destination URI */
	parent_uri=gnome_vfs_uri_new(parent_dir);
	/* localizers: the initial name of a new folder  */
	uri=gnome_vfs_uri_append_file_name(parent_uri, _("New Folder"));
	target_uri_list=g_list_prepend(NULL,uri);
	gnome_vfs_async_xfer(&state->handle,NULL,target_uri_list,GNOME_VFS_XFER_NEW_UNIQUE_DIRECTORY,
		GNOME_VFS_XFER_ERROR_MODE_QUERY,GNOME_VFS_XFER_OVERWRITE_MODE_QUERY,GNOME_VFS_PRIORITY_DEFAULT,
	    velocity_vfs_xfer_newfolder_cb,state,
	    velocity_vfs_xfer_sync_cb,NULL);
	gnome_vfs_uri_list_free(target_uri_list);
	gnome_vfs_uri_unref(parent_uri);
}
void velocity_vfs_delete(GList*item_uris,GtkWindow*parent_view){
	velocity_vfs_xfer_info*xferinfo=g_new0(velocity_vfs_xfer_info,1);
	GList*n=item_uris;
	GList*uri_list=NULL;
	while(n){
		uri_list=g_list_prepend(uri_list,gnome_vfs_uri_new((const char*)n->data));
		n=n->next;
	}
	uri_list=g_list_reverse(uri_list);

	xferinfo->parent=parent_view;
	xferinfo->show_progress_dialog=TRUE;

	xferinfo->error_mode=GNOME_VFS_XFER_ERROR_MODE_QUERY;
	xferinfo->overwrite_mode=GNOME_VFS_XFER_OVERWRITE_MODE_REPLACE;
	xferinfo->type=VELOCITY_VFS_XFER_DELETE;
	
	gnome_vfs_async_xfer(&xferinfo->handle,uri_list,NULL,GNOME_VFS_XFER_DELETE_ITEMS|GNOME_VFS_XFER_RECURSIVE,
		GNOME_VFS_XFER_ERROR_MODE_QUERY,GNOME_VFS_XFER_OVERWRITE_MODE_REPLACE,GNOME_VFS_PRIORITY_DEFAULT,
		velocity_vfs_xfer_handler_cb,(gpointer)xferinfo,
		velocity_vfs_xfer_sync_cb,(gpointer)xferinfo);
	gnome_vfs_uri_list_free(uri_list);
}
void velocity_vfs_empty_trash(GtkWindow*parent,gboolean showprogress,void*on_done,gpointer user_data){
	if(vprefs->locations->trash){
		velocity_vfs_xfer_info*xferinfo=g_new0(velocity_vfs_xfer_info,1);
		GnomeVFSURI*tguri=gnome_vfs_uri_new(vprefs->locations->trash);
		xferinfo->parent=parent;
		xferinfo->type=VELOCITY_VFS_XFER_EMPTY_TRASH;
		xferinfo->error_mode=GNOME_VFS_XFER_ERROR_MODE_QUERY;
		xferinfo->overwrite_mode=GNOME_VFS_XFER_OVERWRITE_MODE_QUERY;
		xferinfo->srclist=g_list_prepend(NULL,(gpointer)tguri);
		xferinfo->destlist=NULL;
		xferinfo->user_data=user_data;
		xferinfo->on_done=on_done;
		if(showprogress)xferinfo->progress=velocity_vfs_progress_new(_("Emptying"),"",NULL,g_list_length(xferinfo->srclist),1,(gpointer)xferinfo);
		gnome_vfs_async_xfer(&xferinfo->handle,xferinfo->srclist,NULL,GNOME_VFS_XFER_EMPTY_DIRECTORIES,
			GNOME_VFS_XFER_ERROR_MODE_QUERY,GNOME_VFS_XFER_OVERWRITE_MODE_REPLACE,GNOME_VFS_PRIORITY_DEFAULT,
			velocity_vfs_xfer_handler_cb,(gpointer)xferinfo,velocity_vfs_xfer_sync_cb,(gpointer)xferinfo);
		gnome_vfs_uri_unref(tguri);
	}
}
static velocity_vfs_monitor_event*velocity_vfs_monitor_event_new(const char*uri,GnomeVFSMonitorEventType type){
	velocity_vfs_monitor_event*event=g_new(velocity_vfs_monitor_event,1);
	event->uri=g_strdup(uri);
	event->type=type;
	return(event);
}
static velocity_vfs_monitor_event_free(velocity_vfs_monitor_event*event){
	g_free(event->uri);
	g_free(event);
}
void velocity_vfs_inform_event(const char*uri,GnomeVFSMonitorEventType type){
	GnomeVFSURI*guri=gnome_vfs_uri_new(uri);
	char*muri=gnome_vfs_uri_to_string(guri,GNOME_VFS_URI_HIDE_USER_NAME|GNOME_VFS_URI_HIDE_PASSWORD);
	velocity_vfs_monitor_event*event=velocity_vfs_monitor_event_new(muri,type);
	velocity_vfs_sys->monitor_events=g_slist_prepend(velocity_vfs_sys->monitor_events,(gpointer)event);
	g_free(muri);
	gnome_vfs_uri_unref(guri);
}
static gboolean velocity_vfs_monitor_idler_cb(gpointer user_data){
	if(velocity_vfs_sys->monitor_events){
		GSList*e=velocity_vfs_sys->monitor_events;
		while(e){
			velocity_vfs_monitor_event*event=(velocity_vfs_monitor_event*)e->data;
			GSList*m=velocity_vfs_sys->monitors;
			char*t=velocity_vfs_get_uri_parent(event->uri);
			char*t2=velocity_vfs_uri_get_matchable(event->uri);
			while(m){
				velocity_vfs_monitor*vfsmon=(velocity_vfs_monitor*)m->data;
				if((!vfsmon->uri)||(strncmp(vfsmon->uri,t,strlen(vfsmon->uri))==0))vfsmon->on_event(vfsmon,event,vfsmon->user_data);
				m=m->next;
			}
			g_free(t2);
			g_free(t);
			velocity_vfs_monitor_event_free(event);
			e=e->next;
		}
		g_slist_free(velocity_vfs_sys->monitor_events);
		velocity_vfs_sys->monitor_events=NULL;
	}
	return(TRUE);
}
velocity_vfs_monitor*velocity_vfs_monitor_new(const char*uri,velocity_vfs_monitor_on_event_cb on_event,gpointer user_data){
	if(on_event){
		velocity_vfs_monitor*vfsmon=g_new0(velocity_vfs_monitor,1);
		if(uri){
			GnomeVFSURI*guri=gnome_vfs_uri_new(uri);
			vfsmon->uri=gnome_vfs_uri_to_string(guri,GNOME_VFS_URI_HIDE_NONE);
			gnome_vfs_uri_unref(guri);
		}else{
			vfsmon->uri=NULL;
		}
		vfsmon->on_event=on_event;
		vfsmon->user_data=user_data;
		velocity_vfs_sys->monitors=g_slist_prepend(velocity_vfs_sys->monitors,(gpointer)vfsmon);
		if(!velocity_vfs_sys->monitor_idler){
			velocity_vfs_sys->monitor_idler=gtk_timeout_add(500,(GtkFunction)velocity_vfs_monitor_idler_cb,NULL);
		}
		VELOCITY_MSG("uri=%s",vfsmon->uri);
		return(vfsmon);
	}
	return(NULL);
}
void velocity_vfs_monitor_free(velocity_vfs_monitor*vfsmon){
	if(vfsmon){
		GSList*n=velocity_vfs_sys->monitors;
		GSList*nl=NULL;
		VELOCITY_MSG("uri=%s",vfsmon->uri);
		while(n){
			if(n->data){
				velocity_vfs_monitor*vfsmon2=(velocity_vfs_monitor*)n->data;
				if(vfsmon2==vfsmon){
					if(vfsmon->uri)g_free(vfsmon->uri);
					g_free(vfsmon);
				}else{
					nl=g_slist_prepend(nl,(gpointer)vfsmon2);
				}
			}
			n=n->next;
		}
		if(velocity_vfs_sys->monitors)g_slist_free(velocity_vfs_sys->monitors);
		velocity_vfs_sys->monitors=nl;
	}
}
