/*
 * Velocity - A GNOME File Manager
 *
 *
 * Copyright (C) (2002) (Kyle Davis) <kyled@blueriver.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef VELOCITY_VFS_H
#define VELOCITY_VFS_H

#include <glib.h>
#include <libgnomevfs/gnome-vfs-mime-handlers.h>
#include <libgnomevfs/gnome-vfs-application-registry.h>
#include "libuuid/uuid.h"
#include "velocity-vfs-object.h"

#define VELOCITY_VFS_MIME_TYPE_DIRECTORY	"x-directory/normal"
#define VELOCITY_VFS_MIME_TYPE_VFOLDER	"x-directory/vfolder-desktop"
#define VELOCITY_VFS_MIME_TYPE_EXECUTABLE	"application/x-executable-binary"

#define GNOME_FILE_TYPES_PROPERTIES	"gnome-file-types-properties"

#define VELOCITY_VFS_LOCAL_URI	"file:///"
#define VELOCITY_VFS_START_HERE_URI	"start-here:///"
#define VELOCITY_VFS_APPLICATIONS_URI	"applications:///"
#define VELOCITY_VFS_APPLICATIONS_ALL_USERS_URI	"applications-all-users:///"
#define VELOCITY_VFS_ALL_APPLICATIONS_URI	"all-applications:///"
#define VELOCITY_VFS_PREFERENCES_URI	"preferences:///"
#define VELOCITY_VFS_PREFERENCES_ALL_USERS_URI	"preferences-all-users:///"
#define VELOCITY_VFS_ALL_PREFERENCES_URI	"all-preferences:///"
#define VELOCITY_VFS_SYSTEM_SETTINGS_URI	"system-settings:///"
#define VELOCITY_VFS_SERVER_SETTINGS_URI	"server-settings:///"
#define VELOCITY_VFS_NETWORK_URI	"network:///"
#define VELOCITY_VFS_SAMBA_URI	"smb:///"
#define VELOCITY_VFS_FONTS_URI	"fonts:///"
#define VELOCITY_VFS_BURN_URI		"burn:///"

typedef enum{VELOCITY_VFS_XFER_NULL=0,
	VELOCITY_VFS_XFER_DELETE,
	VELOCITY_VFS_XFER_EMPTY_TRASH,
	VELOCITY_VFS_XFER_COPY,
	VELOCITY_VFS_XFER_MOVE,
	VELOCITY_VFS_XFER_MOVE_TO_TRASH,
	VELOCITY_VFS_XFER_LINK,
	VELOCITY_VFS_XFER_DUPLICATE,
	VELOCITY_VFS_XFER_NUM}velocity_vfs_xfer_type;

typedef void(*velocity_vfs_xfer_object_done_cb)(const char*new_folder_uri,gpointer data);

typedef struct{
	GtkWidget*widget;
	GtkWidget*lblOperation;
	GtkWidget*lblStatus;
	GtkWidget*progress;

	char*operation;
	char*src_prefix;
	char*dest_prefix;
	
	gulong total_files;
	gulong total_bytes;
	gulong files_done;
	gulong bytes_done;
	
	GTimer*timer;
	//gint64 start_time;
	//guint delayed_close_timeout_id;

	/*Callbacks*/
	gpointer user_data;
	void(*on_cancel)(void*vfsprogress,gpointer user_data);
}velocity_vfs_progress;

typedef struct{
	GtkWindow*parent;
	GnomeVFSAsyncHandle*handle;
	velocity_vfs_progress*progress;
	
	GnomeVFSXferErrorMode error_mode;
	GnomeVFSXferOverwriteMode overwrite_mode;
	velocity_vfs_xfer_type type;
	
	//GHashTable*debuting_uris;
	GList*srclist;
	GList*destlist;
	
	gboolean local_src;
	gboolean local_dest;
	gboolean show_progress_dialog;
	gboolean cancelled;
	
	void(*on_done)(void*xferinfo,gpointer user_data);
	gpointer user_data;
}velocity_vfs_xfer_info;

typedef struct{
	GtkWindow*parent;
	GnomeVFSAsyncHandle*handle;
	void(*on_done)(const char*new_folder_uri,gpointer data);
	gpointer user_data;
}velocity_vfs_xfer_new_folder_info;


typedef struct{
	/*Public*/
	char*uri;
	GnomeVFSMonitorEventType type;
}velocity_vfs_monitor_event;

typedef void(*velocity_vfs_monitor_on_event_cb)(void*monitor,velocity_vfs_monitor_event*event,gpointer user_data);	/*when a file/directory event occurs*/
typedef struct{
	/*Public*/
	char*uri;	/*URI to monitor (NULL means ALL)*/
	velocity_vfs_monitor_on_event_cb on_event;
	/*Private*/
	gpointer user_data;
}velocity_vfs_monitor;

typedef enum{VELOCITY_VFS_FILTER_NONE=0,
	VELOCITY_VFS_FILTER_HIDDEN	=1<<0,
	VELOCITY_VFS_FILTER_BACKUPS	=1<<1,
	VELOCITY_VFS_FILTER_FILES	=1<<2,
	VELOCITY_VFS_FILTER_DIRS	=1<<3
}velocity_vfs_filter_flags;

void velocity_vfs_init(void);
void velocity_vfs_shutdown(void);


/*URI utilities*/
char*velocity_vfs_uri_from_input(const char*uri);
char*velocity_vfs_uri_reformat(const char*uri);
char*velocity_vfs_uri_get_tail(const char*uri);
char*velocity_vfs_ensure_dir_uri(const char*uri);
char*velocity_vfs_get_uri_unescaped(velocity_vfs_obj*vfsobj);
char*velocity_vfs_get_uri(velocity_vfs_obj*vfsobj);
char*velocity_vfs_get_human_readable(const char*uri);
gboolean velocity_vfs_uris_match(const char*uri1,const char*uri2);
gboolean velocity_vfs_uri_has_parent(const char*uri);
char*velocity_vfs_get_uri_parent(const char*uri);
GList*velocity_vfs_get_uri_parents_reversed(const char*uri);
GList*velocity_vfs_get_uri_parents(const char*uri);


/*Object testing functions*/
gboolean velocity_vfs_is_owner(velocity_vfs_obj*vfsobj);
gboolean velocity_vfs_is_in_group(velocity_vfs_obj*vfsobj);
gboolean velocity_vfs_is_local(velocity_vfs_obj*vfsobj);
gboolean velocity_vfs_is_readable(velocity_vfs_obj*vfsobj);
gboolean velocity_vfs_is_writable(velocity_vfs_obj*vfsobj);
gboolean velocity_vfs_is_executable(velocity_vfs_obj*vfsobj);
gboolean velocity_vfs_is_directory(velocity_vfs_obj*vfsobj);
gboolean velocity_vfs_is_fifo(velocity_vfs_obj*vfsobj);
gboolean velocity_vfs_is_socket(velocity_vfs_obj*vfsobj);
gboolean velocity_vfs_is_char_dev(velocity_vfs_obj*vfsobj);
gboolean velocity_vfs_is_block_dev(velocity_vfs_obj*vfsobj);
gboolean velocity_vfs_is_symlink(velocity_vfs_obj*vfsobj);
gboolean velocity_vfs_is_home_dir_uri(const char*uri);
gboolean velocity_vfs_is_trash_dir_uri(const char*uri);
gboolean velocity_vfs_is_desktop_dir_uri(const char*uri);
gboolean velocity_vfs_is_home_dir(velocity_vfs_obj*vfsobj);
gboolean velocity_vfs_is_trash_dir(velocity_vfs_obj*vfsobj);
gboolean velocity_vfs_is_desktop_dir(velocity_vfs_obj*vfsobj);


/*A few syncronous VFS functions*/
GnomeVFSResult velocity_vfs_move(velocity_vfs_obj*vfsobj,const char*new);
GnomeVFSResult velocity_vfs_rename(velocity_vfs_obj*vfsobj,const char*newname);


/*Utilities*/
gboolean velocity_vfs_can_open(velocity_vfs_obj*vfsobj);
/*Get best icon for mime_type and URI.  URI can be NULL*/
char*velocity_vfs_get_mime_icon_uri(const char*mime_type,const char*uri,guint base_size);
GnomeVFSFileSize velocity_vfs_get_free_space(char*uri);
/*Filtering functions.  Returns TRUE if should be filtered.  shell_exp can be NULL*/
gboolean velocity_vfs_filter(GnomeVFSFileInfo*info,const char*shell_exp,velocity_vfs_filter_flags filter_flags);
/*Returns a GList of velocity_vfs_obj's of all non directories beneath the given URI*/
GList*velocity_vfs_directory_list_load_recursive(const char*uri,GnomeVFSFileInfoOptions options);
GnomeVFSMimeApplication*velocity_vfs_mime_application_new_from_cmd(const char*cmd);
GnomeVFSMimeApplication*velocity_vfs_mime_application_new(const char*cmd,const char*mime_type,gboolean requires_terminal,gboolean can_open_multiple_files);
void velocity_vfs_mime_application_open(GnomeVFSMimeApplication*mimeapp,char*uri);
void velocity_vfs_mime_application_open_list(GnomeVFSMimeApplication*mimeapp,GList*uris);


void velocity_vfs_progress_operation_update(velocity_vfs_progress*p);
void velocity_vfs_progress_status_update(velocity_vfs_progress*p,const char*current_file);
void velocity_vfs_progress_bar_update(velocity_vfs_progress*p);
void velocity_vfs_progress_set_totals(velocity_vfs_progress*p,guint total_files,gulong total_bytes);
void velocity_vfs_progress_operation_set_string(velocity_vfs_progress*p,const char*operation_string);
void velocity_vfs_progress_update(velocity_vfs_progress*p);


velocity_vfs_progress*velocity_vfs_progress_new(const char*operation,const char*src_prefix,const char*dest_prefix,guint total_files,gulong total_bytes,gpointer user_data);
void velocity_vfs_progress_free(velocity_vfs_progress*p);

char*velocity_vfs_get_error_string(GnomeVFSResult result);

void velocity_vfs_xfer(GtkWindow*window,GList*srcurilst,GList*desturilst,velocity_vfs_xfer_type action,gboolean show_progress_dialog,void*on_done,gpointer user_data);

int velocity_vfs_xfer_newfolder_cb(GnomeVFSAsyncHandle*handle,GnomeVFSXferProgressInfo*progress_info,gpointer data);
void velocity_vfs_xfer_new_folder(GtkWindow*pwindow,const char*parent_dir,velocity_vfs_xfer_object_done_cb done_callback,gpointer user_data);

void velocity_vfs_delete(GList*item_uris,GtkWindow*parent_view);
void velocity_vfs_empty_trash(GtkWindow*parent,gboolean showprogress,void*on_done,gpointer user_data);

/*These monitors are handled internally, but you can force one to be deleted*/
velocity_vfs_monitor*velocity_vfs_monitor_new(const char*uri,velocity_vfs_monitor_on_event_cb on_event,gpointer user_data);
void velocity_vfs_inform_event(const char*uri,GnomeVFSMonitorEventType type);
void velocity_vfs_monitor_free(velocity_vfs_monitor*vfsmon);
	
#endif
