/*
 * Velocity - A GNOME File Manager
 *
 *
 * Copyright (C) (2002) (Kyle Davis) <kyled@blueriver.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef VELOCITY_VIEW_H
#define VELOCITY_VIEW_H

#include <glib.h>
#include <gdk/gdk.h>
#include <gdk/gdkx.h>
#include <gnome.h>
#include "velocity-preferences.h"
#include "velocity-clipboard.h"
#include "velocity-vfs.h"
#include "velocity-icon.h"
#include "velocity-lasso.h"

#define VELOCITY_VIEW_LOAD_PER_SYNC_LOOP	256
#define VELOCITY_VIEW_RENDER_PER_SYNC_LOOP	128

#define GNOME_APP_INFO	"application/x-gnome-app-info"
#define KDE_APP_INFO	"application/x-kde-app-info"

extern velocity_clipboard*vclipboard;	/*Global clipboard*/

/*View status UCB's (User CallBacks)*/
typedef enum{VELOCITY_VIEW_STATUS_EVENT_INVALID=0,
	VELOCITY_VIEW_STATUS_EVENT_VALIDATE,
	VELOCITY_VIEW_STATUS_EVENT_ERROR,
	VELOCITY_VIEW_STATUS_EVENT_GO,
	VELOCITY_VIEW_STATUS_EVENT_LOADING,
	VELOCITY_VIEW_STATUS_EVENT_DONE,
	VELOCITY_VIEW_STATUS_EVENT_STOP,
	VELOCITY_VIEW_STATUS_EVENT_NEW_WINDOW,
	VELOCITY_VIEW_STATUS_EVENT_NUM}velocity_view_status_event;
typedef void*(*velocity_view_status_event_ucb)(void*vview,velocity_view_status_event vevent,const char*uri,GnomeVFSResult result,gpointer user_data);
/*vview=the view obj,vevent=velocity_view_event,uri=the current URI,result=error result if any,user_data=a pointer you can pass around*/
/*NOTES:  result is only valid for ERROR callbacks
	return a value ONLY if explicitly described to return a value above, all unauthorized returned variables will be ignored*/

/*View behavioral UCB's (User CallBacks)*/
typedef enum{VELOCITY_VIEW_BEHAVIOR_EVENT_INVALID=0,
	VELOCITY_VIEW_BEHAVIOR_EVENT_OBJECT_LOADED,
	VELOCITY_VIEW_BEHAVIOR_EVENT_OBJECT_GET_ICON,
	VELOCITY_VIEW_BEHAVIOR_EVENT_OBJECT_GET_CAPTION,
	VELOCITY_VIEW_BEHAVIOR_EVENT_MOUSE_OVER,
	VELOCITY_VIEW_BEHAVIOR_EVENT_MOUSE_OUT,
	VELOCITY_VIEW_BEHAVIOR_EVENT_CLICK,	/*in general:  left click*/
	VELOCITY_VIEW_BEHAVIOR_EVENT_ACTIVATE,	/*in general:  double click*/
	VELOCITY_VIEW_BEHAVIOR_EVENT_CONTEXT,	/*in general:  right click*/
	VELOCITY_VIEW_BEHAVIOR_EVENT_BACKGROUND_CONTEXT,	/*in general:  right click (on background)*/
	VELOCITY_VIEW_BEHAVIOR_EVENT_KEYBOARD,	/*in general:  any keyboard event, return anything !=NULL to skip default keyhandler*/
	//TODO:  VELOCITY_VIEW_BEHAVIOR_EVENT_KEY{PRESS,RELEASE}
	VELOCITY_VIEW_BEHAVIOR_EVENT_NUM}velocity_view_behavior_event;
typedef void*(*velocity_view_behavior_event_ucb)(void*vview,velocity_view_behavior_event vevent,GdkEvent*event,velocity_vfs_obj*vfsobj,gpointer user_data);
/*vview=the view obj,vevent=velocity_view_behavior_event,event=the GDK event,vfsobj=a vfs object representing icon (if logical),user_data=a pointer you can pass around*/
/*NOTES:  vfsobj is not valid for background context (since there is no icon representing the background this wouldn't be logical)
 * return a value ONLY if explicitly described to return a value above, all unexpected returned variables will be ignored
 */

//TODO:  Tooltip funcs should probably be split into their own functions...but are still a part of the velocity_view
#define VELOCITY_TOOLTIPS_EXT_ID	N_("tooltips")

typedef char*(*velocity_tooltip_item_text_cb)(velocity_vfs_obj*vfsobj);
typedef GdkPixbuf*(*velocity_tooltip_item_pixbuf_cb)(velocity_vfs_obj*vfsobj);

typedef enum{VELOCITY_TOOLTIP_ITEM_TYPE_INVALID=0,
	VELOCITY_TOOLTIP_ITEM_TYPE_TEXT,	/*we're pointing to a velocity_tooltip_item_text_cb*/
	VELOCITY_TOOLTIP_ITEM_TYPE_PIXBUF,	/*we're pointing to a velocity_tooltip_item_pixbuf_cb*/
	VELOCITY_TOOLTIP_ITEM_TYPE_INT,	/*we're pointing to an integer  (See GINT_TO_POINTER and GPOINTER_TO_INT macros)*/
	VELOCITY_TOOLTIP_ITEM_TYPE_NUM}velocity_tooltip_item_type;

typedef struct{
	char*id;
	char*desc;
	velocity_tooltip_item_type type;
	gpointer data;
}velocity_tooltip_item;
GList*velocity_tooltip_items(velocity_vfs_obj*vfsobj);
void velocity_tooltip_add(velocity_vfs_obj*vfsobj,const char*id,const char*desc,velocity_tooltip_item_type type,gpointer data);
void velocity_tooltip_remove(velocity_vfs_obj*vfsobj,const char*id);

typedef struct{
	guint files;
	guint dirs;
	guint objects;
	guint images;	/*Used for async image loading safeness*/
	guint filtered;	/*Items filtered out by view settings*/
}velocity_view_counters;

typedef struct{
	GtkWindow*parent;	/*parent window*/
	GtkWidget*widget;	/*The scrolled window widget*/

	/*Root canvas and a canvas group to be nested within the root canvas.  This is our main drawing area*/
	GtkWidget*canvas;	
	GnomeCanvasGroup*canvas_group;
	GdkPixmap*background;	/*Used for background images in canvas*/

	/*Current URI location (not neccessarily the same text that is in the Location GtkEntry*/
	gchar*clocation;
	gchar*shellexp;	/*A shell expression to filter the names with*/		//TODO:  Finish this properly...

	GList*files;	/*List of velocity_vfs_obj*'s*/
	GList*dirs;	/*List of velocity_vfs_obj*'s-this is used for sorting when dirs go first*/
	GList*qn;	/*Queue node, this way we don't have to search EVERY TIME we want to render*/

	GList*nuke_files;	/*Pointer to old files list until things are ready to be nuked, thus avoiding a huge flicker*/
	GList*nuke_dirs;	/*Pointer to old dirs list until things are ready to be nuked, thus avoiding a huge flicker*/

	/*For multiple selections*/
	velocity_lasso*lasso;
	velocity_clipboard*selection;

	GnomeVFSAsyncHandle*handle;	/*and to keep track of what we're doing, also allows us to cancel easily*/

	guint dequeue_handle;	/*Handle for dequeueing/rendering*/
	guint animation_idle_handle;	/*one ID for the icon animation handler*/
	guint tooltip_handle;	/*tooltip handler idle handle*/
	guint refresh_handle;	/*refresh handler idle handle*/

	gboolean lock;	/*A lock flag so we know when we can safely use our redrawing routine (used during validating)*/

	velocity_icon*animation_handle;	/*Pointer to our animation basic_icon*/

	/*Helps us to keep track of current status*/
	velocity_view_counters read;
	velocity_view_counters total;

	velocity_view_preferences*prefs;
	GdkPoint icon;	/*Posiiton iterator for drawing*/
	GtkBorder border;	/*Border for VFS object drawing area*/

	/*Callbacks*/
	gpointer user_data;
	velocity_view_status_event_ucb on_status_event;
	velocity_view_behavior_event_ucb on_behavior_event;
}velocity_view;

gchar*velocity_view_get_mime_icon_uri(const gchar*mime_type,const gchar*uri,guint base_size);
GdkPixbuf*velocity_view_get_mime_icon(velocity_view*vview,velocity_vfs_obj*vfsobj,guint width,guint height);
velocity_view*velocity_view_new(GtkWindow*parent,gboolean scrollbars);
void set_background_color_ref(GtkWidget *canvas, GdkColor *color);
void velocity_view_append_unsorted(velocity_view*vview,velocity_vfs_obj*vfsobj);
void velocity_view_dequeue(velocity_view*vview);
void velocity_view_dequeue_1(velocity_view*vview);
void velocity_view_set_icon_caption(velocity_view*vview,velocity_vfs_obj*vfsobj,gboolean confine);
void velocity_view_loading_prep(velocity_view*vview);
void velocity_view_dequeue_prep(velocity_view*vview);
void velocity_view_set_location(velocity_view*vview,const char*location);
void velocity_view_soft_refresh(velocity_view*vview,gboolean reload_images); /*This will refresh the icons, but will NOT reread from the VFS*/
void velocity_view_activate(velocity_view*vview);
void velocity_view_open(velocity_view*vview,velocity_vfs_obj*vfsobj);	/*This bypasses the ACTIVATE callback, avoid using*/
void velocity_view_stop(velocity_view*vview);
void velocity_view_show_lasso(velocity_view*vview,gfloat x1,gfloat y1,gfloat x2,gfloat y2);
void velocity_view_clear_last_selection(velocity_view*vview);
void velocity_view_hide_lasso(velocity_view*vview);
void velocity_view_clear_selection(velocity_view*vview);
/*This will call func for every velocity_vfs_obj* in vview*/
void velocity_view_foreach(velocity_view*vview,GFunc func,gpointer user_data);
void velocity_view_free(velocity_view*vview);

/*NOTES:  These vs. the actual variables (vars are in vprefs->view and vprefs->graphics mainly...)
	setting the variables directly will require a refresh before they take effect
	These functions will actually make the changes happen w/o a refresh*/
void velocity_view_ensure_canvas(velocity_view*vview);
void velocity_view_set_background(velocity_view*vview,const char*uri);
void gtk_widget_set_background_color(GtkWidget*w,GdkColor*color);
void velocity_view_set_background_color(velocity_view*vview,GdkColor*color);

#endif
