/*
 * Velocity - A GNOME File Manager
 *
 *
 * Copyright (C) (2002) (Kyle Davis) <kyled@blueriver.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <config.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <gdk/gdk.h>
#include <gdk/gdkprivate.h>
#include <gnome.h>
#include <libgnomeui/gnome-window-icon.h>

#include "callbacks.h"
#include "velocity-window.h"
#include "velocity-sys.h"
#include "velocity-dnd.h"
#include "velocity-factories.h"
#include "velocity-preferences-util.h"
#include "velocity-module-private.h"
#include "velocity-windowman.h"

static void location_drag_data_get_callback(GtkWidget*widget,GdkDragContext*context,GtkSelectionData*selection_data,guint info,guint32 time,gpointer callback_data){
	velocity_window*win=NULL;
	char*uri=NULL;
	g_assert(selection_data!=NULL);
	win=(velocity_window*)callback_data;
	uri=velocity_vfs_uri_reformat(win->view->clocation);
	switch(info){
		case VELOCITY_DND_URI_LIST:
		case VELOCITY_DND_URI:
			VELOCITY_MSG("This only handles one URI/URL right now...FIX IT!");
			gtk_selection_data_set(selection_data,selection_data->target,8,(guchar*)uri,strlen(uri));
			break;
		default:
			g_assert_not_reached();
	}
	g_free(uri);
}

static gboolean location_drag_data_received_callback(GtkWidget*widget,GdkDragContext*context,int x,int y,GtkSelectionData*data,guint info,guint32 time,gpointer callback_data){
	if(data){
		GList*names=gnome_vfs_uri_list_parse(data->data);
		if(!names){
			VELOCITY_ERR("No Drag & Drop URI's");
			gtk_drag_finish(context,FALSE,FALSE,time);
			return(FALSE);
		}else{
			velocity_window*vwin=(velocity_window*)callback_data;
			GList*n=names;
			int c=g_list_length(names);
			while(n){
				GnomeVFSURI*guri=(GnomeVFSURI*)n->data;
				char*curi=gnome_vfs_uri_to_string(guri,GNOME_VFS_URI_HIDE_NONE);
				if(curi){
					if(c==1){
						velocity_window_set_location(vwin,curi);
					}else{
						velocity_windowman_new(curi);
					}
					g_free(curi);
				}
				n=n->next;
			}
			gnome_vfs_uri_list_free(names);
		}
	}
	gtk_drag_finish(context,TRUE,FALSE,time);
	return(TRUE);
}
static void velocity_menu_tools_templates_dir_cb(GtkMenuItem*menuitem,gpointer user_data){
	velocity_view*vview=(velocity_view*)g_object_get_data(G_OBJECT(menuitem),"velocity_view");
	char*uri=(GPOINTER_TO_INT(user_data)==0)?PACKAGE_TEMPLATES_DIR GNOME_VFS_URI_PATH_STR:vprefs->locations->templates;
	velocity_view_set_location(vview,uri);
}
static void velocity_menu_tools_scripts_dir_cb(GtkMenuItem*menuitem,gpointer user_data){
	velocity_view*vview=(velocity_view*)g_object_get_data(G_OBJECT(menuitem),"velocity_view");
	char*uri=(GPOINTER_TO_INT(user_data)==0)?PACKAGE_SCRIPTS_DIR GNOME_VFS_URI_PATH_STR:vprefs->locations->scripts;
	velocity_view_set_location(vview,uri);
}
static void velocity_menu_tools_sendto_dir_cb(GtkMenuItem*menuitem,gpointer user_data){
	velocity_view*vview=(velocity_view*)g_object_get_data(G_OBJECT(menuitem),"velocity_view");
	char*uri=(GPOINTER_TO_INT(user_data)==0)?PACKAGE_SENDTO_DIR GNOME_VFS_URI_PATH_STR:vprefs->locations->sendto;
	velocity_view_set_location(vview,uri);
}
void velocity_window_menubar_create(velocity_window*vwin){
	GtkWidget*menu=NULL;
	GtkWidget*menuitem=NULL;
	void*root_file[]={VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_OPEN,_("_Open"),VELOCITY_FACTORY_TRUE,velocity_menu_open_cb,vwin,
		VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_OPEN,_("Open _New Window"),VELOCITY_FACTORY_TRUE,velocity_menu_open_in_window_cb,NULL,
		VELOCITY_MENU_FACTORY_SEPERATOR,
		VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_QUIT,_("_Quit"),VELOCITY_FACTORY_TRUE,velocity_menu_file_quit_cb,vwin,
		NULL};
	void*root_edit[]={VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_UNDO,_("Un_do"),VELOCITY_FACTORY_FALSE,velocity_toolbar_undo_cb,vwin,
		VELOCITY_MENU_FACTORY_SEPERATOR,
		VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_CUT,_("C_ut"),VELOCITY_FACTORY_TRUE,velocity_menu_cut_cb,vwin->view,
		VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_COPY,_("_Copy"),VELOCITY_FACTORY_TRUE,velocity_menu_copy_cb,vwin->view,
		VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_PASTE,_("_Paste"),VELOCITY_FACTORY_TRUE,velocity_menu_paste_cb,vwin->view,
		VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_CONVERT,_("Re_name"),VELOCITY_FACTORY_TRUE,velocity_menu_rename_cb,vwin->view,
		VELOCITY_MENU_FACTORY_PIXBUF,VELOCITY_TRASH,_("_Trash"),VELOCITY_FACTORY_TRUE,velocity_menu_trash_cb,vwin->view,
		VELOCITY_MENU_FACTORY_SEPERATOR,
		_("Select _All"),VELOCITY_FACTORY_TRUE,velocity_menu_edit_selectall_cb,vwin,
		VELOCITY_MENU_FACTORY_SEPERATOR,
		VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_PROPERTIES,_("P_roperties"),VELOCITY_FACTORY_TRUE,velocity_menu_properties_cb,vwin->view,
		VELOCITY_MENU_FACTORY_SEPERATOR,
		VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_PREFERENCES,_("_File types"),VELOCITY_FACTORY_TRUE,velocity_menu_file_types_cb,vwin->view,
		VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_PREFERENCES,_("_Preferences"),VELOCITY_FACTORY_TRUE,velocity_menu_preferences_cb,vwin->view,
		NULL};
	void*root_view_viewopts[]={VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_REFRESH,_("Re_organize"),VELOCITY_FACTORY_TRUE,velocity_menu_sorting_reorganize_cb,vwin->view,
		VELOCITY_MENU_FACTORY_SEPERATOR,
		VELOCITY_MENU_FACTORY_CHECKBOX,(gpointer)vprefs->view->dirs_first,_("_Directories first"),VELOCITY_FACTORY_TRUE,velocity_menu_sorting_dirs_first_cb,vwin->view,
		VELOCITY_MENU_FACTORY_CHECKBOX,(gpointer)(!vprefs->view->filter&VELOCITY_VFS_FILTER_HIDDEN),_("Show _hidden"),VELOCITY_FACTORY_TRUE,velocity_menu_show_hidden_cb,vwin->view,
		VELOCITY_MENU_FACTORY_SEPERATOR,
		VELOCITY_MENU_FACTORY_CHECKBOX,(gpointer)(vprefs->view->sorting==VELOCITY_VIEW_SORT_NAME),_("by _Name"),VELOCITY_FACTORY_TRUE,velocity_menu_sorting_cb,GINT_TO_POINTER(VELOCITY_VIEW_SORT_NAME),
		VELOCITY_MENU_FACTORY_CHECKBOX,(gpointer)(vprefs->view->sorting==VELOCITY_VIEW_SORT_TYPE),_("by _Type"),VELOCITY_FACTORY_TRUE,velocity_menu_sorting_cb,GINT_TO_POINTER(VELOCITY_VIEW_SORT_TYPE),
		VELOCITY_MENU_FACTORY_CHECKBOX,(gpointer)(vprefs->view->sorting==VELOCITY_VIEW_SORT_SIZE),_("by _Size"),VELOCITY_FACTORY_TRUE,velocity_menu_sorting_cb,GINT_TO_POINTER(VELOCITY_VIEW_SORT_SIZE),
		VELOCITY_MENU_FACTORY_CHECKBOX,(gpointer)(vprefs->view->sorting==VELOCITY_VIEW_SORT_MODIFIED),_("by Last _Modified"),VELOCITY_FACTORY_TRUE,velocity_menu_sorting_cb,GINT_TO_POINTER(VELOCITY_VIEW_SORT_MODIFIED),
		VELOCITY_MENU_FACTORY_CHECKBOX,(gpointer)(vprefs->view->sorting==VELOCITY_VIEW_SORT_ACCESSED),("by Last A_ccessed"),VELOCITY_FACTORY_TRUE,velocity_menu_sorting_cb,GINT_TO_POINTER(VELOCITY_VIEW_SORT_ACCESSED),
		VELOCITY_MENU_FACTORY_CHECKBOX,(gpointer)(vprefs->view->sorting==VELOCITY_VIEW_SORT_CREATED),_("by Date Created"),VELOCITY_FACTORY_TRUE,velocity_menu_sorting_cb,GINT_TO_POINTER(VELOCITY_VIEW_SORT_CREATED),
		NULL};		
	void*root_view[]={VELOCITY_MENU_FACTORY_CHECKBOX,VELOCITY_FACTORY_TRUE,_("Toolbar"),VELOCITY_FACTORY_FALSE,NULL,vwin,
		VELOCITY_MENU_FACTORY_CHECKBOX,VELOCITY_FACTORY_TRUE,_("Location bar"),VELOCITY_FACTORY_FALSE,NULL,vwin,
		VELOCITY_MENU_FACTORY_CHECKBOX,VELOCITY_FACTORY_TRUE,_("Status bar"),VELOCITY_FACTORY_FALSE,NULL,vwin,
		VELOCITY_MENU_FACTORY_SEPERATOR,
		VELOCITY_MENU_FACTORY_SUBFACTORY,root_view_viewopts,GTK_STOCK_PREFERENCES,_("_View options"),VELOCITY_FACTORY_TRUE,NULL,vwin,
		NULL};
	void*root_go[]={VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_GO_BACK,_("_Back"),VELOCITY_FACTORY_TRUE,velocity_toolbar_back_cb,vwin,
		VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_GO_FORWARD,_("_Forward"),VELOCITY_FACTORY_TRUE,velocity_toolbar_forward_cb,vwin,
		VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_GO_UP,_("_Up"),VELOCITY_FACTORY_TRUE,velocity_toolbar_up_cb,vwin,
		VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_STOP,_("_Stop"),VELOCITY_FACTORY_TRUE,velocity_toolbar_stop_cb,vwin,
		VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_REFRESH,_("_Refresh"),VELOCITY_FACTORY_TRUE,velocity_toolbar_refresh_cb,vwin,
		VELOCITY_MENU_FACTORY_PIXBUF,VELOCITY_HOME,_("_Home"),VELOCITY_FACTORY_TRUE,velocity_toolbar_home_cb,vwin,
		NULL};
	void*root_tools_advanced_root[]={VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_NEW,_("System Templates Directory"),VELOCITY_FACTORY_TRUE,velocity_menu_tools_templates_dir_cb,0,
		VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_EXECUTE,_("System Scripts Directory"),VELOCITY_FACTORY_TRUE,velocity_menu_tools_scripts_dir_cb,0,
		VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_JUMP_TO,_("System Send to Directory"),VELOCITY_FACTORY_TRUE,velocity_menu_tools_sendto_dir_cb,0,
		VELOCITY_MENU_FACTORY_SEPERATOR,
		NULL};
	void*root_tools_advanced[]={VELOCITY_MENU_FACTORY_EMBEDDED_FACTORY,(getuid()==0)?root_tools_advanced_root:NULL,
		VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_NEW,_("Templates Directory"),VELOCITY_FACTORY_TRUE,velocity_menu_tools_templates_dir_cb,GINT_TO_POINTER(1),
		VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_EXECUTE,_("Scripts Directory"),VELOCITY_FACTORY_TRUE,velocity_menu_tools_scripts_dir_cb,GINT_TO_POINTER(1),
		VELOCITY_MENU_FACTORY_PIXBUF,GTK_STOCK_JUMP_TO,_("Send to Directory"),VELOCITY_FACTORY_TRUE,velocity_menu_tools_sendto_dir_cb,GINT_TO_POINTER(1),
		NULL};
	void*root_tools[]={VELOCITY_MENU_FACTORY_PIXBUF,VELOCITY_NETWORK,_("_Network"),VELOCITY_FACTORY_TRUE,velocity_menu_tool_network_cb,vwin->view,
		VELOCITY_MENU_FACTORY_SEPERATOR,
		VELOCITY_MENU_FACTORY_PIXBUF,VELOCITY_APPS,_("GNOME _Applications"),VELOCITY_FACTORY_TRUE,velocity_menu_tool_applications_cb,vwin->view,
		VELOCITY_MENU_FACTORY_PIXBUF,VELOCITY_PREFS,_("GNOME _Preferences"),VELOCITY_FACTORY_TRUE,velocity_menu_tool_preferences_cb,vwin->view,
		VELOCITY_MENU_FACTORY_SUBFACTORY,root_tools_advanced,GTK_STOCK_PREFERENCES,_("Ad_vanced"),VELOCITY_FACTORY_TRUE,NULL,NULL,
		NULL};
	void*root_help[]={VELOCITY_MENU_FACTORY_PIXBUF,GNOME_STOCK_ABOUT,_("_About"),VELOCITY_FACTORY_TRUE,velocity_menu_help_about_cb,vwin,
		NULL};
	void*root[]={_("_File"),_("_Edit"),_("_View"),_("_Go"),_("_Tools"),_("_Help"),NULL};
	void**subs[]={root_file,root_edit,root_view,root_go,root_tools,root_help,NULL};
	int m=0;
	vwin->menubar=gtk_menu_bar_new();
	while(root[m]){
		menuitem=gtk_menu_item_new_with_mnemonic(root[m]);
		gtk_menu_shell_append(GTK_MENU_SHELL(vwin->menubar),menuitem);
		if(subs[m]){
			menu=velocity_menu_factory(vwin->view,vwin->menubar,subs[m]);
			gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuitem),menu);
		}
		m++;
	}
	gnome_app_set_menus(GNOME_APP(vwin->widget),GTK_MENU_BAR(vwin->menubar));
	gtk_widget_show_all(vwin->menubar);
}
void velocity_window_toolbar_create(velocity_window*vwin){
	void*stream[]={VELOCITY_TOOLBAR_FACTORY_PIXBUF,GTK_STOCK_GO_BACK,_("Back"),_("Go back"),velocity_toolbar_back_cb,GTK_STOCK_GO_BACK,
		VELOCITY_TOOLBAR_FACTORY_PIXBUF,GTK_STOCK_GO_FORWARD,_("Forward"),_("Go forward"),velocity_toolbar_forward_cb,GTK_STOCK_GO_FORWARD,
		VELOCITY_TOOLBAR_FACTORY_PIXBUF,GTK_STOCK_GO_UP,_("Up"),_("Go to the parent directory"),velocity_toolbar_up_cb,GTK_STOCK_GO_UP,
		VELOCITY_TOOLBAR_FACTORY_SEPERATOR,
		VELOCITY_TOOLBAR_FACTORY_PIXBUF,GTK_STOCK_STOP,_("Stop"),_("Stop scanning"),velocity_toolbar_stop_cb,GTK_STOCK_STOP,
		VELOCITY_TOOLBAR_FACTORY_PIXBUF,GTK_STOCK_REFRESH,_("Refresh"),_("Reread the current directory"),velocity_toolbar_refresh_cb,GTK_STOCK_REFRESH,
		VELOCITY_TOOLBAR_FACTORY_SEPERATOR,
		VELOCITY_TOOLBAR_FACTORY_PIXBUF,GTK_STOCK_CUT,_("Cut"),_("\"Cut\" selected"),velocity_toolbar_cut_cb,GTK_STOCK_CUT,
		VELOCITY_TOOLBAR_FACTORY_PIXBUF,GTK_STOCK_COPY,_("Copy"),_("Copy selected"),velocity_toolbar_copy_cb,GTK_STOCK_COPY,
		VELOCITY_TOOLBAR_FACTORY_PIXBUF,GTK_STOCK_PASTE,_("Paste"),_("Paste clipboard contents"),velocity_toolbar_paste_cb,GTK_STOCK_PASTE,
		VELOCITY_TOOLBAR_FACTORY_PIXBUF,VELOCITY_TRASH,_("Trash"),_("Move selected to trash"),velocity_toolbar_trash_cb,VELOCITY_TRASH,
		VELOCITY_TOOLBAR_FACTORY_PIXBUF,GTK_STOCK_UNDO,_("Undo"),_("Undo last action"),velocity_toolbar_undo_cb,GTK_STOCK_UNDO,
		VELOCITY_TOOLBAR_FACTORY_SEPERATOR,
		VELOCITY_TOOLBAR_FACTORY_PIXBUF,GTK_STOCK_FIND,_("Search"),_("Search for files or folders"),velocity_toolbar_search_cb,GTK_STOCK_FIND,
		VELOCITY_TOOLBAR_FACTORY_SEPERATOR,
		VELOCITY_TOOLBAR_FACTORY_PIXBUF,VELOCITY_HOME,_("Home"),_("Go to your home directory"),velocity_toolbar_home_cb,VELOCITY_HOME,
		NULL};
	vwin->toolbar=velocity_toolbar_factory(vwin->widget,stream,(gpointer)vwin);
	gnome_app_add_toolbar(GNOME_APP(vwin->widget),GTK_TOOLBAR(vwin->toolbar),GNOME_APP_TOOLBAR_NAME,BONOBO_DOCK_ITEM_BEH_EXCLUSIVE,BONOBO_DOCK_TOP,1,0,0);
	g_signal_connect(velocity_toolbar_factory_get_button(vwin->toolbar,GTK_STOCK_REFRESH),"button_press_event",G_CALLBACK(velocity_toolbar_refresh_click_cb),vwin);
	g_signal_connect(velocity_toolbar_factory_get_button(vwin->toolbar,VELOCITY_TRASH),"button_press_event",G_CALLBACK(velocity_toolbar_trash_click_cb),vwin);
	g_signal_connect(velocity_toolbar_factory_get_button(vwin->toolbar,VELOCITY_HOME),"button_press_event",G_CALLBACK(velocity_toolbar_home_click_cb),vwin);
	gtk_widget_set_sensitive(velocity_toolbar_factory_get_button(vwin->toolbar,GTK_STOCK_PASTE),velocity_clipboard_has_items(vclipboard));
	gtk_widget_set_sensitive(velocity_toolbar_factory_get_button(vwin->toolbar,GTK_STOCK_UNDO),FALSE);
}
static void*velocity_window_tree_item_modfunc(velocity_tree_item*vti,velocity_tree_item_mod_type modtype){
	if(modtype==VELOCITY_TREE_ITEM_MOD_SET_ICONS){
		char*defaulturi;
		char*specialuri;
		/*Set icon sizes*/
//TODO:  Decide if this is acceptable behavoir
/*		if(velocity_vfs_uris_match(vti->uri,VELOCITY_VFS_LOCAL_URI)){
			vti->size.x=vti->size.y=48;
		}else{*/
			vti->size.x=vti->size.y=velocity_vfs_uri_has_parent(vti->uri)?24:32;
//		}
		defaulturi=velocity_image_get_uri(VELOCITY_FOLDER,vti->size.x);
		specialuri=velocity_vfs_get_mime_icon_uri(vti->mime_type,vti->uri,vti->size.x);
		if((!specialuri)||(strcmp(defaulturi,specialuri)==0)){
			vti->expand_file=velocity_image_get_uri(VELOCITY_FOLDER_ACCEPT,vti->size.x);
			vti->collapse_file=g_strdup(defaulturi);
		}else{
			vti->expand_file=g_strdup(specialuri);
			vti->collapse_file=g_strdup(specialuri);
		}
		if(specialuri)g_free(specialuri);
		g_free(defaulturi);
	}else if(modtype==VELOCITY_TREE_ITEM_MOD_GET_LABEL){
		return(velocity_vfs_get_human_readable(vti->uri));
	}
	return(NULL);
}
static void velocity_window_nuke_foreach(velocity_vfs_obj*vfsobj,gpointer user_data){
	if(vfsobj->icon){
		velocity_icon_free(vfsobj->icon);
		vfsobj->icon=NULL;
	}
}
static void velocity_window_set_view_profile(GtkOptionMenu*optionmenu,velocity_window*vwin){
	velocity_view_profile*vvp=velocity_view_profile_option_menu_get_profile(optionmenu);
	if(vvp){
		VELOCITY_MSG("view_profile(%s)",vvp->name);
		vwin->view->prefs->profile=vvp;
		velocity_view_foreach(vwin->view,(GFunc)velocity_window_nuke_foreach,NULL);
		velocity_view_soft_refresh(vwin->view,TRUE);
	}
}
static void testing(GnomeVFSMonitorHandle*handle,const gchar*monitor_uri,const gchar*uri,GnomeVFSMonitorEventType event,gpointer user_data){
	char*info;
	switch(event){
		case GNOME_VFS_MONITOR_EVENT_CREATED:
			info=_("Created");
			break;
		case GNOME_VFS_MONITOR_EVENT_CHANGED:
			info=_("Changed");
			break;
		case GNOME_VFS_MONITOR_EVENT_DELETED:
			info=_("Deleted");
			break;
		default:
			info=_("Unknown");
			break;
	}
	VELOCITY_MSG("File(%s), Status(%s)",uri,info);
}
static void on_btnGo_clicked(GtkButton*button,gpointer user_data){
	velocity_window*vwin=(velocity_window*)user_data;
	const char*uri=gtk_entry_get_text(GTK_ENTRY(vwin->location));
	if(uri)velocity_window_set_location(vwin,uri);
}
velocity_window*velocity_window_new(){
	GtkWidget*hbox;
	GtkWidget*lbllocation;
	GtkObject*spinbutton_adj;
	GtkWidget*spinbutton;
	velocity_window*vwin=g_new0(velocity_window,1);
	if(!vprefs)velocity_preferences_init();
	vwin->widget=gnome_app_new("Velocity",_("Velocity"));
	g_signal_connect(vwin->widget,"delete_event",G_CALLBACK(velocity_window_on_quit_cb),vwin);
	gtk_window_set_default_size(GTK_WINDOW(vwin->widget),vprefs->window->size.x,vprefs->window->size.y);
	gnome_window_icon_set_from_file(GTK_WINDOW(vwin->widget),PACKAGE_PIXMAPS_DIR "/" VELOCITY_ICON);
	gnome_app_enable_layout_config(GNOME_APP(vwin->widget),TRUE);
	vwin->view=velocity_view_new(GTK_WINDOW(vwin->widget),TRUE);
	/*Setup our intercomponent callbacks*/
	vwin->view->user_data=vwin;
	vwin->view->on_status_event=(velocity_view_status_event_ucb)velocity_window_on_status_event_cb;
	vwin->view->on_behavior_event=(velocity_view_behavior_event_ucb)velocity_window_on_behavior_event_cb;
	g_free(vwin->view->prefs);
	/*Copy prefs over from our main prefs handler*/
	vwin->view->prefs=g_memdup((gconstpointer)vprefs->view,sizeof(velocity_view_preferences));
	/*Hard code a couple of defaults that make sense for a file manager*/
	vwin->view->prefs->can_open=TRUE;
	vwin->view->prefs->can_drag=TRUE;
	vwin->view->prefs->can_lasso=TRUE;
	vwin->view->prefs->can_cancel=TRUE;
	velocity_window_menubar_create(vwin);
	velocity_window_toolbar_create(vwin);
	hbox=gtk_hbox_new(FALSE,6);
	gnome_app_add_docked(GNOME_APP(vwin->widget),hbox,"hbox",BONOBO_DOCK_ITEM_BEH_EXCLUSIVE|BONOBO_DOCK_ITEM_BEH_NEVER_VERTICAL,BONOBO_DOCK_TOP,2,0,3);
	gtk_container_set_border_width(GTK_CONTAINER(hbox),2);
	lbllocation=gtk_label_new(_("Location:"));
	gtk_box_pack_start(GTK_BOX(hbox),lbllocation,FALSE,FALSE,0);
	gtk_label_set_justify(GTK_LABEL(lbllocation),GTK_JUSTIFY_RIGHT);
	vwin->location=gtk_entry_new();
	g_signal_connect(vwin->location,"activate",G_CALLBACK(on_btnGo_clicked),vwin);
	gtk_box_pack_start(GTK_BOX(hbox),vwin->location,TRUE,TRUE,0);
	/*My first Drag & Drop!*/
	gtk_drag_dest_set(GTK_WIDGET(vwin->location),GTK_DEST_DEFAULT_ALL,velocity_dnd_targets,2,GDK_ACTION_DEFAULT|GDK_ACTION_COPY|GDK_ACTION_MOVE|GDK_ACTION_LINK);
	g_signal_connect(vwin->location,"drag_data_received",G_CALLBACK(location_drag_data_received_callback),vwin);
	vwin->btnGo=gtk_button_new_with_label(_(" Go "));
	gtk_box_pack_start(GTK_BOX(hbox),vwin->btnGo,FALSE,FALSE,0);
	/*My first Drag & Drop! (cont.'d)*/
	gtk_drag_source_set(GTK_WIDGET(vwin->btnGo),GDK_BUTTON1_MASK|GDK_BUTTON3_MASK,velocity_dnd_targets,2,GDK_ACTION_DEFAULT|GDK_ACTION_COPY|GDK_ACTION_LINK);
	g_signal_connect(vwin->btnGo,"drag_data_get",G_CALLBACK(location_drag_data_get_callback),vwin);
	vwin->optionmenu=velocity_view_profile_option_menu_new(vwin->view->prefs->profile->name);
	g_signal_connect(vwin->optionmenu,"changed",G_CALLBACK(velocity_window_set_view_profile),vwin);
	gtk_box_pack_start(GTK_BOX(hbox),vwin->optionmenu,FALSE,FALSE,0);
	gtk_widget_show_all(GTK_WIDGET(hbox));
	vwin->hpane=gtk_hpaned_new();
	gnome_app_set_contents(GNOME_APP(vwin->widget),vwin->hpane);
	gtk_paned_set_position(GTK_PANED(vwin->hpane),vprefs->window->treeview_size);
	vwin->tree=velocity_tree_new((GCompareFunc)strcasecmp,(velocity_tree_item_mod_func)velocity_window_tree_item_modfunc,
		(velocity_tree_item_select_func)velocity_window_tree_cb,(gpointer)vwin);
	gtk_paned_pack1(GTK_PANED(vwin->hpane),vwin->tree->widget,FALSE,TRUE);	
	/*Set up our velocity_view component*/
	gtk_paned_pack2(GTK_PANED(vwin->hpane),vwin->view->widget,TRUE,TRUE);
	gnome_app_set_statusbar(GNOME_APP(vwin->widget),gnome_appbar_new(TRUE,TRUE,GNOME_PREFERENCES_NEVER));
	g_signal_connect(vwin->btnGo,"clicked",G_CALLBACK(on_btnGo_clicked),vwin);

	{
		GnomeVFSMonitorHandle*h;
		gnome_vfs_monitor_add(&h,vprefs->locations->home,GNOME_VFS_MONITOR_DIRECTORY,(GnomeVFSMonitorCallback)testing,NULL);
	}
	gtk_widget_show_all(vwin->widget);
	gtk_window_set_focus(GTK_WINDOW(vwin->widget),vwin->view->widget);	/*This keeps the focus off of the toolbar (looks ugly)*/

	return(vwin);
}
/*void velocity_window_set_theme(velocity_window*vwin,const char*newtheme){
	velocity_view_set_theme(vwin->view,newtheme);
	//TODO:  We need to be sure to save the toolbar's position and orientation eventually...
	gtk_widget_destroy(GTK_WIDGET(gnome_app_get_dock_item_by_name(GNOME_APP(vwin->widget),GNOME_APP_MENUBAR_NAME)));
	GNOME_APP(vwin->widget)->menubar=NULL;	//gtk_widget_destroy doesn't do this and gnome_app_set_menus wants it
	velocity_window_menubar_create(vwin);
	gtk_widget_destroy(GTK_WIDGET(gnome_app_get_dock_item_by_name(GNOME_APP(vwin->widget),GNOME_APP_TOOLBAR_NAME)));
	velocity_window_toolbar_create(vwin);
	//TODO:  we should traverse the tree here to update the theme...
	//gtk_widget_show_all(vwin->toolbar);
}*/
void velocity_window_update_location_entry(velocity_window*vwin,const char*location){
	GnomeVFSURI*safeguri=gnome_vfs_uri_new(location);
	char*safeuri=gnome_vfs_uri_to_string(safeguri,GNOME_VFS_URI_HIDE_USER_NAME|GNOME_VFS_URI_HIDE_PASSWORD);
	char*msafeuri=velocity_vfs_uri_reformat(safeuri);
	char*uemsafeuri=gnome_vfs_unescape_string(msafeuri,"/");
	char*newlocation;
	if(vwin->view->shellexp){
		newlocation=g_strconcat(uemsafeuri,vwin->view->shellexp,NULL);
	}else{
		newlocation=g_strdup(uemsafeuri);
	}
	gnome_vfs_uri_unref(safeguri);
	gtk_entry_set_text(GTK_ENTRY(vwin->location),newlocation);
	g_free(newlocation);
	g_free(uemsafeuri);
	g_free(safeuri);
	g_free(msafeuri);
}
void velocity_window_set_location(velocity_window*vwin,const char*uri){	/*And now the good stuff...*/
	gboolean update=TRUE;
	/*If we were using a module for the view call its destructor here*/
	if(vwin->viewer){
		const char*substr=vwin->view->clocation+strlen(VELOCITY_MODULE_URI_PREFIX);
		GList*n=velocity_module_tracker_get_list();
		while((n)&&(vwin->viewer)){
			if(n->data){
				velocity_module*vmod=(velocity_module*)n->data;
				char*t=g_strconcat(vmod->info->id,":",NULL);
				int l=strlen(t);
				if(strncmp(substr,t,l)==0){
					if(vmod->on_viewer_free){
						vmod->on_viewer_free(vwin->viewer);
					}
					if(vwin->viewer)gtk_widget_destroy(vwin->viewer);
					if(vwin->view->widget)gtk_paned_pack2(GTK_PANED(vwin->hpane),vwin->view->widget,TRUE,TRUE);
					/*See the comment below about gtk_container_remove and "floating" widgets*/
					gtk_widget_unref(vwin->view->widget);
					if((uri)&&(strcmp(vwin->lasturi,uri)==0)){
						update=FALSE;
						g_free(vwin->view->clocation);
						vwin->view->clocation=g_strdup(uri);
					}
					vwin->viewer=NULL;
					g_free(vwin->lasturi);
					vwin->lasturi=NULL;
				}
				g_free(t);
			}
			n=n->next;
		}
	}
	/*In case we need to use a module here*/
	if((uri)&&(strncmp(uri,VELOCITY_MODULE_URI_PREFIX,strlen(VELOCITY_MODULE_URI_PREFIX))==0)){
		const char*substr=uri+strlen(VELOCITY_MODULE_URI_PREFIX);
		GList*n=velocity_module_tracker_get_list();
		while((n)&&(!vwin->viewer)){
			if(n->data){
				velocity_module*vmod=(velocity_module*)n->data;
				char*t=g_strconcat(vmod->info->id,":",NULL);
				int l=strlen(t);
				if(strncmp(substr,t,l)==0){
					if(vmod->on_viewer_new){
						char*vuri=gnome_vfs_unescape_string(substr+l,GNOME_VFS_URI_PATH_STR);
						vwin->viewer=vmod->on_viewer_new(vwin,vuri);
						if(vwin->viewer){
							/*We must ref here because gtk_container_remove will try to unref it if it's "floating"*/
							gtk_widget_ref(vwin->view->widget);
							velocity_window_clear_selection(vwin);
							gtk_container_remove(GTK_CONTAINER(vwin->hpane),vwin->view->widget);
							gtk_paned_pack2(GTK_PANED(vwin->hpane),vwin->viewer,TRUE,TRUE);
							vwin->lasturi=g_strdup(vwin->view->clocation);
							velocity_window_history_append_uri(vwin,uri);
							vwin->view->clocation=g_strdup(uri);
							gtk_entry_set_text(GTK_ENTRY(vwin->location),vuri);
						}
					}
				}
				g_free(t);
			}
			n=n->next;
		}
	}else{
		if(update){
			char*furi=velocity_vfs_uri_from_input(uri);
			velocity_view_set_location(vwin->view,furi);
			if(furi)g_free(furi);
		}else{
			velocity_window_update_location_entry(vwin,uri);
		}
	}
}
void velocity_window_show_lasso(velocity_window*vwin,gfloat x1,gfloat y1,gfloat x2,gfloat y2){
	velocity_view_show_lasso(vwin->view,x1,y1,x2,y2);
}
void velocity_window_hide_lasso(velocity_window*vwin){
	velocity_view_hide_lasso(vwin->view);
}
void velocity_window_clear_selection(velocity_window*vwin){
	velocity_view_clear_selection(vwin->view);
}
typedef struct{
	char*uri;
	GdkPoint scroll;
}velocity_window_history_data;
static velocity_window_history_data*velocity_window_history_data_new(const char*uri,int scrollx,int scrolly){
	velocity_window_history_data*vwhd=g_new(velocity_window_history_data,1);
	g_assert(vwhd!=NULL);
	vwhd->uri=g_strdup(uri);
	vwhd->scroll.x=scrollx;
	vwhd->scroll.y=scrolly;
	return(vwhd);
}
static void velocity_window_history_data_free(velocity_window_history_data*vwhd){
	g_free(vwhd->uri);
	g_free(vwhd);
}
void velocity_window_history_append_uri(velocity_window*vwin,const char*uri){
	/*Check to see if we are at the top of the history stack*/
	gboolean resetptr=(vwin->history==vwin->history_ptr);
	gboolean iscurrent=FALSE;
	if((vwin->history_ptr)&&(vwin->history_ptr->data)){
		velocity_window_history_data*current=(velocity_window_history_data*)vwin->history_ptr->data;
		if(strcmp(current->uri,uri)==0)iscurrent=TRUE;
	}
	/*We shouldn't add something that we are on right now.  eg.  avoid duplicates*/
	if(!iscurrent){
		guint inh=g_list_length(vwin->history);
		if(vwin->history_ptr!=vwin->history){
			/*If we are in the middle of the list somewhere kill the old portion*/
			GList*m;
			GList*n=vwin->history_ptr->prev;
			VELOCITY_MSG("this needs reviewed and retested");
			while(n){
				g_free(n->data);
				n=n->prev;
			}
			vwin->history=vwin->history_ptr;
			if(vwin->history->prev)vwin->history->prev->next=NULL;
/*				m=n;
			while(m){
				velocity_window_history_data_free((velocity_window_history_data*)m->data);
				g_free(m->data);
				m=m->next;
			}*/
			g_list_free(n);
			vwin->history->prev=NULL;
			resetptr=TRUE;
		}
		vwin->history=g_list_prepend(vwin->history,
			(gpointer)velocity_window_history_data_new(uri,
			gtk_adjustment_get_value(gtk_scrolled_window_get_hadjustment(GTK_SCROLLED_WINDOW(vwin->view->widget))),
			gtk_adjustment_get_value(gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(vwin->view->widget)))));
		inh++;
		if(inh>VELOCITY_WINDOW_HISTORY_LENGTH){	/*if we are over the history limit free one off the end of the list*/
			GList*tail=g_list_last(vwin->history);
			velocity_window_history_data_free((velocity_window_history_data*)tail->data);
			g_free(tail->data);
			g_list_free_1(tail);
		}
	}
	if((resetptr)||(!vwin->history_ptr))vwin->history_ptr=vwin->history;
	gtk_widget_set_sensitive(velocity_toolbar_factory_get_button(vwin->toolbar,GTK_STOCK_GO_BACK),(vwin->history_ptr->next!=NULL));
	gtk_widget_set_sensitive(velocity_toolbar_factory_get_button(vwin->toolbar,GTK_STOCK_GO_FORWARD),(vwin->history_ptr->prev!=NULL));
}
void velocity_window_history_back(velocity_window*vwin){
	/*I prepend to the history table so the logic here is reversed*/
	if(vwin->history_ptr){
		vwin->history_ptr=vwin->history_ptr->next;
		if(vwin->history_ptr){
			velocity_window_history_data*vwhd=(velocity_window_history_data*)vwin->history_ptr->data;
			velocity_window_set_location(vwin,vwhd->uri);
		}
	}
}
void velocity_window_history_forward(velocity_window*vwin){
	/*I prepend to the history table so the logic here is reversed*/
	if(vwin->history_ptr){
		vwin->history_ptr=vwin->history_ptr->prev;
		if(vwin->history_ptr){
			velocity_window_history_data*vwhd=(velocity_window_history_data*)vwin->history_ptr->data;
			velocity_window_set_location(vwin,vwhd->uri);
		}
	}
}
void velocity_window_free(velocity_window*vwin){
	if(vwin){
		VELOCITY_MSG("this is probably unfinished");
		if(vwin->widget)gtk_widget_destroy(vwin->widget);
		g_free(vwin);
	}
}
