/*
 * Velocity - A GNOME File Manager
 *
 *
 * Copyright (C) (2002) (Kyle Davis) <kyled@blueriver.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <X11/Xatom.h>
#include <gdk/gdkx.h>
#include <glib.h>
#include <gnome.h>
#include <gconf/gconf.h>
#include <gconf/gconf-client.h>
#include "velocity-windowman.h"
#include "velocity-vfs.h"
#include "velocity-factories.h"
#include "velocity-module-private.h"
#include "callbacks.h"

static GList*winlist=NULL;	/*GList of velocity_window*s */
static GtkWidget*desktop_window=NULL;
static velocity_view*desktop=NULL;
static velocity_vfs_monitor*vfsmon=NULL;
typedef struct{
	velocity_vfs_monitor*monitor;
	velocity_vfs_monitor_event*event;
	gpointer tmp;
}vfs_monitor_foreach_data;

static void on_created_cb(velocity_view*vview,vfs_monitor_foreach_data*data){
	char*puri=velocity_vfs_get_uri_parent(data->event->uri);
	if(velocity_vfs_uris_match(vview->clocation,puri)){
		GnomeVFSFileInfo*vfsinfo=gnome_vfs_file_info_new();
		if(gnome_vfs_get_file_info(data->event->uri,vfsinfo,GNOME_VFS_FILE_INFO_DEFAULT|GNOME_VFS_FILE_INFO_GET_MIME_TYPE|GNOME_VFS_FILE_INFO_FOLLOW_LINKS)==GNOME_VFS_OK){
			velocity_vfs_obj*vfsobj=velocity_vfs_obj_new(vview->clocation,vfsinfo,NULL,vview);
			velocity_view_append_unsorted(vview,vfsobj);
			data->tmp=(gpointer)velocity_vfs_is_directory(vfsobj);
		}
	}
	g_free(puri);
}
static void on_deleted_view_cb(velocity_vfs_obj*vfsobj,vfs_monitor_foreach_data*data){
	velocity_view*vview=(velocity_view*)data->tmp;
	char*vuri=velocity_vfs_get_uri_unescaped(vfsobj);
	if(velocity_vfs_uris_match(data->event->uri,vuri)){
		velocity_icon_hide(vfsobj->icon);
		if(velocity_vfs_is_directory(vfsobj)){
			vview->total.dirs--;
			vview->read.dirs--;
		}else{
			vview->total.files--;
			vview->read.files--;
		}
//		velocity_vfs_obj_free(vfsobj);
		vview->total.objects--;
		vview->read.objects--;
	}
	g_free(vuri);
}
static void on_deleted_cb(velocity_view*vview,vfs_monitor_foreach_data*data){
	data->tmp=(gpointer)vview;
	velocity_view_foreach(vview,(GFunc)on_deleted_view_cb,(gpointer)data);
}
static void vfs_monitor_on_event_cb(velocity_vfs_monitor*monitor,velocity_vfs_monitor_event*event,gpointer user_data){
	vfs_monitor_foreach_data*tmp=g_new(vfs_monitor_foreach_data,1);
	tmp->monitor=monitor;
	tmp->event=event;
	VELOCITY_MSG("Got VFS Monitor event(%i) for (%s)",event->type,event->uri);
	switch(event->type){
		case GNOME_VFS_MONITOR_EVENT_CREATED:
			tmp->tmp=(gpointer)FALSE;
			velocity_windowman_foreach(TRUE,(GFunc)on_created_cb,(gpointer)tmp);
			if(tmp->tmp)velocity_windowman_trees_register(event->uri);
			break;
		case GNOME_VFS_MONITOR_EVENT_DELETED:
			velocity_windowman_foreach(TRUE,(GFunc)on_deleted_cb,(gpointer)tmp);
			velocity_windowman_trees_unregister(event->uri);
			break;
		default:
			VELOCITY_MSG("Unhandled VFS Monitor event(%i)!",event->type);
			break;
	}
	g_free(tmp);
}
void velocity_windowman_new(const char*uri){
	velocity_window*vwin=velocity_window_new();
	if(!vfsmon)vfsmon=velocity_vfs_monitor_new(NULL,(velocity_vfs_monitor_on_event_cb)vfs_monitor_on_event_cb,NULL);
	winlist=g_list_prepend(winlist,vwin);
	velocity_window_set_location(vwin,uri);
}
GList*velocity_windowman_list(){
	return(winlist);
}
void velocity_windowman_set_all(velocity_windowman_set_type set){	//TODO:  This needs a lot more work...
	velocity_window*vwin=NULL;
	GList*n=winlist;
	while(n){
		vwin=(velocity_window*)n->data;
		if(vwin){
			switch(set){
				case VELOCITY_WINDOWMAN_PASTE_OFF:
					gtk_widget_set_sensitive(velocity_toolbar_factory_get_button(vwin->toolbar,GTK_STOCK_PASTE),FALSE);
					break;
				case VELOCITY_WINDOWMAN_PASTE_ON:
					gtk_widget_set_sensitive(velocity_toolbar_factory_get_button(vwin->toolbar,GTK_STOCK_PASTE),TRUE);
					break;
				default:
					break;
			}
		}
		n=n->next;
	}
}
void velocity_windowman_trees_register(const char*uri){
	GList*nw=winlist;
	while(nw){
		if(nw->data){
			velocity_window*vwin=(velocity_window*)nw->data;
			velocity_tree_register_item(vwin->tree,uri);
		}
		nw=nw->next;
	}
}
void velocity_windowman_trees_unregister(const char*uri){
	GList*nw=winlist;
	while(nw){
		velocity_window*vwin=(velocity_window*)nw->data;
		if(vwin)velocity_tree_unregister_item(vwin->tree,uri);
		nw=nw->next;
	}
}
/*This was taken from Nautilus and modified for Velocity*/
static void set_wmspec_desktop_hint(GdkWindow*window){
	GdkAtom atom=gdk_atom_intern("_NET_WM_WINDOW_TYPE_DESKTOP",FALSE);
	gdk_property_change(window,gdk_atom_intern("_NET_WM_WINDOW_TYPE", FALSE),
		gdk_x11_xatom_to_atom(XA_ATOM),32,GDK_PROP_MODE_REPLACE,(guchar *)&atom,1);
}
/*This was taken from Nautilus and modified for Velocity*/
static void set_desktop_window_id(GtkWindow*window){
	/*Tuck the desktop windows xid in the root to indicate we own the desktop.*/
	GdkWindow*root_window=gdk_screen_get_root_window(gtk_window_get_screen(window));
	gtk_window_add_embedded_xid(window,GDK_WINDOW_XWINDOW(root_window));
}
static void velocity_windowman_desktop_realize_cb(GtkWidget*dwin,gpointer user_data){
	velocity_view*desktop=(velocity_view*)user_data;
	gdk_window_set_decorations(dwin->window,0);
	gdk_window_set_functions(dwin->window,0);
	set_wmspec_desktop_hint(dwin->window);
	set_desktop_window_id(GTK_WINDOW(dwin));
}
void velocity_windowman_desktop_enable(){
	if(!vfsmon)vfsmon=velocity_vfs_monitor_new(NULL,(velocity_vfs_monitor_on_event_cb)vfs_monitor_on_event_cb,NULL);
	if(!desktop){
		GConfClient*gcc=gconf_client_get_default();
		gchar*bg=gconf_client_get_string(gcc,"/desktop/gnome/background/picture_filename",NULL);
		GSList*pil=gconf_client_get_list(gcc,"/apps/panel/profiles/default/general/panel_id_list",GCONF_VALUE_STRING,NULL);
		GSList*n=pil;
		if(!vprefs)velocity_preferences_init();
		desktop_window=gtk_window_new(GTK_WINDOW_TOPLEVEL);
		gtk_window_set_wmclass(GTK_WINDOW(desktop_window),"desktop_window","Velocity");
		/*Set the size and position of our desktop window*/
		gtk_window_set_default_size(GTK_WINDOW(desktop_window),gdk_screen_width(),gdk_screen_height());
		/*Add our desktop handler*/
		desktop=velocity_view_new(GTK_WINDOW(desktop_window),FALSE);
		g_free(desktop->prefs);
		desktop->prefs=vprefs->desktop;
		//TODO:  These events need desktop only extentions added....
		desktop->on_status_event=(velocity_view_status_event_ucb)velocity_window_on_status_event_cb;
		desktop->on_behavior_event=(velocity_view_behavior_event_ucb)velocity_window_on_behavior_event_cb;
		gtk_container_add(GTK_CONTAINER(desktop_window),desktop->widget);
		g_signal_connect(desktop_window,"realize",G_CALLBACK(velocity_windowman_desktop_realize_cb),desktop);

		if(bg){
			VELOCITY_MSG("gnome background=%s",bg);
			velocity_view_set_background(desktop,bg);
			g_free(bg);
		}

		/*Measure what the borders should be from gnome-panel settings*/
		//TODO:  Put this in a function and return a GtkBorder
		while(n){
			gchar*pi=(gchar*)n->data;
			gchar*ts=g_strdup_printf("/apps/panel/profiles/default/panels/%s/panel_type",pi);
			gchar*ss=g_strdup_printf("/apps/panel/profiles/default/panels/%s/panel_size",pi);
			gchar*t=gconf_client_get_string(gcc,ts,NULL);	/*Type*/
			gchar*s=gconf_client_get_string(gcc,ss,NULL);	/*Size*/
			if((t)&&(s)){
				/*Size lookup arrays*/
				const gchar*sla_s[]={"panel-size-xx-small","panel-size-x-small",
					"panel-size-small","panel-size-medium",
					"panel-size-large","panel-size-x-large",
					"panel-size-xx-large",NULL};
				const gint sla_i[]={12,24,36,48,64,80,128};
				gint size;
				gint i=0;
				while(sla_s[i]){
					if(strcasecmp(sla_s[i],s)==0)break;
					i++;
				}
				size=sla_i[i];
				VELOCITY_MSG("Panel:  ID(%s), size(%i)",pi,size);
				if(strcasecmp(t,"edge-panel")==0){
					gchar*hs=g_strdup_printf("/apps/panel/profiles/default/panels/%s/panel_hide_state",pi);
					gint h=gconf_client_get_int(gcc,hs,NULL);
					if(h==0){	/*If panel is visible*/
						gchar*es=g_strdup_printf("/apps/panel/profiles/default/panels/%s/screen_edge",pi);
						gchar*e=gconf_client_get_string(gcc,es,NULL);
						if(strcasecmp("panel-edge-top",e)==0){
							desktop->border.top+=size;
						}else if(strcasecmp("panel-edge-right",e)==0){
							desktop->border.right+=size;
						}else if(strcasecmp("panel-edge-bottom",e)==0){
							desktop->border.bottom+=size;
						}else if(strcasecmp("panel-edge-left",e)==0){
							desktop->border.left+=size;
						}
						g_free(e);
						g_free(es);
					}
					g_free(hs);
				}else if(strcasecmp(t,"menu-panel")==0){
					desktop->border.top+=size;
				}
				g_free(t);
				g_free(s);
			}
			g_free(ss);
			g_free(ts);
			g_free(n->data);
			n=n->next;
		}
		g_slist_free(pil);
		VELOCITY_MSG("Panel borders(%i,%i,%i,%i)",desktop->border.top,desktop->border.right,desktop->border.bottom,desktop->border.left);
		
		gtk_widget_show_all(desktop_window);
		velocity_view_set_location(desktop,vprefs->locations->desktop);
	}
}
void velocity_windowman_desktop_disable(){
	if(desktop){
		/*When destroyed the desktop is free'd automatically*/
		gtk_widget_destroy(GTK_WIDGET(desktop_window));
		desktop=NULL;
	}
}
velocity_view*velocity_windowman_desktop(){
	return(desktop);
}
void velocity_windowman_foreach(gboolean include_desktop,GFunc func,gpointer user_data){
	if(func){
		GList*w=velocity_windowman_list();
		while(w){
			if(w->data){
				velocity_window*vwin=(velocity_window*)w->data;
				if(vwin->view)func(vwin->view,user_data);
			}
			w=w->next;
		}
		if((include_desktop)&&(velocity_windowman_desktop()))func(velocity_windowman_desktop(),user_data);
	}
}
void velocity_windowman_free(velocity_window*vwin){
	velocity_window*tstwin=NULL;
	GList*n=winlist;
	GList*nl=NULL;	/*new list*/
	vprefs->window->size.x=vwin->widget->allocation.width;
	vprefs->window->size.y=vwin->widget->allocation.height;
	vprefs->window->treeview_size=GTK_PANED(vwin->hpane)->child1_size;
	while(n){
		tstwin=(velocity_window*)n->data;
		if(tstwin){
			if(tstwin==vwin){
				velocity_window_free(vwin);
			}else{
				nl=g_list_prepend(nl,(gpointer)tstwin);
			}
		}
		n=n->next;
	}
	g_list_free(winlist);
	winlist=nl;
	if((!winlist)&&(!desktop))velocity_windowman_shutdown();
}
void velocity_windowman_free_all(){
	if(winlist){
		GList*n=winlist;
		while(n){
			if(n->data)velocity_window_free((velocity_window*)n->data);
			n=n->next;
		}
		g_list_free(winlist);
		winlist=NULL;
	}
	velocity_windowman_desktop_disable();
	velocity_windowman_shutdown();
}
void velocity_windowman_shutdown(){
	if(winlist){
		velocity_windowman_free_all();
	}else{
		velocity_image_cache_shutdown();
		velocity_preferences_sync();
		velocity_module_tracker_shutdown();
		velocity_vfs_shutdown();
		gtk_exit(0);
	}
}
