/*
 * Velocity - A GNOME File Manager
 *
 *
 * Copyright (C) (2002) (Kyle Davis) <kyled@blueriver.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "velocity-vfs.h"
#include "velocity-view.h"
#include "velocity-windowman.h"
#include "callbacks.h"
/*This is for the static modules*/
#include "velocity-module-private.h"
#include "velocity-misc.h"
#include "velocity-ditem.h"
#include "velocity-template.h"

#define VELOCITY_GNOME_CONFIG_WINDOW_COUNT	"/velocity/general/wincount"
#define VELOCITY_GNOME_CONFIG_WINDOW_URI	"%i/uri"

GList*velocity_static_templates=NULL;

/*the save_yourself handler, you can safely ignore most of the parameters, and just save your session and return TRUE*/
static int save_yourself(GnomeClient*client,int phase,GnomeSaveStyle save_style,int shutdown,GnomeInteractStyle interact_style,int fast,gpointer client_data){
	velocity_window*vwin=NULL;
	GList*n=velocity_windowman_list();
	guint c=0;
	/*get the prefix for our config*/
	const char*prefix=gnome_client_get_config_prefix(client);
	
	/*this is a "discard" command for discarding data from a saved session, usually this will work*/
	char*argv[]={"rm","-r",NULL};
	char*confstr=NULL;
	
	/* Save the state using gnome-config stuff. */
	gnome_config_push_prefix(prefix);
	while(n){
		vwin=(velocity_window*)n->data;
		if(vwin){
			//TODO:  ideas:  store scroll, selected icons, icon positions, etc.
			confstr=g_strdup_printf(VELOCITY_GNOME_CONFIG_WINDOW_URI,c);
			gnome_config_set_string(confstr,vwin->view->clocation); /*This will be a vector for all windows*/
			g_free(confstr);
			c++;
		}
		n=n->next;
	}
	gnome_config_set_int(VELOCITY_GNOME_CONFIG_WINDOW_COUNT,c);
	//TODO:  Save more settings here.... such as # open windows, and then a location for each, scroll offset would be nice, etc...
	gnome_config_pop_prefix();
	gnome_config_sync();
	
	/* Here is the real SM code. We set the argv to the parameters needed to restart/discard the session that
	we've just saved and call the gnome_session_set_*_command to tell the session manager it. */
	argv[2]=gnome_config_get_real_path(prefix);
	gnome_client_set_discard_command(client,3,argv);
	
	/* Set commands to clone and restart this application.  Note that we use the same values for both -- the
	session management code will automatically add whatever magic option is required to set the session
	id on startup. The client_data was set to the command used to start this application when save_yourself handler was connected. */
	argv[0]=(gchar*)client_data;
	gnome_client_set_clone_command(client,1,argv);
	gnome_client_set_restart_command(client,1,argv);
	
	return(TRUE);
}
static void die(GnomeClient*client,gpointer client_data){
	/*Just exit in a friendly way.  We don't need to save any state here, because the session manager
	should have sent us a save_yourself-message before.*/
	//on_exit1_activate(NULL,NULL); //TODO:  This will need to change when we allow multiple windows properly
	velocity_windowman_free_all();
}
int main(int argc,char*argv[]){
	GnomeClient*client=NULL;
	char*uri=NULL;
	char*confstr=NULL;
	guint c=0;
	guint i=0;
#ifdef ENABLE_NLS
	bindtextdomain(PACKAGE,PACKAGE_LOCALE_DIR);
	textdomain(PACKAGE);
#endif
	//TODO:  popt options are much needed...
	gnome_program_init("velocity",VERSION,LIBGNOMEUI_MODULE,argc,argv,
		GNOME_PARAM_APP_PREFIX,"",
		GNOME_PARAM_APP_SYSCONFDIR,"",
		GNOME_PARAM_APP_DATADIR,PACKAGE_DATA_DIR,
		GNOME_PARAM_APP_LIBDIR,"",
		GNOME_PARAM_HUMAN_READABLE_NAME,_("Velocity"),NULL);
	velocity_vfs_init();

	/*Get the master client, that was hopefully connected to the session manager int the 'gnome_init' call.*/
	client=gnome_master_client();
	gnome_client_set_restart_style(client,GNOME_RESTART_IF_RUNNING);

	/*Arrange to be told when something interesting happens.*/
	g_signal_connect(client,"save_yourself",G_CALLBACK(save_yourself),argv[0]);
	g_signal_connect(client,"die",G_CALLBACK(die),NULL);

	velocity_preferences_init();
	/*Load static modules*/
	velocity_module_tracker_register_static(velocity_misc_init,velocity_misc_about,velocity_misc_unload,
		&VELOCITY_STATIC_MODULE_INFO_VAR(misc),&VELOCITY_STATIC_MODULE_GUI_VAR(misc),NULL,
		(velocity_view_behavior_event_ucb)velocity_misc_on_view_behavior_event,NULL,NULL);
	velocity_module_tracker_register_static(velocity_image_init,velocity_image_about,velocity_image_unload,
		&VELOCITY_STATIC_MODULE_INFO_VAR(image),&VELOCITY_STATIC_MODULE_GUI_VAR(image),NULL,
		(velocity_view_behavior_event_ucb)velocity_image_on_view_behavior_event,
		velocity_image_viewer_new,velocity_image_viewer_free);
	velocity_module_tracker_register_static(velocity_ditem_init,velocity_ditem_about,velocity_ditem_unload,
		&VELOCITY_STATIC_MODULE_INFO_VAR(ditem),&VELOCITY_STATIC_MODULE_GUI_VAR(ditem),NULL,
		(velocity_view_behavior_event_ucb)velocity_ditem_on_view_behavior_event,
		NULL,NULL);
	/*Load static templates*/
	velocity_static_templates=g_list_prepend(velocity_static_templates,
		(gpointer)velocity_template_new(_("New Folder"),"1.0",VELOCITY_VFS_MIME_TYPE_DIRECTORY,_("New Folder"),
			_("Handles creation of Directories."),FALSE,
			(velocity_template_activate_cb)velocity_menu_newfolder_cb));
{	/*Load all modules found*/
	//TODO:  This will DEFINATELY need changed...will probably want to check against a module blacklist (aka prefs)
	char*puri[]={PACKAGE_PLUGINS_DIR GNOME_VFS_URI_PATH_STR,vprefs->locations->plugins,NULL};
	int d=0;
	while(puri[d]){
		GList*files=NULL;
		VELOCITY_MSG("Reading plugins from:  %s",puri[d]);
		if(gnome_vfs_directory_list_load(&files,puri[d],GNOME_VFS_FILE_INFO_GET_MIME_TYPE|GNOME_VFS_FILE_INFO_FORCE_FAST_MIME_TYPE|GNOME_VFS_FILE_INFO_FOLLOW_LINKS)==GNOME_VFS_OK){
			GList*n=files;
			while(n){
				if(n->data){
					GnomeVFSFileInfo*vfsinfo=(GnomeVFSFileInfo*)n->data;
					if(strcmp(vfsinfo->mime_type,"application/x-shared-library")==0){
						char*uri=g_strconcat(puri[d],vfsinfo->name,NULL);
						VELOCITY_MSG("  Loading %s",vfsinfo->name);
						velocity_module_tracker_register(uri);
						g_free(uri);
					}
					gnome_vfs_file_info_unref(vfsinfo);
				}
				n=n->next;
			}
		}else{
			VELOCITY_ERR("An error occured trying to read %s\n",puri[d]);
		}
		d++;
	}
}
	/*Enable the desktop if needed*/
	if(vprefs->handle_desktop)velocity_windowman_desktop_enable();

	if(gnome_client_get_previous_id(client)){	/*check if we are restoring our program or running it for the first time*/
		/*we are connected, we will get the prefix under which we saved our session last time and load up our data*/
		gnome_config_push_prefix(gnome_client_get_config_prefix(client));
		c=gnome_config_get_int(VELOCITY_GNOME_CONFIG_WINDOW_COUNT);
		for(i=0;i<c;i++){
			confstr=g_strdup_printf(VELOCITY_GNOME_CONFIG_WINDOW_URI,i);
			uri=gnome_config_get_string(confstr);
			velocity_windowman_new(uri);
			g_free(confstr);
		}
		gnome_config_pop_prefix();
	}else{	/*we are not connected to any session manager, continue normally*/
		//TODO:  we should be able to deal with relative paths on the command line...come on....
		if(argc>1){
			guint i=1;
			while(i<argc){
				if((argv[i][0]!='-')&&(argv[i][1]!='-')){
					VELOCITY_MSG("User specified: %s",argv[i]);
					velocity_windowman_new(argv[i]);
				}
				i++;
			}
		}else{
			/*if needed, open new window in users home directory*/
			if(!velocity_windowman_desktop())
				velocity_windowman_new(NULL);
		}
	}

	gtk_main();
	return(0);
}
